/*****************************************************************************
 *  Rohde&Schwarz DOCSIS Signal Analyzer instrument driver
 *  VXIpnp Instrument Driver
 *  Original Release: July 2021
 *  By: John Reid, Petr Kom�nek
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *  Modification History:
 *    see ChangeLog
 *****************************************************************************/
#include "rsdsa.h"
#include <math.h>
#include <float.h>
#include "rsdsa_utility.h"
#include <visatype.h>
#if defined (_CVI_)
#include <utility.h>
#endif

 /*****************************************************************************
  * Function: rsdsa_init
  * Purpose:  VXIplug&play required function. Calls the
  *           rsdsa_InitWithOptions function.
  *****************************************************************************/
ViStatus rsdsa_init(ViRsrc     resourceName,
    ViBoolean  IDQuery,
    ViBoolean  resetDevice,
    ViSession* newInstrSession)
{
    ViStatus error = VI_SUCCESS;

    if (newInstrSession == NULL)
    {
        (void)RsCore_SetErrorInfo(0, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
            VI_ERROR_PARAMETER4, "Null address for Instrument Handle");

        checkErr(RS_ERROR_INVALID_PARAMETER);
    }

    checkErr(rsdsa_InitWithOptions(resourceName, IDQuery, resetDevice, "", newInstrSession));

Error:
    return error;
}

/*****************************************************************************
 * Function: rsdsa_InitWithOptions
 * Purpose:  This function creates a new RS session and calls the
 *           RsInit function.
 *****************************************************************************/
ViStatus rsdsa_InitWithOptions(ViRsrc     resourceName,
    ViBoolean  IDQuery,
    ViBoolean  resetDevice,
    ViString   optionString,
    ViSession* newInstrSession)
{
    ViStatus            error = VI_SUCCESS;
    RsCoreAttributePtr* attrList = NULL;
    ViSession           instrSession = 0;
    RsCoreSessionPtr    sessionProperties = NULL;

    if (newInstrSession == NULL)
    {
        (void)RsCore_SetErrorInfo(0, VI_FALSE, RS_ERROR_INVALID_PARAMETER, VI_ERROR_PARAMETER5,
            "Null address for Instrument Handle");
        checkErr(RS_ERROR_INVALID_PARAMETER);
    }

    attrList = (RsCoreAttributePtr*)rsdsa_RsCoreAttributeList;

    *newInstrSession = 0;

    checkErr(RsCore_NewSpecificDriver(
        resourceName, // VISA Resource Name
        "rsdsa", // driver prefix
        optionString, // Init Options string, is applied to the session settings
        attrList, // List of all attributes from the rsxxx_AttrPropertiesList
        rsdsa_RsCoreAttributeListLength, // Length of attributes list
        VI_FALSE, //idnModelFullName Model has the full *IDN? query name e.g. true: "DSA101210", VI_FALSE: "DSA" Used in CheckInstrument()
        0, // WriteDelay
        0, // ReadDelay
        100000, // IO Segment Size
        RS_VAL_OPCWAIT_OPCQUERY, // OPC Wait Mode
        RSDSA_OPC_TIMEOUT, // OPC timeout
        15000, // VISA Timeout
        600000, // Self-test timeout
        RS_VAL_BIN_FLOAT_FORMAT_SINGLE_4BYTES, // BinaryFloatNumbersFormat
        RS_VAL_BIN_INTEGER_FORMAT_INT32_4BYTES, // binaryIntegerNumbersFormat
        &instrSession));

    checkErr(RsCore_GetRsSession(instrSession, &sessionProperties));

    // Minimal allowed rscore version is 4.2.0
    checkErr(RsCore_CheckMinimalEngineVersion(instrSession, 4, 2, 0));

    // No SCPI command has been sent yet.
    // Call viClear before sending any SCPI command
    checkErr(RsCore_ViClear(instrSession));

    /* --- Here perform settings that are default for this driver,
    but can be overwritten by the optionsString settings */

    sessionProperties->autoSystErrQuery = VI_FALSE;

    // Parse option string and optionally sets the initial state of the following session attributes
    checkErr(RsCore_ApplyOptionString(instrSession, optionString));
    checkErr(RsCore_BuildRepCapTable(instrSession, rsdsa_RsCoreRepCapTable, rsdsa_RsCoreRepCapTableCount));

    // Query *IDN? string, parse it and set the following attributes:
    // -RS_ATTR_INSTRUMENT_MANUFACTURER
    // - RS_ATTR_INSTRUMENT_MODEL - based on the idnModelFullName true: "SMW200A", false : "SMW"
    // - RS_ATTR_INSTRUMENT_FIRMWARE_REVISION
    checkErr(RsCore_QueryAndParseIDNstring(instrSession, RSDSA_SIMULATION_ID_QUERY, NULL));

    // Query *OPT? string, parse the options, remove the duplicates, sort them and store the result string to RS_ATTR_OPTIONS_LIST
    checkErr(RsCore_QueryAndParseOPTstring(instrSession, RSDSA_SIMULATION_OPT_QUERY, RS_VAL_OPTIONS_PARSING_AUTO));

    // Default Instrument Setup + optional *RST
    if (resetDevice == VI_TRUE)
    {
        checkErr(rsdsa_Reset(instrSession));
    }
    else
    {
        checkErr(rsdsa_DefaultInstrSetup(instrSession));
    }

    if (IDQuery == VI_TRUE)
    {
        checkErr(RsCore_FitsIDNpattern(instrSession, RSDSA_VALID_ID_RESPONSE_STRING, NULL));
    }

    checkErr(rsdsa_CheckStatus(instrSession));

Error:
    RsCore_AfterErrorInInit(instrSession, sessionProperties, error, newInstrSession);
    return error;
}

/// HIFN  Enables the downstream EuroDOCSIS 3.0 (DOCSIS 3.0 SC-QAM J.83 Annex A)
/// HIFN  mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_EnableEuroDOCSIS30Mode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_ENABLE_EURODOCSIS30_MODE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the demodulator lock status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR demodulatorLockStatus/Queries the demodulator lock status.
ViStatus rsdsa_ED30QueryDemodulatorLockStatus(ViSession instrSession,
                                              ViInt32*  demodulatorLockStatus)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_ED30_QUERY_DEMODULATOR_LOCK_STATUS, demodulatorLockStatus),
            2, "Demodulator Lock Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the decoder lock status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR decoderLockStatus/Queries the decoder lock status.
ViStatus rsdsa_ED30QueryDecoderLockStatus(ViSession instrSession,
                                          ViInt32*  decoderLockStatus)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_ED30_QUERY_DECODER_LOCK_STATUS, decoderLockStatus),
            2, "Decoder Lock Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the MPEG lock status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mpegLockStatus/Queries the MPEG lock status.
ViStatus rsdsa_ED30QueryMPEGLockStatus(ViSession instrSession,
                                       ViInt32*  mpegLockStatus)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_ED30_QUERY_MPEG_LOCK_STATUS, mpegLockStatus),
            2, "MPEG Lock Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the input signal power level in the units selected by
/// HIFN  RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR signalPower/Queries the input signal power level in the units selected by
/// HIPAR signalPower/RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_ED30QuerySignalPower(ViSession instrSession,
                                    ViReal64* signalPower)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_ED30_QUERY_SIGNAL_POWER, signalPower),
            2, "Signal Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the frequency offset in Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequencyOffset/Queries the frequency offset in Hz.
ViStatus rsdsa_ED30QueryFrequencyOffset(ViSession instrSession,
                                        ViReal64* frequencyOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_ED30_QUERY_FREQUENCY_OFFSET, frequencyOffset),
            2, "Frequency Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the symbol offset in Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR symbolClockOffset/Queries the symbol offset in Hz.
ViStatus rsdsa_ED30QuerySymbolClockOffset(ViSession instrSession,
                                          ViReal64* symbolClockOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_ED30_QUERY_SYMBOL_CLOCK_OFFSET, symbolClockOffset),
            2, "Symbol Clock Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the channel bit rate in Mbps (megabits per second).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelBitRate/Queries the channel bit rate in Mbps (megabits per second).
ViStatus rsdsa_ED30QueryChannelBitRate(ViSession instrSession,
                                       ViReal64* channelBitRate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_ED30_QUERY_CHANNEL_BIT_RATE, channelBitRate),
            2, "Channel Bit Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the interleaver type for the DOCSIS 3.0 downstream signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR interleaver/Queries the interleaver type for the DOCSIS 3.0 downstream signal.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_ED30QueryInterleaver(ViSession instrSession,
                                    ViInt32   bufferSize,
                                    ViChar    interleaver[],
                                    ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL,
        RSDSA_ATTR_ED30_QUERY_INTERLEAVER,
        RS_VAL_DIRECT_USER_CALL,
        bufferSize, interleaver, bytesRead),
        3, "Interleaver");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activate or deactivate the MER compensation function. Activating this
/// HIFN  feature interrupts the signal analysis. Compensation can be disabled by
/// HIFN  setting this to false, or it may be terminated automatically for
/// HIFN  signals when the MER changes substantially.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Activate or deactivate the MER compensation function. Activating this
/// HIPAR enabled/feature interrupts the signal analysis. Compensation can be disabled by
/// HIPAR enabled/setting this to false, or it may be terminated automatically for
/// HIPAR enabled/signals when the MER changes substantially.
ViStatus rsdsa_ED30ConfigureMERCompensationEnabled(ViSession instrSession,
                                                   ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_ED30_CONFIGURE_MER_COMPENSATION_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the sign preferred for MER numerical display and plot axis
/// HIFN  labels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merSign/Selects the sign preferred for MER numerical display and plot axis
/// HIPAR merSign/labels.
ViStatus rsdsa_ED30ConfigureMERSign(ViSession instrSession,
                                    ViInt32   merSign)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_ED30_CONFIGURE_MER_SIGN, merSign),
            2, "MER Sign");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Query whether MER compensation is currently active for the overall RMS
/// HIFN  MER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rmsCompensatedOverall/Query whether MER compensation is currently active for the overall RMS
/// HIPAR rmsCompensatedOverall/MER.
ViStatus rsdsa_ED30QueryMERRMSCompensatedOverall(ViSession  instrSession,
                                                 ViBoolean* rmsCompensatedOverall)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_ED30_QUERY_MER_RMS_COMPENSATED_OVERALL, rmsCompensatedOverall),
            2, "RMS Compensated Overall");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall RMS equalized MER. For DOCSIS 3.0 downstream
/// HIFN  modes, this is the only RMS MER status applicable.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rmsEqualized/Queries the overall RMS equalized MER. For DOCSIS 3.0 downstream
/// HIPAR rmsEqualized/modes, this is the only RMS MER status applicable.
ViStatus rsdsa_ED30QueryMERRMSEqualized(ViSession instrSession,
                                        ViReal64* rmsEqualized)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_ED30_QUERY_MER_RMS_EQUALIZED, rmsEqualized),
            2, "RMS Equalized");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall peak (lowest) MER. For DOCSIS 3.0 downstream
/// HIFN  modes, this is the only peak MER status applicable.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merPeak/Queries the overall peak (lowest) MER. For DOCSIS 3.0 downstream
/// HIPAR merPeak/modes, this is the only peak MER status applicable.
ViStatus rsdsa_ED30QueryMERPeak(ViSession instrSession,
                                ViReal64* merPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_ED30_QUERY_MER_PEAK, merPeak),
            2, "MER Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the settings for error analysis. Changing error analysis
/// HIFN  settings will reset the error analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR errorAnalysisMode/Specifies the mode of BER measurement.
/// HIPAR manualTestDepth/Defines the test depth of the minimum BER integration. The actual test
/// HIPAR manualTestDepth/depth setting may be automatically changed in automatic error analysis
/// HIPAR manualTestDepth/mode.
/// HIPAR berSource/Selects the BER source used for BER measurements.
/// HIPAR testDepthProgressionSource/Selects the BER source used for test depth progression.
ViStatus rsdsa_ED30ConfigureErrorAnalysis(ViSession instrSession,
                                          ViInt32   errorAnalysisMode,
                                          ViInt32   manualTestDepth,
                                          ViInt32   berSource,
                                          ViInt32   testDepthProgressionSource)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_ED30_CONFIGURE_ERROR_ANALYSIS_MODE, errorAnalysisMode),
            2, "Error Analysis Mode");
    if (errorAnalysisMode == RSDSA_VAL_ERROR_ANALYSIS_MANUAL)
    {
        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_ED30_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH, manualTestDepth),
                3, "Manual Test Depth");
    }
    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_ED30_CONFIGURE_ERROR_ANALYSIS_BER_SOURCE, berSource),
            4, "BER Source");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_ED30_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH_PROGRESSION_SOURCE, testDepthProgressionSource),
            5, "Test Depth Progression Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Performs a BER reset that clears all related counters to zero and
/// HIFN  restarts the BER/CER measurement for the current mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_ED30ResetErrorAnalysis(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_ED30_RESET_ERROR_ANALYSIS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the elapsed time of the current BER/CER measurement: time
/// HIFN  elapsed since the start of the mode, or last BER reset
/// HIFN  (RSDSA_ATTR_ED30_RESET_ERROR_ANALYSIS), or last BER configuration
/// HIFN  change, in seconds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR elapsedTime/Queries the elapsed time of the current BER/CER measurement: time
/// HIPAR elapsedTime/elapsed since the start of the mode, or last BER reset
/// HIPAR elapsedTime/(RSDSA_ATTR_ED30_RESET_ERROR_ANALYSIS), or last BER configuration
/// HIPAR elapsedTime/change, in seconds.
ViStatus rsdsa_ED30QueryBERElapsedTime(ViSession instrSession,
                                       ViReal64* elapsedTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_ED30_QUERY_BER_ELAPSED_TIME, elapsedTime),
            2, "Elapsed Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the percentage of blocks processed for the current BER/CER
/// HIFN  measurement: blocks processed since the start of the mode, or last BER
/// HIFN  reset (CALCulate:[<Mode>:][AVERage:]BERatio:CLEar), or last BER
/// HIFN  configuration change. Expressed as the percentage of blocks processed
/// HIFN  relative to the current test depth. A block is 10^7 bits maximum.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR blocksProcessed/Queries the percentage of blocks processed for the current BER/CER
/// HIPAR blocksProcessed/measurement: blocks processed since the start of the mode, or last BER
/// HIPAR blocksProcessed/reset (CALCulate:[<Mode>:][AVERage:]BERatio:CLEar), or last BER
/// HIPAR blocksProcessed/configuration change. Expressed as the percentage of blocks processed
/// HIPAR blocksProcessed/relative to the current test depth. A block is 10^7 bits maximum.
ViStatus rsdsa_ED30QueryBERBlocksProcessed(ViSession instrSession,
                                           ViReal64* blocksProcessed)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_ED30_QUERY_BER_BLOCKS_PROCESSED, blocksProcessed),
            2, "Blocks Processed");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the pre-Reed-Solomon BER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR berPreRS/Queries the pre-Reed-Solomon BER.
ViStatus rsdsa_ED30QueryBERPreReedSolomon(ViSession instrSession,
                                          ViReal64* berPreRS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_ED30_QUERY_BER_PRE_REED_SOLOMON, berPreRS),
            2, "BER Pre-RS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of bits analyzed at the input of the
/// HIFN  Reed-Solomon decoder for the BER measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR preRSTotalBits/Queries the total number of bits analyzed at the input of the
/// HIPAR preRSTotalBits/Reed-Solomon decoder for the BER measurement.
ViStatus rsdsa_ED30QueryBERPreReedSolomonTotalBits(ViSession instrSession,
                                                   ViReal64* preRSTotalBits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_ED30_QUERY_BER_PRE_REED_SOLOMON_TOTAL_BITS, preRSTotalBits),
            2, "Pre-RS Total Bits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the post-Reed-Solomon BER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR berPostRS/Queries the post-Reed-Solomon BER.
ViStatus rsdsa_ED30QueryBERPostReedSolomon(ViSession instrSession,
                                           ViReal64* berPostRS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_ED30_QUERY_BER_POST_REED_SOLOMON, berPostRS),
            2, "BER Post-RS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of bits analyzed at the output of the
/// HIFN  Reed-Solomon decoder for the BER measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR postRSTotalBits/Queries the total number of bits analyzed at the output of the
/// HIPAR postRSTotalBits/Reed-Solomon decoder for the BER measurement.
ViStatus rsdsa_ED30QueryBERPostReedSolomonTotalBits(ViSession instrSession,
                                                    ViReal64* postRSTotalBits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_ED30_QUERY_BER_POST_REED_SOLOMON_TOTAL_BITS, postRSTotalBits),
            2, "Post-RS Total Bits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of corrected bits at the output of the
/// HIFN  Reed-Solomon decoder for the BER measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR postRSCorrectedBits/Queries the total number of corrected bits at the output of the
/// HIPAR postRSCorrectedBits/Reed-Solomon decoder for the BER measurement.
ViStatus rsdsa_ED30QueryBERPostReedSolomonCorrectedBits(ViSession instrSession,
                                                        ViReal64* postRSCorrectedBits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_ED30_QUERY_BER_POST_REED_SOLOMON_CORRECTED_BITS, postRSCorrectedBits),
            2, "Post-RS Corrected Bits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of uncorrected bits at the output of the
/// HIFN  Reed-Solomon decoder for the BER measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR postRSUncorrectedBits/Queries the total number of uncorrected bits at the output of the
/// HIPAR postRSUncorrectedBits/Reed-Solomon decoder for the BER measurement.
ViStatus rsdsa_ED30QueryBERPostReedSolomonUncorrectedBits(ViSession instrSession,
                                                          ViReal64* postRSUncorrectedBits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_ED30_QUERY_BER_POST_REED_SOLOMON_UNCORRECTED_BITS, postRSUncorrectedBits),
            2, "Post-RS Uncorrected Bits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the corrected CER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cerCorrected/Queries the corrected CER.
ViStatus rsdsa_ED30QueryCERCorrected(ViSession instrSession,
                                     ViReal64* cerCorrected)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_ED30_QUERY_CER_CORRECTED, cerCorrected),
            2, "CER Corrected");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the uncorrected CER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cerUncorrected/Queries the uncorrected CER.
ViStatus rsdsa_ED30QueryCERUncorrected(ViSession instrSession,
                                       ViReal64* cerUncorrected)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_ED30_QUERY_CER_UNCORRECTED, cerUncorrected),
            2, "CER Uncorrected");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of codewords for the CER measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR totalCodewords/Queries the total number of codewords for the CER measurement.
ViStatus rsdsa_ED30QueryCERTotalCodewords(ViSession instrSession,
                                          ViReal64* totalCodewords)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_ED30_QUERY_CER_TOTAL_CODEWORDS, totalCodewords),
            2, "Total Codewords");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of corrected CER codewords.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR correctedCodewords/Queries the total number of corrected CER codewords.
ViStatus rsdsa_ED30QueryCERCorrectedCodewords(ViSession instrSession,
                                              ViReal64* correctedCodewords)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_ED30_QUERY_CER_CORRECTED_CODEWORDS, correctedCodewords),
            2, "Corrected Codewords");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of uncorrected CER codewords.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR uncorrectedCodewords/Queries the total number of uncorrected CER codewords.
ViStatus rsdsa_ED30QueryCERUncorrectedCodewords(ViSession instrSession,
                                                ViReal64* uncorrectedCodewords)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_ED30_QUERY_CER_UNCORRECTED_CODEWORDS, uncorrectedCodewords),
            2, "Uncorrected Codewords");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the constellation plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_ED30DisplayConstellationPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_ED30_DISPLAY_CONSTELLATION_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the I/Q density plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_ED30DisplayIQDensityPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_ED30_DISPLAY_I_Q_DENSITY_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the measurement mode for the I/Q density plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementMode/Selects the measurement mode for the I/Q density plot.
/// HIPAR windowWidth/Specifies the sliding window width in samples for the I/Q density plot
/// HIPAR windowWidth/in sliding window mode.
ViStatus rsdsa_ED30ConfigureIQMeasurementMode(ViSession instrSession,
                                              ViInt32   measurementMode,
                                              ViInt32   windowWidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_ED30_CONFIGURE_I_Q_MEASUREMENT_MODE, measurementMode),
            2, "Measurement Mode");
    if (measurementMode == RSDSA_VAL_IQ_MEASUREMENT_MODE_SLIDING_WINDOW)
    {
        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_ED30_CONFIGURE_I_Q_WINDOW_WIDTH, windowWidth),
                3, "Window Width");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the zoom setting for the I/Q density plot display. Note that
/// HIFN  the x and y origin are the top left corner of the window, as per the
/// HIFN  GUI settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR x/X origin (left-most window point), between 0 and 1.
/// HIPAR y/Y origin (top-most window point), between 0 and 1.
/// HIPAR zoom/Zoom setting, 1 to 128.
ViStatus rsdsa_ED30SetIQDisplayZoom(ViSession instrSession,
                                    ViReal64  x,
                                    ViReal64  y,
                                    ViInt32   zoom)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_CheckRangeTableRangeViReal64(instrSession, &RsDSA_rngIQZoomOrigin, x),
        2, "X");
    viCheckParm(rsdsa_CheckRangeTableRangeViReal64(instrSession, &RsDSA_rngIQZoomOrigin, y),
        3, "Y");
    viCheckParm(rsdsa_CheckRangeTableRangeViInt32(instrSession, &RsDSA_rngIQZoomLevel, zoom),
        4, "Zoom");

    snprintf(param, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%f,%f,%ld", x, y, zoom);

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_HIDDEN_ED30_DISPLAY_I_Q_ZOOM, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Save the constellation I/Q data to the specified file. The file
/// HIFN  extension is .iq.tar. Note that there is no way to abort this command,
/// HIFN  and large file sizes could take a long time to capture depending on the
/// HIFN  input data, so use maxSamples or timeoutSeconds to limit the file size
/// HIFN  or time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/File path, uses .iq.tar extension.
/// HIPAR maxSamples/Sets the I/Q data file maximum size in samples.
/// HIPAR timeoutSeconds/Sets the maximum time in seconds to save data.
ViStatus rsdsa_ED30CaptureIQDataToFile(ViSession instrSession,
                                       ViString  fileName,
                                       ViInt64   maxSamples,
                                       ViReal64  timeoutSeconds)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt64(instrSession, NULL,
        RSDSA_ATTR_ED30_IQ_DATA_EXPORT_MAX_SAMPLES,
        maxSamples),
        3, "Max Samples");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_ED30_IQ_DATA_EXPORT_TIMEOUT, timeoutSeconds),
            4, "Timeout Seconds");

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_ED30_CAPTURE_IQ_DATA_TO_FILENAME, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the MER vs. time plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_ED30DisplayMERVsTimePlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_ED30_DISPLAY_MER_VS_TIME_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a time filter period to be used for displayed MER vs. Time
/// HIFN  plots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR period/Selects a time filter period to be used for displayed MER vs. Time
/// HIPAR period/plots.
ViStatus rsdsa_ED30ConfigureMERVsTimePlotFilterPeriod(ViSession instrSession,
                                                      ViInt32   period)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_ED30_CONFIGURE_MER_VS_TIME_PLOT_FILTER_PERIOD, period),
            2, "Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the BER vs. time plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_ED30DisplayBERVsTimePlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_ED30_DISPLAY_BER_VS_TIME_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a time filter period to be used for displayed BER vs. Time
/// HIFN  plots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR period/Selects a time filter period to be used for displayed BER vs. Time
/// HIPAR period/plots.
ViStatus rsdsa_ED30ConfigureBERVsTimePlotFilterPeriod(ViSession instrSession,
                                                      ViInt32   period)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_ED30_CONFIGURE_BER_VS_TIME_PLOT_FILTER_PERIOD, period),
            2, "Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the POWer vs. time plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_ED30DisplayPowerVsTimePlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_ED30_DISPLAY_POWER_VS_TIME_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a time filter period to be used for displayed Power vs. Time
/// HIFN  plots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR period/Selects a time filter period to be used for displayed Power vs. Time
/// HIPAR period/plots.
ViStatus rsdsa_ED30ConfigurePowerVsTimePlotFilterPeriod(ViSession instrSession,
                                                        ViInt32   period)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_ED30_CONFIGURE_POWER_VS_TIME_PLOT_FILTER_PERIOD, period),
            2, "Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays a downstream or upstream CCDF (Complementary Cumulative
/// HIFN  Distribution Function histogram as per CM-SP-PHYv3.1-I16 sections 9.3.8
/// HIFN  or 9.4.6 respectively). This command may take up to 30 seconds to
/// HIFN  execute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_ED30DisplayCCDFHistogram(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_ED30_DISPLAY_CCDF_HISTOGRAM, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the peak CCDF power level in the units selected by
/// HIFN  RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ccdfPeak/Queries the peak CCDF power level in the units selected by
/// HIPAR ccdfPeak/RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_ED30QueryCCDFPeak(ViSession instrSession,
                                 ViReal64* ccdfPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_ED30_QUERY_CCDF_PEAK, ccdfPeak),
            2, "CCDF Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the channel analysis amplitude/phase vs. subcarrier plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_ED30DisplayAmplitudePhaseVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_ED30_DISPLAY_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Scales the y-axis for the channel analysis amplitude/phase vs.
/// HIFN  subcarrier plot to best fit the measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_ED30AutoScaleAmplitudePhaseVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_ED30_AUTO_SCALE_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the channel analysis amplitude/group delay vs. subcarrier
/// HIFN  plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_ED30DisplayAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_ED30_DISPLAY_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Scales the y-axis for the channel analysis amplitude/group delay vs.
/// HIFN  subcarrier plot to best fit the measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_ED30AutoScaleAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_ED30_AUTO_SCALE_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the channel analysis echo pattern plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_ED30DisplayEchoPatternPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_ED30_DISPLAY_ECHO_PATTERN_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the specified trace data. Plot data is returned as x,y value
/// HIFN  pairs (x1,y1,x2,y2,...). 1054 points are displayed across the DSA
/// HIFN  screen. For I/Q density data, 65535 integers are always returned in row
/// HIFN  order, regardless of the zoom settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Selects repcap Trace.
/// HIPAR bufferSize/Size of output array.
/// HIPAR traceData/Queries the specified trace data. Plot data is returned as x,y value
/// HIPAR traceData/pairs (x1,y1,x2,y2,...). 1054 points are displayed across the DSA
/// HIPAR traceData/screen. For I/Q density data, 65535 integers are always returned in row
/// HIPAR traceData/order, regardless of the zoom settings.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_ED30QueryTraceData(ViSession instrSession,
                                  ViInt32   trace,
                                  ViInt32   bufferSize,
                                  ViReal64  traceData[],
                                  ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxTrace = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Trace", &maxTrace),
        2, "Trace");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, maxTrace),
            2, "Trace");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31U_QUERY_TRACE_DATA,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trace");
    viCheckParm(RsCore_GetAttributeViStringAsViReal64ArrayToUserBuffer(instrSession, repCap,
        RSDSA_ATTR_D31U_QUERY_TRACE_DATA, NULL,
        bufferSize, traceData, bytesRead),
        4, "Trace Data");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the all the messages that are currently in the acquisition log
/// HIFN  and clears the acquisition log. If the acquisition log is empty, the
/// HIFN  number 0, 'No entry', is returned. For each error in the log, the
/// HIFN  aqcuisition log entry message is returned, followed by the date and
/// HIFN  time the entry was added to the log.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR log/Queries the all the messages that are currently in the acquisition log
/// HIPAR log/and clears the acquisition log. If the acquisition log is empty, the
/// HIPAR log/number 0, 'No entry', is returned. For each error in the log, the
/// HIPAR log/aqcuisition log entry message is returned, followed by the date and
/// HIPAR log/time the entry was added to the log.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_ED30QuerySystemAcquisitionLogAll(ViSession instrSession,
                                                ViInt32   bufferSize,
                                                ViChar    log[],
                                                ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL,
        RSDSA_ATTR_SYSTEM_ACQUISITION_LOG_ALL,
        RS_VAL_DIRECT_USER_CALL,
        bufferSize, log, bytesRead),
        3, "Log");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the most recent acquisition log entry and deletes it. If the
/// HIFN  acquisition log is empty, the number 0, 'No entry', is returned.
/// HIFN  Returns the aqcuisition log entry message, followed by the date and
/// HIFN  time the entry was added to the log.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR entry/Queries the most recent acquisition log entry and deletes it. If the
/// HIPAR entry/acquisition log is empty, the number 0, 'No entry', is returned.
/// HIPAR entry/Returns the aqcuisition log entry message, followed by the date and
/// HIPAR entry/time the entry was added to the log.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_ED30QuerySystemAcquisitionLogNext(ViSession instrSession,
                                                 ViInt32   bufferSize,
                                                 ViChar    entry[],
                                                 ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL,
        RSDSA_ATTR_ED30_SYSTEM_ACQUISITION_LOG_NEXT,
        RS_VAL_DIRECT_USER_CALL,
        bufferSize, entry, bytesRead),
        3, "Entry");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the acquisition log for the number of unread entries. If the
/// HIFN  acquisition log is empty, the error number 0, 'No error', is returned.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR logCount/Queries the acquisition log for the number of unread entries. If the
/// HIPAR logCount/acquisition log is empty, the error number 0, 'No error', is returned.
ViStatus rsdsa_ED30QuerySystemAcquisitionLogCount(ViSession instrSession,
                                                  ViInt32*  logCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_ED30_SYSTEM_ACQUISITION_LOG_COUNT, logCount),
            2, "Log Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes all entries in the acquisition log.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_ED30SystemAcquisitionLogClearAll(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_ED30_SYSTEM_ACQUISITION_LOG_CLEAR_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures settings for the EuroDOCSIS 3.0 mode. Some of these
/// HIFN  settings will also affect other instrument modes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequency/Specifies the center frequency of the channel. If the spectrum
/// HIPAR frequency/location is changed by CONFigure:CHANnel:SPECtrum:FREQuency, the center
/// HIPAR frequency/frequency is adapted accordingly.
/// HIPAR constellation/Defines the constellation.
/// HIPAR spectralInversionEnabled/Turns spectral inversion on and off. This inverts the spectrum at the
/// HIPAR spectralInversionEnabled/demodulator. It does not affect any displayed spectrum.
/// HIPAR equalizerState/Controls the equalizer state.
/// HIPAR attenuation/Defines the total attenuation for RF input in dB. If you set the
/// HIPAR attenuation/attenuation manually, it is no longer coupled to the reference level,
/// HIPAR attenuation/but the reference level is coupled to the attenuation. Thus, if the
/// HIPAR attenuation/current reference level is not compatible with an attenuation that has
/// HIPAR attenuation/been set manually, the command also adjusts the reference level.
/// HIPAR attenuation/Therefore, the reference level set in
/// HIPAR attenuation/DISPlay:[<Mode>:][WINDow:]TRACe[<Trace>]:Y[:SCALe]:RLEVel is no longer
/// HIPAR attenuation/applied. Note: The query returns the actual attenuation setting, which
/// HIPAR attenuation/can also be changed by RSDSA_ATTR_ED30_AUTO_LEVEL.
/// HIPAR preamplifierEnabled/Turns the preamplifier on and off. Note: The query returns the actual
/// HIPAR preamplifierEnabled/preamplifier state (which might also be changed by
/// HIPAR preamplifierEnabled/RSDSA_ATTR_ED30_AUTO_LEVEL).
/// HIPAR asiOutputEnabled/Turns the ASI output on and off.
/// HIPAR phaseTrackingLoopBandwidth/Specifies the phase tracking loop bandwidth.
ViStatus rsdsa_ED30ConfigureGeneralSettings(ViSession instrSession,
                                            ViReal64  frequency,
                                            ViInt32   constellation,
                                            ViBoolean spectralInversionEnabled,
                                            ViInt32   equalizerState,
                                            ViReal64  attenuation,
                                            ViBoolean preamplifierEnabled,
                                            ViBoolean asiOutputEnabled,
                                            ViInt32   phaseTrackingLoopBandwidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_ED30_CONFIGURE_FREQUENCY, frequency),
            2, "Frequency");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_ED30_CONFIGURE_CONSTELLATION, constellation),
            3, "Constellation");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_ED30_CONFIGURE_SPECTRAL_INVERSION_ENABLED, spectralInversionEnabled),
            4, "Spectral Inversion Enabled");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_ED30_CONFIGURE_EQUALIZER_STATE, equalizerState),
            5, "Equalizer State");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_ED30_CONFIGURE_ATTENUATION, attenuation),
            6, "Attenuation");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_ED30_CONFIGURE_PREAMPLIFIER_ENABLED, preamplifierEnabled),
            7, "Preamplifier Enabled");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_ED30_CONFIGURE_ASI_OUTPUT_ENABLED, asiOutputEnabled),
            8, "ASI Output Enabled");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_ED30_CONFIGURE_PHASE_TRACKING_LOOP_BANDWIDTH, phaseTrackingLoopBandwidth),
            9, "Phase Tracking Loop Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the list of equalizer coefficients. A pair of signed
/// HIFN  floating-point values is returned for each coefficient, representing
/// HIFN  the real and imaginary parts. The number of coefficient pairs returned
/// HIFN  depends on the mode, the signal bandwidth, and for OFDM/OFDMA, the FFT
/// HIFN  size.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR equalizerCoefficients/Queries the list of equalizer coefficients. A pair of signed
/// HIPAR equalizerCoefficients/floating-point values is returned for each coefficient, representing
/// HIPAR equalizerCoefficients/the real and imaginary parts. The number of coefficient pairs returned
/// HIPAR equalizerCoefficients/depends on the mode, the signal bandwidth, and for OFDM/OFDMA, the FFT
/// HIPAR equalizerCoefficients/size.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_ED30QueryEqualizerCoefficients(ViSession instrSession,
                                              ViInt32   bufferSize,
                                              ViReal64  equalizerCoefficients[],
                                              ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringAsViReal64ArrayToUserBuffer(instrSession, NULL,
        RSDSA_ATTR_ED30_QUERY_EQUALIZER_COEFFICIENTS, NULL,
        bufferSize, equalizerCoefficients, bytesRead),
        3, "Equalizer Coefficients");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Automatically determines the attenuation level and preamplifier state
/// HIFN  (on/off). Query the resultant attenuation level with
/// HIFN  RSDSA_ATTR_ED30_CONFIGURE_ATTENUATION and the resultant preamplifier
/// HIFN  state with RSDSA_ATTR_ED30_CONFIGURE_PREAMPLIFIER_ENABLED.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_ED30AutoLevel(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_ED30_AUTO_LEVEL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the alarm thresholds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rmsMEROverallThreshold/Sets the alarm threshold for RMS MER overall. When this parameter
/// HIPAR rmsMEROverallThreshold/falls below the set threshold, an alarm is generated.
/// HIPAR berPreRSThreshold/Sets the alarm threshold for pre-Reed-Solomon BER. When this parameter
/// HIPAR berPreRSThreshold/exceeds the set threshold, an alarm is generated.
/// HIPAR berPostRSThreshold/Sets the alarm threshold for post-Reed-Solomon BER. When this
/// HIPAR berPostRSThreshold/parameter exceeds the set threshold, an alarm is generated.
ViStatus rsdsa_ED30ConfigureAlarmThresholds(ViSession instrSession,
                                            ViReal64  rmsMEROverallThreshold,
                                            ViReal64  berPreRSThreshold,
                                            ViReal64  berPostRSThreshold)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_ED30_ALARM_THRESHOLD_RMS_MER_OVERALL, rmsMEROverallThreshold),
            2, "RMS MER Overall Threshold");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_ED30_ALARM_THRESHOLD_BER_PRE_REED_SOLOMON, berPreRSThreshold),
            3, "BER Pre-RS Threshold");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_ED30_ALARM_THRESHOLD_BER_POST_REED_SOLOMON, berPostRSThreshold),
            4, "BER Post-RS Threshold");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the downstream DOCSIS 3.0 (DOCSIS 3.0 SC-QAM J.83 Annex B)
/// HIFN  mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_EnableDOCSIS30Mode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_ENABLE_DOCSIS30_MODE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the demodulator lock status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR demodulatorLock/Queries the demodulator lock status.
ViStatus rsdsa_QueryD30DDemodulatorLockStatus(ViSession instrSession,
                                              ViInt32*  demodulatorLock)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30D_QUERY_DEMODULATOR_LOCK_STATUS, demodulatorLock),
            2, "Demodulator Lock");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the decoder lock status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR decoderLock/Queries the decoder lock status.
ViStatus rsdsa_QueryD30DDecoderLockStatus(ViSession instrSession,
                                          ViInt32*  decoderLock)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30D_QUERY_DECODER_LOCK_STATUS, decoderLock),
            2, "Decoder Lock");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the MPEG lock status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mpegLock/Queries the MPEG lock status.
ViStatus rsdsa_QueryD30DMPEGLockStatus(ViSession instrSession,
                                       ViInt32*  mpegLock)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30D_QUERY_MPEG_LOCK_STATUS, mpegLock),
            2, "MPEG Lock");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the input signal power level in the units selected by
/// HIFN  RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR signalPower/Queries the input signal power level in the units selected by
/// HIPAR signalPower/RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_QueryD30DSignalPower(ViSession instrSession,
                                    ViReal64* signalPower)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30D_QUERY_SIGNAL_POWER, signalPower),
            2, "Signal Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the frequency offset in Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequencyOffset/Queries the frequency offset in Hz.
ViStatus rsdsa_QueryD30DFrequencyOffset(ViSession instrSession,
                                        ViReal64* frequencyOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30D_QUERY_FREQUENCY_OFFSET, frequencyOffset),
            2, "Frequency Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the symbol offset in Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR symbolClockOffset/Queries the symbol offset in Hz.
ViStatus rsdsa_QueryD30DSymbolClockOffset(ViSession instrSession,
                                          ViReal64* symbolClockOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30D_QUERY_SYMBOL_CLOCK_OFFSET, symbolClockOffset),
            2, "Symbol Clock Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the channel bit rate in Mbps (megabits per second).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelBitRate/Queries the channel bit rate in Mbps (megabits per second).
ViStatus rsdsa_QueryD30DChannelBitRate(ViSession instrSession,
                                       ViReal64* channelBitRate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30D_QUERY_CHANNEL_BIT_RATE, channelBitRate),
            2, "Channel Bit Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the interleaver type for the DOCSIS 3.0 downstream signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR interleaver/Queries the interleaver type for the DOCSIS 3.0 downstream signal.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_QueryD30DInterleaver(ViSession instrSession,
                                    ViInt32   bufferSize,
                                    ViChar    interleaver[],
                                    ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL,
        RSDSA_ATTR_D30D_QUERY_INTERLEAVER,
        RS_VAL_DIRECT_USER_CALL,
        bufferSize, interleaver, bytesRead),
        3, "Interleaver");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the MER Compensation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Activate or deactivate the MER compensation function. Activating this
/// HIPAR enabled/feature interrupts the signal analysis. Compensation can be disabled by
/// HIPAR enabled/setting this to false, or it may be terminated automatically for
/// HIPAR enabled/signals when the MER changes substantially.
/// HIPAR sign/Selects the sign preferred for MER numerical display and plot axis
/// HIPAR sign/labels.
ViStatus rsdsa_ConfigureD30DMERCompensation(ViSession instrSession,
                                            ViBoolean enabled,
                                            ViInt32   sign)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_D30D_CONFIGURE_MER_COMPENSATION_ENABLED, enabled),
            2, "Enabled");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30D_CONFIGURE_MER_SIGN, sign),
            3, "Sign");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Query whether MER compensation is currently active for the overall RMS
/// HIFN  MER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR compensatedOverall/Query whether MER compensation is currently active for the overall RMS
/// HIPAR compensatedOverall/MER.
ViStatus rsdsa_QueryD30DMERRMSCompensatedOverall(ViSession  instrSession,
                                                 ViBoolean* compensatedOverall)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_D30D_QUERY_MER_RMS_COMPENSATED_OVERALL, compensatedOverall),
            2, "Compensated Overall");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall RMS equalized MER. For DOCSIS 3.0 downstream
/// HIFN  modes, this is the only RMS MER status applicable.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rmsEqualized/Queries the overall RMS equalized MER. For DOCSIS 3.0 downstream
/// HIPAR rmsEqualized/modes, this is the only RMS MER status applicable.
ViStatus rsdsa_QueryD30DMERRMSEqualized(ViSession instrSession,
                                        ViReal64* rmsEqualized)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30D_QUERY_MER_RMS_EQUALIZED, rmsEqualized),
            2, "RMS Equalized");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall peak (lowest) MER. For DOCSIS 3.0 downstream
/// HIFN  modes, this is the only peak MER status applicable.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merPeak/Queries the overall peak (lowest) MER. For DOCSIS 3.0 downstream
/// HIPAR merPeak/modes, this is the only peak MER status applicable.
ViStatus rsdsa_QueryD30DMERPeak(ViSession instrSession,
                                ViReal64* merPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30D_QUERY_MER_PEAK, merPeak),
            2, "MER Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the BER measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Specifies the mode of BER measurement.
/// HIPAR manualTestDepth/Defines the test depth of the minimum BER integration. The query
/// HIPAR manualTestDepth/returns the actual test depth setting, which may be automatically
/// HIPAR manualTestDepth/changed in automatic BER measurement mode.
/// HIPAR berSource/Selects the BER source used for BER measurements.
/// HIPAR progressionSource/Selects the BER source used for test depth progression.
ViStatus rsdsa_ConfigureD30DErrorAnalysis(ViSession instrSession,
                                          ViInt32   mode,
                                          ViInt32   manualTestDepth,
                                          ViInt32   berSource,
                                          ViInt32   progressionSource)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30D_CONFIGURE_ERROR_ANALYSIS_MODE, mode),
            2, "Mode");
    if (mode == RSDSA_VAL_ERROR_ANALYSIS_MANUAL)
    {
        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30D_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH, manualTestDepth),
                3, "Manual Test Depth");
    }
    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30D_CONFIGURE_ERROR_ANALYSIS_BER_SOURCE, berSource),
            4, "BER Source");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30D_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH_PROGRESSION_SOURCE, progressionSource),
            5, "Progression Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the elapsed time of the current BER/CER measurement: time
/// HIFN  elapsed since the start of the mode, or last BER reset
/// HIFN  (RSDSA_ATTR_D30D_RESET_ERROR_ANALYSIS), or last BER configuration
/// HIFN  change, in seconds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR elapsedTime/Queries the elapsed time of the current BER/CER measurement: time
/// HIPAR elapsedTime/elapsed since the start of the mode, or last BER reset
/// HIPAR elapsedTime/(RSDSA_ATTR_D30D_RESET_ERROR_ANALYSIS), or last BER configuration
/// HIPAR elapsedTime/change, in seconds.
ViStatus rsdsa_QueryD30DBERElapsedTime(ViSession instrSession,
                                       ViReal64* elapsedTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30D_QUERY_BER_ELAPSED_TIME, elapsedTime),
            2, "Elapsed Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the percentage of blocks processed for the current BER/CER
/// HIFN  measurement: blocks processed since the start of the mode, or last BER
/// HIFN  reset (CALCulate:[<Mode>:][AVERage:]BERatio:CLEar), or last BER
/// HIFN  configuration change. Expressed as the percentage of blocks processed
/// HIFN  relative to the current test depth. A block is 10^7 bits maximum.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR blocksProcessed/Queries the percentage of blocks processed for the current BER/CER
/// HIPAR blocksProcessed/measurement: blocks processed since the start of the mode, or last BER
/// HIPAR blocksProcessed/reset (CALCulate:[<Mode>:][AVERage:]BERatio:CLEar), or last BER
/// HIPAR blocksProcessed/configuration change. Expressed as the percentage of blocks processed
/// HIPAR blocksProcessed/relative to the current test depth. A block is 10^7 bits maximum.
ViStatus rsdsa_QueryD30DBERBlocksProcessed(ViSession instrSession,
                                           ViReal64* blocksProcessed)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30D_QUERY_BER_BLOCKS_PROCESSED, blocksProcessed),
            2, "Blocks Processed");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Performs a BER reset that clears all related counters to zero and
/// HIFN  restarts the BER/CER measurement for the current mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30DResetErrorAnalysis(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30D_RESET_ERROR_ANALYSIS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the pre-Viterbi BER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR preViterbi/Queries the pre-Viterbi BER.
ViStatus rsdsa_QueryD30DBERPreViterbi(ViSession instrSession,
                                      ViReal64* preViterbi)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30D_QUERY_BER_PRE_VITERBI, preViterbi),
            2, "Pre-Viterbi");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of bits analyzed at the input of the Viterbi
/// HIFN  decoder for the BER measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR totalBits/Queries the total number of bits analyzed at the input of the Viterbi
/// HIPAR totalBits/decoder for the BER measurement.
ViStatus rsdsa_QueryD30DBERPreViterbiTotalBits(ViSession instrSession,
                                               ViReal64* totalBits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30D_QUERY_BER_PRE_VITERBI_TOTAL_BITS, totalBits),
            2, "Total Bits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the post-Viterbi BER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR postViterbi/Queries the post-Viterbi BER.
ViStatus rsdsa_QueryD30DBERPostViterbi(ViSession instrSession,
                                       ViReal64* postViterbi)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30D_QUERY_BER_POST_VITERBI, postViterbi),
            2, "Post-Viterbi");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of bits analyzed at the output of the Viterbi
/// HIFN  decoder for the BER measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR totalBits/Queries the total number of bits analyzed at the output of the Viterbi
/// HIPAR totalBits/decoder for the BER measurement.
ViStatus rsdsa_QueryD30DBERPostViterbiTotalBits(ViSession instrSession,
                                                ViReal64* totalBits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30D_QUERY_BER_POST_VITERBI_TOTAL_BITS, totalBits),
            2, "Total Bits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the post-Reed-Solomon BER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR postReedSolomon/Queries the post-Reed-Solomon BER.
ViStatus rsdsa_QueryD30DQueryBERPostReedSolomon(ViSession instrSession,
                                                ViReal64* postReedSolomon)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30D_QUERY_BER_POST_REED_SOLOMON, postReedSolomon),
            2, "Post-Reed-Solomon");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of corrected bits at the output of the
/// HIFN  Reed-Solomon decoder for the BER measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR correctedBits/Queries the total number of corrected bits at the output of the
/// HIPAR correctedBits/Reed-Solomon decoder for the BER measurement.
ViStatus rsdsa_QueryD30DBERPostReedSolomonCorrectedBits(ViSession instrSession,
                                                        ViReal64* correctedBits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30D_QUERY_BER_POST_REED_SOLOMON_CORRECTED_BITS, correctedBits),
            2, "Corrected Bits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of uncorrected bits at the output of the
/// HIFN  Reed-Solomon decoder for the BER measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR uncorrectedBits/Queries the total number of uncorrected bits at the output of the
/// HIPAR uncorrectedBits/Reed-Solomon decoder for the BER measurement.
ViStatus rsdsa_QueryD30DBERPostReedSolomonUncorrectedBits(ViSession instrSession,
                                                          ViReal64* uncorrectedBits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30D_QUERY_BER_POST_REED_SOLOMON_UNCORRECTED_BITS, uncorrectedBits),
            2, "Uncorrected Bits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the corrected CER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR corrected/Queries the corrected CER.
ViStatus rsdsa_QueryD30DCERCorrected(ViSession instrSession,
                                     ViReal64* corrected)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30D_QUERY_CER_CORRECTED, corrected),
            2, "Corrected");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the uncorrected CER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR uncorrected/Queries the uncorrected CER.
ViStatus rsdsa_QueryD30DCERUncorrected(ViSession instrSession,
                                       ViReal64* uncorrected)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30D_QUERY_CER_UNCORRECTED, uncorrected),
            2, "Uncorrected");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of codewords for the CER measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR totalCodewords/Queries the total number of codewords for the CER measurement.
ViStatus rsdsa_QueryD30DCERTotalCodewords(ViSession instrSession,
                                          ViReal64* totalCodewords)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30D_QUERY_CER_TOTAL_CODEWORDS, totalCodewords),
            2, "Total Codewords");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of corrected CER codewords.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR correctedCodewords/Queries the total number of corrected CER codewords.
ViStatus rsdsa_QueryD30DCERCorrectedCodewords(ViSession instrSession,
                                              ViReal64* correctedCodewords)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30D_QUERY_CER_CORRECTED_CODEWORDS, correctedCodewords),
            2, "Corrected Codewords");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of uncorrected CER codewords.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR uncorrectedCodewords/Queries the total number of uncorrected CER codewords.
ViStatus rsdsa_QueryD30DCERUncorrectedCodewords(ViSession instrSession,
                                                ViReal64* uncorrectedCodewords)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30D_QUERY_CER_UNCORRECTED_CODEWORDS, uncorrectedCodewords),
            2, "Uncorrected Codewords");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the constellation plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30DDisplayConstellationPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30D_DISPLAY_CONSTELLATION_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the I/Q density plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30DDisplayIQDensityPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30D_DISPLAY_I_Q_DENSITY_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the I/Q density plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Selects the measurement mode for the I/Q density plot.
/// HIPAR windowWidth/Specifies the sliding window width in samples for the I/Q density plot
/// HIPAR windowWidth/in sliding window mode.
ViStatus rsdsa_ConfigureD30DIQMeasurement(ViSession instrSession,
                                          ViInt32   mode,
                                          ViInt32   windowWidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30D_CONFIGURE_I_Q_MEASUREMENT_MODE, mode),
            2, "Mode");
    if (mode == RSDSA_VAL_IQ_MEASUREMENT_MODE_SLIDING_WINDOW)
    {
        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30D_CONFIGURE_I_Q_WINDOW_WIDTH, windowWidth),
                3, "Window Width");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the zoom setting for the I/Q density plot display. Note that
/// HIFN  the x and y origin are the top left corner of the window, as per the
/// HIFN  GUI settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR x/X origin (left-most window point), between 0 and 1.
/// HIPAR y/Y origin (top-most window point), between 0 and 1.
/// HIPAR zoom/Zoom setting, 1 to 128.
ViStatus rsdsa_ConfigureD30DIQDisplayZoom(ViSession instrSession,
                                          ViReal64  x,
                                          ViReal64  y,
                                          ViInt32   zoom)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_CheckRangeTableRangeViReal64(instrSession, &RsDSA_rngIQZoomOrigin, x),
        2, "X");
    viCheckParm(rsdsa_CheckRangeTableRangeViReal64(instrSession, &RsDSA_rngIQZoomOrigin, y),
        3, "Y");
    viCheckParm(rsdsa_CheckRangeTableRangeViInt32(instrSession, &RsDSA_rngIQZoomLevel, zoom),
        4, "Zoom");

    snprintf(param, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%f,%f,%ld", x, y, zoom);

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_HIDDEN_D30D_DISPLAY_I_Q_ZOOM, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Save the constellation I/Q data to the specified file. The file
/// HIFN  extension is .iq.tar. Note that there is no way to abort this command,
/// HIFN  and large file sizes could take a long time to capture depending on the
/// HIFN  input data, so use maxSamples or timeoutSeconds to limit the file size
/// HIFN  or time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filename/File path, uses .iq.tar extension.
/// HIPAR maxSamples/Sets the I/Q data file maximum size in samples.
/// HIPAR timeoutSeconds/Sets the maximum time in seconds to save data.
ViStatus rsdsa_CaptureD30DIQDataToFile(ViSession instrSession,
                                       ViString  fileName,
                                       ViInt64   maxSamples,
                                       ViReal64  timeoutSeconds)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt64(instrSession, NULL,
        RSDSA_ATTR_D30D_IQ_DATA_EXPORT_MAX_SAMPLES,
        maxSamples),
        3, "Max Samples");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30D_IQ_DATA_EXPORT_TIMEOUT, timeoutSeconds),
            4, "Timeout Seconds");

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30D_CAPTURE_IQ_DATA_TO_FILENAME, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the MER vs. time plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30DDisplayMERVsTimePlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30D_DISPLAY_MER_VS_TIME_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the BER vs. time plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30DDisplayBERVsTimePlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30D_DISPLAY_BER_VS_TIME_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the POWer vs. time plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30DDisplayPowerVsTimePlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30D_DISPLAY_POWER_VS_TIME_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a time filter period to be used for displayed MER vs. Time
/// HIFN  plots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR period/Selects a time filter period to be used for displayed MER vs. Time plots.
ViStatus rsdsa_ConfigureD30DMERVsTimePlotFilterPeriod(ViSession instrSession,
                                                      ViInt32   period)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30D_CONFIGURE_MER_VS_TIME_PLOT_FILTER_PERIOD, period),
            2, "Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a time filter period to be used for displayed BER vs. Time
/// HIFN  plots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR period/Selects a time filter period to be used for displayed BER vs. Time plots.
ViStatus rsdsa_ConfigureD30DBERVsTimePlotFilterPeriod(ViSession instrSession,
                                                      ViInt32   period)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30D_CONFIGURE_BER_VS_TIME_PLOT_FILTER_PERIOD, period),
            2, "Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a time filter period to be used for displayed Power vs. Time
/// HIFN  plots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR period/Selects a time filter period to be used for displayed Power vs. Time plots.
ViStatus rsdsa_ConfigureD30DPowerVsTimePlotFilterPeriod(ViSession instrSession,
                                                        ViInt32   period)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30D_CONFIGURE_POWER_VS_TIME_PLOT_FILTER_PERIOD, period),
            2, "Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays a downstream or upstream CCDF (Complementary Cumulative
/// HIFN  Distribution Function histogram as per CM-SP-PHYv3.1-I16 sections 9.3.8
/// HIFN  or 9.4.6 respectively). This command may take up to 30 seconds to
/// HIFN  execute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30DDisplayCCDFHistogram(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30D_DISPLAY_CCDF_HISTOGRAM, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the peak CCDF power level in the units selected by
/// HIFN  RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ccdfPeak/Queries the peak CCDF power level in the units selected by
/// HIPAR ccdfPeak/RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_QueryD30DCCDFPeak(ViSession instrSession,
                                 ViReal64* ccdfPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30D_QUERY_CCDF_PEAK, ccdfPeak),
            2, "CCDF Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the channel analysis amplitude/phase vs. subcarrier plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30DDisplayAmplitudePhaseVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30D_DISPLAY_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Scales the y-axis for the channel analysis amplitude/phase vs.
/// HIFN  subcarrier plot to best fit the measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30DAutoScaleAmplitudePhaseVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30D_AUTO_SCALE_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the channel analysis amplitude/group delay vs. subcarrier
/// HIFN  plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30DDisplayAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30D_DISPLAY_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Scales the y-axis for the channel analysis amplitude/group delay vs.
/// HIFN  subcarrier plot to best fit the measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30DAutoScaleAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30D_AUTO_SCALE_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the channel analysis echo pattern plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30DDisplayEchoPatternPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30D_DISPLAY_ECHO_PATTERN_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the specified trace data. Plot data is returned as x,y value
/// HIFN  pairs ( x1,y1,x2,y2,...). 1054 points are displayed across the DSA
/// HIFN  screen. For I/Q density data, 65535 integers are always returned in row
/// HIFN  order, regardless of the zoom settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Selects repcap Trace.
/// HIPAR bufferSize/Size of output array.
/// HIPAR traceData/Queries the specified trace data. Plot data is returned as x,y value
/// HIPAR traceData/pairs ( x1,y1,x2,y2,...). 1054 points are displayed across the DSA
/// HIPAR traceData/screen. For I/Q density data, 65535 integers are always returned in row
/// HIPAR traceData/order, regardless of the zoom settings.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_QueryD30DTraceData(ViSession instrSession,
                                  ViInt32   trace,
                                  ViInt32   bufferSize,
                                  ViReal64  traceData[],
                                  ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxTrace = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Trace", &maxTrace),
        2, "Trace");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, maxTrace),
            2, "Trace");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D30D_QUERY_TRACE_DATA,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trace");
    viCheckParm(RsCore_GetAttributeViStringAsViReal64ArrayToUserBuffer(instrSession, repCap,
        RSDSA_ATTR_D30D_QUERY_TRACE_DATA, NULL,
        bufferSize, traceData, bytesRead),
        4, "Trace Data");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the all the messages that are currently in the acquisition log
/// HIFN  and clears the acquisition log. If the acquisition log is empty, the
/// HIFN  number 0, 'No entry', is returned. For each error in the log, the
/// HIFN  aqcuisition log entry message is returned, followed by the date and
/// HIFN  time the entry was added to the log.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR log/Queries the all the messages that are currently in the acquisition log
/// HIPAR log/and clears the acquisition log. If the acquisition log is empty, the
/// HIPAR log/number 0, 'No entry', is returned. For each error in the log, the
/// HIPAR log/aqcuisition log entry message is returned, followed by the date and
/// HIPAR log/time the entry was added to the log.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_QueryD30DSystemAcquisitionLogAll(ViSession instrSession,
                                                ViInt32   bufferSize,
                                                ViChar    log[],
                                                ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL,
        RSDSA_ATTR_D30D_SYSTEM_ACQUISITION_LOG_ALL,
        RS_VAL_DIRECT_USER_CALL,
        bufferSize, log, bytesRead),
        3, "Log");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the most recent acquisition log entry and deletes it. If the
/// HIFN  acquisition log is empty, the number 0, 'No entry', is returned.
/// HIFN  Returns the aqcuisition log entry message, followed by the date and
/// HIFN  time the entry was added to the log.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR entry/Queries the most recent acquisition log entry and deletes it. If the
/// HIPAR entry/acquisition log is empty, the number 0, 'No entry', is returned.
/// HIPAR entry/Returns the aqcuisition log entry message, followed by the date and
/// HIPAR entry/time the entry was added to the log.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_QueryD30DSystemAcquisitionLogNext(ViSession instrSession,
                                                 ViInt32   bufferSize,
                                                 ViChar    entry[],
                                                 ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL,
        RSDSA_ATTR_D30D_SYSTEM_ACQUISITION_LOG_NEXT,
        RS_VAL_DIRECT_USER_CALL,
        bufferSize, entry, bytesRead),
        3, "Entry");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the acquisition log for the number of unread entries. If the
/// HIFN  acquisition log is empty, the error number 0, 'No error', is returned.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR count/Queries the acquisition log for the number of unread entries. If the
/// HIPAR count/acquisition log is empty, the error number 0, 'No error', is returned.
ViStatus rsdsa_QueryD30DSystemAcquisitionLogCount(ViSession instrSession,
                                                  ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30D_SYSTEM_ACQUISITION_LOG_COUNT, count),
            2, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes all entries in the acquisition log.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30DSystemAcquisitionLogClearAll(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30D_SYSTEM_ACQUISITION_LOG_CLEAR_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Automatically determines the attenuation level and preamplifier state
/// HIFN  (on/off). Query the resultant attenuation level with
/// HIFN  RSDSA_ATTR_D30D_CONFIGURE_ATTENUATION and the resultant preamplifier
/// HIFN  state with RSDSA_ATTR_D30D_CONFIGURE_PREAMPLIFIER_ENABLED.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30DAutoLevel(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30D_AUTO_LEVEL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the DOCSIS 3.0
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequency/Specifies the center frequency of the channel. If the spectrum
/// HIPAR frequency/location is changed by CONFigure:CHANnel:SPECtrum:FREQuency, the center
/// HIPAR frequency/frequency is adapted accordingly.
/// HIPAR constellation/Defines the constellation.
/// HIPAR spectralInversionEnabled/Turns spectral inversion on and off. This inverts the spectrum at the
/// HIPAR spectralInversionEnabled/demodulator. It does not affect any displayed spectrum.
/// HIPAR equalizerState/Controls the equalizer state.
/// HIPAR attenuation/Defines the total attenuation for RF input in dB. If you set the
/// HIPAR attenuation/attenuation manually, it is no longer coupled to the reference level,
/// HIPAR attenuation/but the reference level is coupled to the attenuation. Thus, if the
/// HIPAR attenuation/current reference level is not compatible with an attenuation that has
/// HIPAR attenuation/been set manually, the command also adjusts the reference level.
/// HIPAR attenuation/Therefore, the reference level set in
/// HIPAR attenuation/DISPlay:[<Mode>:][WINDow:]TRACe[<Trace>]:Y[:SCALe]:RLEVel is no longer
/// HIPAR attenuation/applied. Note: The query returns the actual attenuation setting, which
/// HIPAR attenuation/can also be changed by RSDSA_ATTR_ED30_AUTO_LEVEL.
/// HIPAR preamplifierEnabled/Turns the preamplifier on and off. Note: The query returns the actual
/// HIPAR preamplifierEnabled/preamplifier state (which might also be changed by
/// HIPAR preamplifierEnabled/RSDSA_ATTR_D30D_AUTO_LEVEL).
/// HIPAR asiOutputEnabled/Turns the ASI output on and off.
/// HIPAR phaseTrackingLoopBandwidth/Specifies the phase tracking loop bandwidth.
ViStatus rsdsa_ConfigureD30D(ViSession instrSession,
                             ViReal64  frequency,
                             ViInt32   constellation,
                             ViBoolean spectralInversionEnabled,
                             ViInt32   equalizerState,
                             ViReal64  attenuation,
                             ViBoolean preamplifierEnabled,
                             ViBoolean asiOutputEnabled,
                             ViInt32   phaseTrackingLoopBandwidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30D_CONFIGURE_FREQUENCY, frequency),
            2, "Frequency");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30D_CONFIGURE_CONSTELLATION, constellation),
            3, "Constellation");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_D30D_CONFIGURE_SPECTRAL_INVERSION_ENABLED, spectralInversionEnabled),
            4, "Spectral Inversion Enabled");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30D_CONFIGURE_EQUALIZER_STATE, equalizerState),
            5, "Equalizer State");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30D_CONFIGURE_ATTENUATION, attenuation),
            6, "Attenuation");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_D30D_CONFIGURE_PREAMPLIFIER_ENABLED, preamplifierEnabled),
            7, "Preamplifier Enabled");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_D30D_CONFIGURE_ASI_OUTPUT_ENABLED, asiOutputEnabled),
            8, "ASI Output Enabled");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30D_CONFIGURE_PHASE_TRACKING_LOOP_BANDWIDTH, phaseTrackingLoopBandwidth),
            9, "Phase Tracking Loop Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the list of equalizer coefficients. A pair of signed
/// HIFN  floating-point values is returned for each coefficient, representing
/// HIFN  the real and imaginary parts. The number of coefficient pairs returned
/// HIFN  depends on the mode, the signal bandwidth, and for OFDM/OFDMA, the FFT
/// HIFN  size.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR equalizerCoefficients/Queries the list of equalizer coefficients. A pair of signed
/// HIPAR equalizerCoefficients/floating-point values is returned for each coefficient, representing
/// HIPAR equalizerCoefficients/the real and imaginary parts. The number of coefficient pairs returned
/// HIPAR equalizerCoefficients/depends on the mode, the signal bandwidth, and for OFDM/OFDMA, the FFT
/// HIPAR equalizerCoefficients/size.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_QueryD30DEqualizerCoefficients(ViSession instrSession,
                                              ViInt32   bufferSize,
                                              ViReal64  equalizerCoefficients[],
                                              ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringAsViReal64ArrayToUserBuffer(instrSession, NULL,
        RSDSA_ATTR_D30D_QUERY_EQUALIZER_COEFFICIENTS, NULL,
        bufferSize, equalizerCoefficients, bytesRead),
        3, "Equalizer Coefficients");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the DOCSIS 3.0 Alarm Threshold.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rmsMEROverall/Sets the alarm threshold for RMS MER overall. When this parameter
/// HIPAR rmsMEROverall/falls below the set threshold, an alarm is generated.
/// HIPAR berPreViterbi/Sets the alarm threshold for pre-Viterbi BER. When this parameter
/// HIPAR berPreViterbi/exceeds the set threshold, an alarm is generated.
/// HIPAR berPostViterbi/Sets the alarm threshold for post-Viterbi BER. When this parameter
/// HIPAR berPostViterbi/exceeds the set threshold, an alarm is generated.
/// HIPAR berPostReedSolomon/Sets the alarm threshold for post-Reed-Solomon BER. When this
/// HIPAR berPostReedSolomon/parameter exceeds the set threshold, an alarm is generated.
ViStatus rsdsa_ConfigureD30DAlarmThreshold(ViSession instrSession,
                                           ViReal64  rmsMEROverall,
                                           ViReal64  berPreViterbi,
                                           ViReal64  berPostViterbi,
                                           ViReal64  berPostReedSolomon)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30D_ALARM_THRESHOLD_RMS_MER_OVERALL, rmsMEROverall),
            2, "RMS MER Overall");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30D_ALARM_THRESHOLD_BER_PRE_VITERBI, berPreViterbi),
            3, "BER Pre-Viterbi");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30D_ALARM_THRESHOLD_BER_POST_VITERBI, berPostViterbi),
            4, "BER Post-Viterbi");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30D_ALARM_THRESHOLD_BER_POST_REED_SOLOMON, berPostReedSolomon),
            5, "BER Post-Reed-Solomon");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the downstream DOCSIS 3.0 SC-QAM J.83 Annex C mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_EnableSCQAMJ83CMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_ENABLE_SC_QAM_J83C_MODE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the demodulator lock status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR demodulatorLockStatus/Queries the demodulator lock status.
ViStatus rsdsa_D30JQueryDemodulatorLockStatus(ViSession instrSession,
                                              ViInt32*  demodulatorLockStatus)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30J_QUERY_DEMODULATOR_LOCK_STATUS, demodulatorLockStatus),
            2, "Demodulator Lock Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the decoder lock status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR decoderLockStatus/Queries the decoder lock status.
ViStatus rsdsa_D30JQueryDecoderLockStatus(ViSession instrSession,
                                          ViInt32*  decoderLockStatus)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30J_QUERY_DECODER_LOCK_STATUS, decoderLockStatus),
            2, "Decoder Lock Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the MPEG lock status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mpegLockStatus/Queries the MPEG lock status.
ViStatus rsdsa_D30JQueryMPEGLockStatus(ViSession instrSession,
                                       ViInt32*  mpegLockStatus)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30J_QUERY_MPEG_LOCK_STATUS, mpegLockStatus),
            2, "MPEG Lock Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the input signal power level in the units selected by
/// HIFN  RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR signalPower/Queries the input signal power level in the units selected by
/// HIPAR signalPower/RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_D30JQuerySignalPower(ViSession instrSession,
                                    ViReal64* signalPower)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30J_QUERY_SIGNAL_POWER, signalPower),
            2, "Signal Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the frequency offset in Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequencyOffset/Queries the frequency offset in Hz.
ViStatus rsdsa_D30JQueryFrequencyOffset(ViSession instrSession,
                                        ViReal64* frequencyOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30J_QUERY_FREQUENCY_OFFSET, frequencyOffset),
            2, "Frequency Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the symbol offset in Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR symbolClockOffset/Queries the symbol offset in Hz.
ViStatus rsdsa_D30JQuerySymbolClockOffset(ViSession instrSession,
                                          ViReal64* symbolClockOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30J_QUERY_SYMBOL_CLOCK_OFFSET, symbolClockOffset),
            2, "Symbol Clock Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the channel bit rate in Mbps (megabits per second).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelBitRate/Queries the channel bit rate in Mbps (megabits per second).
ViStatus rsdsa_D30JQueryChannelBitRate(ViSession instrSession,
                                       ViReal64* channelBitRate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30J_QUERY_CHANNEL_BIT_RATE, channelBitRate),
            2, "Channel Bit Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the interleaver type for the DOCSIS 3.0 downstream signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR interleaver/Queries the interleaver type for the DOCSIS 3.0 downstream signal.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_D30JQueryInterleaver(ViSession instrSession,
                                    ViInt32   bufferSize,
                                    ViChar    interleaver[],
                                    ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL,
        RSDSA_ATTR_D30J_QUERY_INTERLEAVER,
        RS_VAL_DIRECT_USER_CALL,
        bufferSize, interleaver, bytesRead),
        3, "Interleaver");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activate or deactivate the MER compensation function. Activating this
/// HIFN  feature interrupts the signal analysis. Compensation can be disabled by
/// HIFN  setting this to false, or it may be terminated automatically for
/// HIFN  signals when the MER changes substantially.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Activate or deactivate the MER compensation function. Activating this
/// HIPAR enabled/feature interrupts the signal analysis. Compensation can be disabled by
/// HIPAR enabled/setting this to false, or it may be terminated automatically for
/// HIPAR enabled/signals when the MER changes substantially.
ViStatus rsdsa_D30JConfigureMERCompensationEnabled(ViSession instrSession,
                                                   ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_D30J_CONFIGURE_MER_COMPENSATION_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the sign preferred for MER numerical display and plot axis
/// HIFN  labels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merSign/Selects the sign preferred for MER numerical display and plot axis
/// HIPAR merSign/labels.
ViStatus rsdsa_D30JConfigureMERSign(ViSession instrSession,
                                    ViInt32   merSign)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30J_CONFIGURE_MER_SIGN, merSign),
            2, "MER Sign");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Query whether MER compensation is currently active for the overall RMS
/// HIFN  MER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rmsCompensatedOverall/Query whether MER compensation is currently active for the overall RMS
/// HIPAR rmsCompensatedOverall/MER.
ViStatus rsdsa_D30JQueryMERRMSCompensatedOverall(ViSession  instrSession,
                                                 ViBoolean* rmsCompensatedOverall)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_D30J_QUERY_MER_RMS_COMPENSATED_OVERALL, rmsCompensatedOverall),
            2, "RMS Compensated Overall");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall RMS equalized MER. For DOCSIS 3.0 downstream
/// HIFN  modes, this is the only RMS MER status applicable.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rmsEqualized/Queries the overall RMS equalized MER. For DOCSIS 3.0 downstream
/// HIPAR rmsEqualized/modes, this is the only RMS MER status applicable.
ViStatus rsdsa_D30JQueryMERRMSEqualized(ViSession instrSession,
                                        ViReal64* rmsEqualized)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30J_QUERY_MER_RMS_EQUALIZED, rmsEqualized),
            2, "RMS Equalized");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall peak (lowest) MER. For DOCSIS 3.0 downstream
/// HIFN  modes, this is the only peak MER status applicable.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merPeak/Queries the overall peak (lowest) MER. For DOCSIS 3.0 downstream
/// HIPAR merPeak/modes, this is the only peak MER status applicable.
ViStatus rsdsa_D30JQueryMERPeak(ViSession instrSession,
                                ViReal64* merPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30J_QUERY_MER_PEAK, merPeak),
            2, "MER Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the settings for error analysis. Changing error analysis
/// HIFN  settings will reset the error analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR errorAnalysisMode/Specifies the mode of BER measurement.
/// HIPAR manualTestDepth/Defines the test depth of the minimum BER integration. The actual test
/// HIPAR manualTestDepth/depth setting may be automatically changed in automatic error analysis
/// HIPAR manualTestDepth/mode.
/// HIPAR berSource/Selects the BER source used for BER measurements.
/// HIPAR testDepthProgressionSource/Selects the BER source used for test depth progression.
ViStatus rsdsa_D30JConfigureErrorAnalysis(ViSession instrSession,
                                          ViInt32   errorAnalysisMode,
                                          ViInt32   manualTestDepth,
                                          ViInt32   berSource,
                                          ViInt32   testDepthProgressionSource)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30J_CONFIGURE_ERROR_ANALYSIS_MODE, errorAnalysisMode),
            2, "Error Analysis Mode");
    if (errorAnalysisMode == RSDSA_VAL_ERROR_ANALYSIS_MANUAL)
    {
        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30J_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH, manualTestDepth),
                3, "Manual Test Depth");
    }
    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30J_CONFIGURE_ERROR_ANALYSIS_BER_SOURCE, berSource),
            4, "BER Source");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30J_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH_PROGRESSION_SOURCE, testDepthProgressionSource),
            5, "Test Depth Progression Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Performs a BER reset that clears all related counters to zero and
/// HIFN  restarts the BER/CER measurement for the current mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30JResetErrorAnalysis(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30J_RESET_ERROR_ANALYSIS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the elapsed time of the current BER/CER measurement: time
/// HIFN  elapsed since the start of the mode, or last BER reset
/// HIFN  (RSDSA_ATTR_D30J_RESET_ERROR_ANALYSIS), or last BER configuration
/// HIFN  change, in seconds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR elapsedTime/Queries the elapsed time of the current BER/CER measurement: time
/// HIPAR elapsedTime/elapsed since the start of the mode, or last BER reset
/// HIPAR elapsedTime/(RSDSA_ATTR_D30J_RESET_ERROR_ANALYSIS), or last BER configuration
/// HIPAR elapsedTime/change, in seconds.
ViStatus rsdsa_D30JQueryBERElapsedTime(ViSession instrSession,
                                       ViReal64* elapsedTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30J_QUERY_BER_ELAPSED_TIME, elapsedTime),
            2, "Elapsed Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the percentage of blocks processed for the current BER/CER
/// HIFN  measurement: blocks processed since the start of the mode, or last BER
/// HIFN  reset (CALCulate:[<Mode>:][AVERage:]BERatio:CLEar), or last BER
/// HIFN  configuration change. Expressed as the percentage of blocks processed
/// HIFN  relative to the current test depth. A block is 10^7 bits maximum.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR blocksProcessed/Queries the percentage of blocks processed for the current BER/CER
/// HIPAR blocksProcessed/measurement: blocks processed since the start of the mode, or last BER
/// HIPAR blocksProcessed/reset (CALCulate:[<Mode>:][AVERage:]BERatio:CLEar), or last BER
/// HIPAR blocksProcessed/configuration change. Expressed as the percentage of blocks processed
/// HIPAR blocksProcessed/relative to the current test depth. A block is 10^7 bits maximum.
ViStatus rsdsa_D30JQueryBERBlocksProcessed(ViSession instrSession,
                                           ViReal64* blocksProcessed)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30J_QUERY_BER_BLOCKS_PROCESSED, blocksProcessed),
            2, "Blocks Processed");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the pre-Reed-Solomon BER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR berPreRS/Queries the pre-Reed-Solomon BER.
ViStatus rsdsa_D30JQueryBERPreReedSolomon(ViSession instrSession,
                                          ViReal64* berPreRS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30J_QUERY_BER_PRE_REED_SOLOMON, berPreRS),
            2, "BER Pre-RS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of bits analyzed at the input of the
/// HIFN  Reed-Solomon decoder for the BER measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR preRSTotalBits/Queries the total number of bits analyzed at the input of the
/// HIPAR preRSTotalBits/Reed-Solomon decoder for the BER measurement.
ViStatus rsdsa_D30JQueryBERPreReedSolomonTotalBits(ViSession instrSession,
                                                   ViReal64* preRSTotalBits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30J_QUERY_BER_PRE_REED_SOLOMON_TOTAL_BITS, preRSTotalBits),
            2, "Pre-RS Total Bits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the post-Reed-Solomon BER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR berPostRS/Queries the post-Reed-Solomon BER.
ViStatus rsdsa_D30JQueryBERPostReedSolomon(ViSession instrSession,
                                           ViReal64* berPostRS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30J_QUERY_BER_POST_REED_SOLOMON, berPostRS),
            2, "BER Post-RS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of bits analyzed at the output of the
/// HIFN  Reed-Solomon decoder for the BER measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR postRSTotalBits/Queries the total number of bits analyzed at the output of the
/// HIPAR postRSTotalBits/Reed-Solomon decoder for the BER measurement.
ViStatus rsdsa_D30JQueryBERPostReedSolomonTotalBits(ViSession instrSession,
                                                    ViReal64* postRSTotalBits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30J_QUERY_BER_POST_REED_SOLOMON_TOTAL_BITS, postRSTotalBits),
            2, "Post-RS Total Bits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of corrected bits at the output of the
/// HIFN  Reed-Solomon decoder for the BER measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR postRSCorrectedBits/Queries the total number of corrected bits at the output of the
/// HIPAR postRSCorrectedBits/Reed-Solomon decoder for the BER measurement.
ViStatus rsdsa_D30JQueryBERPostReedSolomonCorrectedBits(ViSession instrSession,
                                                        ViReal64* postRSCorrectedBits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30J_QUERY_BER_POST_REED_SOLOMON_CORRECTED_BITS, postRSCorrectedBits),
            2, "Post-RS Corrected Bits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of uncorrected bits at the output of the
/// HIFN  Reed-Solomon decoder for the BER measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR postRSUncorrectedBits/Queries the total number of uncorrected bits at the output of the
/// HIPAR postRSUncorrectedBits/Reed-Solomon decoder for the BER measurement.
ViStatus rsdsa_D30JQueryBERPostReedSolomonUncorrectedBits(ViSession instrSession,
                                                          ViReal64* postRSUncorrectedBits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30J_QUERY_BER_POST_REED_SOLOMON_UNCORRECTED_BITS, postRSUncorrectedBits),
            2, "Post-RS Uncorrected Bits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the corrected CER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cerCorrected/Queries the corrected CER.
ViStatus rsdsa_D30JQueryCERCorrected(ViSession instrSession,
                                     ViReal64* cerCorrected)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30J_QUERY_CER_CORRECTED, cerCorrected),
            2, "CER Corrected");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the uncorrected CER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cerUncorrected/Queries the uncorrected CER.
ViStatus rsdsa_D30JQueryCERUncorrected(ViSession instrSession,
                                       ViReal64* cerUncorrected)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30J_QUERY_CER_UNCORRECTED, cerUncorrected),
            2, "CER Uncorrected");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of codewords for the CER measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR totalCodewords/Queries the total number of codewords for the CER measurement.
ViStatus rsdsa_D30JQueryCERTotalCodewords(ViSession instrSession,
                                          ViReal64* totalCodewords)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30J_QUERY_CER_TOTAL_CODEWORDS, totalCodewords),
            2, "Total Codewords");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of corrected CER codewords.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR correctedCodewords/Queries the total number of corrected CER codewords.
ViStatus rsdsa_D30JQueryCERCorrectedCodewords(ViSession instrSession,
                                              ViReal64* correctedCodewords)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30J_QUERY_CER_CORRECTED_CODEWORDS, correctedCodewords),
            2, "Corrected Codewords");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of uncorrected CER codewords.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR uncorrectedCodewords/Queries the total number of uncorrected CER codewords.
ViStatus rsdsa_D30JQueryCERUncorrectedCodewords(ViSession instrSession,
                                                ViReal64* uncorrectedCodewords)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30J_QUERY_CER_UNCORRECTED_CODEWORDS, uncorrectedCodewords),
            2, "Uncorrected Codewords");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the constellation plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30JDisplayConstellationPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30J_DISPLAY_CONSTELLATION_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the I/Q density plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30JDisplayIQDensityPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30J_DISPLAY_I_Q_DENSITY_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the measurement mode for the I/Q density plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementMode/Selects the measurement mode for the I/Q density plot.
/// HIPAR windowWidth/Specifies the sliding window width in samples for the I/Q density plot
/// HIPAR windowWidth/in sliding window mode.
ViStatus rsdsa_D30JConfigureIQMeasurementMode(ViSession instrSession,
                                              ViInt32   measurementMode,
                                              ViInt32   windowWidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30J_CONFIGURE_I_Q_MEASUREMENT_MODE, measurementMode),
            2, "Measurement Mode");
    if (measurementMode == RSDSA_VAL_IQ_MEASUREMENT_MODE_SLIDING_WINDOW)
    {
        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30J_CONFIGURE_I_Q_WINDOW_WIDTH, windowWidth),
                3, "Window Width");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the zoom setting for the I/Q density plot display. Note that
/// HIFN  the x and y origin are the top left corner of the window, as per the
/// HIFN  GUI settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR x/X origin (left-most window point), between 0 and 1.
/// HIPAR y/Y origin (top-most window point), between 0 and 1.
/// HIPAR zoom/Zoom setting, 1 to 128.
ViStatus rsdsa_D30JSetIQDisplayZoom(ViSession instrSession,
                                    ViReal64  x,
                                    ViReal64  y,
                                    ViInt32   zoom)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_CheckRangeTableRangeViReal64(instrSession, &RsDSA_rngIQZoomOrigin, x),
        2, "X");
    viCheckParm(rsdsa_CheckRangeTableRangeViReal64(instrSession, &RsDSA_rngIQZoomOrigin, y),
        3, "Y");
    viCheckParm(rsdsa_CheckRangeTableRangeViInt32(instrSession, &RsDSA_rngIQZoomLevel, zoom),
        4, "Zoom");

    snprintf(param, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%f,%f,%ld", x, y, zoom);

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_HIDDEN_D30J_DISPLAY_I_Q_ZOOM, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Save the constellation I/Q data to the specified file. The file
/// HIFN  extension is .iq.tar. Note that there is no way to abort this command,
/// HIFN  and large file sizes could take a long time to capture depending on the
/// HIFN  input data, so use maxSamples or timeoutSeconds to limit the file size
/// HIFN  or time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/File path, uses .iq.tar extension.
/// HIPAR maxSamples/Sets the I/Q data file maximum size in samples.
/// HIPAR timeoutSeconds/Sets the maximum time in seconds to save data.
ViStatus rsdsa_D30JCaptureIQDataToFile(ViSession instrSession,
                                       ViString  fileName,
                                       ViInt64   maxSamples,
                                       ViReal64  timeoutSeconds)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt64(instrSession, NULL,
        RSDSA_ATTR_D30J_IQ_DATA_EXPORT_MAX_SAMPLES,
        maxSamples),
        3, "Max Samples");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30J_IQ_DATA_EXPORT_TIMEOUT, timeoutSeconds),
            4, "Timeout Seconds");

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30J_CAPTURE_IQ_DATA_TO_FILENAME, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the MER vs. time plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30JDisplayMERVsTimePlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30J_DISPLAY_MER_VS_TIME_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a time filter period to be used for displayed MER vs. Time
/// HIFN  plots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR period/Selects a time filter period to be used for displayed MER vs. Time
/// HIPAR period/plots.
ViStatus rsdsa_D30JConfigureMERVsTimePlotFilterPeriod(ViSession instrSession,
                                                      ViInt32   period)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30J_CONFIGURE_MER_VS_TIME_PLOT_FILTER_PERIOD, period),
            2, "Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the BER vs. time plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30JDisplayBERVsTimePlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30J_DISPLAY_BER_VS_TIME_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a time filter period to be used for displayed BER vs. Time
/// HIFN  plots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR period/Selects a time filter period to be used for displayed BER vs. Time
/// HIPAR period/plots.
ViStatus rsdsa_D30JConfigureBERVsTimePlotFilterPeriod(ViSession instrSession,
                                                      ViInt32   period)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30J_CONFIGURE_BER_VS_TIME_PLOT_FILTER_PERIOD, period),
            2, "Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the POWer vs. time plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30JDisplayPowerVsTimePlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30J_DISPLAY_POWER_VS_TIME_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a time filter period to be used for displayed Power vs. Time
/// HIFN  plots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR period/Selects a time filter period to be used for displayed Power vs. Time
/// HIPAR period/plots.
ViStatus rsdsa_D30JConfigurePowerVsTimePlotFilterPeriod(ViSession instrSession,
                                                        ViInt32   period)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30J_CONFIGURE_POWER_VS_TIME_PLOT_FILTER_PERIOD, period),
            2, "Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays a downstream or upstream CCDF (Complementary Cumulative
/// HIFN  Distribution Function histogram as per CM-SP-PHYv3.1-I16 sections 9.3.8
/// HIFN  or 9.4.6 respectively). This command may take up to 30 seconds to
/// HIFN  execute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30JDisplayCCDFHistogram(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30J_DISPLAY_CCDF_HISTOGRAM, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the peak CCDF power level in the units selected by
/// HIFN  RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ccdfPeak/Queries the peak CCDF power level in the units selected by
/// HIPAR ccdfPeak/RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_D30JQueryCCDFPeak(ViSession instrSession,
                                 ViReal64* ccdfPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30J_QUERY_CCDF_PEAK, ccdfPeak),
            2, "CCDF Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the channel analysis amplitude/phase vs. subcarrier plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30JDisplayAmplitudePhaseVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30J_DISPLAY_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Scales the y-axis for the channel analysis amplitude/phase vs.
/// HIFN  subcarrier plot to best fit the measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30JAutoScaleAmplitudePhaseVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30J_AUTO_SCALE_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the channel analysis amplitude/group delay vs. subcarrier
/// HIFN  plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30JDisplayAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30J_DISPLAY_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Scales the y-axis for the channel analysis amplitude/group delay vs.
/// HIFN  subcarrier plot to best fit the measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30JAutoScaleAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30J_AUTO_SCALE_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the channel analysis echo pattern plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30JDisplayEchoPatternPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30J_DISPLAY_ECHO_PATTERN_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the specified trace data. Plot data is returned as x,y value
/// HIFN  pairs (x1,y1,x2,y2,...). 1054 points are displayed across the DSA
/// HIFN  screen. For I/Q density data, 65535 integers are always returned in row
/// HIFN  order, regardless of the zoom settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Selects repcap Trace.
/// HIPAR bufferSize/Size of output array.
/// HIPAR traceData/Queries the specified trace data. Plot data is returned as x,y value
/// HIPAR traceData/pairs (x1,y1,x2,y2,...). 1054 points are displayed across the DSA
/// HIPAR traceData/screen. For I/Q density data, 65535 integers are always returned in row
/// HIPAR traceData/order, regardless of the zoom settings.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_D30JQueryTraceData(ViSession instrSession,
                                  ViInt32   trace,
                                  ViInt32   bufferSize,
                                  ViReal64  traceData[],
                                  ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxTrace = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Trace", &maxTrace),
        2, "Trace");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, maxTrace),
            2, "Trace");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D30J_QUERY_TRACE_DATA,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trace");
    viCheckParm(RsCore_GetAttributeViStringAsViReal64ArrayToUserBuffer(instrSession, repCap,
        RSDSA_ATTR_D30J_QUERY_TRACE_DATA, NULL,
        bufferSize, traceData, bytesRead),
        4, "Trace Data");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the all the messages that are currently in the acquisition log
/// HIFN  and clears the acquisition log. If the acquisition log is empty, the
/// HIFN  number 0, 'No entry', is returned. For each error in the log, the
/// HIFN  aqcuisition log entry message is returned, followed by the date and
/// HIFN  time the entry was added to the log.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR log/Queries the all the messages that are currently in the acquisition log
/// HIPAR log/and clears the acquisition log. If the acquisition log is empty, the
/// HIPAR log/number 0, 'No entry', is returned. For each error in the log, the
/// HIPAR log/aqcuisition log entry message is returned, followed by the date and
/// HIPAR log/time the entry was added to the log.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_D30JQuerySystemAcquisitionLogAll(ViSession instrSession,
                                                ViInt32   bufferSize,
                                                ViChar    log[],
                                                ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL,
        RSDSA_ATTR_D30J_SYSTEM_ACQUISITION_LOG_ALL,
        RS_VAL_DIRECT_USER_CALL,
        bufferSize, log, bytesRead),
        3, "Log");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the most recent acquisition log entry and deletes it. If the
/// HIFN  acquisition log is empty, the number 0, 'No entry', is returned.
/// HIFN  Returns the aqcuisition log entry message, followed by the date and
/// HIFN  time the entry was added to the log.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR entry/Queries the most recent acquisition log entry and deletes it. If the
/// HIPAR entry/acquisition log is empty, the number 0, 'No entry', is returned.
/// HIPAR entry/Returns the aqcuisition log entry message, followed by the date and
/// HIPAR entry/time the entry was added to the log.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_D30JQuerySystemAcquisitionLogNext(ViSession instrSession,
                                                 ViInt32   bufferSize,
                                                 ViChar    entry[],
                                                 ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL,
        RSDSA_ATTR_D30J_SYSTEM_ACQUISITION_LOG_NEXT,
        RS_VAL_DIRECT_USER_CALL,
        bufferSize, entry, bytesRead),
        3, "Entry");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the acquisition log for the number of unread entries. If the
/// HIFN  acquisition log is empty, the error number 0, 'No error', is returned.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR count/Queries the acquisition log for the number of unread entries. If the
/// HIPAR count/acquisition log is empty, the error number 0, 'No error', is returned.
ViStatus rsdsa_D30JQuerySystemAcquisitionLogCount(ViSession instrSession,
                                                  ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30J_SYSTEM_ACQUISITION_LOG_COUNT, count),
            2, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes all entries in the acquisition log.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30JSystemAcquisitionLogClearAll(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30J_SYSTEM_ACQUISITION_LOG_CLEAR_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures settings for the SC-QAM J.83C mode. Some of these settings
/// HIFN  will also affect other instrument modes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequency/Specifies the center frequency of the channel. If the spectrum
/// HIPAR frequency/location is changed by CONFigure:CHANnel:SPECtrum:FREQuency, the center
/// HIPAR frequency/frequency is adapted accordingly.
/// HIPAR constellation/Defines the constellation.
/// HIPAR spectralInversionEnabled/Turns spectral inversion on and off. This inverts the spectrum at the
/// HIPAR spectralInversionEnabled/demodulator. It does not affect any displayed spectrum.
/// HIPAR equalizerState/Controls the equalizer state.
/// HIPAR attenuation/Defines the total attenuation for RF input in dB. If you set the
/// HIPAR attenuation/attenuation manually, it is no longer coupled to the reference level,
/// HIPAR attenuation/but the reference level is coupled to the attenuation. Thus, if the
/// HIPAR attenuation/current reference level is not compatible with an attenuation that has
/// HIPAR attenuation/been set manually, the command also adjusts the reference level.
/// HIPAR attenuation/Therefore, the reference level set in
/// HIPAR attenuation/DISPlay:[<Mode>:][WINDow:]TRACe[<Trace>]:Y[:SCALe]:RLEVel is no longer
/// HIPAR attenuation/applied. Note: The query returns the actual attenuation setting, which
/// HIPAR attenuation/can also be changed by RSDSA_ATTR_D30J_AUTO_LEVEL.
/// HIPAR preamplifierEnabled/Turns the preamplifier on and off. Note: The query returns the actual
/// HIPAR preamplifierEnabled/preamplifier state (which might also be changed by
/// HIPAR preamplifierEnabled/RSDSA_ATTR_ED30_AUTO_LEVEL).
/// HIPAR asiOutputEnabled/Turns the ASI output on and off.
/// HIPAR phaseTrackingLoopBandwidth/Specifies the phase tracking loop bandwidth.
ViStatus rsdsa_D30JConfigureGeneralSettings(ViSession instrSession,
                                            ViReal64  frequency,
                                            ViInt32   constellation,
                                            ViBoolean spectralInversionEnabled,
                                            ViInt32   equalizerState,
                                            ViReal64  attenuation,
                                            ViBoolean preamplifierEnabled,
                                            ViBoolean asiOutputEnabled,
                                            ViInt32   phaseTrackingLoopBandwidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30J_CONFIGURE_FREQUENCY, frequency),
            2, "Frequency");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30J_CONFIGURE_CONSTELLATION, constellation),
            3, "Constellation");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_D30J_CONFIGURE_SPECTRAL_INVERSION_ENABLED, spectralInversionEnabled),
            4, "Spectral Inversion Enabled");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30J_CONFIGURE_EQUALIZER_STATE, equalizerState),
            5, "Equalizer State");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30J_CONFIGURE_ATTENUATION, attenuation),
            6, "Attenuation");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_D30J_CONFIGURE_PREAMPLIFIER_ENABLED, preamplifierEnabled),
            7, "Preamplifier Enabled");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_D30J_CONFIGURE_ASI_OUTPUT_ENABLED, asiOutputEnabled),
            8, "ASI Output Enabled");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30J_CONFIGURE_PHASE_TRACKING_LOOP_BANDWIDTH, phaseTrackingLoopBandwidth),
            9, "Phase Tracking Loop Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the list of equalizer coefficients. A pair of signed
/// HIFN  floating-point values is returned for each coefficient, representing
/// HIFN  the real and imaginary parts. The number of coefficient pairs returned
/// HIFN  depends on the mode, the signal bandwidth, and for OFDM/OFDMA, the FFT
/// HIFN  size.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR equalizerCoefficients/Queries the list of equalizer coefficients. A pair of signed
/// HIPAR equalizerCoefficients/floating-point values is returned for each coefficient, representing
/// HIPAR equalizerCoefficients/the real and imaginary parts. The number of coefficient pairs returned
/// HIPAR equalizerCoefficients/depends on the mode, the signal bandwidth, and for OFDM/OFDMA, the FFT
/// HIPAR equalizerCoefficients/size.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_D30JQueryEqualizerCoefficients(ViSession instrSession,
                                              ViInt32   bufferSize,
                                              ViReal64  equalizerCoefficients[],
                                              ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringAsViReal64ArrayToUserBuffer(instrSession, NULL,
        RSDSA_ATTR_D30J_QUERY_EQUALIZER_COEFFICIENTS, NULL,
        bufferSize, equalizerCoefficients, bytesRead),
        3, "Equalizer Coefficients");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Automatically determines the attenuation level and preamplifier state
/// HIFN  (on/off). Query the resultant attenuation level with
/// HIFN  RSDSA_ATTR_D30J_CONFIGURE_ATTENUATION and the resultant preamplifier
/// HIFN  state with RSDSA_ATTR_D30J_CONFIGURE_PREAMPLIFIER_ENABLED.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30JAutoLevel(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30J_AUTO_LEVEL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the alarm thresholds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rmsMEROverallThreshold/Sets the alarm threshold for RMS MER overall. When this parameter
/// HIPAR rmsMEROverallThreshold/falls below the set threshold, an alarm is generated.
/// HIPAR berPreRSThreshold/Sets the alarm threshold for pre-Reed-Solomon BER. When this parameter
/// HIPAR berPreRSThreshold/exceeds the set threshold, an alarm is generated.
/// HIPAR berPostRSThreshold/Sets the alarm threshold for post-Reed-Solomon BER. When this
/// HIPAR berPostRSThreshold/parameter exceeds the set threshold, an alarm is generated.
ViStatus rsdsa_D30JConfigureAlarmThresholds(ViSession instrSession,
                                            ViReal64  rmsMEROverallThreshold,
                                            ViReal64  berPreRSThreshold,
                                            ViReal64  berPostRSThreshold)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30J_ALARM_THRESHOLD_RMS_MER_OVERALL, rmsMEROverallThreshold),
            2, "RMS MER Overall Threshold");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30J_ALARM_THRESHOLD_BER_PRE_REED_SOLOMON, berPreRSThreshold),
            3, "BER Pre-RS Threshold");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30J_ALARM_THRESHOLD_BER_POST_REED_SOLOMON, berPostRSThreshold),
            4, "BER Post-RS Threshold");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the downstream J.83 QAM mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_EnableJ83QAMMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_ENABLE_J83QAM_MODE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the demodulator lock status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR demodulatorLockStatus/Queries the demodulator lock status.
ViStatus rsdsa_JQDQueryDemodulatorLockStatus(ViSession instrSession,
                                             ViInt32*  demodulatorLockStatus)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_JQD_QUERY_DEMODULATOR_LOCK_STATUS, demodulatorLockStatus),
            2, "Demodulator Lock Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the decoder lock status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR decoderLockStatus/Queries the decoder lock status.
ViStatus rsdsa_JQDQueryDecoderLockStatus(ViSession instrSession,
                                         ViInt32*  decoderLockStatus)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_JQD_QUERY_DECODER_LOCK_STATUS, decoderLockStatus),
            2, "Decoder Lock Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the MPEG lock status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mpegLockStatus/Queries the MPEG lock status.
ViStatus rsdsa_JQDQueryMPEGLockStatus(ViSession instrSession,
                                      ViInt32*  mpegLockStatus)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_JQD_QUERY_MPEG_LOCK_STATUS, mpegLockStatus),
            2, "MPEG Lock Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the input signal power level in the units selected by
/// HIFN  RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR signalPower/Queries the input signal power level in the units selected by
/// HIPAR signalPower/RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_JQDQuerySignalPower(ViSession instrSession,
                                   ViReal64* signalPower)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_JQD_QUERY_SIGNAL_POWER, signalPower),
            2, "Signal Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the frequency offset in Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequencyOffset/Queries the frequency offset in Hz.
ViStatus rsdsa_JQDQueryFrequencyOffset(ViSession instrSession,
                                       ViReal64* frequencyOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_JQD_QUERY_FREQUENCY_OFFSET, frequencyOffset),
            2, "Frequency Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the symbol offset in Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR symbolClockOffset/Queries the symbol offset in Hz.
ViStatus rsdsa_JQDQuerySymbolClockOffset(ViSession instrSession,
                                         ViReal64* symbolClockOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_JQD_QUERY_SYMBOL_CLOCK_OFFSET, symbolClockOffset),
            2, "Symbol Clock Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the channel bit rate in Mbps (megabits per second).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelBitRate/Queries the channel bit rate in Mbps (megabits per second).
ViStatus rsdsa_JQDQueryChannelBitRate(ViSession instrSession,
                                      ViReal64* channelBitRate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_JQD_QUERY_CHANNEL_BIT_RATE, channelBitRate),
            2, "Channel Bit Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the interleaver type for the DOCSIS 3.0 downstream signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR interleaver/Queries the interleaver type for the DOCSIS 3.0 downstream signal.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_JQDQueryInterleaver(ViSession instrSession,
                                   ViInt32   bufferSize,
                                   ViChar    interleaver[],
                                   ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL,
        RSDSA_ATTR_JQD_QUERY_INTERLEAVER,
        RS_VAL_DIRECT_USER_CALL,
        bufferSize, interleaver, bytesRead),
        3, "Interleaver");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activate or deactivate the MER compensation function. Activating this
/// HIFN  feature interrupts the signal analysis. Compensation can be disabled by
/// HIFN  setting this to false, or it may be terminated automatically for
/// HIFN  signals when the MER changes substantially.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Activate or deactivate the MER compensation function. Activating this
/// HIPAR enabled/feature interrupts the signal analysis. Compensation can be disabled by
/// HIPAR enabled/setting this to false, or it may be terminated automatically for
/// HIPAR enabled/signals when the MER changes substantially.
ViStatus rsdsa_JQDConfigureMERCompensationEnabled(ViSession instrSession,
                                                  ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_JQD_CONFIGURE_MER_COMPENSATION_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the sign preferred for MER numerical display and plot axis
/// HIFN  labels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merSign/Selects the sign preferred for MER numerical display and plot axis
/// HIPAR merSign/labels.
ViStatus rsdsa_JQDConfigureMERSign(ViSession instrSession,
                                   ViInt32   merSign)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_JQD_CONFIGURE_MER_SIGN, merSign),
            2, "MER Sign");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Query whether MER compensation is currently active for the overall RMS
/// HIFN  MER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rmsCompensatedOverall/Query whether MER compensation is currently active for the overall RMS
/// HIPAR rmsCompensatedOverall/MER.
ViStatus rsdsa_JQDQueryMERRMSCompensatedOverall(ViSession  instrSession,
                                                ViBoolean* rmsCompensatedOverall)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_JQD_QUERY_MER_RMS_COMPENSATED_OVERALL, rmsCompensatedOverall),
            2, "RMS Compensated Overall");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall RMS equalized MER. For DOCSIS 3.0 downstream
/// HIFN  modes, this is the only RMS MER status applicable.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rmsEqualized/Queries the overall RMS equalized MER. For DOCSIS 3.0 downstream
/// HIPAR rmsEqualized/modes, this is the only RMS MER status applicable.
ViStatus rsdsa_JQDQueryMERRMSEqualized(ViSession instrSession,
                                       ViReal64* rmsEqualized)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_JQD_QUERY_MER_RMS_EQUALIZED, rmsEqualized),
            2, "RMS Equalized");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall peak (lowest) MER. For DOCSIS 3.0 downstream
/// HIFN  modes, this is the only peak MER status applicable.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merPeak/Queries the overall peak (lowest) MER. For DOCSIS 3.0 downstream
/// HIPAR merPeak/modes, this is the only peak MER status applicable.
ViStatus rsdsa_JQDQueryMERPeak(ViSession instrSession,
                               ViReal64* merPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_JQD_QUERY_MER_PEAK, merPeak),
            2, "MER Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the settings for error analysis. Changing error analysis
/// HIFN  settings will reset the error analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR errorAnalysisMode/Specifies the mode of BER measurement.
/// HIPAR manualTestDepth/Defines the test depth of the minimum BER integration. The actual test
/// HIPAR manualTestDepth/depth setting may be automatically changed in automatic error analysis
/// HIPAR manualTestDepth/mode.
/// HIPAR berSource/Selects the BER source used for BER measurements.
/// HIPAR testDepthProgressionSource/Selects the BER source used for test depth progression.
ViStatus rsdsa_JQDConfigureErrorAnalysis(ViSession instrSession,
                                         ViInt32   errorAnalysisMode,
                                         ViInt32   manualTestDepth,
                                         ViInt32   berSource,
                                         ViInt32   testDepthProgressionSource)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_JQD_CONFIGURE_ERROR_ANALYSIS_MODE, errorAnalysisMode),
            2, "Error Analysis Mode");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_JQD_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH, manualTestDepth),
            3, "Manual Test Depth");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_JQD_CONFIGURE_ERROR_ANALYSIS_BER_SOURCE, berSource),
            4, "BER Source");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_JQD_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH_PROGRESSION_SOURCE, testDepthProgressionSource),
            5, "Test Depth Progression Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Performs a BER reset that clears all related counters to zero and
/// HIFN  restarts the BER/CER measurement for the current mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_JQDResetErrorAnalysis(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_JQD_RESET_ERROR_ANALYSIS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the elapsed time of the current BER/CER measurement: time
/// HIFN  elapsed since the start of the mode, or last BER reset
/// HIFN  (RSDSA_ATTR_JQD_RESET_ERROR_ANALYSIS), or last BER configuration
/// HIFN  change, in seconds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR elapsedTime/Queries the elapsed time of the current BER/CER measurement: time
/// HIPAR elapsedTime/elapsed since the start of the mode, or last BER reset
/// HIPAR elapsedTime/(RSDSA_ATTR_JQD_RESET_ERROR_ANALYSIS), or last BER configuration
/// HIPAR elapsedTime/change, in seconds.
ViStatus rsdsa_JQDQueryBERElapsedTime(ViSession instrSession,
                                      ViReal64* elapsedTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_JQD_QUERY_BER_ELAPSED_TIME, elapsedTime),
            2, "Elapsed Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the percentage of blocks processed for the current BER/CER
/// HIFN  measurement: blocks processed since the start of the mode, or last BER
/// HIFN  reset (CALCulate:[<Mode>:][AVERage:]BERatio:CLEar), or last BER
/// HIFN  configuration change. Expressed as the percentage of blocks processed
/// HIFN  relative to the current test depth. A block is 10^7 bits maximum.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR blocksProcessed/Queries the percentage of blocks processed for the current BER/CER
/// HIPAR blocksProcessed/measurement: blocks processed since the start of the mode, or last BER
/// HIPAR blocksProcessed/reset (CALCulate:[<Mode>:][AVERage:]BERatio:CLEar), or last BER
/// HIPAR blocksProcessed/configuration change. Expressed as the percentage of blocks processed
/// HIPAR blocksProcessed/relative to the current test depth. A block is 10^7 bits maximum.
ViStatus rsdsa_JQDQueryBERBlocksProcessed(ViSession instrSession,
                                          ViReal64* blocksProcessed)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_JQD_QUERY_BER_BLOCKS_PROCESSED, blocksProcessed),
            2, "Blocks Processed");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the pre-Reed-Solomon BER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR berPreRS/Queries the pre-Reed-Solomon BER.
ViStatus rsdsa_JQDQueryBERPreReedSolomon(ViSession instrSession,
                                         ViReal64* berPreRS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_JQD_QUERY_BER_PRE_REED_SOLOMON, berPreRS),
            2, "BER Pre-RS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of bits analyzed at the input of the
/// HIFN  Reed-Solomon decoder for the BER measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR preRSTotalBits/Queries the total number of bits analyzed at the input of the
/// HIPAR preRSTotalBits/Reed-Solomon decoder for the BER measurement.
ViStatus rsdsa_JQDQueryBERPreReedSolomonTotalBits(ViSession instrSession,
                                                  ViReal64* preRSTotalBits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_JQD_QUERY_BER_PRE_REED_SOLOMON_TOTAL_BITS, preRSTotalBits),
            2, "Pre-RS Total Bits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the post-Reed-Solomon BER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR berPostRS/Queries the post-Reed-Solomon BER.
ViStatus rsdsa_JQDQueryBERPostReedSolomon(ViSession instrSession,
                                          ViReal64* berPostRS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_JQD_QUERY_BER_POST_REED_SOLOMON, berPostRS),
            2, "BER Post-RS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of corrected bits at the output of the
/// HIFN  Reed-Solomon decoder for the BER measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR postRSCorrectedBits/Queries the total number of corrected bits at the output of the
/// HIPAR postRSCorrectedBits/Reed-Solomon decoder for the BER measurement.
ViStatus rsdsa_JQDQueryBERPostReedSolomonCorrectedBits(ViSession instrSession,
                                                       ViReal64* postRSCorrectedBits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_JQD_QUERY_BER_POST_REED_SOLOMON_CORRECTED_BITS, postRSCorrectedBits),
            2, "Post-RS Corrected Bits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of uncorrected bits at the output of the
/// HIFN  Reed-Solomon decoder for the BER measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR postRSUncorrectedBits/Queries the total number of uncorrected bits at the output of the
/// HIPAR postRSUncorrectedBits/Reed-Solomon decoder for the BER measurement.
ViStatus rsdsa_JQDQueryBERPostReedSolomonUncorrectedBits(ViSession instrSession,
                                                         ViReal64* postRSUncorrectedBits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_JQD_QUERY_BER_POST_REED_SOLOMON_UNCORRECTED_BITS, postRSUncorrectedBits),
            2, "Post-RS Uncorrected Bits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the corrected CER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cerCorrected/Queries the corrected CER.
ViStatus rsdsa_JQDQueryCERCorrected(ViSession instrSession,
                                    ViReal64* cerCorrected)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_JQD_QUERY_CER_CORRECTED, cerCorrected),
            2, "CER Corrected");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the uncorrected CER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cerUncorrected/Queries the uncorrected CER.
ViStatus rsdsa_JQDQueryCERUncorrected(ViSession instrSession,
                                      ViReal64* cerUncorrected)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_JQD_QUERY_CER_UNCORRECTED, cerUncorrected),
            2, "CER Uncorrected");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of codewords for the CER measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR totalCodewords/Queries the total number of codewords for the CER measurement.
ViStatus rsdsa_JQDQueryCERTotalCodewords(ViSession instrSession,
                                         ViReal64* totalCodewords)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_JQD_QUERY_CER_TOTAL_CODEWORDS, totalCodewords),
            2, "Total Codewords");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of corrected CER codewords.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR correctedCodewords/Queries the total number of corrected CER codewords.
ViStatus rsdsa_JQDQueryCERCorrectedCodewords(ViSession instrSession,
                                             ViReal64* correctedCodewords)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_JQD_QUERY_CER_CORRECTED_CODEWORDS, correctedCodewords),
            2, "Corrected Codewords");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of uncorrected CER codewords.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR uncorrectedCodewords/Queries the total number of uncorrected CER codewords.
ViStatus rsdsa_JQDQueryCERUncorrectedCodewords(ViSession instrSession,
                                               ViReal64* uncorrectedCodewords)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_JQD_QUERY_CER_UNCORRECTED_CODEWORDS, uncorrectedCodewords),
            2, "Uncorrected Codewords");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the constellation plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_JQDDisplayConstellationPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_JQD_DISPLAY_CONSTELLATION_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the I/Q density plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_JQDDisplayIQDensityPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_JQD_DISPLAY_I_Q_DENSITY_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the measurement mode for the I/Q density plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementMode/Selects the measurement mode for the I/Q density plot.
/// HIPAR windowWidth/Specifies the sliding window width in samples for the I/Q density plot
/// HIPAR windowWidth/in sliding window mode.
ViStatus rsdsa_JQDConfigureIQMeasurementMode(ViSession instrSession,
                                             ViInt32   measurementMode,
                                             ViInt32   windowWidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_JQD_CONFIGURE_I_Q_MEASUREMENT_MODE, measurementMode),
            2, "Measurement Mode");
    if (measurementMode == RSDSA_VAL_IQ_MEASUREMENT_MODE_SLIDING_WINDOW)
    {
        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_JQD_CONFIGURE_I_Q_WINDOW_WIDTH, windowWidth),
                3, "Window Width");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the zoom setting for the I/Q density plot display. Note that
/// HIFN  the x and y origin are the top left corner of the window, as per the
/// HIFN  GUI settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR x/X origin (left-most window point), between 0 and 1.
/// HIPAR y/Y origin (top-most window point), between 0 and 1.
/// HIPAR zoom/Zoom setting, 1 to 128.
ViStatus rsdsa_JQDSetIQDisplayZoom(ViSession instrSession,
                                   ViReal64  x,
                                   ViReal64  y,
                                   ViInt32   zoom)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_CheckRangeTableRangeViReal64(instrSession, &RsDSA_rngIQZoomOrigin, x),
        2, "X");
    viCheckParm(rsdsa_CheckRangeTableRangeViReal64(instrSession, &RsDSA_rngIQZoomOrigin, y),
        3, "Y");
    viCheckParm(rsdsa_CheckRangeTableRangeViInt32(instrSession, &RsDSA_rngIQZoomLevel, zoom),
        4, "Zoom");

    snprintf(param, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%f,%f,%ld", x, y, zoom);

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_HIDDEN_JQD_DISPLAY_I_Q_ZOOM, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Save the constellation I/Q data to the specified file. The file
/// HIFN  extension is .iq.tar. Note that there is no way to abort this command,
/// HIFN  and large file sizes could take a long time to capture depending on the
/// HIFN  input data, so use maxSamples or timeoutSeconds to limit the file size
/// HIFN  or time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/File path, uses .iq.tar extension.
/// HIPAR maxSamples/Sets the I/Q data file maximum size in samples.
/// HIPAR timeoutSeconds/Sets the maximum time in seconds to save data.
ViStatus rsdsa_JQDCaptureIQDataToFile(ViSession instrSession,
                                      ViString  fileName,
                                      ViInt64   maxSamples,
                                      ViReal64  timeoutSeconds)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt64(instrSession, NULL,
        RSDSA_ATTR_JQD_IQ_DATA_EXPORT_MAX_SAMPLES,
        maxSamples),
        3, "Max Samples");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_JQD_IQ_DATA_EXPORT_TIMEOUT, timeoutSeconds),
            4, "Timeout Seconds");

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_JQD_CAPTURE_IQ_DATA_TO_FILENAME, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the MER vs. time plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_JQDDisplayMERVsTimePlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_JQD_DISPLAY_MER_VS_TIME_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a time filter period to be used for displayed MER vs. Time
/// HIFN  plots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR period/Selects a time filter period to be used for displayed MER vs. Time
/// HIPAR period/plots.
ViStatus rsdsa_JQDConfigureMERVsTimePlotFilterPeriod(ViSession instrSession,
                                                     ViInt32   period)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_JQD_CONFIGURE_MER_VS_TIME_PLOT_FILTER_PERIOD, period),
            2, "Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the BER vs. time plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_JQDDisplayBERVsTimePlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_JQD_DISPLAY_BER_VS_TIME_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a time filter period to be used for displayed BER vs. Time
/// HIFN  plots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR period/Selects a time filter period to be used for displayed BER vs. Time
/// HIPAR period/plots.
ViStatus rsdsa_JQDConfigureBERVsTimePlotFilterPeriod(ViSession instrSession,
                                                     ViInt32   period)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_JQD_CONFIGURE_BER_VS_TIME_PLOT_FILTER_PERIOD, period),
            2, "Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the POWer vs. time plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_JQDDisplayPowerVsTimePlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_JQD_DISPLAY_POWER_VS_TIME_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a time filter period to be used for displayed Power vs. Time
/// HIFN  plots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR period/Selects a time filter period to be used for displayed Power vs. Time
/// HIPAR period/plots.
ViStatus rsdsa_JQDConfigurePowerVsTimePlotFilterPeriod(ViSession instrSession,
                                                       ViInt32   period)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_JQD_CONFIGURE_POWER_VS_TIME_PLOT_FILTER_PERIOD, period),
            2, "Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays a downstream or upstream CCDF (Complementary Cumulative
/// HIFN  Distribution Function histogram as per CM-SP-PHYv3.1-I16 sections 9.3.8
/// HIFN  or 9.4.6 respectively). This command may take up to 30 seconds to
/// HIFN  execute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_JQDDisplayCCDFHistogram(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_JQD_DISPLAY_CCDF_HISTOGRAM, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the peak CCDF power level in the units selected by
/// HIFN  RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ccdfPeak/Queries the peak CCDF power level in the units selected by
/// HIPAR ccdfPeak/RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_JQDQueryCCDFPeak(ViSession instrSession,
                                ViReal64* ccdfPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_JQD_QUERY_CCDF_PEAK, ccdfPeak),
            2, "CCDF Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the channel analysis amplitude/phase vs. subcarrier plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_JQDDisplayAmplitudePhaseVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_JQD_DISPLAY_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Scales the y-axis for the channel analysis amplitude/phase vs.
/// HIFN  subcarrier plot to best fit the measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_JQDAutoScaleAmplitudePhaseVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_JQD_AUTO_SCALE_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the channel analysis amplitude/group delay vs. subcarrier
/// HIFN  plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_JQDDisplayAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_JQD_DISPLAY_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Scales the y-axis for the channel analysis amplitude/group delay vs.
/// HIFN  subcarrier plot to best fit the measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_JQDAutoScaleAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_JQD_AUTO_SCALE_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the channel analysis echo pattern plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_JQDDisplayEchoPatternPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_JQD_DISPLAY_ECHO_PATTERN_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the specified trace data. Plot data is returned as x,y value
/// HIFN  pairs (x1,y1,x2,y2,...). 1054 points are displayed across the DSA
/// HIFN  screen. For I/Q density data, 65535 integers are always returned in row
/// HIFN  order, regardless of the zoom settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Selects repcap Trace.
/// HIPAR bufferSize/Size of output array.
/// HIPAR traceData/Queries the specified trace data. Plot data is returned as x,y value
/// HIPAR traceData/pairs (x1,y1,x2,y2,...). 1054 points are displayed across the DSA
/// HIPAR traceData/screen. For I/Q density data, 65535 integers are always returned in row
/// HIPAR traceData/order, regardless of the zoom settings.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_JQDQueryTraceData(ViSession instrSession,
                                 ViInt32   trace,
                                 ViInt32   bufferSize,
                                 ViReal64  traceData[],
                                 ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxTrace = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Trace", &maxTrace),
        2, "Trace");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, maxTrace),
            2, "Trace");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_JQD_QUERY_TRACE_DATA,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trace");
    viCheckParm(RsCore_GetAttributeViStringAsViReal64ArrayToUserBuffer(instrSession, repCap,
        RSDSA_ATTR_JQD_QUERY_TRACE_DATA, NULL,
        bufferSize, traceData, bytesRead),
        4, "Trace Data");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the all the messages that are currently in the acquisition log
/// HIFN  and clears the acquisition log. If the acquisition log is empty, the
/// HIFN  number 0, 'No entry', is returned. For each error in the log, the
/// HIFN  aqcuisition log entry message is returned, followed by the date and
/// HIFN  time the entry was added to the log.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR log/Queries the all the messages that are currently in the acquisition log
/// HIPAR log/and clears the acquisition log. If the acquisition log is empty, the
/// HIPAR log/number 0, 'No entry', is returned. For each error in the log, the
/// HIPAR log/aqcuisition log entry message is returned, followed by the date and
/// HIPAR log/time the entry was added to the log.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_JQDQuerySystemAcquisitionLogAll(ViSession instrSession,
                                               ViInt32   bufferSize,
                                               ViChar    log[],
                                               ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL,
        RSDSA_ATTR_JQD_SYSTEM_ACQUISITION_LOG_ALL,
        RS_VAL_DIRECT_USER_CALL,
        bufferSize, log, bytesRead),
        3, "Log");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the most recent acquisition log entry and deletes it. If the
/// HIFN  acquisition log is empty, the number 0, 'No entry', is returned.
/// HIFN  Returns the aqcuisition log entry message, followed by the date and
/// HIFN  time the entry was added to the log.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR entry/Queries the most recent acquisition log entry and deletes it. If the
/// HIPAR entry/acquisition log is empty, the number 0, 'No entry', is returned.
/// HIPAR entry/Returns the aqcuisition log entry message, followed by the date and
/// HIPAR entry/time the entry was added to the log.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_JQDQuerySystemAcquisitionLogNext(ViSession instrSession,
                                                ViInt32   bufferSize,
                                                ViChar    entry[],
                                                ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL,
        RSDSA_ATTR_JQD_SYSTEM_ACQUISITION_LOG_NEXT,
        RS_VAL_DIRECT_USER_CALL,
        bufferSize, entry, bytesRead),
        3, "Entry");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the acquisition log for the number of unread entries. If the
/// HIFN  acquisition log is empty, the error number 0, 'No error', is returned.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR count/Queries the acquisition log for the number of unread entries. If the
/// HIPAR count/acquisition log is empty, the error number 0, 'No error', is returned.
ViStatus rsdsa_JQDQuerySystemAcquisitionLogCount(ViSession instrSession,
                                                 ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_JQD_SYSTEM_ACQUISITION_LOG_COUNT, count),
            2, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes all entries in the acquisition log.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_JQDSystemAcquisitionLogClearAll(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_JQD_SYSTEM_ACQUISITION_LOG_CLEAR_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures settings for the J.83 QAM mode. Some of these settings will
/// HIFN  also affect other instrument modes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fec/Selects the J.83 standard to be used in J.83 QAM mode.
/// HIPAR frequency/Specifies the center frequency of the channel. If the spectrum
/// HIPAR frequency/location is changed by CONFigure:CHANnel:SPECtrum:FREQuency, the center
/// HIPAR frequency/frequency is adapted accordingly.
/// HIPAR constellation/Defines the constellation.
/// HIPAR symbolRate/Specifies the symbol rate.
/// HIPAR rollOff/Defines the roll-off period, which depends on the channel bandwidth
/// HIPAR rollOff/and the number of excluded carriers within the channel. The larger the
/// HIPAR rollOff/roll-off period, the more time transmission takes; however, the more
/// HIPAR rollOff/useful subcarriers are available in the frequency domain.
/// HIPAR spectralInversionEnabled/Turns spectral inversion on and off. This inverts the spectrum at the
/// HIPAR spectralInversionEnabled/demodulator. It does not affect any displayed spectrum.
/// HIPAR equalizerState/Controls the equalizer state.
/// HIPAR attenuation/Defines the total attenuation for RF input in dB. If you set the
/// HIPAR attenuation/attenuation manually, it is no longer coupled to the reference level,
/// HIPAR attenuation/but the reference level is coupled to the attenuation. Thus, if the
/// HIPAR attenuation/current reference level is not compatible with an attenuation that has
/// HIPAR attenuation/been set manually, the command also adjusts the reference level.
/// HIPAR attenuation/Therefore, the reference level set in
/// HIPAR attenuation/DISPlay:[<Mode>:][WINDow:]TRACe[<Trace>]:Y[:SCALe]:RLEVel is no longer
/// HIPAR attenuation/applied. Note: The query returns the actual attenuation setting, which
/// HIPAR attenuation/can also be changed by RSDSA_ATTR_ED30_AUTO_LEVEL.
/// HIPAR preamplifierEnabled/Turns the preamplifier on and off. Note: The query returns the actual
/// HIPAR preamplifierEnabled/preamplifier state (which might also be changed by
/// HIPAR preamplifierEnabled/RSDSA_ATTR_JQD_AUTO_LEVEL).
/// HIPAR asiOutputEnabled/Turns the ASI output on and off.
/// HIPAR phaseTrackingLoopBandwidth/Specifies the phase tracking loop bandwidth.
ViStatus rsdsa_JQDConfigureGeneralSettings(ViSession instrSession,
                                           ViInt32   fec,
                                           ViReal64  frequency,
                                           ViInt32   constellation,
                                           ViReal64  symbolRate,
                                           ViInt32   rollOff,
                                           ViBoolean spectralInversionEnabled,
                                           ViInt32   equalizerState,
                                           ViReal64  attenuation,
                                           ViBoolean preamplifierEnabled,
                                           ViBoolean asiOutputEnabled,
                                           ViInt32   phaseTrackingLoopBandwidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_JQD_CONFIGURE_FEC, fec),
            2, "FEC");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_JQD_CONFIGURE_FREQUENCY, frequency),
            3, "Frequency");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_JQD_CONFIGURE_CONSTELLATION, constellation),
            4, "Constellation");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_JQD_CONFIGURE_SYMBOL_RATE, symbolRate),
            5, "Symbol Rate");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_JQD_CONFIGURE_ROLL_OFF, rollOff),
            6, "Roll Off");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_JQD_CONFIGURE_SPECTRAL_INVERSION_ENABLED, spectralInversionEnabled),
            7, "Spectral Inversion Enabled");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_JQD_CONFIGURE_EQUALIZER_STATE, equalizerState),
            8, "Equalizer State");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_JQD_CONFIGURE_ATTENUATION, attenuation),
            9, "Attenuation");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_JQD_CONFIGURE_PREAMPLIFIER_ENABLED, preamplifierEnabled),
            10, "Preamplifier Enabled");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_JQD_CONFIGURE_ASI_OUTPUT_ENABLED, asiOutputEnabled),
            11, "ASI Output Enabled");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_JQD_CONFIGURE_PHASE_TRACKING_LOOP_BANDWIDTH, phaseTrackingLoopBandwidth),
            12, "Phase Tracking Loop Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the list of equalizer coefficients. A pair of signed
/// HIFN  floating-point values is returned for each coefficient, representing
/// HIFN  the real and imaginary parts. The number of coefficient pairs returned
/// HIFN  depends on the mode, the signal bandwidth, and for OFDM/OFDMA, the FFT
/// HIFN  size.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR equalizerCoefficients/Queries the list of equalizer coefficients. A pair of signed
/// HIPAR equalizerCoefficients/floating-point values is returned for each coefficient, representing
/// HIPAR equalizerCoefficients/the real and imaginary parts. The number of coefficient pairs returned
/// HIPAR equalizerCoefficients/depends on the mode, the signal bandwidth, and for OFDM/OFDMA, the FFT
/// HIPAR equalizerCoefficients/size.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_JQDQueryEqualizerCoefficients(ViSession instrSession,
                                             ViInt32   bufferSize,
                                             ViReal64  equalizerCoefficients[],
                                             ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringAsViReal64ArrayToUserBuffer(instrSession, NULL,
        RSDSA_ATTR_JQD_QUERY_EQUALIZER_COEFFICIENTS, NULL,
        bufferSize, equalizerCoefficients, bytesRead),
        3, "Equalizer Coefficients");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Automatically determines the attenuation level and preamplifier state
/// HIFN  (on/off). Query the resultant attenuation level with
/// HIFN  RSDSA_ATTR_JQD_CONFIGURE_ATTENUATION and the resultant preamplifier
/// HIFN  state with RSDSA_ATTR_JQD_CONFIGURE_PREAMPLIFIER_ENABLED.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_JQDAutoLevel(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_JQD_AUTO_LEVEL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the alarm thresholds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rmsMEROverallThreshold/Sets the alarm threshold for RMS MER overall. When this parameter
/// HIPAR rmsMEROverallThreshold/falls below the set threshold, an alarm is generated.
/// HIPAR berPreRSThreshold/Sets the alarm threshold for pre-Reed-Solomon BER. When this parameter
/// HIPAR berPreRSThreshold/exceeds the set threshold, an alarm is generated.
/// HIPAR berPostRSThreshold/Sets the alarm threshold for post-Reed-Solomon BER. When this
/// HIPAR berPostRSThreshold/parameter exceeds the set threshold, an alarm is generated.
ViStatus rsdsa_JQDConfigureAlarmThresholds(ViSession instrSession,
                                           ViReal64  rmsMEROverallThreshold,
                                           ViReal64  berPreRSThreshold,
                                           ViReal64  berPostRSThreshold)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_JQD_ALARM_THRESHOLD_RMS_MER_OVERALL, rmsMEROverallThreshold),
            2, "RMS MER Overall Threshold");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_JQD_ALARM_THRESHOLD_BER_PRE_REED_SOLOMON, berPreRSThreshold),
            3, "BER Pre-RS Threshold");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_JQD_ALARM_THRESHOLD_BER_POST_REED_SOLOMON, berPostRSThreshold),
            4, "BER Post-RS Threshold");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the downstream DOCSIS 3.1 OFDM mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_EnableDOCSIS31OFDMMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_ENABLE_DOCSIS_31_OFDM_MODE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a profile to be used as a data filter for displayed status and
/// HIFN  plots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR profileFilter/Selects a profile to be used as a data filter for displayed status and
/// HIPAR profileFilter/plots.
ViStatus rsdsa_D31DConfigureProfileFilter(ViSession instrSession,
                                          ViInt32   profileFilter)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31D_CONFIGURE_PROFILE_FILTER, profileFilter),
            2, "D31D Configure Profile Filter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the PLC lock status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR plcLockStatus/Queries the PLC lock status.
ViStatus rsdsa_D31DQueryPLCLockStatus(ViSession instrSession,
                                      ViInt32*  plcLockStatus)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_PLC_LOCK_STATUS, plcLockStatus),
            2, "PLC Lock Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the demodulator lock status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR demodulatorLockStatus/Queries the demodulator lock status.
ViStatus rsdsa_D31DQueryDemodulatorLockStatus(ViSession instrSession,
                                              ViInt32*  demodulatorLockStatus)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_DEMODULATOR_LOCK_STATUS, demodulatorLockStatus),
            2, "Demodualator Lock Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the input signal power level in the units selected by
/// HIFN  RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR signalPower/Queries the input signal power level in the units selected by
/// HIPAR signalPower/RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_D31DQuerySignalPower(ViSession instrSession,
                                    ViReal64* signalPower)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_SIGNAL_POWER, signalPower),
            2, "Signal Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the OFDM occupied bandwidth in Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR occupiedBandwidth/Queries the OFDM occupied bandwidth in Hz.
ViStatus rsdsa_D31DQueryOccupiedBandwidth(ViSession instrSession,
                                          ViReal64* occupiedBandwidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_OCCUPIED_BANDWIDTH, occupiedBandwidth),
            2, "Occupied Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the input signal power level, per 6 MHz, in the units selected
/// HIFN  by RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR signalPowerPer6MHz/Queries the input signal power level, per 6 MHz, in the units selected
/// HIPAR signalPowerPer6MHz/by RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_D31DQuerySignalPowerPer6MHz(ViSession instrSession,
                                           ViReal64* signalPowerPer6MHz)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_SIGNAL_POWER_PER_6MHZ, signalPowerPer6MHz),
            2, "Signal Power Per 6 MHz");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the frequency offset in Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequencyOffset/Queries the frequency offset in Hz.
ViStatus rsdsa_D31DQueryFrequencyOffset(ViSession instrSession,
                                        ViReal64* frequencyOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_FREQUENCY_OFFSET, frequencyOffset),
            2, "Frequency Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the symbol offset in Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR symbolClockOffset/Queries the symbol offset in Hz.
ViStatus rsdsa_D31DQuerySymbolClockOffset(ViSession instrSession,
                                          ViReal64* symbolClockOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_SYMBOL_CLOCK_OFFSET, symbolClockOffset),
            2, "Symbol Clock Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of NCP CRC errors.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ncpCRCErrors/Queries the number of NCP CRC errors.
ViStatus rsdsa_D31DQueryNCPCRCErrors(ViSession instrSession,
                                     ViInt32*  ncpCRCErrors)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_NCP_CRC_ERRORS, ncpCRCErrors),
            2, "NCP CRC Errors");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the payload data rate in Mbps (megabits per second) for the
/// HIFN  profile currently selected by DISPlay:[<Mode>:][WINDow:]PFILter
/// HIFN  <Profile>
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR payloadDataRateProfile/Queries the payload data rate in Mbps (megabits per second) for the
/// HIPAR payloadDataRateProfile/profile currently selected by DISPlay:[<Mode>:][WINDow:]PFILter
/// HIPAR payloadDataRateProfile/<Profile>
ViStatus rsdsa_D31DQueryPayloadDataRateProfile(ViSession instrSession,
                                               ViReal64* payloadDataRateProfile)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_PAYLOAD_DATA_RATE_PROFILE, payloadDataRateProfile),
            2, "Payload Data Rate Profile");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activate or deactivate the MER compensation function. Activating this
/// HIFN  feature interrupts the signal analysis. Compensation can be disabled by
/// HIFN  setting this to false, or it may be terminated automatically for
/// HIFN  signals when the MER changes substantially.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Activate or deactivate the MER compensation function. Activating this
/// HIPAR enabled/feature interrupts the signal analysis. Compensation can be disabled by
/// HIPAR enabled/setting this to false, or it may be terminated automatically for
/// HIPAR enabled/signals when the MER changes substantially.
ViStatus rsdsa_D31DConfigureMERCompensationEnabled(ViSession instrSession,
                                                   ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_D31D_CONFIGURE_MER_COMPENSATION_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the sign preferred for MER numerical display and plot axis
/// HIFN  labels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merSign/Selects the sign preferred for MER numerical display and plot axis
/// HIPAR merSign/labels.
ViStatus rsdsa_D31DConfigureMERSign(ViSession instrSession,
                                    ViInt32   merSign)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31D_CONFIGURE_MER_SIGN, merSign),
            2, "MER Sign");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Query whether MER compensation is currently active for the overall RMS
/// HIFN  MER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rmsCompensatedOverall/Query whether MER compensation is currently active for the overall RMS
/// HIPAR rmsCompensatedOverall/MER.
ViStatus rsdsa_D31DQueryMERRMSCompensatedOverall(ViSession  instrSession,
                                                 ViBoolean* rmsCompensatedOverall)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_MER_RMS_COMPENSATED_OVERALL, rmsCompensatedOverall),
            2, "RMS Compensated Overall");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall RMS partially equalized MER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rmsPartialyEqualized/Queries the overall RMS partially equalized MER.
ViStatus rsdsa_D31DQueryMERRMSPartialyEqualized(ViSession instrSession,
                                                ViReal64* rmsPartialyEqualized)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_MER_RMS_PARTIALY_EQUALIZED, rmsPartialyEqualized),
            2, "RMS Partially Equalized");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall RMS equalized MER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rmsEqualized/Queries the overall RMS equalized MER.
ViStatus rsdsa_D31DQueryMERRMSEqualized(ViSession instrSession,
                                        ViReal64* rmsEqualized)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_MER_RMS_EQUALIZED, rmsEqualized),
            2, "RMS Equalized");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall peak (lowest) MER. For DOCSIS 3.0 downstream
/// HIFN  modes, this is the only peak MER status applicable.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR peak/Queries the overall peak (lowest) MER. For DOCSIS 3.0 downstream
/// HIPAR peak/modes, this is the only peak MER status applicable.
ViStatus rsdsa_D31DQueryMERPeak(ViSession instrSession,
                                ViReal64* peak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_MER_PEAK, peak),
            2, "Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the profile A RMS MER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR profileARMS/Queries the profile A RMS MER.
ViStatus rsdsa_D31DQueryMERProfileARMS(ViSession instrSession,
                                       ViReal64* profileARMS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_MER_PROFILE_A_RMS, profileARMS),
            2, "Profile A RMS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Query whether MER compensation is currently active for the Profile A
/// HIFN  RMS MER quantity.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR profileARMSCompensated/Query whether MER compensation is currently active for the Profile A
/// HIPAR profileARMSCompensated/RMS MER quantity.
ViStatus rsdsa_D31DQueryMERProfileARMSCompensated(ViSession  instrSession,
                                                  ViBoolean* profileARMSCompensated)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_MER_PROFILE_A_RMS_COMPENSATED, profileARMSCompensated),
            2, "Profile A RMS Compensated");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall peak (lowest) MER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR profileAPeak/Queries the overall peak (lowest) MER.
ViStatus rsdsa_D31DQueryMERProfileAPeak(ViSession instrSession,
                                        ViReal64* profileAPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_MER_PROFILE_A_PEAK, profileAPeak),
            2, "Profile A Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the constellations for profile A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR profileAConstellations/Queries the constellations for profile A.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_D31DQueryProfileAConstellations(ViSession instrSession,
                                               ViInt32   bufferSize,
                                               ViChar    profileAConstellations[],
                                               ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_PROFILE_A_CONSTELLATIONS, RS_VAL_DIRECT_USER_CALL, bufferSize, profileAConstellations, bytesRead),
        3, "Profile A Constellations");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the RMS MER for the profile currently selected by
/// HIFN  DISPlay:[<Mode>:][WINDow:]PFILter <Profile>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR profileFilterRMS/Queries the RMS MER for the profile currently selected by
/// HIPAR profileFilterRMS/DISPlay:[<Mode>:][WINDow:]PFILter <Profile>.
ViStatus rsdsa_D31DQueryMERProfileFilterRMS(ViSession instrSession,
                                            ViReal64* profileFilterRMS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_MER_PROFILE_FILTER_RMS, profileFilterRMS),
            2, "Profile Filter RMS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Query whether MER compensation is currently active for the current
/// HIFN  profile RMS MER quantity.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR profileFilterRMSCompensated/Query whether MER compensation is currently active for the current
/// HIPAR profileFilterRMSCompensated/profile RMS MER quantity.
ViStatus rsdsa_D31DQueryMERProfileFilterRMSCompensated(ViSession  instrSession,
                                                       ViBoolean* profileFilterRMSCompensated)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_MER_PROFILE_FILTER_RMS_COMPENSATED, profileFilterRMSCompensated),
            2, "Profile Filter RMS Compensated");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall peak (lowest) MER for the profile currently
/// HIFN  selected by DISPlay:[<Mode>:][WINDow:]PFILter <Profile>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR profileFilterPeak/Queries the overall peak (lowest) MER for the profile currently
/// HIPAR profileFilterPeak/selected by DISPlay:[<Mode>:][WINDow:]PFILter <Profile>.
ViStatus rsdsa_D31DQueryMERProfileFilterPeak(ViSession instrSession,
                                             ViReal64* profileFilterPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_MER_PROFILE_FILTER_PEAK, profileFilterPeak),
            2, "Profile Filter Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the constellations for the selected profile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR profileFilterConstellations/Queries the constellations for the selected profile.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_D31DQueryProfileFilterConstellations(ViSession instrSession,
                                                    ViInt32   bufferSize,
                                                    ViChar    profileFilterConstellations[],
                                                    ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_PROFILE_FILTER_CONSTELLATIONS, RS_VAL_DIRECT_USER_CALL, bufferSize, profileFilterConstellations, bytesRead),
        3, "Profile Filter Constellations");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the RMS MER for the pilot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR pilotRMS/Queries the RMS MER for the pilot.
ViStatus rsdsa_D31DQueryMERPilotRMS(ViSession instrSession,
                                    ViReal64* pilotRMS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_MER_PILOT_RMS, pilotRMS),
            2, "Pilot RMS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Query whether MER compensation is currently active for the pilot RMS
/// HIFN  MER quantity.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR pilotRMSCompensated/Query whether MER compensation is currently active for the pilot RMS
/// HIPAR pilotRMSCompensated/MER quantity.
ViStatus rsdsa_D31DQueryMERPilotRMSCompensated(ViSession  instrSession,
                                               ViBoolean* pilotRMSCompensated)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_MER_PILOT_RMS_COMPENSATED, pilotRMSCompensated),
            2, "Pilot RMS Compensated");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the peak (lowest) MER for the pilot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR pilotPeak/Queries the peak (lowest) MER for the pilot.
ViStatus rsdsa_D31DQueryMERPilotPeak(ViSession instrSession,
                                     ViReal64* pilotPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_MER_PILOT_PEAK, pilotPeak),
            2, "Pilot Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the constellations for the pilot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR pilotConstellations/Queries the constellations for the pilot.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_D31DQueryPilotConstellations(ViSession instrSession,
                                            ViInt32   bufferSize,
                                            ViChar    pilotConstellations[],
                                            ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_PILOT_CONSTELLATIONS, RS_VAL_DIRECT_USER_CALL, bufferSize, pilotConstellations, bytesRead),
        3, "Pilot Constellations");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the RMS MER for the PLC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR plcRMS/Queries the RMS MER for the PLC.
ViStatus rsdsa_D31DQueryMERPLCRMS(ViSession instrSession,
                                  ViReal64* plcRMS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_MER_PLC_RMS, plcRMS),
            2, "PLC RMS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Query whether MER compensation is currently active for the PLC RMS MER
/// HIFN  quantity.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR plcRMSCompensated/Query whether MER compensation is currently active for the PLC RMS MER
/// HIPAR plcRMSCompensated/quantity.
ViStatus rsdsa_D31DQueryMERPLCRMSCompensated(ViSession  instrSession,
                                             ViBoolean* plcRMSCompensated)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_MER_PLC_RMS_COMPENSATED, plcRMSCompensated),
            2, "PLC RMS Compensated");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the peak (lowest) MER for the PLC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR plcPeak/Queries the peak (lowest) MER for the PLC.
ViStatus rsdsa_D31DQueryMERPLCPeak(ViSession instrSession,
                                   ViReal64* plcPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_MER_PLC_PEAK, plcPeak),
            2, "PLC Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the constellations for the PLC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR plcConstellations/Queries the constellations for the PLC.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_D31DQueryPLCConstellations(ViSession instrSession,
                                          ViInt32   bufferSize,
                                          ViChar    plcConstellations[],
                                          ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_PLC_CONSTELLATIONS, RS_VAL_DIRECT_USER_CALL, bufferSize, plcConstellations, bytesRead),
        3, "PLC Constellations");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the RMS MER for the NCP.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ncpRMS/Queries the RMS MER for the NCP.
ViStatus rsdsa_D31DQueryMERNCPRMS(ViSession instrSession,
                                  ViReal64* ncpRMS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_MER_NCP_RMS, ncpRMS),
            2, "NCP RMS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Query whether MER compensation is currently active for the NCP RMS MER
/// HIFN  quantity.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ncpRMSCompensated/Query whether MER compensation is currently active for the NCP RMS MER
/// HIPAR ncpRMSCompensated/quantity.
ViStatus rsdsa_D31DQueryMERNCPRMSCompensated(ViSession  instrSession,
                                             ViBoolean* ncpRMSCompensated)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_MER_NCP_RMS_COMPENSATED, ncpRMSCompensated),
            2, "NCP RMS Compensated");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the peak (lowest) MER for the NCP.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ncpPeak/Queries the peak (lowest) MER for the NCP.
ViStatus rsdsa_D31DQueryMERNCPPeak(ViSession instrSession,
                                   ViReal64* ncpPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_MER_NCP_PEAK, ncpPeak),
            2, "NCP Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the constellations for the NCP.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR ncpConstellations/Queries the constellations for the NCP.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_D31DQueryNCPConstellations(ViSession instrSession,
                                          ViInt32   bufferSize,
                                          ViChar    ncpConstellations[],
                                          ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_NCP_CONSTELLATIONS, RS_VAL_DIRECT_USER_CALL, bufferSize, ncpConstellations, bytesRead),
        3, "NCP Constellations");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the RMS MER for zero-bit loaded subcarriers (i.e., no
/// HIFN  profile).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR zblRMS/Queries the RMS MER for zero-bit loaded subcarriers (i.e., no
/// HIPAR zblRMS/profile).
ViStatus rsdsa_D31DQueryMERZeroBitLoadedRMS(ViSession instrSession,
                                            ViReal64* zblRMS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_MER_ZERO_BIT_LOADED_RMS, zblRMS),
            2, "ZBL RMS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Query whether MER compensation is currently active for the zero-bit
/// HIFN  loaded RMS MER quantity.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR zblRMSCompensated/Query whether MER compensation is currently active for the zero-bit
/// HIPAR zblRMSCompensated/loaded RMS MER quantity.
ViStatus rsdsa_D31DQueryMERZeroBitLoadedRMSCompensated(ViSession  instrSession,
                                                       ViBoolean* zblRMSCompensated)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_MER_ZERO_BIT_LOADED_RMS_COMPENSATED, zblRMSCompensated),
            2, "ZBL RMS Compensated");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the peak (lowest) MER for zero-bit loaded subcarriers (i.e.,
/// HIFN  no profile).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR zblPeak/Queries the peak (lowest) MER for zero-bit loaded subcarriers (i.e.,
/// HIPAR zblPeak/no profile).
ViStatus rsdsa_D31DQueryMERZeroBitLoadedPeak(ViSession instrSession,
                                             ViReal64* zblPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_MER_ZERO_BIT_LOADED_PEAK, zblPeak),
            2, "ZBL Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Performs a BER reset that clears all related counters to zero and
/// HIFN  restarts the BER/CER measurement for the current mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31DResetErrorAnalysis(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31D_RESET_ERROR_ANALYSIS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the elapsed time of the current BER/CER measurement: time
/// HIFN  elapsed since the start of the mode, or last BER reset
/// HIFN  (RSDSA_ATTR_D31D_RESET_ERROR_ANALYSIS), or last BER configuration
/// HIFN  change, in seconds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR elapsedTime/Queries the elapsed time of the current BER/CER measurement: time
/// HIPAR elapsedTime/elapsed since the start of the mode, or last BER reset
/// HIPAR elapsedTime/(RSDSA_ATTR_D31D_RESET_ERROR_ANALYSIS), or last BER configuration
/// HIPAR elapsedTime/change, in seconds.
ViStatus rsdsa_D31DQueryBERElapsedTime(ViSession instrSession,
                                       ViReal64* elapsedTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_BER_ELAPSED_TIME, elapsedTime),
            2, "Elapsed Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the BER source used for BER measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR source/Selects the BER source used for BER measurements.
ViStatus rsdsa_D31DConfigureErrorAnalysisBERSource(ViSession instrSession,
                                                   ViInt32   source)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31D_CONFIGURE_ERROR_ANALYSIS_BER_SOURCE, source),
            2, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the PLC BER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR berPLC/Queries the PLC BER.
ViStatus rsdsa_D31DQueryBERPLC(ViSession instrSession,
                               ViReal64* berPLC)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_BER_PLC, berPLC),
            2, "BER PLC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of PLC bits analyzed for the BER measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR plcTotalBits/Queries the total number of PLC bits analyzed for the BER measurement.
ViStatus rsdsa_D31DQueryBERPLCTotalBits(ViSession instrSession,
                                        ViReal64* plcTotalBits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_BER_PLC_TOTAL_BITS, plcTotalBits),
            2, "PLC Total Bits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the NCP BER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR berNCP/Queries the NCP BER.
ViStatus rsdsa_D31DQueryBERNCP(ViSession instrSession,
                               ViReal64* berNCP)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_BER_NCP, berNCP),
            2, "BER NCP");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of NCP bits analyzed for the BER measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ncpTotalBits/Queries the total number of NCP bits analyzed for the BER measurement.
ViStatus rsdsa_D31DQueryBERNCPTotalBits(ViSession instrSession,
                                        ViReal64* ncpTotalBits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_BER_NCP_TOTAL_BITS, ncpTotalBits),
            2, "NCP Total Bits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the PLC CER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cerPLC/Queries the PLC CER.
ViStatus rsdsa_D31DQueryCERPLC(ViSession instrSession,
                               ViReal64* cerPLC)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_CER_PLC, cerPLC),
            2, "CER PLC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total PLC codewords processed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR plcTotalCodewords/Queries the total PLC codewords processed.
ViStatus rsdsa_D31DQueryCERPLCTotalCodewords(ViSession instrSession,
                                             ViReal64* plcTotalCodewords)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_CER_PLC_TOTAL_CODEWORDS, plcTotalCodewords),
            2, "PLC Total Codewords");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the NCP CER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cerNCP/Queries the NCP CER.
ViStatus rsdsa_D31DQueryCERNCP(ViSession instrSession,
                               ViReal64* cerNCP)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_CER_NCP, cerNCP),
            2, "CER NCP");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total NCP codewords processed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ncpTotalCodewords/Queries the total NCP codewords processed.
ViStatus rsdsa_D31DQueryCERNCPTotalCodewords(ViSession instrSession,
                                             ViReal64* ncpTotalCodewords)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_CER_NCP_TOTAL_CODEWORDS, ncpTotalCodewords),
            2, "NCP Total Codewords");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the mode of BER measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Specifies the mode of BER measurement.
/// HIPAR manualTestDepth/Defines the test depth of the minimum BER integration. The actual test
/// HIPAR manualTestDepth/depth setting may be automatically changed in automatic error analysis
/// HIPAR manualTestDepth/mode.
/// HIPAR testDepthSource/Selects the BER source used for test depth progression.
ViStatus rsdsa_D31DConfigureErrorAnalysisProfile(ViSession instrSession,
                                                 ViInt32   mode,
                                                 ViInt32   manualTestDepth,
                                                 ViInt32   testDepthSource)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31D_CONFIGURE_ERROR_ANALYSIS_PROFILE_BER_MODE, mode),
            2, "Mode");

    if (mode == RSDSA_VAL_ERROR_ANALYSIS_MANUAL)
    {
        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31D_CONFIGURE_ERROR_ANALYSIS_PROFILE_TEST_DEPTH, manualTestDepth),
                3, "Manual Test Depth");
    }

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31D_CONFIGURE_ERROR_ANALYSIS_PROFILE_TEST_DEPTH_SOURCE, testDepthSource),
            4, "Test Depth Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the percentage of blocks processed for the current BER/CER
/// HIFN  measurement: blocks processed since the start of the mode, or last BER
/// HIFN  reset (CALCulate:[<Mode>:][AVERage:]BERatio:CLEar), or last BER
/// HIFN  configuration change. Expressed as the percentage of blocks processed
/// HIFN  relative to the current test depth. A block is 10^7 bits maximum.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR blocksProcessed/Queries the percentage of blocks processed for the current BER/CER
/// HIPAR blocksProcessed/measurement: blocks processed since the start of the mode, or last BER
/// HIPAR blocksProcessed/reset (CALCulate:[<Mode>:][AVERage:]BERatio:CLEar), or last BER
/// HIPAR blocksProcessed/configuration change. Expressed as the percentage of blocks processed
/// HIPAR blocksProcessed/relative to the current test depth. A block is 10^7 bits maximum.
ViStatus rsdsa_D31DQueryProfileBERBlocksProcessed(ViSession instrSession,
                                                  ViReal64* blocksProcessed)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_PROFILE_BER_BLOCKS_PROCESSED, blocksProcessed),
            2, "Blocks Processed");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the pre-LDPC BER for the profile currently selected by
/// HIFN  DISPlay:[<Mode>:][WINDow:]PFILter <Profile>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR berPreLDPC/Queries the pre-LDPC BER for the profile currently selected by
/// HIPAR berPreLDPC/DISPlay:[<Mode>:][WINDow:]PFILter <Profile>.
ViStatus rsdsa_D31DQueryProfileBERPreLDPC(ViSession instrSession,
                                          ViReal64* berPreLDPC)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_PROFILE_BER_PRE_LDPC, berPreLDPC),
            2, "BER Pre-LDPC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of bits analyzed at the input of the LDPC
/// HIFN  decoder for the BER measurement, for the profile currently selected by
/// HIFN  DISPlay:[<Mode>:][WINDow:]PFILter <Profile>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR preLDPCTotalBits/Queries the total number of bits analyzed at the input of the LDPC
/// HIPAR preLDPCTotalBits/decoder for the BER measurement, for the profile currently selected by
/// HIPAR preLDPCTotalBits/DISPlay:[<Mode>:][WINDow:]PFILter <Profile>.
ViStatus rsdsa_D31DQueryProfileBERPreLDPCTotalBits(ViSession instrSession,
                                                   ViReal64* preLDPCTotalBits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_PROFILE_BER_PRE_LDPC_TOTAL_BITS, preLDPCTotalBits),
            2, "Pre-LDPC Total Bits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the post-LDPC BER for the profile currently selected by
/// HIFN  DISPlay:[<Mode>:][WINDow:]PFILter <Profile>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR berPostLDPC/Queries the post-LDPC BER for the profile currently selected by
/// HIPAR berPostLDPC/DISPlay:[<Mode>:][WINDow:]PFILter <Profile>.
ViStatus rsdsa_D31DQueryProfileBERPostLDPC(ViSession instrSession,
                                           ViReal64* berPostLDPC)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_PROFILE_BER_POST_LDPC, berPostLDPC),
            2, "BER Post-LDPC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of bits analyzed at the output of the LDPC
/// HIFN  decoder for the BER measurement, for the profile currently selected by
/// HIFN  DISPlay:[<Mode>:][WINDow:]PFILter <Profile>. Note that the bits for
/// HIFN  undecodable codewords are not included in this total.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR postLDPCTotalBits/Queries the total number of bits analyzed at the output of the LDPC
/// HIPAR postLDPCTotalBits/decoder for the BER measurement, for the profile currently selected by
/// HIPAR postLDPCTotalBits/DISPlay:[<Mode>:][WINDow:]PFILter <Profile>. Note that the bits for
/// HIPAR postLDPCTotalBits/undecodable codewords are not included in this total.
ViStatus rsdsa_D31DQueryProfileBERPostLDPCTotalBits(ViSession instrSession,
                                                    ViReal64* postLDPCTotalBits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_PROFILE_BER_POST_LDPC_TOTAL_BITS, postLDPCTotalBits),
            2, "Post-LDPC Total Bits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the post-BCH BER for the profile currently selected by
/// HIFN  DISPlay:[<Mode>:][WINDow:]PFILter <Profile>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR berPostBCH/Queries the post-BCH BER for the profile currently selected by
/// HIPAR berPostBCH/DISPlay:[<Mode>:][WINDow:]PFILter <Profile>.
ViStatus rsdsa_D31DQueryProfileBERPostBCH(ViSession instrSession,
                                          ViReal64* berPostBCH)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_PROFILE_BER_POST_BCH, berPostBCH),
            2, "BER Post-BCH");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of bits analyzed at the output of the BCH
/// HIFN  decoder for the BER measurement, for the profile currently selected by
/// HIFN  DISPlay:[<Mode>:][WINDow:]PFILter <Profile>. Note that the bits for
/// HIFN  undecodable codewords are not included in this total.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR postBCHTotalBits/Queries the total number of bits analyzed at the output of the BCH
/// HIPAR postBCHTotalBits/decoder for the BER measurement, for the profile currently selected by
/// HIPAR postBCHTotalBits/DISPlay:[<Mode>:][WINDow:]PFILter <Profile>. Note that the bits for
/// HIPAR postBCHTotalBits/undecodable codewords are not included in this total.
ViStatus rsdsa_D31DQueryProfileBERPostBCHTotalBits(ViSession instrSession,
                                                   ViReal64* postBCHTotalBits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_PROFILE_BER_POST_BCH_TOTAL_BITS, postBCHTotalBits),
            2, "Post-BCH Total Bits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the post-LDPC CER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cerPostLDPC/Queries the post-LDPC CER.
ViStatus rsdsa_D31DQueryProfileCERPostLDPC(ViSession instrSession,
                                           ViReal64* cerPostLDPC)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_PROFILE_CER_POST_LDPC, cerPostLDPC),
            2, "CER Post-LDPC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total post-LDPC codewords processed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR postLDPCTotalCodewords/Queries the total post-LDPC codewords processed.
ViStatus rsdsa_D31DQueryProfileCERPostLDPCTotalCodewords(ViSession instrSession,
                                                         ViReal64* postLDPCTotalCodewords)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_PROFILE_CER_POST_LDPC_TOTAL_CODEWORDS, postLDPCTotalCodewords),
            2, "Post-LDPC Total Codewords");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of codeword errors at the output of the LDPC
/// HIFN  decoder.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR postLDPCCodewordErrors/Queries the total number of codeword errors at the output of the LDPC
/// HIPAR postLDPCCodewordErrors/decoder.
ViStatus rsdsa_D31DQueryProfileCERPostLDPCCodewordErrors(ViSession instrSession,
                                                         ViReal64* postLDPCCodewordErrors)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_PROFILE_CER_POST_LDPC_CODEWORD_ERRORS, postLDPCCodewordErrors),
            2, "Post-LDPC Codewords Error");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the post-BCH CER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cerPostBCH/Queries the post-BCH CER.
ViStatus rsdsa_D31DQueryProfileCERPostBCH(ViSession instrSession,
                                          ViReal64* cerPostBCH)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_PROFILE_CER_POST_BCH, cerPostBCH),
            2, "CER Post-BCH");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total post-BCH codewords processed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR postBCHTotalCodewords/Queries the total post-BCH codewords processed.
ViStatus rsdsa_D31DQueryProfileCERPostBCHTotalCodewords(ViSession instrSession,
                                                        ViReal64* postBCHTotalCodewords)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_PROFILE_CER_POST_BCH_TOTAL_CODEWORDS, postBCHTotalCodewords),
            2, "Post-BCH Total Codewords");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of codeword errors at the output of the BCH
/// HIFN  decoder.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR postBCHCodewordErrors/Queries the total number of codeword errors at the output of the BCH
/// HIPAR postBCHCodewordErrors/decoder.
ViStatus rsdsa_D31DQueryProfileCERPostBCHCodewordErrors(ViSession instrSession,
                                                        ViReal64* postBCHCodewordErrors)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_PROFILE_CER_POST_BCH_CODEWORD_ERRORS, postBCHCodewordErrors),
            2, "Post-BCH Codewords Errors");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of bits in the average payload per codeword,
/// HIFN  for the profile currently selected by DISPlay:[<Mode>:][WINDow:]PFILter
/// HIFN  <Profile>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR avgPayloadPerCodeword/Queries the total number of bits in the average payload per codeword,
/// HIPAR avgPayloadPerCodeword/for the profile currently selected by DISPlay:[<Mode>:][WINDow:]PFILter
/// HIPAR avgPayloadPerCodeword/<Profile>.
ViStatus rsdsa_D31DQueryProfileCERAvgPayloadPerCodeword(ViSession instrSession,
                                                        ViReal64* avgPayloadPerCodeword)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_PROFILE_CER_AVG_PAYLOAD_PER_CODEWORD, avgPayloadPerCodeword),
            2, "Avg Payload Per Codeword");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the average number of LDPC iterations.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR avgLDPCIterations/Queries the average number of LDPC iterations.
ViStatus rsdsa_D31DQueryProfileCERAvgLDPCIterations(ViSession instrSession,
                                                    ViReal64* avgLDPCIterations)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_PROFILE_CER_AVG_LDPC_ITERATIONS, avgLDPCIterations),
            2, "Avg LDPC Iterations");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the constellation plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31DDisplayConstellationPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31D_DISPLAY_CONSTELLATION_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the I/Q density plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31DDisplayIQDensityPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31D_DISPLAY_I_Q_DENSITY_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the measurement mode for the I/Q density plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementMode/Selects the measurement mode for the I/Q density plot.
/// HIPAR windowWidth/Specifies the sliding window width in samples for the I/Q density plot
/// HIPAR windowWidth/in sliding window mode.
ViStatus rsdsa_D31DConfigureIQMeasurementMode(ViSession instrSession,
                                              ViInt32   measurementMode,
                                              ViInt32   windowWidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31D_CONFIGURE_I_Q_MEASUREMENT_MODE, measurementMode),
            2, "Measurement Mode");

    if (measurementMode == RSDSA_VAL_IQ_MEASUREMENT_MODE_SLIDING_WINDOW)
        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31D_CONFIGURE_I_Q_WINDOW_WIDTH, windowWidth),
                3, "Window Width");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the zoom setting for the I/Q density plot display. Note that
/// HIFN  the x and y origin are the top left corner of the window, as per the
/// HIFN  GUI settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR x/X origin (left-most window point), between 0 and 1.
/// HIPAR y/Y origin (top-most window point), between 0 and 1.
/// HIPAR zoom/Zoom setting, 1 to 128.
ViStatus rsdsa_D31DSetIQDisplayZoom(ViSession instrSession,
                                    ViReal64  x,
                                    ViReal64  y,
                                    ViInt32   zoom)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_CheckRangeTableRangeViReal64(instrSession, &RsDSA_rngIQZoomOrigin, x),
        2, "X");
    viCheckParm(rsdsa_CheckRangeTableRangeViReal64(instrSession, &RsDSA_rngIQZoomOrigin, y),
        3, "Y");
    viCheckParm(rsdsa_CheckRangeTableRangeViInt32(instrSession, &RsDSA_rngIQZoomLevel, zoom),
        4, "Zoom");

    snprintf(param, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%f,%f,%ld", x, y, zoom);

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_HIDDEN_D31D_DISPLAY_I_Q_ZOOM, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Save the constellation I/Q data to the specified file. The file
/// HIFN  extension is .iq.tar. Note that there is no way to abort this command,
/// HIFN  and large file sizes could take a long time to capture depending on the
/// HIFN  input data, so use maxSamples or timeoutSeconds to limit the file size
/// HIFN  or time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/File path, uses .iq.tar extension.
/// HIPAR maxSamples/Sets the I/Q data file maximum size in samples.
/// HIPAR timeoutSeconds/Sets the maximum time in seconds to save data.
ViStatus rsdsa_D31DCaptureIQDataToFile(ViSession instrSession,
                                       ViString  fileName,
                                       ViInt64   maxSamples,
                                       ViReal64  timeoutSeconds)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt64(instrSession, NULL, RSDSA_ATTR_D31D_IQ_DATA_EXPORT_MAX_SAMPLES, maxSamples),
        3, "Max Samples");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_IQ_DATA_EXPORT_TIMEOUT, timeoutSeconds),
            4, "Timeout Seconds");

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31D_CAPTURE_IQ_DATA_TO_FILENAME, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the MER vs. time plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31DDisplayMERVsTimePlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31D_DISPLAY_MER_VS_TIME_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a time filter period to be used for displayed MER vs. Time
/// HIFN  plots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR period/Selects a time filter period to be used for displayed MER vs. Time
/// HIPAR period/plots.
ViStatus rsdsa_D31DConfigureMERVsTimePlotFilterPeriod(ViSession instrSession,
                                                      ViInt32   period)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31D_CONFIGURE_MER_VS_TIME_PLOT_FILTER_PERIOD, period),
            2, "Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the BER vs. time plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31DDisplayBERVsTimePlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31D_DISPLAY_BER_VS_TIME_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a time filter period to be used for displayed BER vs. Time
/// HIFN  plots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR period/Selects a time filter period to be used for displayed BER vs. Time
/// HIPAR period/plots.
ViStatus rsdsa_D31DConfigureBERVsTimePlotFilterPeriod(ViSession instrSession,
                                                      ViInt32   period)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31D_CONFIGURE_BER_VS_TIME_PLOT_FILTER_PERIOD, period),
            2, "Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the POWer vs. time plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31DDisplayPowerVsTimePlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31D_DISPLAY_POWER_VS_TIME_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a time filter period to be used for displayed Power vs. Time
/// HIFN  plots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR period/Selects a time filter period to be used for displayed Power vs. Time
/// HIPAR period/plots.
ViStatus rsdsa_D31DConfigurePowerVsTimePlotFilterPeriod(ViSession instrSession,
                                                        ViInt32   period)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31D_CONFIGURE_POWER_VS_TIME_PLOT_FILTER_PERIOD, period),
            2, "Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the MER vs. Subcarrier plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31DDisplayMERVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31D_DISPLAY_MER_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays a downstream or upstream CCDF (Complementary Cumulative
/// HIFN  Distribution Function histogram as per CM-SP-PHYv3.1-I16 sections 9.3.8
/// HIFN  or 9.4.6 respectively). This command may take up to 30 seconds to
/// HIFN  execute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31DDisplayCCDFHistogram(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31D_DISPLAY_CCDF_HISTOGRAM, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the peak CCDF power level in the units selected by
/// HIFN  RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ccdfPeak/Queries the peak CCDF power level in the units selected by
/// HIPAR ccdfPeak/RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_D31DQueryCCDFPeak(ViSession instrSession,
                                 ViReal64* ccdfPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_QUERY_CCDF_PEAK, ccdfPeak),
            2, "CCDF Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the spectrum analysis plot for DOCSIS 3.1 Downstream mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31DDisplaySpectrumAnalysisPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31D_DISPLAY_SPECTRUM_ANALYSIS_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns off all markers including delta markers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31DSpectrumAnalysisAllMarkersOff(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_ALL_MARKERS_OFF, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns off all delta markers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31DSpectrumAnalysisAllDeltaMarkersOff(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_ALL_DELTA_MARKERS_OFF, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Clears all masks, including both the lower and upper masks.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31DSpectrumAnalysisClearAllMasks(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_CLEAR_ALL_MASKS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns all masks on or off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR displayAllMasksEnabled/Turns all masks on or off.
ViStatus rsdsa_D31DSpectrumAnalysisConfigureDisplayAllMasksEnabled(
                            ViSession instrSession,
                            ViBoolean displayAllMasksEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DISPLAY_ALL_MASKS_ENABLED, displayAllMasksEnabled),
            2, "Display All Masks Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures amplitude settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR referenceLevel/Defines the reference level for the displayed trace in the units
/// HIPAR referenceLevel/selected by RSDSA_ATTR_POWER_UNITS. This setting is only used when
/// HIPAR referenceLevel/automatic y-axis scaling is turned off. If the reference level is set,
/// HIPAR referenceLevel/then the attenuation set in INPut:ATTenuation is no longer applied.
/// HIPAR scalePerDivision/Defines the scale per division in the units selected by
/// HIPAR scalePerDivision/RSDSA_ATTR_POWER_UNITS to be used for the y-axis for all trace
/// HIPAR scalePerDivision/displays, which all have 10 divisions on the y-axis. Note that this
/// HIPAR scalePerDivision/command sets the scale per division, whereas the user of the DSA GUI
/// HIPAR scalePerDivision/sets the entire range for the vertical axis.
ViStatus rsdsa_D31DSpectrumAnalysisConfigureAmplitudeSettings(
                            ViSession instrSession,
                            ViReal64  referenceLevel,
                            ViReal64  scalePerDivision)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_CONFIGURE_AMPLITUDE_REFERENCE_LEVEL, referenceLevel),
            2, "Reference Level");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_CONFIGURE_AMPLITUDE_SCALE_PER_DIVISION, scalePerDivision),
            3, "Scale Per Division");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Scales the y-axis for trace displays by automatically adjusting the
/// HIFN  y-axis reference level and scale per division to best fit the
/// HIFN  measurement results. Otherwise, the y-axis can be manually scaled using
/// HIFN  DISPlay:[<Mode>:][WINDow:]TRACe[<Trace>]:Y[:SCALe]:RLEVel and
/// HIFN  DISPlay:[<Mode>:][WINDow:]TRACe[<Trace>]:Y[:SCALe]:PDIVision.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31DSpectrumAnalysisAutoScaleYAxis(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_AUTO_SCALE_Y_AXIS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures bandwidth settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoAverageCountEnabled/When turned on, the DSA automatically determines the average setting.
/// HIPAR manualAverageCount/Defines the number of averages. A value of 1 indicates no averaging.
/// HIPAR manualAverageCount/Note: For OFDM spectrum analysis, setting an average that is not a
/// HIPAR manualAverageCount/multiple of 128 may cause artifacts between pilots.
ViStatus rsdsa_D31DSpectrumAnalysisConfigureBandwidthSettings(
                            ViSession instrSession,
                            ViInt32   manualAverageCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_CONFIGURE_MANUAL_AVERAGE_COUNT, manualAverageCount),
            2, "Manual Average Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures measurement settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR captureMode/Selects whether to use continuous captures or single trace captures.
/// HIPAR traceHold/Selects the measurement mode for the displayed trace. The DSA applies
/// HIPAR traceHold/minimum/maximum hold over multiple single traces initiated by
/// HIPAR traceHold/DISPlay:[<Mode>:][WINDow:]TRACe[<Trace>]:RSINgle command.
ViStatus rsdsa_D31DSpectrumAnalysisConfigureMeasurementsSettings(
                            ViSession instrSession,
                            ViInt32   captureMode,
                            ViInt32   traceHold)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_CONFIGURE_CAPTURE_MODE, captureMode),
            2, "Capture Mode");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_CONFIGURE_TRACE_HOLD, traceHold),
            3, "Trace Hold");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restarts a single sweep, rearming for the next trigger, if applicable.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31DSpectrumAnalysisRunSingleSweep(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_RUN_SINGLE_SWEEP, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns individual markers on and off. If the specified marker is a
/// HIFN  delta marker, it is turned into a normal marker. The default position
/// HIFN  is at the maximum peak.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
/// HIPAR state/Turns individual markers on and off. If the specified marker is a
/// HIPAR state/delta marker, it is turned into a normal marker. The default position
/// HIPAR state/is at the maximum peak.
ViStatus rsdsa_D31DSpectrumAnalysisConfigureMarkerState(ViSession instrSession,
                                                        ViInt32   marker,
                                                        ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_MARKER_STATE,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, repCap, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_MARKER_STATE, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a marker to a particular coordinate on the x-axis. If the
/// HIFN  specified marker is turned off, it is turned on first. If the specified
/// HIFN  marker is a delta marker, it is turned into a normal marker.
/// HIFN     The units are Hz (frequency domain) or s (time domain, for zero span
/// HIFN  mode).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
/// HIPAR xPosition/Moves a marker to a particular coordinate on the x-axis. If the
/// HIPAR xPosition/specified marker is turned off, it is turned on first. If the specified
/// HIPAR xPosition/marker is a delta marker, it is turned into a normal marker.
ViStatus rsdsa_D31DSpectrumAnalysisConfigureMarkerXPosition(ViSession instrSession,
                                                            ViInt32   marker,
                                                            ViReal64  xPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_MARKER_X_POSITION,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_MARKER_X_POSITION, xPosition),
            3, "X Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the position of a marker on the y-axis. If the specified
/// HIFN  marker is turned off, it is turned on first. If the specified marker is
/// HIFN  a delta marker, it is turned into a normal marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
/// HIPAR yPosition/Queries the position of a marker on the y-axis. If the specified
/// HIPAR yPosition/marker is turned off, it is turned on first. If the specified marker is
/// HIPAR yPosition/a delta marker, it is turned into a normal marker.
ViStatus rsdsa_D31DSpectrumAnalysisQueryMarkerYPosition(ViSession instrSession,
                                                        ViInt32   marker,
                                                        ViReal64* yPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_MARKER_Y_POSITION,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_MARKER_Y_POSITION, yPosition),
            3, "Y Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a marker to the highest amplitude displayed. If the specified
/// HIFN  marker is turned off, it is turned on first. If the specified marker is
/// HIFN  a delta marker, it is turned into a normal marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_D31DSpectrumAnalysisMarkerToMaximumPeak(ViSession instrSession,
                                                       ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_MARKER_TO_MAXIMUM_PEAK,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_MARKER_TO_MAXIMUM_PEAK, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a marker to the next peak to the left of the current marker
/// HIFN  position. If the specified marker is turned off, it is turned on first.
/// HIFN  If the specified marker is a delta marker, it is turned into a normal
/// HIFN  marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_D31DSpectrumAnalysisMarkerToNextPeakLeft(ViSession instrSession,
                                                        ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_LEFT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_LEFT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a marker to the next peak to the right of the current marker
/// HIFN  position. If the specified marker is turned off, it is turned on first.
/// HIFN  If the specified marker is a delta marker, it is turned into a normal
/// HIFN  marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_D31DSpectrumAnalysisMarkerToNextPeakRight(ViSession instrSession,
                                                         ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_RIGHT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_RIGHT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a marker to the next displayed point to the left of the current
/// HIFN  marker position. If the specified marker is turned off, it is turned on
/// HIFN  first. If the specified marker is a delta marker, it is turned into a
/// HIFN  normal marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_D31DSpectrumAnalysisMarkerToNextPointLeft(ViSession instrSession,
                                                         ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_LEFT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_LEFT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a marker to the next displayed point to the right of the current
/// HIFN  marker position. If the specified marker is turned off, it is turned on
/// HIFN  first. If the specified marker is a delta marker, it is turned into a
/// HIFN  normal marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_D31DSpectrumAnalysisMarkerToNextPointRight(ViSession instrSession,
                                                          ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_RIGHT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_RIGHT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the center frequency to the frequency of a marker. If the
/// HIFN  specified marker is turned off, it is turned on first. If the specified
/// HIFN  marker is a delta marker, it is turned into a normal marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_D31DSpectrumAnalysisMarkerSetCenterFrequency(ViSession instrSession,
                                                            ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_MARKER_SET_CENTER_FREQUENCY,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_MARKER_SET_CENTER_FREQUENCY, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the reference level to the power level of a marker. If the
/// HIFN  specified marker is turned off, it is turned on first. If the specified
/// HIFN  marker is a delta marker, it is turned into a normal marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_D31DSpectrumAnalysisMarkerSetPowerReferenceLevel(
                            ViSession instrSession,
                            ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_MARKER_SET_POWER_REFERENCE_LEVEL,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_MARKER_SET_POWER_REFERENCE_LEVEL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns delta markers on and off. If the specified marker is a normal
/// HIFN  marker, it is turned into a delta marker. The default position is at
/// HIFN  the maximum peak.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deltaMarker/Selects repcap Delta.
/// HIPAR markerState/Turns delta markers on and off. If the specified marker is a normal
/// HIPAR markerState/marker, it is turned into a delta marker. The default position is at
/// HIPAR markerState/the maximum peak.
ViStatus rsdsa_D31DSpectrumAnalysisConfigureDeltaMarkerState(ViSession instrSession,
                                                             ViInt32   deltaMarker,
                                                             ViBoolean markerState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deltaMarker, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_STATE,
            "Delta", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, repCap, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_STATE, markerState),
            3, "Marker State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a reference marker for a delta marker. Values of delta marker
/// HIFN  are relative to the reference marker. If the specified reference marker
/// HIFN  is turned off, it is turned on first. If the specified delta marker is
/// HIFN  turned off, it is turned on first. If the specified delta marker is a
/// HIFN  normal marker, it is turned into a delta marker. If the specified
/// HIFN  reference marker is the same as the specified delta marker, the marker
/// HIFN  is changed to a normal marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deltaMarker/Selects repcap Delta.
/// HIPAR linkedReferenceMarker/Selects a reference marker for a delta marker. Values of delta marker
/// HIPAR linkedReferenceMarker/are relative to the reference marker. If the specified reference marker
/// HIPAR linkedReferenceMarker/is turned off, it is turned on first. If the specified delta marker is
/// HIPAR linkedReferenceMarker/turned off, it is turned on first. If the specified delta marker is a
/// HIPAR linkedReferenceMarker/normal marker, it is turned into a delta marker. If the specified
/// HIPAR linkedReferenceMarker/reference marker is the same as the specified delta marker, the marker
/// HIPAR linkedReferenceMarker/is changed to a normal marker.
ViStatus rsdsa_D31DSpectrumAnalysisConfigureDeltaMarkerLinkedReferenceMarker(
                            ViSession instrSession,
                            ViInt32   deltaMarker,
                            ViInt32   linkedReferenceMarker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deltaMarker, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_LINKED_REFERENCE_MARKER,
            "Delta", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    if (linkedReferenceMarker == RSDSA_VAL_REFERENCE_MARKER_NORMAL)
        linkedReferenceMarker = deltaMarker;

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, repCap, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_LINKED_REFERENCE_MARKER, linkedReferenceMarker),
            3, "Linked Reference Marker");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to a particular absolute coordinate on the
/// HIFN  x-axis. If the specified marker is turned off, it is turned on first.
/// HIFN  If the specified marker is a normal marker, it is turned into a delta
/// HIFN  marker. The query returns the absolute position of the delta marker.
/// HIFN     The units are Hz (frequency domain) or s (time domain, for zero span
/// HIFN  mode).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deltaMarker/Selects repcap Delta.
/// HIPAR absoluteXPosition/Moves a delta marker to a particular absolute coordinate on the
/// HIPAR absoluteXPosition/x-axis. If the specified marker is turned off, it is turned on first.
/// HIPAR absoluteXPosition/If the specified marker is a normal marker, it is turned into a delta
/// HIPAR absoluteXPosition/marker. The query returns the absolute position of the delta marker.
ViStatus rsdsa_D31DSpectrumAnalysisConfigureDeltaMarkerAbsoluteXPosition(
                            ViSession instrSession,
                            ViInt32   deltaMarker,
                            ViReal64  absoluteXPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deltaMarker, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_ABSOLUTE_X_POSITION,
            "Delta", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_ABSOLUTE_X_POSITION, absoluteXPosition),
            3, "Absolute X Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to a particular relative coordinate on the
/// HIFN  x-axis. If the specified marker is turned off, it is turned on first.
/// HIFN  If the specified marker is a normal marker, it is turned into a delta
/// HIFN  marker. The query returns the relative position of the delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deltaMarker/Selects repcap Delta.
/// HIPAR relativeXPosition/Moves a delta marker to a particular relative coordinate on the
/// HIPAR relativeXPosition/x-axis. If the specified marker is turned off, it is turned on first.
/// HIPAR relativeXPosition/If the specified marker is a normal marker, it is turned into a delta
/// HIPAR relativeXPosition/marker. The query returns the relative position of the delta marker.
ViStatus rsdsa_D31DSpectrumAnalysisConfigureDeltaMarkerRelativeXPosition(
                            ViSession instrSession,
                            ViInt32   deltaMarker,
                            ViReal64  relativeXPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deltaMarker, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_RELATIVE_X_POSITION,
            "Delta", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_RELATIVE_X_POSITION, relativeXPosition),
            3, "Relative X Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the absolute position of a delta marker on the y-axis. If the
/// HIFN  specified marker is turned off, it is turned on first. If the specified
/// HIFN  marker is a normal marker, it is turned into a delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deltaMarker/Selects repcap Delta.
/// HIPAR yPosition/Queries the absolute position of a delta marker on the y-axis. If the
/// HIPAR yPosition/specified marker is turned off, it is turned on first. If the specified
/// HIPAR yPosition/marker is a normal marker, it is turned into a delta marker.
ViStatus rsdsa_D31DSpectrumAnalysisQueryDeltaMarkerYPosition(ViSession instrSession,
                                                             ViInt32   deltaMarker,
                                                             ViReal64* yPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deltaMarker, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_Y_POSITION,
            "Delta", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_Y_POSITION, yPosition),
            3, "Y Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to the highest amplitude displayed. If the
/// HIFN  specified marker is turned off, it is turned on first. If the specified
/// HIFN  marker is a normal marker, it is turned into a delta marker.
/// HIFN     CALCulate:[<Mode>:]DELTamarker<Delta>[:FUNCtion]:MAXimum[:PEAK]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deltaMarker/Selects repcap Delta.
ViStatus rsdsa_D31DSpectrumAnalysisDeltaMarkerToMaximumPeak(ViSession instrSession,
                                                            ViInt32   deltaMarker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deltaMarker, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_MAXIMUM_PEAK,
            "Delta", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_MAXIMUM_PEAK, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to the next peak to the left of the current
/// HIFN  marker position. If the specified marker is turned off, it is turned on
/// HIFN  first. If the specified marker is a normal marker, it is turned into a
/// HIFN  delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deltaMarker/Selects repcap Delta.
ViStatus rsdsa_D31DSpectrumAnalysisDeltaMarkerToNextPeakLeft(ViSession instrSession,
                                                             ViInt32   deltaMarker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deltaMarker, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_LEFT,
            "Delta", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_LEFT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to the next peak to the right of the current
/// HIFN  marker position. If the specified marker is turned off, it is turned on
/// HIFN  first. If the specified marker is a normal marker, it is turned into a
/// HIFN  delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deltaMarker/Selects repcap Delta.
ViStatus rsdsa_D31DSpectrumAnalysisDeltaMarkerToNextPeakRight(
                            ViSession instrSession,
                            ViInt32   deltaMarker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deltaMarker, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_RIGHT,
            "Delta", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_RIGHT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to the next displayed point to the left of the
/// HIFN  current marker position. If the specified marker is turned off, it is
/// HIFN  turned on first. If the specified marker is a normal marker, it is
/// HIFN  turned into a delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deltaMarker/Selects repcap Delta.
ViStatus rsdsa_D31DSpectrumAnalysisDeltaMarkerToNextPointLeft(
                            ViSession instrSession,
                            ViInt32   deltaMarker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deltaMarker, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_LEFT,
            "Delta", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_LEFT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to the next displayed point to the right of the
/// HIFN  current marker position. If the specified marker is turned off, it is
/// HIFN  turned on first. If the specified marker is a normal marker, it is
/// HIFN  turned into a delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deltaMarker/Selects repcap Delta.
ViStatus rsdsa_D31DSpectrumAnalysisDeltaMarkerToNextPointRight(
                            ViSession instrSession,
                            ViInt32   deltaMarker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deltaMarker, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_RIGHT,
            "Delta", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_RIGHT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the center frequency to the frequency of a delta marker. If the
/// HIFN  specified marker is turned off, it is turned on first. If the specified
/// HIFN  marker is a normal marker, it is turned into a delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deltaMarker/Selects repcap Delta.
ViStatus rsdsa_D31DSpectrumAnalysisDeltaMarkerSetCenterFrequency(
                            ViSession instrSession,
                            ViInt32   deltaMarker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deltaMarker, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_SET_CENTER_FREQUENCY,
            "Delta", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_SET_CENTER_FREQUENCY, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the reference level to the power level of a delta marker. If the
/// HIFN  specified marker is turned off, it is turned on first. If the specified
/// HIFN  marker is a normal marker, it is turned into a delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deltaMarker/Selects repcap Delta.
ViStatus rsdsa_D31DSpectrumAnalysisDeltaMarkerSetPowerReferenceLevel(
                            ViSession instrSession,
                            ViInt32   deltaMarker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deltaMarker, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_SET_POWER_REFERENCE_LEVEL,
            "Delta", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_SET_POWER_REFERENCE_LEVEL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the bandpower between a specified delta marker and its
/// HIFN  reference marker in the frequency domain. An error is generated if the
/// HIFN  specified marker is turned off, or if the specified marker is a normal
/// HIFN  marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deltaMarker/Selects repcap Delta.
/// HIPAR bandwidthPowerResult/Queries the bandpower between a specified delta marker and its
/// HIPAR bandwidthPowerResult/reference marker in the frequency domain. An error is generated if the
/// HIPAR bandwidthPowerResult/specified marker is turned off, or if the specified marker is a normal
/// HIPAR bandwidthPowerResult/marker.
ViStatus rsdsa_D31DSpectrumAnalysisQueryDeltaMarkerBandwidthPowerResult(
                            ViSession instrSession,
                            ViInt32   deltaMarker,
                            ViReal64* bandwidthPowerResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deltaMarker, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_BANDWIDTH_POWER_RESULT,
            "Delta", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_BANDWIDTH_POWER_RESULT, bandwidthPowerResult),
            3, "Bandwidth Power Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns individual masks on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mask/Selects repcap Mask.
/// HIPAR visible/Turns individual masks on and off.
ViStatus rsdsa_D31DSpectrumAnalysisConfigureDisplayMaskVisible(
                            ViSession instrSession,
                            ViInt32   mask,
                            ViBoolean visible)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMask = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Mask", &maxMask),
        2, "Mask");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mask, 1, maxMask),
            2, "Mask");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_CONFIGURE_DISPLAY_MASK_VISIBLE,
            "Mask", mask - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Mask");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, repCap, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_CONFIGURE_DISPLAY_MASK_VISIBLE, visible),
            3, "Visible");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the specified lower mask with 1 to 20 points. Each point in
/// HIFN  the mask configuration must have a corresponding X and Y value. All
/// HIFN  frequency (X) entries are in Hz, and all amplitude (Y) entries are in
/// HIFN  the units selected by RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mask/Selects repcap Mask.
/// HIPAR points/Defines the length of the X and Y values arrays. A mask definition may
/// HIPAR points/have up to 20 points defined.
/// HIPAR xValues/Defines the frequency (x) values of the points in the display mask.
/// HIPAR xValues/All frequency values are in Hz.
/// HIPAR yValues/Defines the amplitude (y) values of the points in the display mask.
/// HIPAR yValues/All amplitude values are in the units selected by
/// HIPAR yValues/RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_D31DSpectrumAnalysisConfigureLowerDisplayMask(ViSession instrSession,
                                                             ViInt32   mask,
                                                             ViInt32   points,
                                                             ViReal64  xValues[],
                                                             ViReal64  yValues[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMask = 0;
    ViChar   valuesParameter[RS_MAX_MESSAGE_BUF_SIZE] = { '\0' };
    ViChar*  p2buffer = valuesParameter;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Mask", &maxMask),
        2, "Mask");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mask, 1, maxMask),
            2, "Mask");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DISPLAY_MASK_LOWER_CONFIGURATION,
            "Mask", mask - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Mask");

    for (int i = 0; i < points; i++)
    {
        p2buffer += snprintf(valuesParameter, RS_MAX_MESSAGE_BUF_SIZE - strlen(valuesParameter), "%f,%f", xValues[i], yValues[i]);
    }
    *(--p2buffer) = '\0';

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DISPLAY_MASK_LOWER_CONFIGURATION, valuesParameter));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the specified upper mask with 1 to 20 points. Each point in
/// HIFN  the mask configuration must have a corresponding X and Y value. All
/// HIFN  frequency (X) entries are in Hz, and all amplitude (Y) entries are in
/// HIFN  the units selected by RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mask/Selects repcap Mask.
/// HIPAR points/Defines the length of the X and Y values arrays. A mask definition may
/// HIPAR points/have up to 20 points defined.
/// HIPAR xValues/Defines the frequency (x) values of the points in the display mask.
/// HIPAR xValues/All frequency values are in Hz.
/// HIPAR yValues/Defines the amplitude (y) values of the points in the display mask.
/// HIPAR yValues/All amplitude values are in the units selected by
/// HIPAR yValues/RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_D31DSpectrumAnalysisConfigureUpperDisplayMask(ViSession instrSession,
                                                             ViInt32   mask,
                                                             ViInt32   points,
                                                             ViReal64  xValues[],
                                                             ViReal64  yValues[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMask = 0;
    ViChar   valuesParameter[RS_MAX_MESSAGE_BUF_SIZE] = { '\0' };
    ViChar*  p2buffer = valuesParameter;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Mask", &maxMask),
        2, "Mask");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mask, 1, maxMask),
            2, "Mask");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DISPLAY_MASK_UPPER_CONFIGURATION,
            "Mask", mask - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Mask");

    for (int i = 0; i < points; i++)
    {
        p2buffer += snprintf(valuesParameter, RS_MAX_MESSAGE_BUF_SIZE - strlen(valuesParameter), "%f,%f", xValues[i], yValues[i]);
    }
    *(--p2buffer) = '\0';

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DISPLAY_MASK_UPPER_CONFIGURATION, valuesParameter));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Clears specified mask, including both the lower and upper masks.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mask/Selects repcap Mask.
ViStatus rsdsa_D31DSpectrumAnalysisClearMask(ViSession instrSession,
                                             ViInt32   mask)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMask = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Mask", &maxMask),
        2, "Mask");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mask, 1, maxMask),
            2, "Mask");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_CLEAR_MASK,
            "Mask", mask - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Mask");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_CLEAR_MASK, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the channel analysis amplitude/phase vs. subcarrier plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31DDisplayAmplitudePhaseVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31D_DISPLAY_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Scales the y-axis for the channel analysis amplitude/phase vs.
/// HIFN  subcarrier plot to best fit the measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31DAutoScaleAmplitudePhaseVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31D_AUTO_SCALE_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the channel analysis amplitude/group delay vs. subcarrier
/// HIFN  plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31DDisplayAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31D_DISPLAY_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Scales the y-axis for the channel analysis amplitude/group delay vs.
/// HIFN  subcarrier plot to best fit the measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31DAutoScaleAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31D_AUTO_SCALE_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the channel analysis echo pattern plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31DDisplayEchoPatternPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31D_DISPLAY_ECHO_PATTERN_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the specified trace data. Plot data is returned as x,y value
/// HIFN  pairs (x1,y1,x2,y2,...). 1054 points are displayed across the DSA
/// HIFN  screen. For I/Q density data, 65535 integers are always returned in row
/// HIFN  order, regardless of the zoom settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Selects repcap Trace.
/// HIPAR bufferSize/Size of output array.
/// HIPAR traceData/Queries the specified trace data. Plot data is returned as x,y value
/// HIPAR traceData/pairs (x1,y1,x2,y2,...). 1054 points are displayed across the DSA
/// HIPAR traceData/screen. For I/Q density data, 65535 integers are always returned in row
/// HIPAR traceData/order, regardless of the zoom settings.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_D31DQueryTraceData(ViSession instrSession,
                                  ViInt32   trace,
                                  ViInt32   bufferSize,
                                  ViReal64  traceData[],
                                  ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxTrace = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Trace", &maxTrace),
        2, "Trace");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, maxTrace),
            2, "Trace");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31D_QUERY_TRACE_DATA,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trace");
    viCheckParm(RsCore_GetAttributeViStringAsViReal64ArrayToUserBuffer(instrSession, repCap,
        RSDSA_ATTR_D31D_QUERY_TRACE_DATA, NULL,
        bufferSize, traceData, bytesRead),
        4, "Trace Data");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the all the messages that are currently in the acquisition log
/// HIFN  and clears the acquisition log. If the acquisition log is empty, the
/// HIFN  number 0, 'No entry', is returned. For each error in the log, the
/// HIFN  aqcuisition log entry message is returned, followed by the date and
/// HIFN  time the entry was added to the log.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR log/Queries the all the messages that are currently in the acquisition log
/// HIPAR log/and clears the acquisition log. If the acquisition log is empty, the
/// HIPAR log/number 0, 'No entry', is returned. For each error in the log, the
/// HIPAR log/aqcuisition log entry message is returned, followed by the date and
/// HIPAR log/time the entry was added to the log.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_D31DQuerySystemAcquisitionLogAll(ViSession instrSession,
                                                ViInt32   bufferSize,
                                                ViChar    log[],
                                                ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSDSA_ATTR_D31D_SYSTEM_ACQUISITION_LOG_ALL, RS_VAL_DIRECT_USER_CALL, bufferSize, log, bytesRead),
        3, "Log");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the most recent acquisition log entry and deletes it. If the
/// HIFN  acquisition log is empty, the number 0, 'No entry', is returned.
/// HIFN  Returns the aqcuisition log entry message, followed by the date and
/// HIFN  time the entry was added to the log.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR entry/Queries the most recent acquisition log entry and deletes it. If the
/// HIPAR entry/acquisition log is empty, the number 0, 'No entry', is returned.
/// HIPAR entry/Returns the aqcuisition log entry message, followed by the date and
/// HIPAR entry/time the entry was added to the log.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_D31DQuerySystemAcquisitionLogNext(ViSession instrSession,
                                                 ViInt32   bufferSize,
                                                 ViChar    entry[],
                                                 ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSDSA_ATTR_D31D_SYSTEM_ACQUISITION_LOG_NEXT, RS_VAL_DIRECT_USER_CALL, bufferSize, entry, bytesRead),
        3, "Entry");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the acquisition log for the number of unread entries. If the
/// HIFN  acquisition log is empty, the error number 0, 'No error', is returned.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR count/Queries the acquisition log for the number of unread entries. If the
/// HIPAR count/acquisition log is empty, the error number 0, 'No error', is returned.
ViStatus rsdsa_D31DQuerySystemAcquisitionLogCount(ViSession instrSession,
                                                  ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31D_SYSTEM_ACQUISITION_LOG_COUNT, count),
            2, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes all entries in the acquisition log.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31DSystemAcquisitionLogClearAll(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31D_SYSTEM_ACQUISITION_LOG_CLEAR_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Capture the DOCSIS decoded payload to the specified file. Note that
/// HIFN  there is no way to abort this command, and large file sizes could take
/// HIFN  a long time to capture depending on the input data, so use filter and
/// HIFN  maxFileSize to limit the file size.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/File path, uses .out extension.
/// HIPAR filter/The filter to apply to the export data.
/// HIPAR maxFileSize/The max file size for the payload export file, up to 20MB.
ViStatus rsdsa_D31DCapturePayloadToFile(ViSession instrSession,
                                        ViString  fileName,
                                        ViInt32   filter,
                                        ViInt32   maxFileSize)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31D_PAYLOAD_DATA_EXPORT_FILTER, filter),
            3, "Filter");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31D_PAYLOAD_DATA_EXPORT_MAX_FILE_SIZE, maxFileSize),
            4, "Max File Size");

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31D_CAPTURE_PAYLOAD_TO_FILENAME, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a parameter to be used as a data filter for displayed
/// HIFN  constellation, I/Q density, MER vs. Time, and channel analysis plots,
/// HIFN  where applicable.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR dataFilter/Selects a parameter to be used as a data filter for displayed
/// HIPAR dataFilter/constellation, I/Q density, MER vs. Time, and channel analysis plots,
/// HIPAR dataFilter/where applicable.
ViStatus rsdsa_D31DConfigurePlotDataFilter(ViSession instrSession,
                                           ViInt32   dataFilter)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31D_CONFIGURE_PLOT_DATA_FILTER, dataFilter),
            2, "Data Filter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the J.83 standard to be used in J.83 QAM mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fec/Selects the J.83 standard to be used in J.83 QAM mode.
ViStatus rsdsa_ConfigureCMRangingFEC(ViSession instrSession,
                                     ViInt32   fec)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_CONFIGURE_FEC, fec),
            2, "FEC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures signal settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequency/Specifies the center frequency of the channel. If the spectrum
/// HIPAR frequency/location is changed by CONFigure:CHANnel:SPECtrum:FREQuency, the center
/// HIPAR frequency/frequency is adapted accordingly.
/// HIPAR subcarrier0Frequency/Specifies the center frequency of subcarrier 0 of the channel, which
/// HIPAR subcarrier0Frequency/defines the beginning of the OFDM spectrum. The default value is
/// HIPAR subcarrier0Frequency/derived from the current center frequency. If the center frequency is
/// HIPAR subcarrier0Frequency/changed by CONFigure:CHANnel:FREQuency,
/// HIPAR equalizerState/Controls the equalizer state.
/// HIPAR prefilterEnabled/Controls a prefilter that is constructed based on the subcarrier
/// HIPAR prefilterEnabled/layout of the channel, which allows filtering out of unwanted nearby
/// HIPAR prefilterEnabled/interferers or adjacent channels.
/// HIPAR attenuation/Defines the total attenuation for RF input in dB. If you set the
/// HIPAR attenuation/attenuation manually, it is no longer coupled to the reference level,
/// HIPAR attenuation/but the reference level is coupled to the attenuation. Thus, if the
/// HIPAR attenuation/current reference level is not compatible with an attenuation that has
/// HIPAR attenuation/been set manually, the command also adjusts the reference level.
/// HIPAR attenuation/Therefore, the reference level set in
/// HIPAR attenuation/DISPlay:[<Mode>:][WINDow:]TRACe[<Trace>]:Y[:SCALe]:RLEVel is no longer
/// HIPAR attenuation/applied. Note: The query returns the actual attenuation setting, which
/// HIPAR attenuation/can also be changed by RSDSA_ATTR_ED30_AUTO_LEVEL.
/// HIPAR preamplifierEnabled/Turns the preamplifier on and off. Note: The query returns the actual
/// HIPAR preamplifierEnabled/preamplifier state (which might also be changed by
/// HIPAR preamplifierEnabled/RSDSA_ATTR_D31D_AUTO_LEVEL).
ViStatus rsdsa_D31DConfigureSignalSettings(ViSession instrSession,
                                           ViReal64  frequency,
                                           ViReal64  subcarrier0Frequency,
                                           ViInt32   equalizerState,
                                           ViBoolean prefilterEnabled,
                                           ViReal64  attenuation,
                                           ViBoolean preamplifierEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_CONFIGURE_FREQUENCY, frequency),
            2, "Frequency");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_CONFIGURE_SUBCARRIER_0_FREQUENCY, subcarrier0Frequency),
            3, "Subcarrier 0 Frequency");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31D_CONFIGURE_EQUALIZER_STATE, equalizerState),
            4, "Equalizer State");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_D31D_CONFIGURE_PREFILTER_ENABLED, prefilterEnabled),
            5, "Prefilter Enabled");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_CONFIGURE_ATTENUATION, attenuation),
            6, "Attenuation");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_D31D_CONFIGURE_PREAMPLIFIER_ENABLED, preamplifierEnabled),
            7, "Preamplifier Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the list of equalizer coefficients. A pair of signed
/// HIFN  floating-point values is returned for each coefficient, representing
/// HIFN  the real and imaginary parts. The number of coefficient pairs returned
/// HIFN  depends on the mode, the signal bandwidth, and for OFDM/OFDMA, the FFT
/// HIFN  size.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR equalizerCoefficients/Queries the list of equalizer coefficients. A pair of signed
/// HIPAR equalizerCoefficients/floating-point values is returned for each coefficient, representing
/// HIPAR equalizerCoefficients/the real and imaginary parts. The number of coefficient pairs returned
/// HIPAR equalizerCoefficients/depends on the mode, the signal bandwidth, and for OFDM/OFDMA, the FFT
/// HIPAR equalizerCoefficients/size.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_D31DQueryEqualizerCoefficients(ViSession instrSession,
                                              ViInt32   bufferSize,
                                              ViReal64  equalizerCoefficients[],
                                              ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringAsViReal64ArrayToUserBuffer(instrSession, NULL,
        RSDSA_ATTR_D31D_QUERY_EQUALIZER_COEFFICIENTS, NULL,
        bufferSize, equalizerCoefficients, bytesRead),
        3, "Equalizer Coefficients");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Automatically determines the attenuation level and preamplifier state
/// HIFN  (on/off). Query the resultant attenuation level with
/// HIFN  RSDSA_ATTR_D31D_CONFIGURE_ATTENUATION and the resultant preamplifier
/// HIFN  state with RSDSA_ATTR_D31D_CONFIGURE_PREAMPLIFIER_ENABLED.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31DAutoLevel(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31D_AUTO_LEVEL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures acquisition settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR acquisitionMode/Specifies the OFDM acquisition mode as automatic (less configuration
/// HIPAR acquisitionMode/settings needed) or manual (all configuration settings needed).
/// HIPAR subcarrierSpacing/Defines the FFT length, which determines the subcarrier spacing and
/// HIPAR subcarrierSpacing/other parameters. This control does not apply to OFDM automatic
/// HIPAR subcarrierSpacing/acquisition mode.
/// HIPAR cyclicPrefixLength/Defines the cyclic prefix length, which determines where the useful
/// HIPAR cyclicPrefixLength/data starts and allows the application to detect delay spreads during
/// HIPAR cyclicPrefixLength/transmission. The longer the delay spread, the longer the CP must be.
/// HIPAR cyclicPrefixLength/This command and query do not apply to OFDM automatic acquisition mode.
/// HIPAR cyclicPrefixLength/
/// HIPAR timeInterleaverDepth/Specifies the OFDM time interleaver depth. This control does not apply
/// HIPAR timeInterleaverDepth/to OFDM automatic acquisition mode.
/// HIPAR subcarrierConfigFile/Loads the OFDM subcarrier configuration file, which contains
/// HIPAR subcarrierConfigFile/downstream subcarrier configuration information. If the command
/// HIPAR subcarrierConfigFile/specifies an empty string, the subcarrier configuration file selection
/// HIPAR subcarrierConfigFile/is cleared. This command and query do not apply to OFDM automatic
/// HIPAR subcarrierConfigFile/acquisition mode.
ViStatus rsdsa_D31DConfigureAcquisitionSettings(ViSession instrSession,
                                                ViInt32   acquisitionMode,
                                                ViInt32   subcarrierSpacing,
                                                ViInt32   cyclicPrefixLength,
                                                ViInt32   timeInterleaverDepth,
                                                ViString  subcarrierConfigFile)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31D_CONFIGURE_ACQUISITION_MODE, acquisitionMode),
            2, "Acquisition Mode");

    if (acquisitionMode == RSDSA_VAL_CHANNEL_ACQUISITION_MODE_MANUAL)
    {
        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31D_CONFIGURE_SUBCARRIER_SPACING, subcarrierSpacing),
                3, "Subcarrier Spacing");

        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31D_CONFIGURE_CYCLIC_PREFIX_LENGTH, cyclicPrefixLength),
                4, "Cyclic Prefix Length");

        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31D_CONFIGURE_TIME_INTERLEAVER_DEPTH, timeInterleaverDepth),
                5, "Time Interleaver Depth");

        viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31D_CONFIGURE_SUBCARRIER_CONFIG_FILE, subcarrierConfigFile),
                6, "Subcarrier Config File");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the SFP+ network output on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Turns the SFP+ network output on and off.
ViStatus rsdsa_D31DConfigureSFPOutputEnabled(ViSession instrSession,
                                             ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_D31D_CONFIGURE_SFP_OUTPUT_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables and configures the SFP+ PLC UDP port.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Turns the SFP+ PLC UDP port on and off.
/// HIPAR udpPort/Configures the UDP port to be used for SFP+ PLC output. The UDP ports
/// HIPAR udpPort/must be uniquely assigned to downstream OFDM profiles/PLC and upstream
/// HIPAR udpPort/DOCSIS IUCs.
ViStatus rsdsa_D31DConfigureSFPPLCUDPPort(ViSession instrSession,
                                          ViBoolean enabled,
                                          ViInt32   udpPort)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_D31D_CONFIGURE_SFP_PLC_UDP_PORT_ENABLED, enabled),
            2, "Enabled");

    if (enabled)
        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31D_CONFIGURE_SFP_PLC_UDP_PORT, udpPort),
                3, "UDP Port");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables and configures the SFP+ UDP port for the selected profile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR profile/Selects repcap Profile.
/// HIPAR enabled/Turns the SFP+ profile UDP port on and off.
/// HIPAR udpPort/Configures the UDP port to be used for SFP+ profile output. The UDP
/// HIPAR udpPort/ports must be uniquely assigned to downstream OFDM profiles/PLC and
/// HIPAR udpPort/upstream DOCSIS IUCs.
ViStatus rsdsa_D31DConfigureSFPProfileUDPPort(ViSession instrSession,
                                              ViInt32   profile,
                                              ViBoolean enabled,
                                              ViInt32   udpPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxProfile = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Profile", &maxProfile),
        2, "Profile");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, profile, 1, maxProfile),
            2, "Profile");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31D_CONFIGURE_SFP_PROFILE_UDP_PORT_ENABLED,
            "Profile", profile - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Profile");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, repCap, RSDSA_ATTR_D31D_CONFIGURE_SFP_PROFILE_UDP_PORT_ENABLED, enabled),
            3, "Enabled");

    if (enabled)
        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, repCap, RSDSA_ATTR_D31D_CONFIGURE_SFP_PROFILE_UDP_PORT, udpPort),
                4, "UDP Port");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the alarm thresholds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rmsMEROverall/Sets the alarm threshold for RMS MER overall. When this parameter
/// HIPAR rmsMEROverall/falls below the set threshold, an alarm is generated.
/// HIPAR berPreLDPC/Sets the alarm threshold for pre-LDPC BER. When this parameter exceeds
/// HIPAR berPreLDPC/the set threshold, an alarm is generated.
/// HIPAR berPostLDPC/Sets the alarm threshold for post-LDPC BER. When this parameter
/// HIPAR berPostLDPC/exceeds the set threshold, an alarm is generated.
/// HIPAR berPostBCH/Sets the alarm threshold for post-BCH BER. When this parameter exceeds
/// HIPAR berPostBCH/the set threshold, an alarm is generated.
/// HIPAR berPLC/Sets the alarm threshold for PLC BER. When this parameter exceeds the
/// HIPAR berPLC/set threshold, an alarm is generated.
/// HIPAR berNCP/Sets the alarm threshold for NCP BER. When this parameter exceeds the
/// HIPAR berNCP/set threshold, an alarm is generated.
ViStatus rsdsa_D31DConfigureAlarmThresholds(ViSession instrSession,
                                            ViReal64  rmsMEROverall,
                                            ViReal64  berPreLDPC,
                                            ViReal64  berPostLDPC,
                                            ViReal64  berPostBCH,
                                            ViReal64  berPLC,
                                            ViReal64  berNCP)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_ALARM_THRESHOLD_RMS_MER_OVERALL, rmsMEROverall),
            2, "RMS MER Overall");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_ALARM_THRESHOLD_BER_PRE_LDPC, berPreLDPC),
            3, "BER Pre-LDPC");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_ALARM_THRESHOLD_BER_POST_LDPC, berPostLDPC),
            4, "BER Post-LDPC");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_ALARM_THRESHOLD_BER_POST_BCH, berPostBCH),
            5, "BER Post-BCH");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_ALARM_THRESHOLD_BER_PLC, berPLC),
            6, "BER PLC");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31D_ALARM_THRESHOLD_BER_NCP, berNCP),
            7, "BER NCP");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the downstream spectrum analysis mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_EnableDownstreamSpectrumAnalysis(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_ENABLE_DOWNSTREAM_SPECTRUM_ANALYSIS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the center frequency in Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR centerFrequency/Defines the center frequency in Hz.
ViStatus rsdsa_ConfigureDSSpecAnCenterFrequency(ViSession instrSession,
                                                ViReal64  centerFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_CONFIGURE_CENTER_FREQUENCY, centerFrequency),
            2, "Center Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the Downstream Spectrum Analyzer start and stop
/// HIFN  frequencies.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR startFrequency/Defines a start frequency for measurements in the frequency domain in
/// HIPAR startFrequency/Hz.
/// HIPAR stopFrequency/Defines a stop frequency for measurements in the frequency domain in
/// HIPAR stopFrequency/Hz.
ViStatus rsdsa_ConfigureDSSpecAnFrequency(ViSession instrSession,
                                          ViReal64  startFrequency,
                                          ViReal64  stopFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_CONFIGURE_START_FREQUENCY, startFrequency),
            2, "Start Frequency");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_CONFIGURE_STOP_FREQUENCY, stopFrequency),
            3, "Stop Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Clears all stored display states.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsDSA_init or
/// HIPAR instrSession/RsDSA_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_DSSpecAnClearAllDisplayStates(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_CLEAR_ALL_DISPLAY_STATES, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the Downstream Spectrum Analyzer span.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sweepTime/Defines the sweep time in seconds, which is only available for zero
/// HIPAR sweepTime/span mode. Minimum and maximum sweep times are dynamically determined
/// HIPAR sweepTime/by the DSA based on RBW and the sample rate.
/// HIPAR frequencySpan/Defines the frequency span in Hz, which must be compatible with the
/// HIPAR frequencySpan/RBW setting.
ViStatus rsdsa_ConfigureDSSpecAnSpan(ViSession instrSession,
                                     ViReal64  sweepTime,
                                     ViReal64  frequencySpan)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_CONFIGURE_ZERO_SPAN_SWEEP_TIME, sweepTime),
            2, "Sweep Time");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_CONFIGURE_FREQUENCY_SPAN, frequencySpan),
            3, "Frequency Span");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restores the full span as per current mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_DSSpecAnFrequencySpanFull(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_SET_FREQUENCY_SPAN_FULL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Set to minimum span (10 kHz).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_DSSpecAnFrequencySpanMinimum(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_SET_FREQUENCY_SPAN_MINIMUM, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the Downstream Spectrum Analyzer Zero Span.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Sets zero span (time domain). Turning on zero span mode overrides any
/// HIPAR enabled/ACP or phase noise mode setting.
/// HIPAR sampleRate/Defines the sample rate in Hz, which is only available for zero span
/// HIPAR sampleRate/mode.
ViStatus rsdsa_ConfigureDSSpecAnZeroSpan(ViSession instrSession,
                                         ViBoolean enabled,
                                         ViReal64  sampleRate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_CONFIGURE_ZERO_SPAN_ENABLED, enabled),
            2, "Enabled");
    if (enabled)
    {
        viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_CONFIGURE_ZERO_SPAN_SAMPLE_RATE, sampleRate),
                3, "Sample Rate");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the Downstream Amplitude Reference Level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR referenceLevel/Defines the reference level for the displayed trace in the units
/// HIPAR referenceLevel/selected by RSDSA_ATTR_POWER_UNITS. This setting is only used when
/// HIPAR referenceLevel/automatic y-axis scaling is turned off. If the reference level is set,
/// HIPAR referenceLevel/then the attenuation set in INPut:ATTenuation is no longer applied.
/// HIPAR scalePerDivision/Defines the scale per division in the units selected by
/// HIPAR scalePerDivision/RSDSA_ATTR_POWER_UNITS to be used for the y-axis for all trace
/// HIPAR scalePerDivision/displays, which all have 10 divisions on the y-axis. Note that this
/// HIPAR scalePerDivision/command sets the scale per division, whereas the user of the DSA GUI
/// HIPAR scalePerDivision/sets the entire range for the vertical axis.
ViStatus rsdsa_ConfigureDSSpecAnAmplitude(ViSession instrSession,
                                          ViReal64  referenceLevel,
                                          ViReal64  scalePerDivision)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_CONFIGURE_REFERENCE_LEVEL, referenceLevel),
            2, "Reference Level");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_CONFIGURE_SCALE_PER_DIVISION, scalePerDivision),
            3, "Scale Per Division");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Scales the y-axis for trace displays by automatically adjusting the
/// HIFN  y-axis reference level and scale per division to best fit the
/// HIFN  measurement results. Otherwise, the y-axis can be manually scaled using
/// HIFN  RSDSA_ATTR_DS_SPECAN_CONFIGURE_REFERENCE_LEVEL and
/// HIFN  RSDSA_ATTR_DS_SPECAN_CONFIGURE_SCALE_PER_DIVISION.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_DSSpecAnAutoScaleYAxis(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_AUTO_SCALE_Y_AXIS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the Downstream Spectrum Analyzer RBW.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoRBWEnabled/When turned on, the DSA automatically determines the resolution
/// HIPAR autoRBWEnabled/bandwidth based on the span.
/// HIPAR manualRBW/Defines the resolution bandwidth, which must be compatible with the
/// HIPAR manualRBW/span setting: 10 Hz to 3 MHz, or 10 Hz to 204.8 MHz for
ViStatus rsdsa_ConfigureDSSpecAnRBW(ViSession instrSession,
                                    ViBoolean autoRBWEnabled,
                                    ViReal64  manualRBW)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_CONFIGURE_AUTO_RBW_ENABLED, autoRBWEnabled),
            2, "Auto RBW Enabled");
    if (!autoRBWEnabled)
    {
        viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_CONFIGURE_MANUAL_RBW, manualRBW),
                2, "Manual RBW");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the Downstream Spectrum Analyzer RBW.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoVBWEnabled/When turned on, the DSA automatically determines the video bandwidth
/// HIPAR autoVBWEnabled/based on the span.
/// HIPAR manualVBW/Defines the video bandwidth in Hz and automatically calculates the
/// HIPAR manualVBW/number of averages, overriding the manual setting of averages.
ViStatus rsdsa_ConfigureDSSpecAnVBW(ViSession instrSession,
                                    ViBoolean autoVBWEnabled,
                                    ViReal64  manualVBW)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_CONFIGURE_AUTO_VBW_ENABLED, autoVBWEnabled),
            2, "Auto VBW Enabled");
    if (!autoVBWEnabled)
    {
        viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_CONFIGURE_MANUAL_VBW, manualVBW),
                3, "Manual VBW");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configure DS SpecAn Average Count.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoAverageCount/When turned on, the DSA automatically determines the average setting.
/// HIPAR averageCount/Defines the number of averages. A value of 1 indicates no averaging.
/// HIPAR averageCount/Note: For OFDM spectrum analysis, setting an average that is not a
/// HIPAR averageCount/multiple of 128 may cause artifacts between pilots.
ViStatus rsdsa_ConfigureDSSpecAnAverageCount(ViSession instrSession,
                                             ViBoolean autoAverageCount,
                                             ViInt32   averageCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_CONFIGURE_AUTO_AVERAGE_COUNT, autoAverageCount),
            2, "Auto Average Count");
    if (!autoAverageCount)
    {
        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_CONFIGURE_MANUAL_AVERAGE_COUNT, averageCount),
                3, "Average Count");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the trace detector.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR detector/Selects the detection method for the displayed trace.
ViStatus rsdsa_ConfigureDSSpecAnDetector(ViSession instrSession,
                                         ViInt32   detector)
{
    ViStatus error = VI_SUCCESS;

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_CONFIGURE_DETECTOR, detector),
            2, "Detector");

Error:
    return error;
}

/// HIFN  Configures the Downstream Spectrum Analyzer Measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR captureMode/Selects whether to use continuous captures or single trace captures.
/// HIPAR hold/Selects the measurement mode for the displayed trace. The DSA applies
/// HIPAR hold/minimum/maximum hold over multiple single traces initiated by
/// HIPAR hold/DISPlay:[<Mode>:][WINDow:]TRACe[<Trace>]:RSINgle command.
/// HIPAR phaseNoise/Turns phase noise mode on or off. Setting the state to ON overrides
/// HIPAR phaseNoise/any zero span or ACP mode setting.
ViStatus rsdsa_ConfigureDSSpecAnMeasurement(ViSession instrSession,
                                            ViInt32   captureMode,
                                            ViInt32   hold,
                                            ViBoolean phaseNoise)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_CONFIGURE_CAPTURE_MODE, captureMode),
            2, "Capture Mode");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_CONFIGURE_TRACE_HOLD, hold),
            3, "Hold");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_CONFIGURE_PHASE_NOISE_MODE_ENABLED, phaseNoise),
            4, "Phase Noise");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restarts a single sweep, rearming for the next trigger, if applicable.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_DSSpecAnRunSingleSweep(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_RUN_SINGLE_SWEEP, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the Downstream Spectrum Analzyer ACP Mode.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Turns Adjacent Channel Power (ACP) mode on or off. Commanding the
/// HIPAR enabled/state to ON overrides any zero span or phase noise mode setting.
/// HIPAR frequencyGridSpacing/Selects the frequency grid for Adjacent Channel Power (ACP) mode.
/// HIPAR frequencyGridSpacing/Selecting 6, 7, or 8 MHz results in a span of 54, 63, or 72 MHz
/// HIPAR frequencyGridSpacing/respectively.
/// HIPAR gridSpacing/Selects the frequency grid for Adjacent Channel Power (ACP) mode.
/// HIPAR gridSpacing/Selecting 6, 7, or 8 MHz results in a span of 54, 63, or 72 MHz
/// HIPAR gridSpacing/respectively.
/// HIPAR referenceChannel/Selects the reference channel for Adjacent Channel Power (ACP) mode.
/// HIPAR referenceChannel/All power values in dBc are calculated with respect to the specified
/// HIPAR referenceChannel/reference carrier.
ViStatus rsdsa_ConfigureDSSpecAnACPMode(ViSession instrSession,
                                        ViBoolean enabled,
                                        ViInt32   gridSpacing,
                                        ViInt32   referenceChannel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_CONFIGURE_ACP_MODE_ENABLED, enabled),
            2, "Enabled");
    if (enabled)
    {
        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_CONFIGURE_ACP_GRID_SPACING, gridSpacing),
                3, "Grid Spacing");

        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_CONFIGURE_ACP_REFERENCE_CHANNEL, referenceChannel),
                4, "Reference Channel");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the powers measured for each ACP channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR powerResult/Queries the powers measured for each ACP channel.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_QueryDSSpecAnACPPowerResult(ViSession instrSession,
                                           ViInt32   bufferSize,
                                           ViReal64  powerResult[],
                                           ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  response = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringAsViReal64ArrayToUserBuffer(instrSession, NULL,
        RSDSA_ATTR_DS_SPECAN_QUERY_ACP_POWER_RESULT,
        NULL, bufferSize, powerResult, bytesRead),
        3, "Power Result");

Error:
    if (response) free(response);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Saves the ACP data to the specified CSV file. The file contents are
/// HIFN  the same as the output from
/// HIFN  DISPlay:[<Mode>:][WINDow:]TRACe[<Trace>]:ACP:RESult?.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/File path and name for CSV data to be saved.
ViStatus rsdsa_DSSpecAnSaveACPPowerResult(ViSession instrSession,
                                          ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_HIDDEN_DS_SPECAN_SAVE_ACP_POWER_RESULT, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configure Downstream Spectrum Analyzer Phase Noise.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR startOffset/Define the start offset, which defines the start of the frequency band
/// HIPAR startOffset/over which the phase noise measurement is to be made and is specified
/// HIPAR startOffset/as a delta from the target carrier. It is specified as an offset in Hz
/// HIPAR startOffset/from the target carrier.
/// HIPAR stopOffset/Define the stop offset, which defines the end of the frequency band
/// HIPAR stopOffset/over which the phase noise measurement is to be made and is specified
/// HIPAR stopOffset/as a delta from the target carrier. It is specified as an offset in Hz
/// HIPAR stopOffset/from the target carrier.
ViStatus rsdsa_ConfigureDSSpecAnPhaseNoiseOffset(ViSession instrSession,
                                                 ViReal64  startOffset,
                                                 ViReal64  stopOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_CONFIGURE_PHASE_NOISE_START_OFFSET, startOffset),
            2, "Start Offset");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_CONFIGURE_PHASE_NOISE_STOP_OFFSET, stopOffset),
            3, "Stop Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the integrated phase noise in dBc.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR integratedResult/Queries the integrated phase noise in dBc.
ViStatus rsdsa_QueryDSSpecAnPhaseNoiseIntegratedResult(ViSession instrSession,
                                                       ViReal64* integratedResult)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_QUERY_PHASE_NOISE_INTEGRATED_RESULT, integratedResult),
            2, "Integrated Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the spot phase noise in dBc/Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR spotResult/Queries the spot phase noise in dBc/Hz.
ViStatus rsdsa_QueryDSSpecAnPhaseNoiseSpotResult(ViSession instrSession,
                                                 ViReal64* spotResult)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_QUERY_PHASE_NOISE_SPOT_RESULT, spotResult),
            2, "Spot Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the bandwidth that is used for making the integrated phase
/// HIFN  noise measurement in Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bandwidth/Queries the bandwidth that is used for making the integrated phase
/// HIPAR bandwidth/noise measurement in Hz.
ViStatus rsdsa_QueryDSSpecAnPhaseNoiseBandwidth(ViSession instrSession,
                                                ViReal64* bandwidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_QUERY_PHASE_NOISE_BANDWIDTH, bandwidth),
            2, "Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns off all markers including delta markers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_DSSpecAnAllMarkersOff(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_ALL_MARKERS_OFF, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables the marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
/// HIPAR enabled/Turns individual markers on and off. If the specified marker is a
/// HIPAR enabled/delta marker, it is turned into a normal marker. The default position
/// HIPAR enabled/is at the maximum peak.
ViStatus rsdsa_ConfigureDSSpecAnMarkerEnabled(ViSession instrSession,
                                              ViInt32   marker,
                                              ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_MARKER_STATE,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, repCap, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_MARKER_STATE, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the marker X position.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
/// HIPAR xPosition/Moves a marker to a particular coordinate on the x-axis. If the
/// HIPAR xPosition/specified marker is turned off, it is turned on first. If the specified
/// HIPAR xPosition/marker is a delta marker, it is turned into a normal marker.
ViStatus rsdsa_ConfigureDSSpecAnMarkerXPosition(ViSession instrSession,
                                                ViInt32   marker,
                                                ViReal64  xPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_MARKER_X_POSITION,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_MARKER_X_POSITION, xPosition),
            3, "X Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the position of a marker on the y-axis. If the specified
/// HIFN  marker is turned off, it is turned on first. If the specified marker is
/// HIFN  a delta marker, it is turned into a normal marker.
/// HIFN     The units are determined by RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
/// HIPAR yPosition/Queries the position of a marker on the y-axis. If the specified
/// HIPAR yPosition/marker is turned off, it is turned on first. If the specified marker is
/// HIPAR yPosition/a delta marker, it is turned into a normal marker.
ViStatus rsdsa_QueryDSSpecAnMarkerYPosition(ViSession instrSession,
                                            ViInt32   marker,
                                            ViReal64* yPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_MARKER_Y_POSITION,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_MARKER_Y_POSITION, yPosition),
            3, "Y Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a marker to the highest amplitude displayed. If the specified
/// HIFN  marker is turned off, it is turned on first. If the specified marker is
/// HIFN  a delta marker, it is turned into a normal marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_DSSpecAnMarkerToMaximumPeak(ViSession instrSession,
                                           ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_MARKER_TO_MAXIMUM_PEAK,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_MARKER_TO_MAXIMUM_PEAK, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a marker to the next peak to the left of the current marker
/// HIFN  position. If the specified marker is turned off, it is turned on first.
/// HIFN  If the specified marker is a delta marker, it is turned into a normal
/// HIFN  marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_DSSpecAnMarkerToNextPeakLeft(ViSession instrSession,
                                            ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_LEFT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_LEFT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a marker to the next peak to the right of the current marker
/// HIFN  position. If the specified marker is turned off, it is turned on first.
/// HIFN  If the specified marker is a delta marker, it is turned into a normal
/// HIFN  marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_DSSpecAnMarkerToNextPeakRight(ViSession instrSession,
                                             ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_RIGHT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_RIGHT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a marker to the next displayed point to the left of the current
/// HIFN  marker position. If the specified marker is turned off, it is turned on
/// HIFN  first. If the specified marker is a delta marker, it is turned into a
/// HIFN  normal marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_DSSpecAnMarkerToNextPointLeft(ViSession instrSession,
                                             ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_LEFT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_LEFT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a marker to the next displayed point to the right of the current
/// HIFN  marker position. If the specified marker is turned off, it is turned on
/// HIFN  first. If the specified marker is a delta marker, it is turned into a
/// HIFN  normal marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_DSSpecAnMarkerToNextPointRight(ViSession instrSession,
                                              ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_RIGHT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_RIGHT, repCap));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the center frequency to the frequency of a marker. If the
/// HIFN  specified marker is turned off, it is turned on first. If the specified
/// HIFN  marker is a delta marker, it is turned into a normal marker.
/// HIFN     CALCulate:[<Mode>:]MARKer<Marker>[:FUNCtion][:MARKer]:CENTer
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_DSSpecAnMarkerSetCenterFrequency(ViSession instrSession,
                                                ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_MARKER_SET_CENTER_FREQUENCY,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_MARKER_SET_CENTER_FREQUENCY, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the marker to the zero span trigger point at the center of the
/// HIFN  plot. If the specified marker is turned off, it is turned on first. If
/// HIFN  the specified marker is a delta marker, it is turned into a normal
/// HIFN  marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_DSSpecAnZeroSpanMarkerToTriggerPoint(ViSession instrSession,
                                                    ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DS_SPECAN_ZERO_SPAN_MARKER_TO_TRIGGER_POINT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_DS_SPECAN_ZERO_SPAN_MARKER_TO_TRIGGER_POINT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the reference level to the power level of a marker. If the
/// HIFN  specified marker is turned off, it is turned on first. If the specified
/// HIFN  marker is a delta marker, it is turned into a normal marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_DSSpecAnMarkerSetPowerReferenceLevel(ViSession instrSession,
                                                    ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_MARKER_SET_POWER_REFERENCE_LEVEL,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_MARKER_SET_POWER_REFERENCE_LEVEL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns off all delta markers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_DSSpecAnAllDeltaMarkersOff(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_ALL_DELTA_MARKERS_OFF, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the specified delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
/// HIPAR enabled/Turns delta markers on and off. If the specified marker is a normal
/// HIPAR enabled/marker, it is turned into a delta marker. The default position is at
/// HIPAR enabled/the maximum peak.
/// HIPAR linkedReferenceMarker/Selects a reference marker for a delta marker. Values of delta marker
/// HIPAR linkedReferenceMarker/are relative to the reference marker. If the specified reference marker
/// HIPAR linkedReferenceMarker/is turned off, it is turned on first. If the specified delta marker is
/// HIPAR linkedReferenceMarker/turned off, it is turned on first. If the specified delta marker is a
/// HIPAR linkedReferenceMarker/normal marker, it is turned into a delta marker. If the specified
/// HIPAR linkedReferenceMarker/reference marker is the same as the specified delta marker, the marker
/// HIPAR linkedReferenceMarker/is changed to a normal marker.
/// HIPAR absoluteXPosition/Moves a delta marker to a particular absolute coordinate on the
/// HIPAR absoluteXPosition/x-axis. If the specified marker is turned off, it is turned on first.
/// HIPAR absoluteXPosition/If the specified marker is a normal marker, it is turned into a delta
/// HIPAR absoluteXPosition/marker. The query returns the absolute position of the delta marker.
ViStatus rsdsa_ConfigureDSSpecAnDeltaMarkerEnabled(ViSession instrSession,
                                                   ViInt32   delta,
                                                   ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_STATE,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, repCap, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_STATE, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the linked reference marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
/// HIPAR linkedReferenceMarker/Selects a reference marker for a delta marker. Values of delta marker
/// HIPAR linkedReferenceMarker/are relative to the reference marker. If the specified reference marker
/// HIPAR linkedReferenceMarker/is turned off, it is turned on first. If the specified delta marker is
/// HIPAR linkedReferenceMarker/turned off, it is turned on first. If the specified delta marker is a
/// HIPAR linkedReferenceMarker/normal marker, it is turned into a delta marker. If the specified
/// HIPAR linkedReferenceMarker/reference marker is the same as the specified delta marker, the marker
/// HIPAR linkedReferenceMarker/is changed to a normal marker.
ViStatus rsdsa_ConfigureDSSpecAnDeltaMarkerLinkedReferenceMarker(
                            ViSession instrSession,
                            ViInt32   delta,
                            ViInt32   linkedReferenceMarker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_LINKED_REFERENCE_MARKER,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    if (linkedReferenceMarker == RSDSA_VAL_REFERENCE_MARKER_NORMAL)
        linkedReferenceMarker = delta;

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, repCap, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_LINKED_REFERENCE_MARKER, linkedReferenceMarker),
            3, "Linked Reference Marker");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to a particular absolute coordinate on the
/// HIFN  x-axis. If the specified marker is turned off, it is turned on first.
/// HIFN  If the specified marker is a normal marker, it is turned into a delta
/// HIFN  marker. The query returns the absolute position of the delta marker.
/// HIFN     The units are Hz (frequency domain) or s (time domain, for zero span
/// HIFN  mode).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
/// HIPAR absoluteXPosition/Moves a delta marker to a particular absolute coordinate on the
/// HIPAR absoluteXPosition/x-axis. If the specified marker is turned off, it is turned on first.
/// HIPAR absoluteXPosition/If the specified marker is a normal marker, it is turned into a delta
/// HIPAR absoluteXPosition/marker. The query returns the absolute position of the delta marker.
ViStatus rsdsa_ConfigureDSSpecAnDeltaMarkerAbsoluteXPosition(ViSession instrSession,
                                                             ViInt32   delta,
                                                             ViReal64  absoluteXPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_ABSOLUTE_X_POSITION,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_ABSOLUTE_X_POSITION, absoluteXPosition),
            3, "Absolute X Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to a particular relative coordinate on the
/// HIFN  x-axis. If the specified marker is turned off, it is turned on first.
/// HIFN  If the specified marker is a normal marker, it is turned into a delta
/// HIFN  marker. The query returns the relative position of the delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
/// HIPAR relativeXPosition/Moves a delta marker to a particular relative coordinate on the
/// HIPAR relativeXPosition/x-axis. If the specified marker is turned off, it is turned on first.
/// HIPAR relativeXPosition/If the specified marker is a normal marker, it is turned into a delta
/// HIPAR relativeXPosition/marker. The query returns the relative position of the delta marker.
ViStatus rsdsa_ConfigureDSSpecAnDeltaMarkerRelativeXPosition(ViSession instrSession,
                                                             ViInt32   delta,
                                                             ViReal64  relativeXPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_RELATIVE_X_POSITION,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_RELATIVE_X_POSITION, relativeXPosition),
            3, "Relative X Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the absolute position of a delta marker on the y-axis. If the
/// HIFN  specified marker is turned off, it is turned on first. If the specified
/// HIFN  marker is a normal marker, it is turned into a delta marker.
/// HIFN     The units are determined by RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
/// HIPAR yPosition/Queries the absolute position of a delta marker on the y-axis. If the
/// HIPAR yPosition/specified marker is turned off, it is turned on first. If the specified
/// HIPAR yPosition/marker is a normal marker, it is turned into a delta marker.
ViStatus rsdsa_QueryDSSpecAnDeltaMarkerYPosition(ViSession instrSession,
                                                 ViInt32   delta,
                                                 ViReal64* yPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_Y_POSITION,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_Y_POSITION, yPosition),
            3, "Y Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to the highest amplitude displayed. If the
/// HIFN  specified marker is turned off, it is turned on first. If the specified
/// HIFN  marker is a normal marker, it is turned into a delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
ViStatus rsdsa_DSSpecAnDeltaMarkerToMaximumPeak(ViSession instrSession,
                                                ViInt32   delta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_STATE,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_MAXIMUM_PEAK, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to the next peak to the left of the current
/// HIFN  marker position. If the specified marker is turned off, it is turned on
/// HIFN  first. If the specified marker is a normal marker, it is turned into a
/// HIFN  delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
ViStatus rsdsa_DSSpecAnDeltaMarkerToNextPeakLeft(ViSession instrSession,
                                                 ViInt32   delta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_LEFT,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_LEFT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to the next peak to the right of the current
/// HIFN  marker position. If the specified marker is turned off, it is turned on
/// HIFN  first. If the specified marker is a normal marker, it is turned into a
/// HIFN  delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
ViStatus rsdsa_DSSpecAnDeltaMarkerToNextPeakRight(ViSession instrSession,
                                                  ViInt32   delta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_RIGHT,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_RIGHT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to the next displayed point to the left of the
/// HIFN  current marker position. If the specified marker is turned off, it is
/// HIFN  turned on first. If the specified marker is a normal marker, it is
/// HIFN  turned into a delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
ViStatus rsdsa_DSSpecAnDeltaMarkerToNextPointLeft(ViSession instrSession,
                                                  ViInt32   delta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_LEFT,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_LEFT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to the next displayed point to the right of the
/// HIFN  current marker position. If the specified marker is turned off, it is
/// HIFN  turned on first. If the specified marker is a normal marker, it is
/// HIFN  turned into a delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
ViStatus rsdsa_DSSpecAnDeltaMarkerToNextPointRight(ViSession instrSession,
                                                   ViInt32   delta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_RIGHT,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_RIGHT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the delta marker to the zero span trigger point at the center of
/// HIFN  the plot. If the specified marker is turned off, it is turned on first.
/// HIFN  If the specified marker is a normal marker, it is turned into a delta
/// HIFN  marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
ViStatus rsdsa_DSSpecAnZeroSpanDeltaMarkerToTriggerPoint(ViSession instrSession,
                                                         ViInt32   delta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DS_SPECAN_ZERO_SPAN_DELTA_MARKER_TO_TRIGGER_POINT,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_DS_SPECAN_ZERO_SPAN_DELTA_MARKER_TO_TRIGGER_POINT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the center frequency to the frequency of a delta marker. If the
/// HIFN  specified marker is turned off, it is turned on first. If the specified
/// HIFN  marker is a normal marker, it is turned into a delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
ViStatus rsdsa_DSSpecAnDeltaMarkerSetCenterFrequency(ViSession instrSession,
                                                     ViInt32   delta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_SET_CENTER_FREQUENCY,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_SET_CENTER_FREQUENCY, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the reference level to the power level of a delta marker. If the
/// HIFN  specified marker is turned off, it is turned on first. If the specified
/// HIFN  marker is a normal marker, it is turned into a delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
ViStatus rsdsa_DSSpecAnDeltaMarkerSetPowerReferenceLevel(ViSession instrSession,
                                                         ViInt32   delta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_SET_POWER_REFERENCE_LEVEL,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_SET_POWER_REFERENCE_LEVEL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the bandpower between a specified delta marker and its
/// HIFN  reference marker in the frequency domain. An error is generated if the
/// HIFN  specified marker is turned off, or if the specified marker is a normal
/// HIFN  marker.
/// HIFN     The units are selected by RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
/// HIPAR bandwidthPowerResult/Queries the bandpower between a specified delta marker and its
/// HIPAR bandwidthPowerResult/reference marker in the frequency domain. An error is generated if the
/// HIPAR bandwidthPowerResult/specified marker is turned off, or if the specified marker is a normal
/// HIPAR bandwidthPowerResult/marker.
ViStatus rsdsa_QueryDSSpecAnDeltaMarkerBandwidthPowerResult(ViSession instrSession,
                                                            ViInt32   delta,
                                                            ViReal64* bandwidthPowerResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_BANDWIDTH_POWER_RESULT,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_BANDWIDTH_POWER_RESULT, bandwidthPowerResult),
            3, "Bandwidth Power Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores a complete set of spectrum analyzer configuration settings to
/// HIFN  the specified state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR savedState/Selects repcap SavedState.
ViStatus rsdsa_DSSpecAnDisplayStateStore(ViSession instrSession,
                                         ViInt32   savedState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxSavedState = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "SavedState", &maxSavedState),
        2, "Saved State");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, savedState, 1, maxSavedState),
            2, "Saved State");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DS_SPECAN_DISPLAY_STATE_STORE,
            "SavedState", savedState - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Saved State");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_DS_SPECAN_DISPLAY_STATE_STORE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Clears specified state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR savedState/Selects repcap SavedState.
ViStatus rsdsa_DSSpecAnDisplayStateClear(ViSession instrSession,
                                         ViInt32   savedState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxSavedState = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "SavedState", &maxSavedState),
        2, "Saved State");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, savedState, 1, maxSavedState),
            2, "Saved State");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DS_SPECAN_DISPLAY_STATE_CLEAR,
            "SavedState", savedState - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Saved State");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_DS_SPECAN_DISPLAY_STATE_CLEAR, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Recalls and applies a complete set of spectrum analyzer configuration
/// HIFN  settings from the specified state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR savedState/Selects repcap SavedState.
ViStatus rsdsa_DSSpecAnDisplayStateRecall(ViSession instrSession,
                                          ViInt32   savedState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxSavedState = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "SavedState", &maxSavedState),
        2, "Saved State");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, savedState, 1, maxSavedState),
            2, "Saved State");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DS_SPECAN_DISPLAY_STATE_RECALL,
            "SavedState", savedState - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Saved State");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_DS_SPECAN_DISPLAY_STATE_RECALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the specified trace data. Plot data is returned as x,y value
/// HIFN  pairs ( x1,y1,x2,y2,...). 1054 points are displayed across the DSA
/// HIFN  screen. For I/Q density data, 65535 integers are always returned in row
/// HIFN  order, regardless of the zoom settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Selects repcap Trace.
/// HIPAR bufferSize/Size of output array.
/// HIPAR dsSpecAnQueryTraceData/Queries the specified trace data. Plot data is returned as x,y value
/// HIPAR dsSpecAnQueryTraceData/pairs ( x1,y1,x2,y2,...). 1054 points are displayed across the DSA
/// HIPAR dsSpecAnQueryTraceData/screen. For I/Q density data, 65535 integers are always returned in row
/// HIPAR dsSpecAnQueryTraceData/order, regardless of the zoom settings.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_QueryDSSpecAnTraceData(ViSession instrSession,
                                      ViInt32   trace,
                                      ViInt32   bufferSize,
                                      ViReal64  traceData[],
                                      ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxTrace = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Trace", &maxTrace),
        2, "Trace");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, maxTrace),
            2, "Trace");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DS_SPECAN_QUERY_TRACE_DATA,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trace");
    viCheckParm(RsCore_GetAttributeViStringAsViReal64ArrayToUserBuffer(instrSession, repCap,
        RSDSA_ATTR_DS_SPECAN_QUERY_TRACE_DATA, NULL,
        bufferSize, traceData, bytesRead),
        4, "Trace Data");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns all masks on or off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR allMasksEnabled/Turns all masks on or off.
ViStatus rsdsa_ConfigureDSSpecAnDisplayAllMasksEnabled(ViSession instrSession,
                                                       ViBoolean allMasksEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DISPLAY_ALL_MASKS_ENABLED, allMasksEnabled),
            2, "All Masks Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Clears all masks, including both the lower and upper masks.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_DSSpecAnClearAllMasks(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_CLEAR_ALL_MASKS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Clears specified mask, including both the lower and upper masks.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mask/Selects repcap Mask.
ViStatus rsdsa_DSSpecAnClearMask(ViSession instrSession,
                                 ViInt32   mask)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMask = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Mask", &maxMask),
        2, "Mask");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mask, 1, maxMask),
            2, "Mask");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DS_SPECAN_CLEAR_MASK,
            "Mask", mask - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Mask");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_DS_SPECAN_CLEAR_MASK, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns individual masks on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mask/Selects repcap Mask.
/// HIPAR maskVisible/Turns individual masks on and off.
ViStatus rsdsa_ConfigureDSSpecAnMaskVisible(ViSession instrSession,
                                            ViInt32   mask,
                                            ViBoolean maskVisible)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMask = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Mask", &maxMask),
        2, "Mask");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mask, 1, maxMask),
            2, "Mask");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DS_SPECAN_MASK_VISIBLE,
            "Mask", mask - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Mask");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, repCap, RSDSA_ATTR_DS_SPECAN_MASK_VISIBLE, maskVisible),
            3, "Mask Visible");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the specified lower mask with 1 to 20 points. Each point in
/// HIFN  the mask configuration must have a corresponding X and Y value. All
/// HIFN  frequency (X) entries are in Hz, and all amplitude (Y) entries are in
/// HIFN  the units selected by RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mask/Selects repcap Mask.
/// HIPAR points/Defines the length of the X and Y values arrays. A mask definition may
/// HIPAR points/have up to 20 points defined.
/// HIPAR xValues/An array of frequency (X axis) entries for the points of the lower
/// HIPAR xValues/mask, in Hz.
/// HIPAR yValues/An array of amplitude (Y axis) entries for the points of the lower
/// HIPAR yValues/mask, in the units specified by RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_DSSpecAnConfigureLowerDisplayMask(ViSession instrSession,
                                                 ViInt32   mask,
                                                 ViInt32   points,
                                                 ViReal64  xValues[],
                                                 ViReal64  yValues[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMask = 0;
    ViChar   param[RS_MAX_MESSAGE_BUF_SIZE] = { '\0' };
    ViChar*  p2buf = param;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Mask", &maxMask),
        2, "Mask");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mask, 1, maxMask),
            2, "Mask");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_HIDDEN_DS_SPECAN_MASK_LOWER_CONFIGURATION,
            "Mask", mask - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Mask");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, points, RSDSA_VAL_MASK_POINTS_MIN, RSDSA_VAL_MASK_POINTS_MAX),
            3, "Points");

    for (int i = 0; i < points; i++)
    {
        p2buf += snprintf(p2buf, RS_MAX_MESSAGE_BUF_SIZE - strlen(param), "%f,%f,", xValues[i], yValues[i]);
    }
    *(--p2buf) = '\0'; // Remove trailing comma

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_HIDDEN_DS_SPECAN_MASK_LOWER_CONFIGURATION, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the specified upper mask with 1 to 20 points. Each point in
/// HIFN  the mask configuration must have a corresponding X and Y value. All
/// HIFN  frequency (X) entries are in Hz, and all amplitude (Y) entries are in
/// HIFN  the units selected by RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mask/Selects repcap Mask.
/// HIPAR points/Defines the length of the X and Y values arrays. A mask definition may
/// HIPAR points/have up to 20 points defined.
/// HIPAR xValues/An array of frequency (X axis) entries for the points of the upper
/// HIPAR xValues/mask, in Hz.
/// HIPAR yValues/An array of amplitude (Y axis) entries for the points of the upper
/// HIPAR yValues/mask, in the units specified by RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_DSSpecAnConfigureUpperDisplayMask(ViSession instrSession,
                                                 ViInt32   mask,
                                                 ViInt32   points,
                                                 ViReal64  xValues[],
                                                 ViReal64  yValues[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMask = 0;
    ViChar   param[RS_MAX_MESSAGE_BUF_SIZE] = { '\0' };
    ViChar*  p2buf = param;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Mask", &maxMask),
        2, "Mask");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mask, 1, maxMask),
            2, "Mask");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_HIDDEN_DS_SPECAN_MASK_UPPER_CONFIGURATION,
            "Mask", mask - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Mask");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, points, RSDSA_VAL_MASK_POINTS_MIN, RSDSA_VAL_MASK_POINTS_MAX),
            3, "Points");

    for (int i = 0; i < points; i++)
    {
        p2buf += snprintf(p2buf, RS_MAX_MESSAGE_BUF_SIZE - strlen(param), "%f,%f,", xValues[i], yValues[i]);
    }
    *(--p2buf) = '\0'; // Remove trailing comma

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_HIDDEN_DS_SPECAN_MASK_UPPER_CONFIGURATION, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the downstream DOCSIS 3.0 (DOCSIS 3.0 SC-QAM J.83 Annex B)
/// HIFN  mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_EnableDOCSIS30ATDMAMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_ENABLE_DOCSIS_30_ATDMA_MODE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an IUC to be used as a data filter for displayed status and
/// HIFN  plots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR iuc/Selects an IUC to be used as a data filter for displayed status and
/// HIPAR iuc/plots.
ViStatus rsdsa_ConfigureD30AMapGenIUC(ViSession instrSession,
                                      ViInt32   iuc)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_MAP_GEN_IUC, iuc),
            2, "IUC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the burst demodulator lock status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstDemodulatorLock/Queries the burst demodulator lock status.
ViStatus rsdsa_QueryD30ABurstDemodulatorLockStatus(ViSession instrSession,
                                                   ViInt32*  burstDemodulatorLock)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_QUERY_BURST_DEMODULATOR_LOCK_STATUS, burstDemodulatorLock),
            2, "Burst Demodulator Lock");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the forward error correction (FEC) lock status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fecLock/Queries the forward error correction (FEC) lock status.
ViStatus rsdsa_QueryD30AFECLockStatus(ViSession instrSession,
                                      ViInt32*  fecLock)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_QUERY_FEC_LOCK_STATUS, fecLock),
            2, "FEC Lock");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the burst receive power level in the units selected by
/// HIFN  RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstReceivePower/Queries the burst receive power level in the units selected by
/// HIPAR burstReceivePower/RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_QueryD30ABurstReceivePower(ViSession instrSession,
                                          ViReal64* burstReceivePower)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_QUERY_BURST_RECEIVE_POWER, burstReceivePower),
            2, "Burst Receive Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the input signal power level, per 1.6 MHz, in the units
/// HIFN  selected by RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR signalPowerPer1600kHz/Queries the input signal power level, per 1.6 MHz, in the units
/// HIPAR signalPowerPer1600kHz/selected by RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_QueryD30ASignalPowerPer1600kHz(ViSession instrSession,
                                              ViReal64* signalPowerPer1600kHz)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_QUERY_SIGNAL_POWER_PER_1600KHZ, signalPowerPer1600kHz),
            2, "Signal Power Per 1600kHz");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the frequency offset in Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequencyOffset/Queries the frequency offset in Hz.
ViStatus rsdsa_QueryD30AFrequencyOffset(ViSession instrSession,
                                        ViReal64* frequencyOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_QUERY_FREQUENCY_OFFSET, frequencyOffset),
            2, "Frequency Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the burst timing offset from the time the burst was detected
/// HIFN  and the time the burst was supposed to occur, in seconds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstTimingOffset/Queries the burst timing offset from the time the burst was detected
/// HIPAR burstTimingOffset/and the time the burst was supposed to occur, in seconds.
ViStatus rsdsa_QueryD30ABurstTimingOffset(ViSession instrSession,
                                          ViReal64* burstTimingOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_QUERY_BURST_TIMING_OFFSET, burstTimingOffset),
            2, "Burst Timing Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the payload data rate in Mbps (megabits per second) for the
/// HIFN  IUC currently selected by DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR payloadDataRateIUC/Queries the payload data rate in Mbps (megabits per second) for the
/// HIPAR payloadDataRateIUC/IUC currently selected by DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>.
ViStatus rsdsa_QueryD30APayloadDataRateIUC(ViSession instrSession,
                                           ViReal64* payloadDataRateIUC)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_QUERY_PAYLOAD_DATA_RATE_IUC, payloadDataRateIUC),
            2, "Payload Data Rate IUC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of minislots received.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR minislotsReceived/Queries the number of minislots received.
ViStatus rsdsa_QueryD30AMinislotsReceived(ViSession instrSession,
                                          ViInt64*  minislotsReceived)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt64(instrSession, NULL,
        RSDSA_ATTR_D30A_QUERY_MINISLOTS_RECEIVED,
        minislotsReceived),
        2, "Minislots Received");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the percentage of valid bursts out of all bursts received over
/// HIFN  the last one second period.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstCoverage/Queries the percentage of valid bursts out of all bursts received over
/// HIPAR burstCoverage/the last one second period.
ViStatus rsdsa_QueryD30ABurstCoverage(ViSession instrSession,
                                      ViReal64* burstCoverage)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_QUERY_BURST_COVERAGE, burstCoverage),
            2, "Burst Coverage");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the sign preferred for MER numerical display and plot axis
/// HIFN  labels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merSign/Selects the sign preferred for MER numerical display and plot axis
/// HIPAR merSign/labels.
ViStatus rsdsa_ConfigureD30AMERSign(ViSession instrSession,
                                    ViInt32   merSign)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_MER_SIGN, merSign),
            2, "MER Sign");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall RMS equalized MER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merRMS/Queries the overall RMS equalized MER.
ViStatus rsdsa_QueryD30AMERRMS(ViSession instrSession,
                               ViReal64* merRMS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_QUERY_MER_RMS, merRMS),
            2, "MER RMS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall peak (lowest) MER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merPeak/Queries the overall peak (lowest) MER.
ViStatus rsdsa_QueryD30AMERPeak(ViSession instrSession,
                                ViReal64* merPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_QUERY_MER_PEAK, merPeak),
            2, "MER Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the RMS MER for the IUC currently selected by
/// HIFN  DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merIUCRMS/Queries the RMS MER for the IUC currently selected by
/// HIPAR merIUCRMS/DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>.
ViStatus rsdsa_QueryD30AMERIUCRMS(ViSession instrSession,
                                  ViReal64* merIUCRMS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_QUERY_MER_IUC_RMS, merIUCRMS),
            2, "MER IUC RMS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the peak (lowest) MER for the IUC currently selected by
/// HIFN  DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merIUCPeak/Queries the peak (lowest) MER for the IUC currently selected by
/// HIPAR merIUCPeak/DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>.
ViStatus rsdsa_QueryD30AMERIUCPeak(ViSession instrSession,
                                   ViReal64* merIUCPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_QUERY_MER_IUC_PEAK, merIUCPeak),
            2, "merIUCPeak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the I/Q amplitude imbalance in dB.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR amplitudedB/Queries the I/Q amplitude imbalance in dB.
ViStatus rsdsa_QueryD30AIQImbalanceAmplitudedB(ViSession instrSession,
                                               ViReal64* amplitudedB)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_QUERY_IQ_IMBALANCE_AMPLITUDE_DB, amplitudedB),
            2, "Amplitude dB");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the I/Q amplitude imbalance in percent.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR amplitudePercent/Queries the I/Q amplitude imbalance in percent.
ViStatus rsdsa_QueryD30AIQImbalanceAmplitudePercent(ViSession instrSession,
                                                    ViReal64* amplitudePercent)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_QUERY_IQ_IMBALANCE_AMPLITUDE_PERCENT, amplitudePercent),
            2, "Amplitude Percent");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the I/Q imbalance quadrature error in degrees.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR quadratureError/Queries the I/Q imbalance quadrature error in degrees.
ViStatus rsdsa_QueryD30AIQImbalanceQuadratureError(ViSession instrSession,
                                                   ViReal64* quadratureError)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_QUERY_IQ_IMBALANCE_QUADRATURE_ERROR, quadratureError),
            2, "Quadrature Error");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the D30A Error Analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR errorAnalysisMode/Specifies the mode of BER measurement.
/// HIPAR manualTestDepth/Defines the test depth of the minimum BER integration. The query
/// HIPAR manualTestDepth/returns the actual test depth setting, which may be automatically
/// HIPAR manualTestDepth/changed in automatic BER measurement mode.
/// HIPAR testDepthSource/Selects the BER source used for BER measurements.
/// HIPAR progressionSource/Selects the BER source used for test depth progression.
ViStatus rsdsa_ConfigureD30AErrorAnalysis(ViSession instrSession,
                                          ViInt32   errorAnalysisMode,
                                          ViInt32   manualTestDepth,
                                          ViInt32   testDepthSource,
                                          ViInt32   progressionSource)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_ERROR_ANALYSIS_MODE, errorAnalysisMode),
            2, "Error Analysis Mode");
    if (errorAnalysisMode == RSDSA_VAL_ERROR_ANALYSIS_MANUAL)
    {
        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH, manualTestDepth),
                3, "Manual Test Depth");
    }
    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH_SOURCE, testDepthSource),
            4, "Test Depth Source");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH_PROGRESSION_SOURCE, progressionSource),
            5, "Progression Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Performs a BER reset that clears all related counters to zero and
/// HIFN  restarts the BER/CER measurement for the current mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_ResetD30AErrorAnalysis(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30A_RESET_ERROR_ANALYSIS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the elapsed time of the current BER/CER measurement: time
/// HIFN  elapsed since the start of the mode, or last BER reset
/// HIFN  (RSDSA_ATTR_D30A_RESET_ERROR_ANALYSIS), or last BER configuration
/// HIFN  change, in seconds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR elapsedTime/Queries the elapsed time of the current BER/CER measurement: time
/// HIPAR elapsedTime/elapsed since the start of the mode, or last BER reset
/// HIPAR elapsedTime/(RSDSA_ATTR_D30A_RESET_ERROR_ANALYSIS), or last BER configuration
/// HIPAR elapsedTime/change, in seconds.
ViStatus rsdsa_QueryD30ABERElapsedTime(ViSession instrSession,
                                       ViReal64* elapsedTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_QUERY_BER_ELAPSED_TIME, elapsedTime),
            2, "Elapsed Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the percentage of blocks processed for the current BER/CER
/// HIFN  measurement: blocks processed since the start of the mode, or last BER
/// HIFN  reset (CALCulate:[<Mode>:][AVERage:]BERatio:CLEar), or last BER
/// HIFN  configuration change. Expressed as the percentage of blocks processed
/// HIFN  relative to the current test depth. A block is 10^7 bits maximum.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR blocksProcessed/Queries the percentage of blocks processed for the current BER/CER
/// HIPAR blocksProcessed/measurement: blocks processed since the start of the mode, or last BER
/// HIPAR blocksProcessed/reset (CALCulate:[<Mode>:][AVERage:]BERatio:CLEar), or last BER
/// HIPAR blocksProcessed/configuration change. Expressed as the percentage of blocks processed
/// HIPAR blocksProcessed/relative to the current test depth. A block is 10^7 bits maximum.
ViStatus rsdsa_QueryD30ABERBlocksProcessed(ViSession instrSession,
                                           ViReal64* blocksProcessed)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_QUERY_BER_BLOCKS_PROCESSED, blocksProcessed),
            2, "Blocks Processed");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of bursts expected since the signal was acquired.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstsExpected/Queries the number of bursts expected since the signal was acquired.
ViStatus rsdsa_QueryD30ABERBurstsExpected(ViSession instrSession,
                                          ViReal64* burstsExpected)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_QUERY_BER_BURSTS_EXPECTED, burstsExpected),
            2, "Bursts Expected");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of bursts received since the signal was acquired.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstsReceived/Queries the number of bursts received since the signal was acquired.
ViStatus rsdsa_QueryD30ABERBurstsReceived(ViSession instrSession,
                                          ViReal64* burstsReceived)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_QUERY_BER_BURSTS_RECEIVED, burstsReceived),
            2, "Bursts Received");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the pre-Reed-Solomon BER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR preReedSolomon/Queries the pre-Reed-Solomon BER.
ViStatus rsdsa_QueryD30ABERPreReedSolomon(ViSession instrSession,
                                          ViReal64* berPreRS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_QUERY_BER_PRE_REED_SOLOMON, berPreRS),
            2, "BER Pre-RS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of bits analyzed at the input of the
/// HIFN  Reed-Solomon decoder for the BER measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR preReedSolomonTotalBits/Queries the total number of bits analyzed at the input of the
/// HIPAR preReedSolomonTotalBits/Reed-Solomon decoder for the BER measurement.
ViStatus rsdsa_QueryD30ABERPreReedSolomonTotalBits(ViSession instrSession,
                                                   ViReal64* preRSTotalBits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_QUERY_BER_PRE_REED_SOLOMON_TOTAL_BITS, preRSTotalBits),
            2, "Pre-RS Total Bits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the post-Reed-Solomon BER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR postReedSolomon/Queries the post-Reed-Solomon BER.
ViStatus rsdsa_QueryD30ABERPostReedSolomon(ViSession instrSession,
                                           ViReal64* berPostRS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_QUERY_BER_POST_REED_SOLOMON, berPostRS),
            2, "BER Post-RS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of corrected bits at the output of the
/// HIFN  Reed-Solomon decoder for the BER measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR correctedBits/Queries the total number of corrected bits at the output of the
/// HIPAR correctedBits/Reed-Solomon decoder for the BER measurement.
ViStatus rsdsa_QueryD30ABERPostReedSolomonCorrectedBits(ViSession instrSession,
                                                        ViReal64* correctedBits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_QUERY_BER_POST_REED_SOLOMON_CORRECTED_BITS, correctedBits),
            2, "Corrected Bits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of uncorrected bits at the output of the
/// HIFN  Reed-Solomon decoder for the BER measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR uncorrectedBits/Queries the total number of uncorrected bits at the output of the
/// HIPAR uncorrectedBits/Reed-Solomon decoder for the BER measurement.
ViStatus rsdsa_QueryD30ABERPostReedSolomonUncorrectedBits(ViSession instrSession,
                                                          ViReal64* uncorrectedBits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_QUERY_BER_POST_REED_SOLOMON_UNCORRECTED_BITS, uncorrectedBits),
            2, "Uncorrected Bits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the corrected CER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR corrected/Queries the corrected CER.
ViStatus rsdsa_QueryD30ACERCorrected(ViSession instrSession,
                                     ViReal64* corrected)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_QUERY_CER_CORRECTED, corrected),
            2, "Corrected");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the uncorrected CER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR uncorrected/Queries the uncorrected CER.
ViStatus rsdsa_QueryD30ACERUncorrected(ViSession instrSession,
                                       ViReal64* uncorrected)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_QUERY_CER_UNCORRECTED, uncorrected),
            2, "Uncorrected");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of codewords for the CER measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR totalCodewords/Queries the total number of codewords for the CER measurement.
ViStatus rsdsa_QueryD30ACERTotalCodewords(ViSession instrSession,
                                          ViReal64* totalCodewords)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_QUERY_CER_TOTAL_CODEWORDS, totalCodewords),
            2, "Total Codewords");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of corrected CER codewords.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR correctedCodewords/Queries the total number of corrected CER codewords.
ViStatus rsdsa_QueryD30ACERCorrectedCodewords(ViSession instrSession,
                                              ViReal64* correctedCodewords)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_QUERY_CER_CORRECTED_CODEWORDS, correctedCodewords),
            2, "Corrected Codewords");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of uncorrected CER codewords.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR uncorrectedCodewords/Queries the total number of uncorrected CER codewords.
ViStatus rsdsa_QueryD30ACERUncorrectedCodewords(ViSession instrSession,
                                                ViReal64* uncorrectedCodewords)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_QUERY_CER_UNCORRECTED_CODEWORDS, uncorrectedCodewords),
            2, "Uncorrected Codewords");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of bits in the average payload per codeword,
/// HIFN  for the IUC currently selected by DISPlay:[<Mode>:][WINDow:]IUCFilter
/// HIFN  <Iuc>. For OFDMA, this command also applies to the codeword length
/// HIFN  currently selected by DISPlay:[<Mode>:][WINDow:]CLFilter <CwLength>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR payloadPerCodewordPerIUC/Queries the total number of bits in the average payload per codeword,
/// HIPAR payloadPerCodewordPerIUC/for the IUC currently selected by DISPlay:[<Mode>:][WINDow:]IUCFilter
/// HIPAR payloadPerCodewordPerIUC/<Iuc>. For OFDMA, this command also applies to the codeword length
/// HIPAR payloadPerCodewordPerIUC/currently selected by DISPlay:[<Mode>:][WINDow:]CLFilter <CwLength>.
ViStatus rsdsa_QueryD30ACERAvgPayloadPerCodewordPerIUC(ViSession instrSession,
                                                       ViReal64* payloadPerCodewordPerIUC)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_QUERY_CER_AVG_PAYLOAD_PER_CODEWORD_PER_IUC, payloadPerCodewordPerIUC),
            2, "Payload Per Codeword Per IUC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the percentage of codewords covered (i.e., not discarded), for
/// HIFN  the IUC currently selected by DISPlay:[<Mode>:][WINDow:]IUCFilter
/// HIFN  <Iuc>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR codewordCoverageIUC/Queries the percentage of codewords covered (i.e., not discarded), for
/// HIPAR codewordCoverageIUC/the IUC currently selected by DISPlay:[<Mode>:][WINDow:]IUCFilter
/// HIPAR codewordCoverageIUC/<Iuc>.
ViStatus rsdsa_QueryD30ACERCodewordCoverageIUC(ViSession instrSession,
                                               ViReal64* codewordCoverageIUC)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_QUERY_CER_CODEWORD_COVERAGE_IUC, codewordCoverageIUC),
            2, "Codeword Coverage IUC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the constellation plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30ADisplayConstellationPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30A_DISPLAY_CONSTELLATION_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the I/Q density plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30ADisplayIQDensityPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30A_DISPLAY_I_Q_DENSITY_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the I/Q density plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementMode/Selects the measurement mode for the I/Q density plot.
/// HIPAR windowWidth/Specifies the sliding window width in samples for the I/Q density plot
/// HIPAR windowWidth/in sliding window mode.
ViStatus rsdsa_ConfigureD30AIQ(ViSession instrSession,
                               ViInt32   measurementMode,
                               ViInt32   windowWidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_I_Q_MEASUREMENT_MODE, measurementMode),
            2, "Measurement Mode");
    if (measurementMode == RSDSA_VAL_IQ_MEASUREMENT_MODE_SLIDING_WINDOW)
    {
        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_I_Q_WINDOW_WIDTH, windowWidth),
                3, "Window Width");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the zoom setting for the I/Q density plot display. Note that
/// HIFN  the x and y origin are the top left corner of the window, as per the
/// HIFN  GUI settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR x/X origin (left-most window point), between 0 and 1.
/// HIPAR y/Y origin (top-most window point), between 0 and 1.
/// HIPAR zoom/Zoom setting, 1 to 128.
ViStatus rsdsa_ConfigureD30AIQDisplayZoom(ViSession instrSession,
                                          ViReal64  x,
                                          ViReal64  y,
                                          ViInt32   zoom)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_CheckRangeTableRangeViReal64(instrSession, &RsDSA_rngIQZoomOrigin, x),
        2, "X");
    viCheckParm(rsdsa_CheckRangeTableRangeViReal64(instrSession, &RsDSA_rngIQZoomOrigin, y),
        3, "Y");
    viCheckParm(rsdsa_CheckRangeTableRangeViInt32(instrSession, &RsDSA_rngIQZoomLevel, zoom),
        4, "Zoom");

    snprintf(param, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%.12lG,%.12lG,%ld", x, y, zoom);

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_HIDDEN_D30A_DISPLAY_I_Q_ZOOM, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Save the constellation I/Q data to the specified file. The file
/// HIFN  extension is .iq.tar. Note that there is no way to abort this command,
/// HIFN  and large file sizes could take a long time to capture depending on the
/// HIFN  input data, so use maxSamples or timeoutSeconds to limit the file size
/// HIFN  or time.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/File path, uses .iq.tar extension.
/// HIPAR maxSamples/Sets the I/Q data file maximum size in samples.
/// HIPAR timeoutSeconds/Sets the maximum time in seconds to save data.
ViStatus rsdsa_D30ACaptureIQDataToFile(ViSession instrSession,
                                       ViString  fileName,
                                       ViInt64   maxSamples,
                                       ViReal64  timeoutSeconds)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt64(instrSession, NULL,
        RSDSA_ATTR_D30A_IQ_DATA_EXPORT_MAX_SAMPLES,
        maxSamples),
        3, "Max Samples");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_IQ_DATA_EXPORT_TIMEOUT, timeoutSeconds),
            4, "Timeout");

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30A_CAPTURE_IQ_DATA_TO_FILENAME, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the MER vs. time plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30ADisplayMERVsTimePlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30A_DISPLAY_MER_VS_TIME_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a time filter period to be used for displayed MER vs. Time
/// HIFN  plots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR period/Selects a time filter period to be used for displayed MER vs. Time plots.
ViStatus rsdsa_ConfigureD30AMERVsTimePlotFilterPeriod(ViSession instrSession,
                                                      ViInt32   period)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_MER_VS_TIME_PLOT_FILTER_PERIOD, period),
            2, "Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the BER vs. time plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30ADisplayBERVsTimePlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30A_DISPLAY_BER_VS_TIME_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a time filter period to be used for displayed BER vs. Time
/// HIFN  plots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR period/Selects a time filter period to be used for displayed BER vs. Time plots.
ViStatus rsdsa_ConfigureD30ABERVsTimePlotFilterPeriod(ViSession instrSession,
                                                      ViInt32   period)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_BER_VS_TIME_PLOT_FILTER_PERIOD, period),
            2, "Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the POWer vs. time plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30ADisplayPowerVsTimePlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30A_DISPLAY_POWER_VS_TIME_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a time filter period to be used for displayed Power vs. Time
/// HIFN  plots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR period/Selects a time filter period to be used for displayed Power vs. Time plots.
ViStatus rsdsa_ConfigureD30APowerVsTimePlotFilterPeriod(ViSession instrSession,
                                                        ViInt32   period)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_POWER_VS_TIME_PLOT_FILTER_PERIOD, period),
            2, "Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays a downstream or upstream CCDF (Complementary Cumulative
/// HIFN  Distribution Function histogram as per CM-SP-PHYv3.1-I16 sections 9.3.8
/// HIFN  or 9.4.6 respectively). This command may take up to 30 seconds to
/// HIFN  execute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30ADisplayCCDFHistogram(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30A_DISPLAY_CCDF_HISTOGRAM, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the peak CCDF power level in the units selected by
/// HIFN  RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ccdfPeak/Queries the peak CCDF power level in the units selected by
/// HIPAR ccdfPeak/RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_QueryD30ACCDFPeak(ViSession instrSession,
                                 ViReal64* ccdfPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_QUERY_CCDF_PEAK, ccdfPeak),
            2, "CCDF Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the channel analysis amplitude/phase vs. subcarrier plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30ADisplayAmplitudePhaseVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30A_DISPLAY_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Scales the y-axis for the channel analysis amplitude/phase vs.
/// HIFN  subcarrier plot to best fit the measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30AAutoScaleAmplitudePhaseVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30A_AUTO_SCALE_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the channel analysis amplitude/group delay vs. subcarrier
/// HIFN  plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30ADisplayAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30A_DISPLAY_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Scales the y-axis for the channel analysis amplitude/group delay vs.
/// HIFN  subcarrier plot to best fit the measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30AAutoScaleAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30A_AUTO_SCALE_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the channel analysis echo pattern plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30ADisplayEchoPatternPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30A_DISPLAY_ECHO_PATTERN_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the all the messages that are currently in the acquisition log
/// HIFN  and clears the acquisition log. If the acquisition log is empty, the
/// HIFN  number 0, 'No entry', is returned. For each error in the log, the
/// HIFN  aqcuisition log entry message is returned, followed by the date and
/// HIFN  time the entry was added to the log.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR log/Queries the all the messages that are currently in the acquisition log
/// HIPAR log/and clears the acquisition log. If the acquisition log is empty, the
/// HIPAR log/number 0, 'No entry', is returned. For each error in the log, the
/// HIPAR log/aqcuisition log entry message is returned, followed by the date and
/// HIPAR log/time the entry was added to the log.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_QueryD30ASystemAcquisitionLogAll(ViSession instrSession,
                                                ViInt32   bufferSize,
                                                ViChar    log[],
                                                ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL,
        RSDSA_ATTR_D30A_SYSTEM_ACQUISITION_LOG_ALL,
        RS_VAL_DIRECT_USER_CALL,
        bufferSize, log, bytesRead),
        3, "Log");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the most recent acquisition log entry and deletes it. If the
/// HIFN  acquisition log is empty, the number 0, 'No entry', is returned.
/// HIFN  Returns the aqcuisition log entry message, followed by the date and
/// HIFN  time the entry was added to the log.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR entry/Queries the most recent acquisition log entry and deletes it. If the
/// HIPAR entry/acquisition log is empty, the number 0, 'No entry', is returned.
/// HIPAR entry/Returns the aqcuisition log entry message, followed by the date and
/// HIPAR entry/time the entry was added to the log.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_QueryD30ASystemAcquisitionLogNext(ViSession instrSession,
                                                 ViInt32   bufferSize,
                                                 ViChar    entry[],
                                                 ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL,
        RSDSA_ATTR_D30A_SYSTEM_ACQUISITION_LOG_NEXT,
        RS_VAL_DIRECT_USER_CALL,
        bufferSize, entry, bytesRead),
        3, "Entry");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the acquisition log for the number of unread entries. If the
/// HIFN  acquisition log is empty, the error number 0, 'No error', is returned.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR count/Queries the acquisition log for the number of unread entries. If the
/// HIPAR count/acquisition log is empty, the error number 0, 'No error', is returned.
ViStatus rsdsa_QueryD30ASystemAcquisitionLogCount(ViSession instrSession,
                                                  ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_SYSTEM_ACQUISITION_LOG_COUNT, count),
            2, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes all entries in the acquisition log.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30ASystemAcquisitionLogClearAll(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30A_SYSTEM_ACQUISITION_LOG_CLEAR_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Automatically determines the attenuation level and preamplifier state
/// HIFN  (on/off). Query the resultant attenuation level with
/// HIFN  RSDSA_ATTR_D30A_CONFIGURE_ATTENUATION and the resultant preamplifier
/// HIFN  state with RSDSA_ATTR_D30A_CONFIGURE_PREAMPLIFIER_ENABLED.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D30AAutoLevel(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30A_AUTO_LEVEL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the DOCSIS 3.0 ATDMA upstream mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequency/Specifies the center frequency of the channel. If the spectrum
/// HIPAR frequency/location is changed by CONFigure:CHANnel:SPECtrum:FREQuency, the center
/// HIPAR frequency/frequency is adapted accordingly.
/// HIPAR constellation/Defines the constellation.
/// HIPAR channelSymbolRate/Specifies the symbol rate.
/// HIPAR lastCodewordLength/Specifies the last codeword length.
/// HIPAR guardTimeSize/Specifies the guard time size.
/// HIPAR minislotSize/Specifies the minislot size.
ViStatus rsdsa_ConfigureD30AGeneralSettings(ViSession instrSession,
                                            ViReal64  frequency,
                                            ViInt32   constellation,
                                            ViInt32   channelSymbolRate,
                                            ViInt32   lastCodewordLength,
                                            ViInt32   guardTimeSize,
                                            ViInt32   minislotSize)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_FREQUENCY, frequency),
            2, "Frequency");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_CONSTELLATION, constellation),
            3, "Constellation");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_CHANNEL_SYMBOL_RATE, channelSymbolRate),
            4, "Channel Symbol Rate");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_LAST_CODEWORD_LENGTH, lastCodewordLength),
            5, "Last Codeword Length");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_GUARD_TIME_SIZE, guardTimeSize),
            6, "Guard Time Size");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_MINISLOT_SIZE, minislotSize),
            7, "Minislot Size");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the FEC parameter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fecParameterT/Specifies the FEC parameter T.
/// HIPAR fecParameterK/Specifies the FEC parameter k.
ViStatus rsdsa_ConfigureD30AFECParameter(ViSession instrSession,
                                         ViInt32   fecParameterT,
                                         ViInt32   fecParameterK)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_FEC_PARAMETER_T, fecParameterT),
            2, "FEC Parameter T");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_FEC_PARAMETER_K, fecParameterK),
            3, "FEC Parameter K");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the preamble pattern.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR pattern/Specifies the preamble pattern.
/// HIPAR length/Specifies the preamble length.
/// HIPAR offset/Specifies the preamble offset.
/// HIPAR type/Specifies the preamble type.
ViStatus rsdsa_ConfigureD30APreamble(ViSession instrSession,
                                     ViString  pattern,
                                     ViInt32   length,
                                     ViInt32   offset,
                                     ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_PREAMBLE_PATTERN, pattern),
            2, "Pattern");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_PREAMBLE_LENGTH, length),
            3, "Length");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_PREAMBLE_OFFSET, offset),
            4, "Offset");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_PREAMBLE_TYPE, type),
            5, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the scrambling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Turns scrambling on and off.
/// HIPAR seed/Specifies the scrambler seed.
ViStatus rsdsa_ConfigureD30AScrambler(ViSession instrSession,
                                      ViBoolean enabled,
                                      ViString  seed)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_SCRAMBLER_ENABLED, enabled),
            2, "Enabled");

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_SCRAMBLER_SEED, seed),
            3, "Seed");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures Reed-Solomon error correction.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Specifies the Reed-Solomon interleaver mode.
/// HIPAR interleaverDepth/Specifies the Reed-Solomon interleaver depth, used when the
/// HIPAR interleaverDepth/Reed-Solomon interleaver mode is fixed. Valid range is 2 to
/// HIPAR interleaverDepth/floor[2048/(k+2T)].
/// HIPAR interleaverBlockSize/Specifies the Reed-Solomon interleaver block size, used when the
/// HIPAR interleaverBlockSize/Reed-Solomon interleaver mode is dynamic. Valid range is 2(k+2T) to
/// HIPAR interleaverBlockSize/2048.
ViStatus rsdsa_ConfigureD30AReedSolomon(ViSession instrSession,
                                        ViInt32   mode,
                                        ViInt32   interleaverDepth,
                                        ViInt32   interleaverBlockSize)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_REED_SOLOMON_MODE, mode),
            2, "Mode");
    switch (mode)
    {
    case RSDSA_VAL_REED_SOLOMON_MODE_FIXED:
        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_REED_SOLOMON_INTERLEAVER_DEPTH, interleaverDepth),
                3, "Interleaver Depth");
        break;
    case RSDSA_VAL_REED_SOLOMON_MODE_DYNAMIC:
        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_REED_SOLOMON_INTERLEAVER_BLOCK_SIZE, interleaverBlockSize),
                4, "Interleaver Block Size");
        break;
    default:
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the minislot size.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR d30aConfigureMinislotSize/Specifies the minislot size.
ViStatus rsdsa_ConfigureD30AMinislotSize(ViSession instrSession,
                                         ViInt32   minislotSize)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_MINISLOT_SIZE, minislotSize),
            2, "Minislot Size");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the signal path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR equalizerState/Controls the equalizer state.
/// HIPAR attenuation/Defines the total attenuation for RF input in dB. If you set the
/// HIPAR attenuation/attenuation manually, it is no longer coupled to the reference level,
/// HIPAR attenuation/but the reference level is coupled to the attenuation. Thus, if the
/// HIPAR attenuation/current reference level is not compatible with an attenuation that has
/// HIPAR attenuation/been set manually, the command also adjusts the reference level.
/// HIPAR attenuation/Therefore, the reference level set in
/// HIPAR attenuation/DISPlay:[<Mode>:][WINDow:]TRACe[<Trace>]:Y[:SCALe]:RLEVel is no longer
/// HIPAR attenuation/applied. Note: The query returns the actual attenuation setting, which
/// HIPAR attenuation/can also be changed by RSDSA_ATTR_D30A_AUTO_LEVEL.
/// HIPAR preamplifierEnabled/Turns the preamplifier on and off. Note: The query returns the actual
/// HIPAR preamplifierEnabled/preamplifier state (which might also be changed by
/// HIPAR preamplifierEnabled/RSDSA_ATTR_D30A_AUTO_LEVEL).
ViStatus rsdsa_ConfigureD30ASignalPath(ViSession instrSession,
                                       ViInt32   equalizerState,
                                       ViReal64  attenuation,
                                       ViBoolean preamplifierEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_EQUALIZER_STATE, equalizerState),
            2, "Equalizer State");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_ATTENUATION, attenuation),
            3, "Attenuation");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_PREAMPLIFIER_ENABLED, preamplifierEnabled),
            4, "Preamplifier Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the scheduling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR grantSize/This command is only valid in standalone receiver mode. In standalone
/// HIPAR grantSize/mode there are no MAP messages going into the system, so the user must
/// HIPAR grantSize/specify details of the burst scheduling. This parameter lets the
/// HIPAR grantSize/receiver know how many minislots worth of data it
/// HIPAR startingMinislot/This command is only valid in standalone receiver mode. In standalone
/// HIPAR startingMinislot/mode there are no MAP messages going into the system, so the user must
/// HIPAR startingMinislot/specify details of the burst scheduling. This parameter lets the
/// HIPAR startingMinislot/receiver know how many minislots it has to wait for
/// HIPAR scheduling/This command is only valid in standalone receiver mode. In standalone
/// HIPAR scheduling/mode there are no MAP messages going into the system, so the user must
/// HIPAR scheduling/specify details of the burst scheduling. This parameter determines of
/// HIPAR scheduling/the receiver should wait for a single burst on
/// HIPAR repeatInterval/This command is only valid in standalone receiver mode. In standalone
/// HIPAR repeatInterval/mode there are no MAP messages going into the system, so the user must
/// HIPAR repeatInterval/specify details of the burst scheduling. This parameter determines
/// HIPAR repeatInterval/periodic repeat interval that burst will arrive in
/// HIPAR numberOfBursts/This command is only valid in standalone receiver mode. In standalone
/// HIPAR numberOfBursts/mode there are no MAP messages going into the system, so the user must
/// HIPAR numberOfBursts/specify details of the burst scheduling. This parameter determines the
/// HIPAR numberOfBursts/number of bursts to capture. This
/// HIPAR rangingMode/Allows the receiver to lock to a burst that does not occur exactly on
/// HIPAR rangingMode/a minislot boundary. The burst timing offset parameter can then be used
/// HIPAR rangingMode/to zero in on the correct timing at which point the receiver has
/// HIPAR rangingMode/achieved timing accuracy and ranging mode can
/// HIPAR untimedBurstLength/The untimed burst length parameter is only valid when the receiver is
/// HIPAR untimedBurstLength/set to ranging mode. This determines how many minislots in the
/// HIPAR untimedBurstLength/configured grant are used to carry the actual burst. When in ranging
/// HIPAR untimedBurstLength/mode, the cable modem will not be accurate in time
ViStatus rsdsa_ConfigureD30AScheduling(ViSession instrSession,
                                       ViInt32   grantSize,
                                       ViInt32   startingMinislot,
                                       ViInt32   scheduling,
                                       ViInt32   repeatInterval,
                                       ViInt32   numberOfBursts,
                                       ViBoolean rangingMode,
                                       ViInt32   untimedBurstLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_GRANT_SIZE, grantSize),
            2, "Grant Size");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_STARTING_MINISLOT, startingMinislot),
            3, "Starting Minislot");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_SCHEDULING, scheduling),
            4, "Scheduling");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_REPEAT_INTERVAL, repeatInterval),
            5, "Repeat Interval");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_NUMBER_OF_BURSTS, numberOfBursts),
            6, "Number Of Bursts");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_RANGING_MODE_ENABLED, rangingMode),
            7, "Ranging Mode");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_UNTIMED_BURST_LENGTH, untimedBurstLength),
            8, "Untimed Burst Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the trigger input offset for upstream DOCSIS modes. Sets
/// HIFN  the offset applied to the internal DOCSIS clock upon the system
/// HIFN  detecting an external input trigger. It is used to synchronize the
/// HIFN  timing for the reception of upstream bursts. Burst timing impairments
/// HIFN  can be simulated by manipulating this offset. Note this offset is
/// HIFN  additive on the triggering system offset, which is set via the zerospan
/// HIFN  mode. See section 3.9 for triggering details.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR inputTriggerOffset/Configures the trigger input offset for upstream DOCSIS modes. Sets
/// HIPAR inputTriggerOffset/the offset applied to the internal DOCSIS clock upon the system
/// HIPAR inputTriggerOffset/detecting an external input trigger. It is used to synchronize the
/// HIPAR inputTriggerOffset/timing for the reception of upstream bursts. Burst
ViStatus rsdsa_ConfigureD30AInputTriggerOffset(ViSession instrSession,
                                               ViReal64  inputTriggerOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_INPUT_TRIGGER_OFFSET, inputTriggerOffset),
            2, "Input Trigger Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the alarm thresholds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rmsMEROverall/Sets the alarm threshold for RMS MER overall. When this parameter
/// HIPAR rmsMEROverall/falls below the set threshold, an alarm is generated.
/// HIPAR berPreReedSolomon/Sets the alarm threshold for pre-Reed-Solomon BER. When this parameter
/// HIPAR berPreReedSolomon/exceeds the set threshold, an alarm is generated.
/// HIPAR berPostReedSolomon/Sets the alarm threshold for post-Reed-Solomon BER. When this
/// HIPAR berPostReedSolomon/parameter exceeds the set threshold, an alarm is generated.
ViStatus rsdsa_ConfigureD30AAlarmThreshold(ViSession instrSession,
                                           ViReal64  rmsMEROverall,
                                           ViReal64  berPreRS,
                                           ViReal64  berPostRS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_ALARM_THRESHOLD_RMS_MER_OVERALL, rmsMEROverall),
            2, "RMS MER Overall");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_ALARM_THRESHOLD_BER_PRE_REED_SOLOMON, berPreRS),
            3, "BER Pre-RS");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D30A_ALARM_THRESHOLD_BER_POST_REED_SOLOMON, berPostRS),
            4, "BER Post-RS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the number of bursts used for the moving average applied to
/// HIFN  overall MER readings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merBursts/Specifies the number of bursts used for the moving average applied to
/// HIPAR merBursts/overall MER readings.
ViStatus rsdsa_ConfigureD30AMERBursts(ViSession instrSession,
                                      ViInt32   merBursts)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_MER_BURSTS, merBursts),
            2, "MER Bursts");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the SFP+ network output on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sfpOutputEnabled/Turns the SFP+ network output on and off.
ViStatus rsdsa_ConfigureD30ASFPOutputEnabled(ViSession instrSession,
                                             ViBoolean sfpOutputEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_D30A_CONFIGURE_SFP_OUTPUT_ENABLED, sfpOutputEnabled),
            2, "SFP Output Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the SFP+ IUC UDP port on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR iuc_atdma/Selects repcap IUC_ATDMA.
/// HIPAR enabled/Turns the SFP+ IUC UDP port on and off.
/// HIPAR sfpIUCUDPPort/Configures the UDP port to be used for SFP+ IUC output. The UDP ports
/// HIPAR sfpIUCUDPPort/must be uniquely assigned to downstream OFDM profiles/PLC and upstream
/// HIPAR sfpIUCUDPPort/DOCSIS IUCs.
ViStatus rsdsa_ConfigureD30ASFPIUCUDPPort(ViSession instrSession,
                                          ViInt32   iuc,
                                          ViBoolean enabled,
                                          ViInt32   udpPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxIUC = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "IUC_ATDMA", &maxIUC),
        2, "IUC_ATDMA");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, iuc, 1, maxIUC),
            2, "IUC_ATDMA");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D30A_CONFIGURE_SFP_IUC_UDP_PORT_ENABLED,
            "IUC_ATDMA", iuc - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "IUC_ATDMA");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, repCap, RSDSA_ATTR_D30A_CONFIGURE_SFP_IUC_UDP_PORT_ENABLED, enabled),
            3, "Enabled");
    if (enabled)
    {
        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, repCap, RSDSA_ATTR_D30A_CONFIGURE_SFP_IUC_UDP_PORT, udpPort),
                4, "UDP Port");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the downstream spectrum analysis mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_EnableUpstreamSpectrumAnalysis(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_ENABLE_UPSTREAM_SPECTRUM_ANALYSIS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns off all markers including delta markers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_USSpecAnAllMarkersOff(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_ALL_MARKERS_OFF, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns off all delta markers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_USSpecAnAllDeltaMarkersOff(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_ALL_DELTA_MARKERS_OFF, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Clears all stored display states.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_USSpecAnClearAllDisplayStates(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_US_SPECAN_CLEAR_ALL_DISPLAY_STATES, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Clears all masks, including both the lower and upper masks.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_USSpecAnClearAllMasks(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_CLEAR_ALL_MASKS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns all masks on or off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Turns all masks on or off.
ViStatus rsdsa_ConfigureUSSpecAnDisplayAllMasksEnabled(ViSession instrSession,
                                                       ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DISPLAY_ALL_MASKS_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the center frequency in Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR centerFrequency/Defines the center frequency in Hz.
ViStatus rsdsa_ConfigureUSSpecAnCenterFrequency(ViSession instrSession,
                                                ViReal64  centerFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_US_SPECAN_CONFIGURE_CENTER_FREQUENCY, centerFrequency),
            2, "Fenter Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the start and stop frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR startFrequency/Defines a start frequency for measurements in the frequency domain in
/// HIPAR startFrequency/Hz.
/// HIPAR stopFrequency/Defines a stop frequency for measurements in the frequency domain in
/// HIPAR stopFrequency/Hz.
ViStatus rsdsa_ConfigureUSSpecAnFrequency(ViSession instrSession,
                                          ViReal64  startFrequency,
                                          ViReal64  stopFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_US_SPECAN_CONFIGURE_START_FREQUENCY, startFrequency),
            2, "Start Frequency");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_US_SPECAN_CONFIGURE_STOP_FREQUENCY, stopFrequency),
            3, "Stop Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the Zero Span.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Sets zero span (time domain). Turning on zero span mode overrides any
/// HIPAR enabled/ACP or phase noise mode setting.
/// HIPAR sweepTime/Defines the sweep time in seconds, which is only available for zero
/// HIPAR sweepTime/span mode. Minimum and maximum sweep times are dynamically determined
/// HIPAR sweepTime/by the DSA based on RBW and the sample rate.
/// HIPAR sampleRate/Defines the sample rate in Hz, which is only available for zero span
/// HIPAR sampleRate/mode.
ViStatus rsdsa_ConfigureUSSpecAnZeroSpan(ViSession instrSession,
                                         ViBoolean enabled,
                                         ViReal64  sweepTime,
                                         ViReal64  sampleRate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_US_SPECAN_CONFIGURE_ZERO_SPAN_ENABLED, enabled),
            2, "Enabled");
    if (enabled)
    {
        viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_US_SPECAN_CONFIGURE_ZERO_SPAN_SWEEP_TIME, sweepTime),
                3, "Sweep Time");

        viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_US_SPECAN_CONFIGURE_ZERO_SPAN_SAMPLE_RATE, sampleRate),
                4, "Sample Rate");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the frequency span in Hz, which must be compatible with the
/// HIFN  RBW setting.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequencySpan/Defines the frequency span in Hz, which must be compatible with the
/// HIPAR frequencySpan/RBW setting.
ViStatus rsdsa_ConfigureUSSpecAnFrequencySpan(ViSession instrSession,
                                              ViReal64  frequencySpan)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_US_SPECAN_CONFIGURE_FREQUENCY_SPAN, frequencySpan),
            2, "Frequency Span");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restores the full span as per current mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_USSpecAnSetFrequencySpanFull(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_US_SPECAN_SET_FREQUENCY_SPAN_FULL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Set to minimum span (10 kHz).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_USSpecAnSetFrequencySpanMinimum(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_US_SPECAN_SET_FREQUENCY_SPAN_MINIMUM, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the phase noise offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR startOffset/Define the start offset, which defines the start of the frequency band
/// HIPAR startOffset/over which the phase noise measurement is to be made and is specified
/// HIPAR startOffset/as a delta from the target carrier. It is specified as an offset in Hz
/// HIPAR startOffset/from the target carrier.
/// HIPAR stopOffset/Define the stop offset, which defines the end of the frequency band
/// HIPAR stopOffset/over which the phase noise measurement is to be made and is specified
/// HIPAR stopOffset/as a delta from the target carrier. It is specified as an offset in Hz
/// HIPAR stopOffset/from the target carrier.
ViStatus rsdsa_ConfigureUSSpecAnPhaseNoiseOffset(ViSession instrSession,
                                                 ViReal64  startOffset,
                                                 ViReal64  stopOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_US_SPECAN_CONFIGURE_PHASE_NOISE_START_OFFSET, startOffset),
            2, "Start Offset");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_US_SPECAN_CONFIGURE_PHASE_NOISE_STOP_OFFSET, stopOffset),
            3, "Stop Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the spot phase noise in dBc/Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR spotResult/Queries the spot phase noise in dBc/Hz.
ViStatus rsdsa_QueryUSSpecAnPhaseNoiseSpotResult(ViSession instrSession,
                                                 ViReal64* spotResult)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_US_SPECAN_QUERY_PHASE_NOISE_SPOT_RESULT, spotResult),
            2, "Spot Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the integrated phase noise in dBc.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR phaseNoiseIntegratedResult/Queries the integrated phase noise in dBc.
ViStatus rsdsa_QueryUSSpecAnPhaseNoiseIntegratedResult(ViSession instrSession,
                                                       ViReal64* integratedResult)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_US_SPECAN_QUERY_PHASE_NOISE_INTEGRATED_RESULT, integratedResult),
            2, "Integrated Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the bandwidth that is used for making the integrated phase
/// HIFN  noise measurement in Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bandwidth/Queries the bandwidth that is used for making the integrated phase
/// HIPAR bandwidth/noise measurement in Hz.
ViStatus rsdsa_QueryUSSpecAnPhaseNoiseBandwidth(ViSession instrSession,
                                                ViReal64* bandwidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_US_SPECAN_QUERY_PHASE_NOISE_BANDWIDTH, bandwidth),
            2, "Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays a downstream or upstream CCDF (Complementary Cumulative
/// HIFN  Distribution Function histogram as per CM-SP-PHYv3.1-I16 sections 9.3.8
/// HIFN  or 9.4.6 respectively). This command may take up to 30 seconds to
/// HIFN  execute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_USSpecAnDisplayCCDFHistogram(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_US_SPECAN_DISPLAY_CCDF_HISTOGRAM, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns Complementary Cumulative Distribution Function (CCDF) mode on or
/// HIFN  off. Setting the state to ON overrides any zero span, phase noise, or
/// HIFN  ACP mode setting.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ccdfModeEnabled/Turns Complementary Cumulative Distribution Function (CCDF) mode on or
/// HIPAR ccdfModeEnabled/off. Setting the state to ON overrides any zero span, phase noise, or
/// HIPAR ccdfModeEnabled/ACP mode setting.
ViStatus rsdsa_ConfigureUSSpecAnCCDFModeEnabled(ViSession instrSession,
                                                ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_US_SPECAN_CONFIGURE_CCDF_MODE_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the peak CCDF power level in the units selected by
/// HIFN  RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR usSpecAnQueryCCDFPeak/Queries the peak CCDF power level in the units selected by
/// HIPAR usSpecAnQueryCCDFPeak/RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_QueryUSSpecAnCCDFPeak(ViSession instrSession,
                                     ViReal64* ccdfPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_US_SPECAN_QUERY_CCDF_PEAK, ccdfPeak),
            2, "CCDF Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Scales the y-axis for trace displays by automatically adjusting the
/// HIFN  y-axis reference level and scale per division to best fit the
/// HIFN  measurement results. Otherwise, the y-axis can be manually scaled using
/// HIFN  RSDSA_ATTR_US_SPECAN_CONFIGURE_REFERENCE_LEVEL and
/// HIFN  RSDSA_ATTR_US_SPECAN_CONFIGURE_SCALE_PER_DIVISION.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_USSpecAnAutoScaleYAxis(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_US_SPECAN_AUTO_SCALE_Y_AXIS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the amplitude.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR referenceLevel/Defines the reference level for the displayed trace in the units
/// HIPAR referenceLevel/selected by RSDSA_ATTR_POWER_UNITS. This setting is only used when
/// HIPAR referenceLevel/automatic y-axis scaling is turned off. If the reference level is set,
/// HIPAR referenceLevel/then the attenuation set in INPut:ATTenuation is no longer applied.
/// HIPAR scalePerDivision/Defines the scale per division in the units selected by
/// HIPAR scalePerDivision/RSDSA_ATTR_POWER_UNITS to be used for the y-axis for all trace
/// HIPAR scalePerDivision/displays, which all have 10 divisions on the y-axis. Note that this
/// HIPAR scalePerDivision/command sets the scale per division, whereas the user of the DSA GUI
/// HIPAR scalePerDivision/sets the entire range for the vertical axis.
ViStatus rsdsa_ConfigureUSSpecAnAmplitude(ViSession instrSession,
                                          ViReal64  referenceLevel,
                                          ViInt32   scalePerDivision)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_US_SPECAN_CONFIGURE_REFERENCE_LEVEL, referenceLevel),
            2, "Reference Level");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_US_SPECAN_CONFIGURE_SCALE_PER_DIVISION, scalePerDivision),
            3, "Scale Per Division");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the RBW.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoRBW/When turned on, the DSA automatically determines the resolution
/// HIPAR autoRBW/bandwidth based on the span.
/// HIPAR manualRBW/Defines the resolution bandwidth, which must be compatible with the
/// HIPAR manualRBW/span setting: 10 Hz to 3 MHz, or 10 Hz to 204.8 MHz for
ViStatus rsdsa_ConfigureUSSpecAnRBW(ViSession instrSession,
                                    ViBoolean autoRBW,
                                    ViReal64  manualRBW)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_US_SPECAN_CONFIGURE_AUTO_RBW_ENABLED, autoRBW),
            2, "Auto RBW");
    if (!autoRBW)
    {
        viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_US_SPECAN_CONFIGURE_MANUAL_RBW, manualRBW),
                3, "Manual RBW");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the VBW.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoVBW/When turned on, the DSA automatically determines the video bandwidth
/// HIPAR autoVBW/based on the span.
/// HIPAR manualVBW/Defines the video bandwidth in Hz and automatically calculates the
/// HIPAR manualVBW/number of averages, overriding the manual setting of averages.
ViStatus rsdsa_ConfigureUSSpecAnVBW(ViSession instrSession,
                                    ViBoolean autoVBW,
                                    ViReal64  manualVBW)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_US_SPECAN_CONFIGURE_AUTO_VBW_ENABLED, autoVBW),
            2, "Auto VBW");

    if (!autoVBW)
    {
        viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_US_SPECAN_CONFIGURE_MANUAL_VBW, manualVBW),
                3, "Manual VBW");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the average count.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoAverageCount/When turned on, the DSA automatically determines the average setting.
/// HIPAR manualAverageCount/Defines the number of averages. A value of 1 indicates no averaging.
/// HIPAR manualAverageCount/Note: For OFDM spectrum analysis, setting an average that is not a
/// HIPAR manualAverageCount/multiple of 128 may cause artifacts between pilots.
ViStatus rsdsa_ConfigureUSSpecAnAverageCount(ViSession instrSession,
                                             ViBoolean autoAverageCount,
                                             ViInt32   manualAverageCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_US_SPECAN_CONFIGURE_AUTO_AVERAGE_COUNT, autoAverageCount),
            2, "Auto Average Count");
    if (!autoAverageCount)
    {
        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_US_SPECAN_CONFIGURE_MANUAL_AVERAGE_COUNT, manualAverageCount),
                3, "Manual Average Count");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restarts a single sweep, rearming for the next trigger, if applicable.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_USSpecAnRunSingleSweep(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_US_SPECAN_RUN_SINGLE_SWEEP, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the trace detector.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR detector/Selects the detection method for the displayed trace.
ViStatus rsdsa_ConfigureUSSpecAnDetector(ViSession instrSession,
                                         ViInt32   detector)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_US_SPECAN_CONFIGURE_DETECTOR, detector),
            2, "Detector");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR captureMode/Selects whether to use continuous captures or single trace captures.
/// HIPAR traceHold/Selects the measurement mode for the displayed trace. The DSA applies
/// HIPAR traceHold/minimum/maximum hold over multiple single traces initiated by
/// HIPAR traceHold/DISPlay:[<Mode>:][WINDow:]TRACe[<Trace>]:RSINgle command.
/// HIPAR acpMode/Turns Adjacent Channel Power (ACP) mode on or off. Commanding the
/// HIPAR acpMode/state to ON overrides any zero span or phase noise mode setting.
/// HIPAR phaseNoiseMode/Turns phase noise mode on or off. Setting the state to ON overrides
/// HIPAR phaseNoiseMode/any zero span or ACP mode setting.
ViStatus rsdsa_ConfigureUSSpecAnMeasurement(ViSession instrSession,
                                            ViInt32   captureMode,
                                            ViInt32   traceHold,
                                            ViBoolean acpMode,
                                            ViBoolean phaseNoiseMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_US_SPECAN_CONFIGURE_CAPTURE_MODE, captureMode),
            2, "Capture Mode");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_US_SPECAN_CONFIGURE_TRACE_HOLD, traceHold),
            3, "Trace Hold");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_US_SPECAN_CONFIGURE_ACP_MODE_ENABLED, acpMode),
            4, "ACP Mode");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_US_SPECAN_CONFIGURE_PHASE_NOISE_MODE_ENABLED, phaseNoiseMode),
            5, "Phase Noise Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables the marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
/// HIPAR enabled/Turns individual markers on and off. If the specified marker is a
/// HIPAR enabled/delta marker, it is turned into a normal marker. The default position
/// HIPAR enabled/is at the maximum peak.
ViStatus rsdsa_ConfigureUSSpecAnMarkerEnabled(ViSession instrSession,
                                              ViInt32   marker,
                                              ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_MARKER_STATE,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, repCap, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_MARKER_STATE, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the marker X position.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
/// HIPAR xPosition/Moves a marker to a particular coordinate on the x-axis. If the
/// HIPAR xPosition/specified marker is turned off, it is turned on first. If the specified
/// HIPAR xPosition/marker is a delta marker, it is turned into a normal marker.
ViStatus rsdsa_ConfigureUSSpecAnMarkerXPosition(ViSession instrSession,
                                                ViInt32   marker,
                                                ViReal64  xPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_MARKER_X_POSITION,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_MARKER_X_POSITION, xPosition),
            3, "X Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the position of a marker on the y-axis. If the specified
/// HIFN  marker is turned off, it is turned on first. If the specified marker is
/// HIFN  a delta marker, it is turned into a normal marker.
/// HIFN     The units are determined by RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
/// HIPAR yPosition/Queries the position of a marker on the y-axis. If the specified
/// HIPAR yPosition/marker is turned off, it is turned on first. If the specified marker is
/// HIPAR yPosition/a delta marker, it is turned into a normal marker.
ViStatus rsdsa_QueryUSSpecAnMarkerYPosition(ViSession instrSession,
                                            ViInt32   marker,
                                            ViReal64* yPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_MARKER_Y_POSITION,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_MARKER_Y_POSITION, yPosition),
            3, "Y Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a marker to the highest amplitude displayed. If the specified
/// HIFN  marker is turned off, it is turned on first. If the specified marker is
/// HIFN  a delta marker, it is turned into a normal marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_USSpecAnMarkerToMaximumPeak(ViSession instrSession,
                                           ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_MARKER_TO_MAXIMUM_PEAK,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_MARKER_TO_MAXIMUM_PEAK, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a marker to the next peak to the left of the current marker
/// HIFN  position. If the specified marker is turned off, it is turned on first.
/// HIFN  If the specified marker is a delta marker, it is turned into a normal
/// HIFN  marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_USSpecAnMarkerToNextPeakLeft(ViSession instrSession,
                                            ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_LEFT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_LEFT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a marker to the next peak to the right of the current marker
/// HIFN  position. If the specified marker is turned off, it is turned on first.
/// HIFN  If the specified marker is a delta marker, it is turned into a normal
/// HIFN  marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_USSpecAnMarkerToNextPeakRight(ViSession instrSession,
                                             ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_RIGHT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_RIGHT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a marker to the next displayed point to the left of the current
/// HIFN  marker position. If the specified marker is turned off, it is turned on
/// HIFN  first. If the specified marker is a delta marker, it is turned into a
/// HIFN  normal marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_USSpecAnMarkerToNextPointLeft(ViSession instrSession,
                                             ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_LEFT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_LEFT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a marker to the next displayed point to the right of the current
/// HIFN  marker position. If the specified marker is turned off, it is turned on
/// HIFN  first. If the specified marker is a delta marker, it is turned into a
/// HIFN  normal marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_USSpecAnMarkerToNextPointRight(ViSession instrSession,
                                              ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_RIGHT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_RIGHT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the center frequency to the frequency of a marker. If the
/// HIFN  specified marker is turned off, it is turned on first. If the specified
/// HIFN  marker is a delta marker, it is turned into a normal marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_USSpecAnMarkerSetCenterFrequency(ViSession instrSession,
                                                ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_MARKER_SET_CENTER_FREQUENCY,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_MARKER_SET_CENTER_FREQUENCY, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the marker to the zero span trigger point at the center of the
/// HIFN  plot. If the specified marker is turned off, it is turned on first. If
/// HIFN  the specified marker is a delta marker, it is turned into a normal
/// HIFN  marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_USSpecAnZeroSpanMarkerToTriggerPoint(ViSession instrSession,
                                                    ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_US_SPECAN_ZERO_SPAN_MARKER_TO_TRIGGER_POINT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_US_SPECAN_ZERO_SPAN_MARKER_TO_TRIGGER_POINT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the reference level to the power level of a marker. If the
/// HIFN  specified marker is turned off, it is turned on first. If the specified
/// HIFN  marker is a delta marker, it is turned into a normal marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_USSpecAnMarkerSetPowerReferenceLevel(ViSession instrSession,
                                                    ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_MARKER_SET_POWER_REFERENCE_LEVEL,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_MARKER_SET_POWER_REFERENCE_LEVEL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
/// HIPAR enabled/Turns delta markers on and off. If the specified marker is a normal
/// HIPAR enabled/marker, it is turned into a delta marker. The default position is at
/// HIPAR enabled/the maximum peak.
ViStatus rsdsa_ConfigureUSSpecAnDeltaMarkerEnabled(ViSession instrSession,
                                                   ViInt32   delta,
                                                   ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_STATE,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, repCap, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_STATE, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
/// HIPAR linkedReferenceMarker/Selects a reference marker for a delta marker. Values of delta marker
/// HIPAR linkedReferenceMarker/are relative to the reference marker. If the specified reference marker
/// HIPAR linkedReferenceMarker/is turned off, it is turned on first. If the specified delta marker is
/// HIPAR linkedReferenceMarker/turned off, it is turned on first. If the specified delta marker is a
/// HIPAR linkedReferenceMarker/normal marker, it is turned into a delta marker. If the specified
/// HIPAR linkedReferenceMarker/reference marker is the same as the specified delta marker, the marker
/// HIPAR linkedReferenceMarker/is changed to a normal marker.
ViStatus rsdsa_ConfigureUSSpecAnDeltaMarkerLinkedReferenceMarker(
                            ViSession instrSession,
                            ViInt32   delta,
                            ViInt32   linkedReferenceMarker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_LINKED_REFERENCE_MARKER,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    if (linkedReferenceMarker == RSDSA_VAL_REFERENCE_MARKER_NORMAL)
        linkedReferenceMarker = delta;

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, repCap, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_LINKED_REFERENCE_MARKER, linkedReferenceMarker),
            3, "Linked Reference Marker");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to a particular absolute coordinate on the
/// HIFN  x-axis. If the specified marker is turned off, it is turned on first.
/// HIFN  If the specified marker is a normal marker, it is turned into a delta
/// HIFN  marker. The query returns the absolute position of the delta marker.
/// HIFN     The units are Hz (frequency domain) or s (time domain, for zero span
/// HIFN  mode).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
/// HIPAR absoluteXPosition/Moves a delta marker to a particular absolute coordinate on the
/// HIPAR absoluteXPosition/x-axis. If the specified marker is turned off, it is turned on first.
/// HIPAR absoluteXPosition/If the specified marker is a normal marker, it is turned into a delta
/// HIPAR absoluteXPosition/marker. The query returns the absolute position of the delta marker.
ViStatus rsdsa_ConfigureUSSpecAnDeltaMarkerAbsoluteXPosition(ViSession instrSession,
                                                             ViInt32   delta,
                                                             ViReal64  absoluteXPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_ABSOLUTE_X_POSITION,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_ABSOLUTE_X_POSITION, absoluteXPosition),
            3, "Absolute X Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to a particular relative coordinate on the
/// HIFN  x-axis. If the specified marker is turned off, it is turned on first.
/// HIFN  If the specified marker is a normal marker, it is turned into a delta
/// HIFN  marker. The query returns the relative position of the delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
/// HIPAR relativeXPosition/Moves a delta marker to a particular relative coordinate on the
/// HIPAR relativeXPosition/x-axis. If the specified marker is turned off, it is turned on first.
/// HIPAR relativeXPosition/If the specified marker is a normal marker, it is turned into a delta
/// HIPAR relativeXPosition/marker. The query returns the relative position of the delta marker.
ViStatus rsdsa_ConfigureUSSpecAnDeltaMarkerRelativeXPosition(ViSession instrSession,
                                                             ViInt32   delta,
                                                             ViReal64  relativeXPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_RELATIVE_X_POSITION,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_RELATIVE_X_POSITION, relativeXPosition),
            3, "Relative X Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the absolute position of a delta marker on the y-axis. If the
/// HIFN  specified marker is turned off, it is turned on first. If the specified
/// HIFN  marker is a normal marker, it is turned into a delta marker.
/// HIFN     The units are determined by RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
/// HIPAR yPosition/Queries the absolute position of a delta marker on the y-axis. If the
/// HIPAR yPosition/specified marker is turned off, it is turned on first. If the specified
/// HIPAR yPosition/marker is a normal marker, it is turned into a delta marker.
ViStatus rsdsa_QueryUSSpecAnDeltaMarkerYPosition(ViSession instrSession,
                                                 ViInt32   delta,
                                                 ViReal64* yPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_Y_POSITION,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_Y_POSITION, yPosition),
            3, "Y Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to the highest amplitude displayed. If the
/// HIFN  specified marker is turned off, it is turned on first. If the specified
/// HIFN  marker is a normal marker, it is turned into a delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
ViStatus rsdsa_USSpecAnDeltaMarkerToMaximumPeak(ViSession instrSession,
                                                ViInt32   delta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_MAXIMUM_PEAK,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_MAXIMUM_PEAK, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to the next peak to the left of the current
/// HIFN  marker position. If the specified marker is turned off, it is turned on
/// HIFN  first. If the specified marker is a normal marker, it is turned into a
/// HIFN  delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
ViStatus rsdsa_USSpecAnDeltaMarkerToNextPeakLeft(ViSession instrSession,
                                                 ViInt32   delta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_LEFT,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_LEFT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to the next peak to the right of the current
/// HIFN  marker position. If the specified marker is turned off, it is turned on
/// HIFN  first. If the specified marker is a normal marker, it is turned into a
/// HIFN  delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
ViStatus rsdsa_USSpecAnDeltaMarkerToNextPeakRight(ViSession instrSession,
                                                  ViInt32   delta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_RIGHT,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_RIGHT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to the next displayed point to the left of the
/// HIFN  current marker position. If the specified marker is turned off, it is
/// HIFN  turned on first. If the specified marker is a normal marker, it is
/// HIFN  turned into a delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
ViStatus rsdsa_USSpecAnDeltaMarkerToNextPointLeft(ViSession instrSession,
                                                  ViInt32   delta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_LEFT,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_LEFT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to the next displayed point to the right of the
/// HIFN  current marker position. If the specified marker is turned off, it is
/// HIFN  turned on first. If the specified marker is a normal marker, it is
/// HIFN  turned into a delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
ViStatus rsdsa_USSpecAnDeltaMarkerToNextPointRight(ViSession instrSession,
                                                   ViInt32   delta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_RIGHT,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_RIGHT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the delta marker to the zero span trigger point at the center of
/// HIFN  the plot. If the specified marker is turned off, it is turned on first.
/// HIFN  If the specified marker is a normal marker, it is turned into a delta
/// HIFN  marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
ViStatus rsdsa_USSpecAnZeroSpanDeltaMarkerToTriggerPoint(ViSession instrSession,
                                                         ViInt32   delta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_US_SPECAN_ZERO_SPAN_DELTA_MARKER_TO_TRIGGER_POINT,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_US_SPECAN_ZERO_SPAN_DELTA_MARKER_TO_TRIGGER_POINT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the center frequency to the frequency of a delta marker. If the
/// HIFN  specified marker is turned off, it is turned on first. If the specified
/// HIFN  marker is a normal marker, it is turned into a delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
ViStatus rsdsa_USSpecAnDeltaMarkerSetCenterFrequency(ViSession instrSession,
                                                     ViInt32   delta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_SET_CENTER_FREQUENCY,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_SET_CENTER_FREQUENCY, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the reference level to the power level of a delta marker. If the
/// HIFN  specified marker is turned off, it is turned on first. If the specified
/// HIFN  marker is a normal marker, it is turned into a delta marker.REFerence
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
ViStatus rsdsa_USSpecAnDeltaMarkerSetPowerReferenceLevel(ViSession instrSession,
                                                         ViInt32   delta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_SET_POWER_REFERENCE_LEVEL,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_SET_POWER_REFERENCE_LEVEL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the bandpower between a specified delta marker and its
/// HIFN  reference marker in the frequency domain. An error is generated if the
/// HIFN  specified marker is turned off, or if the specified marker is a normal
/// HIFN  marker.
/// HIFN     The units are selected by RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
/// HIPAR powerResult/Queries the bandpower between a specified delta marker and its
/// HIPAR powerResult/reference marker in the frequency domain. An error is generated if the
/// HIPAR powerResult/specified marker is turned off, or if the specified marker is a normal
/// HIPAR powerResult/marker.
ViStatus rsdsa_QueryUSSpecAnDeltaMarkerBandwidthPowerResult(ViSession instrSession,
                                                            ViInt32   delta,
                                                            ViReal64* powerResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_SET_POWER_REFERENCE_LEVEL,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_BANDWIDTH_POWER_RESULT, powerResult),
            3, "Power Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores a complete set of spectrum analyzer configuration settings to
/// HIFN  the specified state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR savedState/Selects repcap SavedState.
ViStatus rsdsa_USSpecAnDisplayStateStore(ViSession instrSession,
                                         ViInt32   savedState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxSavedState = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "SavedState", &maxSavedState),
        2, "Saved State");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, savedState, 1, maxSavedState),
            2, "Saved State");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_US_SPECAN_DISPLAY_STATE_STORE,
            "SavedState", savedState - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Saved State");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_US_SPECAN_DISPLAY_STATE_STORE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Clears specified state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR savedState/Selects repcap SavedState.
ViStatus rsdsa_USSpecAnDisplayStateClear(ViSession instrSession,
                                         ViInt32   savedState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxSavedState = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "SavedState", &maxSavedState),
        2, "Saved State");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, savedState, 1, maxSavedState),
            2, "Saved State");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_US_SPECAN_DISPLAY_STATE_CLEAR,
            "SavedState", savedState - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Saved State");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_US_SPECAN_DISPLAY_STATE_CLEAR, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Recalls and applies a complete set of spectrum analyzer configuration
/// HIFN  settings from the specified state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR savedState/Selects repcap SavedState.
ViStatus rsdsa_USSpecAnDisplayStateRecall(ViSession instrSession,
                                          ViInt32   savedState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxSavedState = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "SavedState", &maxSavedState),
        2, "Saved State");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, savedState, 1, maxSavedState),
            2, "Saved State");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_US_SPECAN_DISPLAY_STATE_RECALL,
            "SavedState", savedState - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Saved State");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_US_SPECAN_DISPLAY_STATE_RECALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the specified trace data. Plot data is returned as x,y value
/// HIFN  pairs ( x1,y1,x2,y2,...). 1054 points are displayed across the DSA
/// HIFN  screen. For I/Q density data, 65535 integers are always returned in row
/// HIFN  order, regardless of the zoom settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Selects repcap Trace.
/// HIPAR bufferSize/Size of output array.
/// HIPAR traceData/Queries the specified trace data. Plot data is returned as x,y value
/// HIPAR traceData/pairs ( x1,y1,x2,y2,...). 1054 points are displayed across the DSA
/// HIPAR traceData/screen. For I/Q density data, 65535 integers are always returned in row
/// HIPAR traceData/order, regardless of the zoom settings.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_QueryUSSpecAnTraceData(ViSession instrSession,
                                      ViInt32   trace,
                                      ViInt32   bufferSize,
                                      ViReal64  traceData[],
                                      ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxTrace = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Trace", &maxTrace),
        2, "Trace");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, maxTrace),
            2, "Trace");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DS_SPECAN_QUERY_TRACE_DATA,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trace");
    viCheckParm(RsCore_GetAttributeViStringAsViReal64ArrayToUserBuffer(instrSession, repCap,
        RSDSA_ATTR_DS_SPECAN_QUERY_TRACE_DATA, NULL,
        bufferSize, traceData, bytesRead),
        4, "Trace Data");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Clears specified mask, including both the lower and upper masks.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mask/Selects repcap Mask.
ViStatus rsdsa_USSpecAnClearMask(ViSession instrSession,
                                 ViInt32   mask)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMask = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Mask", &maxMask),
        2, "Mask");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mask, 1, maxMask),
            2, "Mask");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_US_SPECAN_CLEAR_MASK,
            "Mask", mask - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Mask");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_US_SPECAN_CLEAR_MASK, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns individual masks on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mask/Selects repcap Mask.
/// HIPAR maskVisible/Turns individual masks on and off.
ViStatus rsdsa_ConfigureUSSpecAnMaskVisible(ViSession instrSession,
                                            ViInt32   mask,
                                            ViBoolean maskVisible)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMask = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Mask", &maxMask),
        2, "Mask");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mask, 1, maxMask),
            2, "Mask");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_US_SPECAN_MASK_VISIBLE,
            "Mask", mask - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Mask");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, repCap, RSDSA_ATTR_US_SPECAN_MASK_VISIBLE, maskVisible),
            3, "Mask Visible");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the specified lower mask with 1 to 20 points. Each point in
/// HIFN  the mask configuration must have a corresponding X and Y value. All
/// HIFN  frequency (X) entries are in Hz, and all amplitude (Y) entries are in
/// HIFN  the units selected by RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mask/Selects repcap Mask.
/// HIPAR points/Defines the length of the X and Y values arrays. A mask definition may
/// HIPAR points/have up to 20 points defined.
/// HIPAR xValues/An array of frequency (X axis) entries for the points of the lower
/// HIPAR xValues/mask, in Hz.
/// HIPAR yValues/An array of amplitude (Y axis) entries for the points of the lower
/// HIPAR yValues/mask, in the units specified by RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_USSpecAnConfigureLowerDisplayMask(ViSession instrSession,
                                                 ViInt32   mask,
                                                 ViInt32   points,
                                                 ViReal64  xValues[],
                                                 ViReal64  yValues[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMask = 0;
    ViChar   param[RS_MAX_MESSAGE_BUF_SIZE] = { '\0' };
    ViChar*  p2buf = param;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Mask", &maxMask),
        2, "Mask");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mask, 1, maxMask),
            2, "Mask");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_HIDDEN_US_SPECAN_MASK_LOWER_CONFIGURATION,
            "Mask", mask - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Mask");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, points, RSDSA_VAL_MASK_POINTS_MIN, RSDSA_VAL_MASK_POINTS_MAX),
            3, "Points");

    for (int i = 0; i < points; i++)
    {
        p2buf += snprintf(p2buf, RS_MAX_MESSAGE_BUF_SIZE - strlen(param), "%f,%f,", xValues[i], yValues[i]);
    }
    *(--p2buf) = '\0'; // Remove trailing comma

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_HIDDEN_US_SPECAN_MASK_LOWER_CONFIGURATION, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the specified upper mask with 1 to 20 points. Each point in
/// HIFN  the mask configuration must have a corresponding X and Y value. All
/// HIFN  frequency (X) entries are in Hz, and all amplitude (Y) entries are in
/// HIFN  the units selected by RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mask/Selects repcap Mask.
/// HIPAR points/Defines the length of the X and Y values arrays. A mask definition may
/// HIPAR points/have up to 20 points defined.
/// HIPAR xValues/An array of frequency (X axis) entries for the points of the upper
/// HIPAR xValues/mask, in Hz.
/// HIPAR yValues/An array of amplitude (Y axis) entries for the points of the upper
/// HIPAR yValues/mask, in the units specified by RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_USSpecAnConfigureUpperDisplayMask(ViSession instrSession,
                                                 ViInt32   mask,
                                                 ViInt32   points,
                                                 ViReal64  xValues[],
                                                 ViReal64  yValues[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMask = 0;
    ViChar   param[RS_MAX_MESSAGE_BUF_SIZE] = { '\0' };
    ViChar*  p2buf = param;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Mask", &maxMask),
        2, "Mask");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mask, 1, maxMask),
            2, "Mask");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_HIDDEN_US_SPECAN_MASK_UPPER_CONFIGURATION,
            "Mask", mask - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Mask");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, points, RSDSA_VAL_MASK_POINTS_MIN, RSDSA_VAL_MASK_POINTS_MAX),
            3, "Points");

    for (int i = 0; i < points; i++)
    {
        p2buf += snprintf(p2buf, RS_MAX_MESSAGE_BUF_SIZE - strlen(param), "%f,%f,", xValues[i], yValues[i]);
    }
    *(--p2buf) = '\0'; // Remove trailing comma

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_HIDDEN_US_SPECAN_MASK_UPPER_CONFIGURATION, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the downstream spectrum analysis mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_EnableDynamicUpstreamAnalysis(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_ENABLE_DYNAMIC_UPSTREAM_ANALYSIS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restarts dynamic upstream analysis mode distance estimation, which
/// HIFN  reinitiates the acquisition of MAPs and attempts to rematch bursts to
/// HIFN  the MAP information.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_DUAnalysisRestartDistanceEstimation(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_RESTART_DISTANCE_ESTIMATION, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an IUC to be used as a data filter for displayed status and
/// HIFN  plots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filterIUC/Selects an IUC to be used as a data filter for displayed status and
/// HIPAR filterIUC/plots.
ViStatus rsdsa_DUAnalysisConfigureFilterIUC(ViSession instrSession,
                                            ViInt32   filterIUC)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_FILTER_IUC, filterIUC),
            2, "Filter IUC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a SID (14-bit value in hexadecimal) to be used as a data
/// HIFN  filter for displayed status and plots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filterSID/Selects a SID (14-bit value in hexadecimal) to be used as a data
/// HIPAR filterSID/filter for displayed status and plots.
ViStatus rsdsa_DUAnalysisConfigureFilterSID(ViSession instrSession,
                                            ViString  filterSID)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_FILTER_SID, filterSID),
            2, "Filter SID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the burst demodulator lock status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstDemodulatorLockStatus/Queries the burst demodulator lock status.
ViStatus rsdsa_DUAnalysisQueryBurstDemodulatorLockStatus(ViSession instrSession,
                                                         ViInt32*  burstDemodulatorLockStatus)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_BURST_DEMODULATOR_LOCK_STATUS, burstDemodulatorLockStatus),
            2, "Burst Demodulator Lock Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the forward error correction (FEC) lock status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fecLockStatus/Queries the forward error correction (FEC) lock status.
ViStatus rsdsa_DUAnalysisQueryFECLockStatus(ViSession instrSession,
                                            ViInt32*  fecLockStatus)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_FEC_LOCK_STATUS, fecLockStatus),
            2, "FEC Lock Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the MPEG lock status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mpegLockStatus/Queries the MPEG lock status.
ViStatus rsdsa_DUAnalysisQueryMPEGLockStatus(ViSession instrSession,
                                             ViInt32*  mpegLockStatus)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_MPEG_LOCK_STATUS, mpegLockStatus),
            2, "MPEG Lock Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the PLC lock status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR plcLockStatus/Queries the PLC lock status.
ViStatus rsdsa_DUAnalysisQueryPLCLockStatus(ViSession instrSession,
                                            ViInt32*  plcLockStatus)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_PLC_LOCK_STATUS, plcLockStatus),
            2, "PLC Lock Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the burst receive power level in the units selected by
/// HIFN  RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstReceivePower/Queries the burst receive power level in the units selected by
/// HIPAR burstReceivePower/RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_DUAnalysisQueryBurstReceivePower(ViSession instrSession,
                                                ViReal64* burstReceivePower)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_BURST_RECEIVE_POWER, burstReceivePower),
            2, "Burst Receive Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the frequency offset in Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequencyOffset/Queries the frequency offset in Hz.
ViStatus rsdsa_DUAnalysisQueryFrequencyOffset(ViSession instrSession,
                                              ViReal64* frequencyOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_FREQUENCY_OFFSET, frequencyOffset),
            2, "Frequency Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the burst timing offset from the time the burst was detected
/// HIFN  and the time the burst was supposed to occur, in seconds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstTimingOffset/Queries the burst timing offset from the time the burst was detected
/// HIPAR burstTimingOffset/and the time the burst was supposed to occur, in seconds.
ViStatus rsdsa_DUAnalysisQueryBurstTimingOffset(ViSession instrSession,
                                                ViReal64* burstTimingOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_BURST_TIMING_OFFSET, burstTimingOffset),
            2, "Burst Timing Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the payload data rate in Mbps (megabits per second) for the
/// HIFN  IUC currently selected by DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR payloadDataRateIUC/Queries the payload data rate in Mbps (megabits per second) for the
/// HIPAR payloadDataRateIUC/IUC currently selected by DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>.
ViStatus rsdsa_DUAnalysisQueryPayloadDataRateIUC(ViSession instrSession,
                                                 ViReal64* payloadDataRateIUC)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_PAYLOAD_DATA_RATE_IUC, payloadDataRateIUC),
            2, "Payload Data Rate IUC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of minislots received.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR minislotsReceived/Queries the number of minislots received.
ViStatus rsdsa_DUAnalysisQueryMinislotsReceived(ViSession instrSession,
                                                ViInt64*  minislotsReceived)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt64(instrSession, NULL,
        RSDSA_ATTR_DU_ANALYSIS_QUERY_MINISLOTS_RECEIVED,
        minislotsReceived),
        2, "Minislots Received");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the percentage of valid bursts out of all bursts received over
/// HIFN  the last one second period.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstCoverage/Queries the percentage of valid bursts out of all bursts received over
/// HIPAR burstCoverage/the last one second period.
ViStatus rsdsa_DUAnalysisQueryBurstCoverage(ViSession instrSession,
                                            ViReal64* burstCoverage)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_BURST_COVERAGE, burstCoverage),
            2, "Burst Coverage");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the upstream mode detected via the downstream signal in
/// HIFN  dynamic upstream analysis mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR upstreamMode/Queries the upstream mode detected via the downstream signal in
/// HIPAR upstreamMode/dynamic upstream analysis mode.
ViStatus rsdsa_DUAnalysisQueryDetectedUpstreamMode(ViSession instrSession,
                                                   ViInt32*  upstreamMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_DETECTED_UPSTREAM_MODE, upstreamMode),
            2, "Upstream Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the sign preferred for MER numerical display and plot axis
/// HIFN  labels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sign/Selects the sign preferred for MER numerical display and plot axis
/// HIPAR sign/labels.
ViStatus rsdsa_DUAnalysisConfigureMERSign(ViSession instrSession,
                                          ViInt32   sign)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_MER_SIGN, sign),
            2, "Sign");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall RMS equalized MER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merRMS/Queries the overall RMS equalized MER.
ViStatus rsdsa_DUAnalysisQueryMERRMS(ViSession instrSession,
                                     ViReal64* merRMS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_MER_RMS, merRMS),
            2, "MER RMS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall RMS partially equalized MER. For OFDMA mode, it is
/// HIFN  only valid for data bursts with grant sizes greater than 100 using
/// HIFN  pilots 4 or 11.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merRMSPartialyEqualized/Queries the overall RMS partially equalized MER. For OFDMA mode, it is
/// HIPAR merRMSPartialyEqualized/only valid for data bursts with grant sizes greater than 100 using
/// HIPAR merRMSPartialyEqualized/pilots 4 or 11.
ViStatus rsdsa_DUAnalysisQueryMERRMSPartialyEqualized(ViSession instrSession,
                                                      ViReal64* merRMSPartialyEqualized)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_MER_RMS_PARTIALY_EQUALIZED, merRMSPartialyEqualized),
            2, "MER RMS Partialy Equalized");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall peak (lowest) MER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merPeak/Queries the overall peak (lowest) MER.
ViStatus rsdsa_DUAnalysisQueryMERPeak(ViSession instrSession,
                                      ViReal64* merPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_MER_PEAK, merPeak),
            2, "MER Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the RMS MER for the pilot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merPilotRMS/Queries the RMS MER for the pilot.
ViStatus rsdsa_DUAnalysisQueryMERPilotRMS(ViSession instrSession,
                                          ViReal64* merPilotRMS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_MER_PILOT_RMS, merPilotRMS),
            2, "MER Pilot RMS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the peak (lowest) MER for the pilot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merPilotPeak/Queries the peak (lowest) MER for the pilot.
ViStatus rsdsa_DUAnalysisQueryMERPilotPeak(ViSession instrSession,
                                           ViReal64* merPilotPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_MER_PILOT_PEAK, merPilotPeak),
            2, "MER Pilot Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the constellations for the pilot.?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR constellations/Queries the constellations for the pilot.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_DUAnalysisQueryPilotConstellations(ViSession instrSession,
                                                  ViInt32   bufferSize,
                                                  ViChar    constellations[],
                                                  ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL,
        RSDSA_ATTR_DU_ANALYSIS_QUERY_PILOT_CONSTELLATIONS,
        RS_VAL_DIRECT_USER_CALL,
        bufferSize, constellations, bytesRead),
        3, "Constellations");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the RMS MER for the complementary pilot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merComplementaryPilotRMS/Queries the RMS MER for the complementary pilot.
ViStatus rsdsa_DUAnalysisQueryMERComplementaryPilotRMS(ViSession instrSession,
                                                       ViReal64* merComplementaryPilotRMS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_MER_COMPLEMENTARY_PILOT_RMS, merComplementaryPilotRMS),
            2, "MER Complementary Pilot RMS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the peak (lowest) MER for the complementary pilot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merComplementaryPilotPeak/Queries the peak (lowest) MER for the complementary pilot.
ViStatus rsdsa_DUAnalysisQueryMERComplementaryPilotPeak(ViSession instrSession,
                                                        ViReal64* merComplementaryPilotPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_MER_COMPLEMENTARY_PILOT_PEAK, merComplementaryPilotPeak),
            2, "MER Complementary Pilot Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the constellations for the complementary pilot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR constellations/Queries the constellations for the complementary pilot.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_DUAnalysisQueryComplementaryPilotConstellations(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    constellations[],
                            ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL,
        RSDSA_ATTR_DU_ANALYSIS_QUERY_COMPLEMENTARY_PILOT_CONSTELLATIONS,
        RS_VAL_DIRECT_USER_CALL,
        bufferSize, constellations, bytesRead),
        3, "Constellations");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the RMS MER for the SID currently selected by
/// HIFN  DISPlay:[DUANalysis:][WINDow:]SIDFilter <Sid> and the IUC currently
/// HIFN  selected by DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merFilteredRMS/Queries the RMS MER for the SID currently selected by
/// HIPAR merFilteredRMS/DISPlay:[DUANalysis:][WINDow:]SIDFilter <Sid> and the IUC currently
/// HIPAR merFilteredRMS/selected by DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>.
ViStatus rsdsa_DUAnalysisQueryMERFilteredRMS(ViSession instrSession,
                                             ViReal64* merFilteredRMS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_MER_FILTERED_RMS, merFilteredRMS),
            2, "MER Filtered RMS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the peak (lowest) MER for the SID currently selected by
/// HIFN  DISPlay:[DUANalysis:][WINDow:]SIDFilter <Sid> and the IUC currently
/// HIFN  selected by DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merFilteredPeak/Queries the peak (lowest) MER for the SID currently selected by
/// HIPAR merFilteredPeak/DISPlay:[DUANalysis:][WINDow:]SIDFilter <Sid> and the IUC currently
/// HIPAR merFilteredPeak/selected by DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>.
ViStatus rsdsa_DUAnalysisQueryMERFilteredPeak(ViSession instrSession,
                                              ViReal64* merFilteredPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_MER_FILTERED_PEAK, merFilteredPeak),
            2, "MER Filtered Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the I/Q amplitude imbalance in dB.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR amplitudedB/Queries the I/Q amplitude imbalance in dB.
ViStatus rsdsa_DUAnalysisQueryIQImbalanceAmplitudedB(ViSession instrSession,
                                                     ViReal64* amplitudedB)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_IQ_IMBALANCE_AMPLITUDE_DB, amplitudedB),
            2, "Amplitude dB");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the I/Q amplitude imbalance in percent.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR amplitudePercent/Queries the I/Q amplitude imbalance in percent.
ViStatus rsdsa_DUAnalysisQueryIQImbalanceAmplitudePercent(ViSession instrSession,
                                                          ViReal64* amplitudePercent)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_IQ_IMBALANCE_AMPLITUDE_PERCENT, amplitudePercent),
            2, "Amplitude Percent");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the I/Q imbalance quadrature error in degrees.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR quadratureError/Queries the I/Q imbalance quadrature error in degrees.
ViStatus rsdsa_DUAnalysisQueryIQImbalanceQuadratureError(ViSession instrSession,
                                                         ViReal64* quadratureError)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_IQ_IMBALANCE_QUADRATURE_ERROR, quadratureError),
            2, "Quadrature Error");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the channel ID for downstream RF inputs.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR downstreamChannelID/Queries the channel ID for downstream RF inputs.
ViStatus rsdsa_DUAnalysisQueryDownstreamChannelID(ViSession instrSession,
                                                  ViInt32*  downstreamChannelID)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_DOWNSTREAM_CHANNEL_ID, downstreamChannelID),
            2, "Downstream Channel ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the channel ID for upstream RF inputs.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR upstreamChannelID/Queries the channel ID for upstream RF inputs.
ViStatus rsdsa_DUAnalysisQueryUpstreamChannelID(ViSession instrSession,
                                                ViInt32*  upstreamChannelID)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_UPSTREAM_CHANNEL_ID, upstreamChannelID),
            2, "Upstream Channel ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the settings for error analysis. Changing error analysis
/// HIFN  settings will reset the error analysis.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR errorAnalysisMode/Specifies the mode of BER measurement.
/// HIPAR manualTestDepth/Defines the test depth of the minimum BER integration. The actual test
/// HIPAR manualTestDepth/depth setting may be automatically changed in automatic error analysis
/// HIPAR manualTestDepth/mode.
/// HIPAR testDepthSource/Selects the BER source used for BER measurements.
/// HIPAR testDepthProgressionSource/Selects the BER source used for test depth progression.
ViStatus rsdsa_DUAnalysisConfigureErrorAnalysis(ViSession instrSession,
                                                ViInt32   errorAnalysisMode,
                                                ViInt32   manualTestDepth,
                                                ViInt32   testDepthSource,
                                                ViInt32   testDepthProgressionSource)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_ERROR_ANALYSIS_MODE, errorAnalysisMode),
            2, "Error Analysis Mode");
    if (errorAnalysisMode == RSDSA_VAL_ERROR_ANALYSIS_MANUAL)
    {
        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH, manualTestDepth),
                3, "Manual Test Depth");
    }
    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH_SOURCE, testDepthSource),
            4, "Test Depth Source");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH_PROGRESSION_SOURCE, testDepthProgressionSource),
            5, "Test Depth Progression Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Performs a BER reset that clears all related counters to zero and
/// HIFN  restarts the BER/CER measurement for the current mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_DUAnalysisResetErrorAnalysis(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_RESET_ERROR_ANALYSIS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the elapsed time of the current BER/CER measurement: time
/// HIFN  elapsed since the start of the mode, or last BER reset
/// HIFN  (RSDSA_ATTR_D31U_RESET_ERROR_ANALYSIS), or last BER configuration
/// HIFN  change, in seconds.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR elapsedTime/Queries the elapsed time of the current BER/CER measurement: time
/// HIPAR elapsedTime/elapsed since the start of the mode, or last BER reset
/// HIPAR elapsedTime/(RSDSA_ATTR_D31U_RESET_ERROR_ANALYSIS), or last BER configuration
/// HIPAR elapsedTime/change, in seconds.
ViStatus rsdsa_DUAnalysisQueryBERElapsedTime(ViSession instrSession,
                                             ViReal64* elapsedTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_BER_ELAPSED_TIME, elapsedTime),
            2, "Elapsed Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the percentage of blocks processed for the current BER/CER
/// HIFN  measurement: blocks processed since the start of the mode, or last BER
/// HIFN  reset (CALCulate:[<Mode>:][AVERage:]BERatio:CLEar), or last BER
/// HIFN  configuration change. Expressed as the percentage of blocks processed
/// HIFN  relative to the current test depth. A block is 10^7 bits maximum.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR blocksProcessed/Queries the percentage of blocks processed for the current BER/CER
/// HIPAR blocksProcessed/measurement: blocks processed since the start of the mode, or last BER
/// HIPAR blocksProcessed/reset (CALCulate:[<Mode>:][AVERage:]BERatio:CLEar), or last BER
/// HIPAR blocksProcessed/configuration change. Expressed as the percentage of blocks processed
/// HIPAR blocksProcessed/relative to the current test depth. A block is 10^7 bits maximum.
ViStatus rsdsa_DUAnalysisQueryBERBlocksProcessed(ViSession instrSession,
                                                 ViReal64* blocksProcessed)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_BER_BLOCKS_PROCESSED, blocksProcessed),
            2, "Blocks Processed");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of bursts expected since the signal was acquired.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstsExpected/Queries the number of bursts expected since the signal was acquired.
ViStatus rsdsa_DUAnalysisQueryBERBurstsExpected(ViSession instrSession,
                                                ViReal64* burstsExpected)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_BER_BURSTS_EXPECTED, burstsExpected),
            2, "Bursts Expected");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of bursts received since the signal was acquired.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstsReceived/Queries the number of bursts received since the signal was acquired.
ViStatus rsdsa_DUAnalysisQueryBERBurstsReceived(ViSession instrSession,
                                                ViReal64* burstsReceived)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_BER_BURSTS_RECEIVED, burstsReceived),
            2, "Bursts Received");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the pre-Reed-Solomon BER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR berPreRS/Queries the pre-Reed-Solomon BER.
ViStatus rsdsa_DUAnalysisQueryBERPreReedSolomon(ViSession instrSession,
                                                ViReal64* berPreRS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_BER_PRE_REED_SOLOMON, berPreRS),
            2, "BER Pre-RS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of bits analyzed at the input of the
/// HIFN  Reed-Solomon decoder for the BER measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR preRSTotalBits/Queries the total number of bits analyzed at the input of the
/// HIPAR preRSTotalBits/Reed-Solomon decoder for the BER measurement.
ViStatus rsdsa_DUAnalysisQueryBERPreReedSolomonTotalBits(ViSession instrSession,
                                                         ViReal64* preRSTotalBits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_BER_PRE_REED_SOLOMON_TOTAL_BITS, preRSTotalBits),
            2, "Pre-RS Total Bits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the post-Reed-Solomon BER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR berPostRS/Queries the post-Reed-Solomon BER.
ViStatus rsdsa_DUAnalysisQueryBERPostReedSolomon(ViSession instrSession,
                                                 ViReal64* berPostRS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_BER_POST_REED_SOLOMON, berPostRS),
            2, "BER Post-RS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of corrected bits at the output of the
/// HIFN  Reed-Solomon decoder for the BER measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR postRSCorrectedBits/Queries the total number of corrected bits at the output of the
/// HIPAR postRSCorrectedBits/Reed-Solomon decoder for the BER measurement.
ViStatus rsdsa_DUAnalysisQueryBERPostReedSolomonCorrectedBits(
                            ViSession instrSession,
                            ViReal64* postRSCorrectedBits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_BER_POST_REED_SOLOMON_CORRECTED_BITS, postRSCorrectedBits),
            2, "Post-RS Corrected Bits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of uncorrected bits at the output of the
/// HIFN  Reed-Solomon decoder for the BER measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR postRSUncorrectedBits/Queries the total number of uncorrected bits at the output of the
/// HIPAR postRSUncorrectedBits/Reed-Solomon decoder for the BER measurement.
ViStatus rsdsa_DUAnalysisQueryBERPostReedSolomonUncorrectedBits(
                            ViSession instrSession,
                            ViReal64* postRSUncorrectedBits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_BER_POST_REED_SOLOMON_UNCORRECTED_BITS, postRSUncorrectedBits),
            2, "Post-RS Uncorrected Bits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the pre-LDPC BER for the IUC currently selected by
/// HIFN  DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR berIUCPreLDPC/Queries the pre-LDPC BER for the IUC currently selected by
/// HIPAR berIUCPreLDPC/DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>.
ViStatus rsdsa_DUAnalysisQueryBERIUCPreLDPC(ViSession instrSession,
                                            ViReal64* berIUCPreLDPC)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_BER_IUC_PRE_LDPC, berIUCPreLDPC),
            2, "BER IUC Pre-LDPC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of bits analyzed at the input of the LDPC
/// HIFN  decoder for the BER measurement, for the IUC currently selected by
/// HIFN  DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR iucPreLDPCTotalBits/Queries the total number of bits analyzed at the input of the LDPC
/// HIPAR iucPreLDPCTotalBits/decoder for the BER measurement, for the IUC currently selected by
/// HIPAR iucPreLDPCTotalBits/DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>.
ViStatus rsdsa_DUAnalysisQueryBERIUCPreLDPCTotalBits(ViSession instrSession,
                                                     ViReal64* iucPreLDPCTotalBits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_BER_IUC_PRE_LDPC_TOTAL_BITS, iucPreLDPCTotalBits),
            2, "IUC Pre-LDPC Total Bits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the post-LDPC BER for the IUC currently selected by
/// HIFN  DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR berIUCPostLDPC/Queries the post-LDPC BER for the IUC currently selected by
/// HIPAR berIUCPostLDPC/DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>.
ViStatus rsdsa_DUAnalysisQueryBERIUCPostLDPC(ViSession instrSession,
                                             ViReal64* berIUCPostLDPC)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_BER_IUC_POST_LDPC, berIUCPostLDPC),
            2, "BER IUC Post-LDPC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of bits analyzed at the output of the LDPC
/// HIFN  decoder for the BER measurement, for the IUC currently selected by
/// HIFN  DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>. Note that the bits for
/// HIFN  undecodable codewords are not included in this total.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR iucPostLDPCTotalBits/Queries the total number of bits analyzed at the output of the LDPC
/// HIPAR iucPostLDPCTotalBits/decoder for the BER measurement, for the IUC currently selected by
/// HIPAR iucPostLDPCTotalBits/DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>. Note that the bits for
/// HIPAR iucPostLDPCTotalBits/undecodable codewords are not included in this total.
ViStatus rsdsa_DUAnalysisQueryBERIUCPostLDPCTotalBits(ViSession instrSession,
                                                      ViReal64* iucPostLDPCTotalBits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_BER_IUC_POST_LDPC_TOTAL_BITS, iucPostLDPCTotalBits),
            2, "IUC Post-LDPC Total Bits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the codeword length(s) to be used for displayed status (i.e.,
/// HIFN  codeword length filter).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR codewordLengthFilter/Selects the codeword length(s) to be used for displayed status (i.e.,
/// HIPAR codewordLengthFilter/codeword length filter).
ViStatus rsdsa_DUAnalysisConfigureCodewordLengthFilter(ViSession instrSession,
                                                       ViInt32   codewordLengthFilter)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_CODEWORD_LENGTH_FILTER, codewordLengthFilter),
            2, "Codeword Length Filter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the corrected CER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cerCorrected/Queries the corrected CER.
ViStatus rsdsa_DUAnalysisQueryCERCorrected(ViSession instrSession,
                                           ViReal64* cerCorrected)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_CER_CORRECTED, cerCorrected),
            2, "CER Corrected");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the uncorrected CER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cerUncorrected/Queries the uncorrected CER.
ViStatus rsdsa_DUAnalysisQueryCERUncorrected(ViSession instrSession,
                                             ViReal64* cerUncorrected)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_CER_UNCORRECTED, cerUncorrected),
            2, "CER Uncorrected");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of codewords for the CER measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR totalCodewords/Queries the total number of codewords for the CER measurement.
ViStatus rsdsa_DUAnalysisQueryCERTotalCodewords(ViSession instrSession,
                                                ViReal64* totalCodewords)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_CER_TOTAL_CODEWORDS, totalCodewords),
            2, "Total Codewords");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of corrected CER codewords.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR correctedCodewords/Queries the total number of corrected CER codewords.
ViStatus rsdsa_DUAnalysisQueryCERCorrectedCodewords(ViSession instrSession,
                                                    ViReal64* correctedCodewords)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_CER_CORRECTED_CODEWORDS, correctedCodewords),
            2, "Corrected Codewords");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of uncorrected CER codewords.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR uncorrectedCodewords/Queries the total number of uncorrected CER codewords.
ViStatus rsdsa_DUAnalysisQueryCERUncorrectedCodewords(ViSession instrSession,
                                                      ViReal64* uncorrectedCodewords)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_CER_UNCORRECTED_CODEWORDS, uncorrectedCodewords),
            2, "Uncorrected Codewords");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the post-LDPC CER, for the codeword length currently selected
/// HIFN  by DISPlay:[<Mode>:][WINDow:]CLFilter <CwLength>.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cerPostLDPC/Queries the post-LDPC CER, for the codeword length currently selected
/// HIPAR cerPostLDPC/by DISPlay:[<Mode>:][WINDow:]CLFilter <CwLength>.
ViStatus rsdsa_DUAnalysisQueryCERPostLDPC(ViSession instrSession,
                                          ViReal64* cerPostLDPC)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_CER_POST_LDPC, cerPostLDPC),
            2, "CER Post-LDPC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of post-LDPC codeword errors, for the
/// HIFN  codeword length currently selected by
/// HIFN  DISPlay:[<Mode>:][WINDow:]CLFilter <CwLength>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR postLDPCCodewordErrors/Queries the total number of post-LDPC codeword errors, for the
/// HIPAR postLDPCCodewordErrors/codeword length currently selected by
/// HIPAR postLDPCCodewordErrors/DISPlay:[<Mode>:][WINDow:]CLFilter <CwLength>.
ViStatus rsdsa_DUAnalysisQueryCERPostLDPCCodewordErrors(ViSession instrSession,
                                                        ViReal64* postLDPCCodewordErrors)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_CER_POST_LDPC_CODEWORD_ERRORS, postLDPCCodewordErrors),
            2, "Post-LDPC Codeword Errors");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the average number of LDPC iterations, for the codeword length
/// HIFN  currently selected by DISPlay:[<Mode>:][WINDow:]CLFilter <CwLength>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR averageLDPCIterations/Queries the average number of LDPC iterations, for the codeword length
/// HIPAR averageLDPCIterations/currently selected by DISPlay:[<Mode>:][WINDow:]CLFilter <CwLength>.
ViStatus rsdsa_DUAnalysisQueryCERAverageLDPCIterations(ViSession instrSession,
                                                       ViReal64* averageLDPCIterations)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_CER_AVERAGE_LDPC_ITERATIONS, averageLDPCIterations),
            2, "Average LDPC Iterations");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of bits in the average payload per codeword,
/// HIFN  for the IUC currently selected by DISPlay:[<Mode>:][WINDow:]IUCFilter
/// HIFN  <Iuc>. For OFDMA, this command also applies to the codeword length
/// HIFN  currently selected by DISPlay:[<Mode>:][WINDow:]CLFilter <CwLength>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR avgPayload/Queries the total number of bits in the average payload per codeword,
/// HIPAR avgPayload/for the IUC currently selected by DISPlay:[<Mode>:][WINDow:]IUCFilter
/// HIPAR avgPayload/<Iuc>. For OFDMA, this command also applies to the codeword length
/// HIPAR avgPayload/currently selected by DISPlay:[<Mode>:][WINDow:]CLFilter <CwLength>.
ViStatus rsdsa_DUAnalysisQueryCERAvgPayloadPerCodewordPerIUC(ViSession instrSession,
                                                             ViReal64* avgPayload)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_CER_AVG_PAYLOAD_PER_CODEWORD_PER_IUC, avgPayload),
            2, "Avg Payload");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the percentage of codewords covered (i.e., not discarded), for
/// HIFN  the IUC currently selected by DISPlay:[<Mode>:][WINDow:]IUCFilter
/// HIFN  <Iuc>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR codewordCoverageIUC/Queries the percentage of codewords covered (i.e., not discarded), for
/// HIPAR codewordCoverageIUC/the IUC currently selected by DISPlay:[<Mode>:][WINDow:]IUCFilter
/// HIPAR codewordCoverageIUC/<Iuc>.
ViStatus rsdsa_DUAnalysisQueryCERCodewordCoverageIUC(ViSession instrSession,
                                                     ViReal64* codewordCoverageIUC)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_CER_CODEWORD_COVERAGE_IUC, codewordCoverageIUC),
            2, "Codeword Coverage IUC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the constellation plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_DUAnalysisDisplayConstellationPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_DISPLAY_CONSTELLATION_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the I/Q density plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_DUAnalysisDisplayIQDensityPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_DISPLAY_I_Q_DENSITY_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the measurement mode for the I/Q density plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementMode/Selects the measurement mode for the I/Q density plot.
/// HIPAR windowWidth/Specifies the sliding window width in samples for the I/Q density plot
/// HIPAR windowWidth/in sliding window mode.
ViStatus rsdsa_DUAnalysisConfigureIQMeasurementMode(ViSession instrSession,
                                                    ViInt32   measurementMode,
                                                    ViInt32   windowWidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_I_Q_MEASUREMENT_MODE, measurementMode),
            2, "Measurement Mode");
    if (measurementMode == RSDSA_VAL_IQ_MEASUREMENT_MODE_SLIDING_WINDOW)
    {
        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_I_Q_WINDOW_WIDTH, windowWidth),
                3, "Window Width");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the zoom setting for the I/Q density plot display. Note that
/// HIFN  the x and y origin are the top left corner of the window, as per the
/// HIFN  GUI settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR x/X origin (left-most window point), between 0 and 1.
/// HIPAR y/Y origin (top-most window point), between 0 and 1.
/// HIPAR zoom/Zoom setting, 1 to 128.
ViStatus rsdsa_DUAnalysisSetIQDisplayZoom(ViSession instrSession,
                                          ViReal64  x,
                                          ViReal64  y,
                                          ViInt32   zoom)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_CheckRangeTableRangeViReal64(instrSession, &RsDSA_rngIQZoomOrigin, x),
        2, "X");
    viCheckParm(rsdsa_CheckRangeTableRangeViReal64(instrSession, &RsDSA_rngIQZoomOrigin, y),
        3, "Y");
    viCheckParm(rsdsa_CheckRangeTableRangeViInt32(instrSession, &RsDSA_rngIQZoomLevel, zoom),
        4, "Zoom");

    snprintf(param, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%f,%f,%ld", x, y, zoom);

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_HIDDEN_DU_ANALYSIS_DISPLAY_I_Q_ZOOM, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Save the constellation I/Q data to the specified file. The file
/// HIFN  extension is .iq.tar. Note that there is no way to abort this command,
/// HIFN  and large file sizes could take a long time to capture depending on the
/// HIFN  input data, so use maxSamples or timeoutSeconds to limit the file size
/// HIFN  or time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/File path, uses .iq.tar extension.
/// HIPAR maxSamples/Sets the I/Q data file maximum size in samples.
/// HIPAR timeoutSeconds/Sets the maximum time in seconds to save data.
ViStatus rsdsa_DUAnalysisCaptureIQDataToFile(ViSession instrSession,
                                             ViString  fileName,
                                             ViInt64   maxSamples,
                                             ViReal64  timeoutSeconds)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt64(instrSession, NULL,
        RSDSA_ATTR_DU_ANALYSIS_IQ_DATA_EXPORT_MAX_SAMPLES,
        maxSamples),
        3, "Max Samples");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_IQ_DATA_EXPORT_TIMEOUT, timeoutSeconds),
            4, "Timeout Seconds");

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_CAPTURE_IQ_DATA_TO_FILENAME, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the MER vs. time plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_DUAnalysisDisplayMERVsTimePlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_DISPLAY_MER_VS_TIME_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a time filter period to be used for displayed MER vs. Time
/// HIFN  plots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR period/Selects a time filter period to be used for displayed MER vs. Time
/// HIPAR period/plots.
ViStatus rsdsa_DUAnalysisConfigureMERVsTimePlotFilterPeriod(ViSession instrSession,
                                                            ViInt32   period)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_MER_VS_TIME_PLOT_FILTER_PERIOD, period),
            2, "Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the BER vs. time plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_DUAnalysisDisplayBERVsTimePlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_DISPLAY_BER_VS_TIME_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a time filter period to be used for displayed BER vs. Time
/// HIFN  plots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR period/Selects a time filter period to be used for displayed BER vs. Time
/// HIPAR period/plots.
ViStatus rsdsa_DUAnalysisConfigureBERVsTimePlotFilterPeriod(ViSession instrSession,
                                                            ViInt32   period)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_BER_VS_TIME_PLOT_FILTER_PERIOD, period));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the POWer vs. time plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_DUAnalysisDisplayPowerVsTimePlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_DISPLAY_POWER_VS_TIME_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a time filter period to be used for displayed Power vs. Time
/// HIFN  plots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR period/Selects a time filter period to be used for displayed Power vs. Time
/// HIPAR period/plots.
ViStatus rsdsa_DUAnalysisConfigurePowerVsTimePlotFilterPeriod(
                            ViSession instrSession,
                            ViInt32   period)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_POWER_VS_TIME_PLOT_FILTER_PERIOD, period),
            2, "Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the MER vs. Subcarrier plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_DUAnalysisDisplayMERVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_DISPLAY_MER_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the MER vs. minislot plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_DUAnalysisDisplayMERVsMinislotPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_DISPLAY_MER_VS_MINISLOT_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays a downstream or upstream CCDF (Complementary Cumulative
/// HIFN  Distribution Function histogram as per CM-SP-PHYv3.1-I16 sections 9.3.8
/// HIFN  or 9.4.6 respectively). This command may take up to 30 seconds to
/// HIFN  execute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_DUAnalysisDisplayCCDFHistogram(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_DISPLAY_CCDF_HISTOGRAM, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the peak CCDF power level in the units selected by
/// HIFN  RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ccdfPeak/Queries the peak CCDF power level in the units selected by
/// HIPAR ccdfPeak/RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_DUAnalysisQueryCCDFPeak(ViSession instrSession,
                                       ViReal64* ccdfPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_QUERY_CCDF_PEAK, ccdfPeak),
            2, "CCDF Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the channel analysis amplitude/phase vs. subcarrier plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_DUAnalysisDisplayAmplitudePhaseVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_DISPLAY_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Scales the y-axis for the channel analysis amplitude/phase vs.
/// HIFN  subcarrier plot to best fit the measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_DUAnalysisAutoScaleAmplitudePhaseVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_AUTO_SCALE_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the channel analysis amplitude/group delay vs. subcarrier
/// HIFN  plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_DUAnalysisDisplayAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_DISPLAY_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Scales the y-axis for the channel analysis amplitude/group delay vs.
/// HIFN  subcarrier plot to best fit the measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_DUAnalysisAutoScaleAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_AUTO_SCALE_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the channel analysis echo pattern plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_DUAnalysisDisplayEchoPatternPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_DISPLAY_ECHO_PATTERN_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the spectrum analysis plot for DOCSIS 3.1 Downstream mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_DUAnalysisDisplaySpectrumAnalysisPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_DISPLAY_SPECTRUM_ANALYSIS_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns off all markers including delta markers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_DUAnalysisAllMarkersOff(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_ALL_MARKERS_OFF, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns off all delta markers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_DUAnalysisAllDeltaMarkersOff(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_ALL_DELTA_MARKERS_OFF, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Clears all masks, including both the lower and upper masks.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_DUAnalysisClearAllMasks(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_CLEAR_ALL_MASKS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns all masks on or off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Turns all masks on or off.
ViStatus rsdsa_DUAnalysisConfigureDisplayAllMasksEnabled(ViSession instrSession,
                                                         ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DISPLAY_ALL_MASKS_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures amplitude settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR referenceLevel/Defines the reference level for the displayed trace in the units
/// HIPAR referenceLevel/selected by RSDSA_ATTR_POWER_UNITS. This setting is only used when
/// HIPAR referenceLevel/automatic y-axis scaling is turned off. If the reference level is set,
/// HIPAR referenceLevel/then the attenuation set in INPut:ATTenuation is no longer applied.
/// HIPAR scalePerDivision/Defines the scale per division in the units selected by
/// HIPAR scalePerDivision/RSDSA_ATTR_POWER_UNITS to be used for the y-axis for all trace
/// HIPAR scalePerDivision/displays, which all have 10 divisions on the y-axis. Note that this
/// HIPAR scalePerDivision/command sets the scale per division, whereas the user of the DSA GUI
/// HIPAR scalePerDivision/sets the entire range for the vertical axis.
ViStatus rsdsa_DUAnalysisConfigureAmplitudeSettings(ViSession instrSession,
                                                    ViReal64  referenceLevel,
                                                    ViInt32   scalePerDivision)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_REFERENCE_LEVEL, referenceLevel),
            2, "Reference Level");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_SCALE_PER_DIVISION, scalePerDivision),
            3, "Scale Per Division");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Scales the y-axis for trace displays by automatically adjusting the
/// HIFN  y-axis reference level and scale per division to best fit the
/// HIFN  measurement results. Otherwise, the y-axis can be manually scaled using
/// HIFN  RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_REFERENCE_LEVEL and
/// HIFN  RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_SCALE_PER_DIVISION.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_DUAnalysisAutoScaleYAxis(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_AUTO_SCALE_Y_AXIS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures bandwidth settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoAverageCount/When turned on, the DSA automatically determines the average setting.
/// HIPAR manualAverageCount/Defines the number of averages. A value of 1 indicates no averaging.
/// HIPAR manualAverageCount/Note: For OFDM spectrum analysis, setting an average that is not a
/// HIPAR manualAverageCount/multiple of 128 may cause artifacts between pilots.
ViStatus rsdsa_DUAnalysisConfigureBandwidthSettings(ViSession instrSession,
                                                    ViBoolean autoAverageCount,
                                                    ViInt32   manualAverageCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_AUTO_AVERAGE_COUNT, autoAverageCount),
            2, "Auto Average Count");
    if (!autoAverageCount)
    {
        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_MANUAL_AVERAGE_COUNT, manualAverageCount),
                3, "Manual Average Count");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures measurement settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR captureMode/Selects whether to use continuous captures or single trace captures.
/// HIPAR traceHold/Selects the measurement mode for the displayed trace. The DSA applies
/// HIPAR traceHold/minimum/maximum hold over multiple single traces initiated by
/// HIPAR traceHold/DISPlay:[<Mode>:][WINDow:]TRACe[<Trace>]:RSINgle command.
ViStatus rsdsa_DUAnalysisConfigureMeasurementsSettings(ViSession instrSession,
                                                       ViInt32   captureMode,
                                                       ViInt32   traceHold)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_CAPTURE_MODE, captureMode),
            2, "Capture Mode");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_TRACE_MODE, traceHold),
            3, "Trace Hold");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restarts a single sweep, rearming for the next trigger, if applicable.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_DUAnalysisRunSingleSweep(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_RUN_SINGLE_SWEEP, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns individual markers on and off. If the specified marker is a
/// HIFN  delta marker, it is turned into a normal marker. The default position
/// HIFN  is at the maximum peak.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
/// HIPAR enabled/Turns individual markers on and off. If the specified marker is a
/// HIPAR enabled/delta marker, it is turned into a normal marker. The default position
/// HIPAR enabled/is at the maximum peak.
ViStatus rsdsa_DUAnalysisConfigureMarkerState(ViSession instrSession,
                                              ViInt32   marker,
                                              ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_STATE,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, repCap, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_STATE, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a marker to a particular coordinate on the x-axis. If the
/// HIFN  specified marker is turned off, it is turned on first. If the specified
/// HIFN  marker is a delta marker, it is turned into a normal marker.
/// HIFN     The units are Hz (frequency domain) or s (time domain, for zero span
/// HIFN  mode).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
/// HIPAR xPosition/Moves a marker to a particular coordinate on the x-axis. If the
/// HIPAR xPosition/specified marker is turned off, it is turned on first. If the specified
/// HIPAR xPosition/marker is a delta marker, it is turned into a normal marker.
ViStatus rsdsa_DUAnalysisConfigureMarkerXPosition(ViSession instrSession,
                                                  ViInt32   marker,
                                                  ViReal64  xPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_X_POSITION,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_X_POSITION, xPosition),
            3, "X Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the position of a marker on the y-axis. If the specified
/// HIFN  marker is turned off, it is turned on first. If the specified marker is
/// HIFN  a delta marker, it is turned into a normal marker.
/// HIFN     The units are determined by RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
/// HIPAR yPosition/Queries the position of a marker on the y-axis. If the specified
/// HIPAR yPosition/marker is turned off, it is turned on first. If the specified marker is
/// HIPAR yPosition/a delta marker, it is turned into a normal marker.
ViStatus rsdsa_DUAnalysisQueryMarkerYPosition(ViSession instrSession,
                                              ViInt32   marker,
                                              ViReal64* yPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_X_POSITION,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_Y_POSITION, yPosition),
            3, "Y Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a marker to the highest amplitude displayed. If the specified
/// HIFN  marker is turned off, it is turned on first. If the specified marker is
/// HIFN  a delta marker, it is turned into a normal marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_DUAnalysisMarkerToMaximumPeak(ViSession instrSession,
                                             ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_TO_MAXIMUM_PEAK,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_TO_MAXIMUM_PEAK, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a marker to the next peak to the left of the current marker
/// HIFN  position. If the specified marker is turned off, it is turned on first.
/// HIFN  If the specified marker is a delta marker, it is turned into a normal
/// HIFN  marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_DUAnalysisMarkerToNextPeakLeft(ViSession instrSession,
                                              ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_LEFT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_LEFT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a marker to the next peak to the right of the current marker
/// HIFN  position. If the specified marker is turned off, it is turned on first.
/// HIFN  If the specified marker is a delta marker, it is turned into a normal
/// HIFN  marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_DUAnalysisMarkerToNextPeakRight(ViSession instrSession,
                                               ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_RIGHT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_RIGHT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a marker to the next displayed point to the left of the current
/// HIFN  marker position. If the specified marker is turned off, it is turned on
/// HIFN  first. If the specified marker is a delta marker, it is turned into a
/// HIFN  normal marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_DUAnalysisMarkerToNextPointLeft(ViSession instrSession,
                                               ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_LEFT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_LEFT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a marker to the next displayed point to the right of the current
/// HIFN  marker position. If the specified marker is turned off, it is turned on
/// HIFN  first. If the specified marker is a delta marker, it is turned into a
/// HIFN  normal marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_DUAnalysisMarkerToNextPointRight(ViSession instrSession,
                                                ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_RIGHT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_RIGHT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the reference level to the power level of a marker. If the
/// HIFN  specified marker is turned off, it is turned on first. If the specified
/// HIFN  marker is a delta marker, it is turned into a normal marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_DUAnalysisMarkerSetPowerReferenceLevel(ViSession instrSession,
                                                      ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_SET_POWER_REFERENCE_LEVEL,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_SET_POWER_REFERENCE_LEVEL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns delta markers on and off. If the specified marker is a normal
/// HIFN  marker, it is turned into a delta marker. The default position is at
/// HIFN  the maximum peak.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deltaMarker/Selects repcap Delta.
/// HIPAR enabled/Turns delta markers on and off. If the specified marker is a normal
/// HIPAR enabled/marker, it is turned into a delta marker. The default position is at
/// HIPAR enabled/the maximum peak.
ViStatus rsdsa_DUAnalysisConfigureDeltaMarkerState(ViSession instrSession,
                                                   ViInt32   deltaMarker,
                                                   ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaMarker),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deltaMarker, 1, maxDeltaMarker),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_STATE,
            "Delta", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, repCap, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_STATE, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a reference marker for a delta marker. Values of delta marker
/// HIFN  are relative to the reference marker. If the specified reference marker
/// HIFN  is turned off, it is turned on first. If the specified delta marker is
/// HIFN  turned off, it is turned on first. If the specified delta marker is a
/// HIFN  normal marker, it is turned into a delta marker. If the specified
/// HIFN  reference marker is the same as the specified delta marker, the marker
/// HIFN  is changed to a normal marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deltaMarker/Selects repcap Delta.
/// HIPAR linkedReferenceMarker/Selects a reference marker for a delta marker. Values of delta marker
/// HIPAR linkedReferenceMarker/are relative to the reference marker. If the specified reference marker
/// HIPAR linkedReferenceMarker/is turned off, it is turned on first. If the specified delta marker is
/// HIPAR linkedReferenceMarker/turned off, it is turned on first. If the specified delta marker is a
/// HIPAR linkedReferenceMarker/normal marker, it is turned into a delta marker. If the specified
/// HIPAR linkedReferenceMarker/reference marker is the same as the specified delta marker, the marker
/// HIPAR linkedReferenceMarker/is changed to a normal marker.
ViStatus rsdsa_DUAnalysisConfigureDeltaMarkerLinkedReferenceMarker(
                            ViSession instrSession,
                            ViInt32   deltaMarker,
                            ViInt32   linkedReferenceMarker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaMarker),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deltaMarker, 1, maxDeltaMarker),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_LINKED_REFERENCE_MARKER,
            "Delta", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    if (linkedReferenceMarker == RSDSA_VAL_REFERENCE_MARKER_NORMAL)
        linkedReferenceMarker = deltaMarker;

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, repCap, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_LINKED_REFERENCE_MARKER, linkedReferenceMarker),
            3, "Linked Reference Marker");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to a particular absolute coordinate on the
/// HIFN  x-axis. If the specified marker is turned off, it is turned on first.
/// HIFN  If the specified marker is a normal marker, it is turned into a delta
/// HIFN  marker. The query returns the absolute position of the delta marker.
/// HIFN     The units are Hz (frequency domain) or s (time domain, for zero span
/// HIFN  mode).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deltaMarker/Selects repcap Delta.
/// HIPAR absoluteXPosition/Moves a delta marker to a particular absolute coordinate on the
/// HIPAR absoluteXPosition/x-axis. If the specified marker is turned off, it is turned on first.
/// HIPAR absoluteXPosition/If the specified marker is a normal marker, it is turned into a delta
/// HIPAR absoluteXPosition/marker. The query returns the absolute position of the delta marker.
ViStatus rsdsa_DUAnalysisConfigureDeltaMarkerAbsoluteXPosition(
                            ViSession instrSession,
                            ViInt32   deltaMarker,
                            ViReal64  absoluteXPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaMarker),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deltaMarker, 1, maxDeltaMarker),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_ABSOLUTE_X_POSITION,
            "Delta", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_ABSOLUTE_X_POSITION, absoluteXPosition),
            3, "Absolute X Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to a particular relative coordinate on the
/// HIFN  x-axis. If the specified marker is turned off, it is turned on first.
/// HIFN  If the specified marker is a normal marker, it is turned into a delta
/// HIFN  marker. The query returns the relative position of the delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deltaMarker/Selects repcap Delta.
/// HIPAR relativeXPosition/Moves a delta marker to a particular relative coordinate on the
/// HIPAR relativeXPosition/x-axis. If the specified marker is turned off, it is turned on first.
/// HIPAR relativeXPosition/If the specified marker is a normal marker, it is turned into a delta
/// HIPAR relativeXPosition/marker. The query returns the relative position of the delta marker.
ViStatus rsdsa_DUAnalysisConfigureDeltaMarkerRelativeXPosition(
                            ViSession instrSession,
                            ViInt32   deltaMarker,
                            ViReal64  relativeXPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaMarker),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deltaMarker, 1, maxDeltaMarker),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_RELATIVE_X_POSITION,
            "Delta", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_RELATIVE_X_POSITION, relativeXPosition),
            3, "Relative X Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the absolute position of a delta marker on the y-axis. If the
/// HIFN  specified marker is turned off, it is turned on first. If the specified
/// HIFN  marker is a normal marker, it is turned into a delta marker.
/// HIFN     The units are determined by RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deltaMarker/Selects repcap Delta.
/// HIPAR yPosition/Queries the absolute position of a delta marker on the y-axis. If the
/// HIPAR yPosition/specified marker is turned off, it is turned on first. If the specified
/// HIPAR yPosition/marker is a normal marker, it is turned into a delta marker.
ViStatus rsdsa_DUAnalysisQueryDeltaMarkerYPosition(ViSession instrSession,
                                                   ViInt32   deltaMarker,
                                                   ViReal64* yPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaMarker),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deltaMarker, 1, maxDeltaMarker),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_Y_POSITION,
            "Delta", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_Y_POSITION, yPosition),
            2, "Y Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to the highest amplitude displayed. If the
/// HIFN  specified marker is turned off, it is turned on first. If the specified
/// HIFN  marker is a normal marker, it is turned into a delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deltaMarker/Selects repcap Delta.
ViStatus rsdsa_DUAnalysisDeltaMarkerToMaximumPeak(ViSession instrSession,
                                                  ViInt32   deltaMarker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaMarker),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deltaMarker, 1, maxDeltaMarker),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_MAXIMUM_PEAK,
            "Delta", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_MAXIMUM_PEAK, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to the next peak to the left of the current
/// HIFN  marker position. If the specified marker is turned off, it is turned on
/// HIFN  first. If the specified marker is a normal marker, it is turned into a
/// HIFN  delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deltaMarker/Selects repcap Delta.
ViStatus rsdsa_DUAnalysisDeltaMarkerToNextPeakLeft(ViSession instrSession,
                                                   ViInt32   deltaMarker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaMarker),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deltaMarker, 1, maxDeltaMarker),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_LEFT,
            "Delta", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_LEFT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to the next peak to the right of the current
/// HIFN  marker position. If the specified marker is turned off, it is turned on
/// HIFN  first. If the specified marker is a normal marker, it is turned into a
/// HIFN  delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deltaMarker/Selects repcap Delta.
ViStatus rsdsa_DUAnalysisDeltaMarkerToNextPeakRight(ViSession instrSession,
                                                    ViInt32   deltaMarker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaMarker),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deltaMarker, 1, maxDeltaMarker),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_RIGHT,
            "Delta", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_RIGHT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to the next displayed point to the left of the
/// HIFN  current marker position. If the specified marker is turned off, it is
/// HIFN  turned on first. If the specified marker is a normal marker, it is
/// HIFN  turned into a delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deltaMarker/Selects repcap Delta.
ViStatus rsdsa_DUAnalysisDeltaMarkerToNextPointLeft(ViSession instrSession,
                                                    ViInt32   deltaMarker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaMarker),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deltaMarker, 1, maxDeltaMarker),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_LEFT,
            "Delta", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_LEFT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to the next displayed point to the right of the
/// HIFN  current marker position. If the specified marker is turned off, it is
/// HIFN  turned on first. If the specified marker is a normal marker, it is
/// HIFN  turned into a delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deltaMarker/Selects repcap Delta.
ViStatus rsdsa_DUAnalysisDeltaMarkerToNextPointRight(ViSession instrSession,
                                                     ViInt32   deltaMarker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaMarker),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deltaMarker, 1, maxDeltaMarker),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_RIGHT,
            "Delta", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_RIGHT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the reference level to the power level of a delta marker. If the
/// HIFN  specified marker is turned off, it is turned on first. If the specified
/// HIFN  marker is a normal marker, it is turned into a delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deltaMarker/Selects repcap Delta.
ViStatus rsdsa_DUAnalysisDeltaMarkerSetPowerReferenceLevel(ViSession instrSession,
                                                           ViInt32   deltaMarker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaMarker),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deltaMarker, 1, maxDeltaMarker),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_SET_POWER_REFERENCE_LEVEL,
            "Delta", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_SET_POWER_REFERENCE_LEVEL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the bandpower between a specified delta marker and its
/// HIFN  reference marker in the frequency domain. An error is generated if the
/// HIFN  specified marker is turned off, or if the specified marker is a normal
/// HIFN  marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deltaMarker/Selects repcap Delta.
/// HIPAR bandwidthPowerResult/Queries the bandpower between a specified delta marker and its
/// HIPAR bandwidthPowerResult/reference marker in the frequency domain. An error is generated if the
/// HIPAR bandwidthPowerResult/specified marker is turned off, or if the specified marker is a normal
/// HIPAR bandwidthPowerResult/marker.
ViStatus rsdsa_DUAnalysisQueryDeltaMarkerBandwidthPowerResult(
                            ViSession instrSession,
                            ViInt32   deltaMarker,
                            ViReal64* bandwidthPowerResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaMarker),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deltaMarker, 1, maxDeltaMarker),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_SET_POWER_REFERENCE_LEVEL,
            "Delta", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_BANDWIDTH_POWER_RESULT, bandwidthPowerResult),
            3, "Bandwidth Power Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns individual masks on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mask/Selects repcap Mask.
/// HIPAR visible/Turns individual masks on and off.
ViStatus rsdsa_DUAnalysisConfigureMaskVisible(ViSession instrSession,
                                              ViInt32   mask,
                                              ViBoolean visible)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMask = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Mask", &maxMask),
        2, "Mask");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mask, 1, maxMask),
            2, "Mask");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DU_ANALYSIS_MASK_VISIBLE,
            "Mask", mask - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Mask");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, repCap, RSDSA_ATTR_DU_ANALYSIS_MASK_VISIBLE, visible),
            3, "Visible");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the specified lower mask with 1 to 20 points. Each point in
/// HIFN  the mask configuration must have a corresponding X and Y value. All
/// HIFN  frequency (X) entries are in Hz, and all amplitude (Y) entries are in
/// HIFN  the units selected by RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mask/Selects repcap Mask.
/// HIPAR points/Defines the length of the X and Y values arrays. A mask definition may
/// HIPAR points/have up to 20 points defined.
/// HIPAR xValues/An array of frequency (X axis) entries for the points of the lower
/// HIPAR xValues/mask, in Hz.
/// HIPAR yValues/An array of amplitude (Y axis) entries for the points of the lower
/// HIPAR yValues/mask, in the units specified by RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_DUAnalysisConfigureLowerDisplayMask(ViSession instrSession,
                                                   ViInt32   mask,
                                                   ViInt32   points,
                                                   ViReal64  xValues[],
                                                   ViReal64  yValues[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMask = 0;
    ViChar   param[RS_MAX_MESSAGE_BUF_SIZE] = { '\0' };
    ViChar*  p2buf = param;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Mask", &maxMask),
        2, "Mask");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mask, 1, maxMask),
            2, "Mask");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_HIDDEN_DU_ANALYSIS_MASK_LOWER_CONFIGURATION,
            "Mask", mask - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Mask");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, points, RSDSA_VAL_MASK_POINTS_MIN, RSDSA_VAL_MASK_POINTS_MAX),
            3, "Points");

    for (int i = 0; i < points; i++)
    {
        p2buf += snprintf(p2buf, RS_MAX_MESSAGE_BUF_SIZE - strlen(param), "%f,%f,", xValues[i], yValues[i]);
    }
    *(--p2buf) = '\0'; // Remove trailing comma

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_HIDDEN_DU_ANALYSIS_MASK_LOWER_CONFIGURATION, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the specified upper mask with 1 to 20 points. Each point in
/// HIFN  the mask configuration must have a corresponding X and Y value. All
/// HIFN  frequency (X) entries are in Hz, and all amplitude (Y) entries are in
/// HIFN  the units selected by RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mask/Selects repcap Mask.
/// HIPAR points/Defines the length of the X and Y values arrays. A mask definition may
/// HIPAR points/have up to 20 points defined.
/// HIPAR xValues/An array of frequency (X axis) entries for the points of the upper
/// HIPAR xValues/mask, in Hz.
/// HIPAR yValues/An array of amplitude (Y axis) entries for the points of the upper
/// HIPAR yValues/mask, in the units specified by RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_DUAnalysisConfigureUpperDisplayMask(ViSession instrSession,
                                                   ViInt32   mask,
                                                   ViInt32   points,
                                                   ViReal64  xValues[],
                                                   ViReal64  yValues[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMask = 0;
    ViChar   param[RS_MAX_MESSAGE_BUF_SIZE] = { '\0' };
    ViChar*  p2buf = param;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Mask", &maxMask),
        2, "Mask");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mask, 1, maxMask),
            2, "Mask");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_HIDDEN_DU_ANALYSIS_MASK_UPPER_CONFIGURATION,
            "Mask", mask - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Mask");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, points, RSDSA_VAL_MASK_POINTS_MIN, RSDSA_VAL_MASK_POINTS_MAX),
            3, "Points");

    for (int i = 0; i < points; i++)
    {
        p2buf += snprintf(p2buf, RS_MAX_MESSAGE_BUF_SIZE - strlen(param), "%f,%f,", xValues[i], yValues[i]);
    }
    *(--p2buf) = '\0'; // Remove trailing comma

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_HIDDEN_DU_ANALYSIS_MASK_UPPER_CONFIGURATION, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Clears specified mask, including both the lower and upper masks.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mask/Selects repcap Mask.
ViStatus rsdsa_DUAnalysisClearMask(ViSession instrSession,
                                   ViInt32   mask)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMask = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Mask", &maxMask),
        2, "Mask");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mask, 1, maxMask),
            2, "Mask");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DU_ANALYSIS_CLEAR_MASK,
            "Mask", mask - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Mask");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_DU_ANALYSIS_CLEAR_MASK, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the specified trace data. Plot data is returned as x,y value
/// HIFN  pairs (x1,y1,x2,y2,...). 1054 points are displayed across the DSA
/// HIFN  screen. For I/Q density data, 65535 integers are always returned in row
/// HIFN  order, regardless of the zoom settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Selects repcap Trace.
/// HIPAR bufferSize/Size of output array.
/// HIPAR traceData/Queries the specified trace data. Plot data is returned as x,y value
/// HIPAR traceData/pairs (x1,y1,x2,y2,...). 1054 points are displayed across the DSA
/// HIPAR traceData/screen. For I/Q density data, 65535 integers are always returned in row
/// HIPAR traceData/order, regardless of the zoom settings.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_DUAnalysisQueryTraceData(ViSession instrSession,
                                        ViInt32   trace,
                                        ViInt32   bufferSize,
                                        ViReal64  traceData[],
                                        ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxTrace = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Trace", &maxTrace),
        2, "Trace");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, maxTrace),
            2, "Trace");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DU_ANALYSIS_QUERY_TRACE_DATA,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trace");
    viCheckParm(RsCore_GetAttributeViStringAsViReal64ArrayToUserBuffer(instrSession, repCap,
        RSDSA_ATTR_DU_ANALYSIS_QUERY_TRACE_DATA, NULL,
        bufferSize, traceData, bytesRead),
        4, "Trace Data");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the all the messages that are currently in the acquisition log
/// HIFN  and clears the acquisition log. If the acquisition log is empty, the
/// HIFN  number 0, 'No entry', is returned. For each error in the log, the
/// HIFN  aqcuisition log entry message is returned, followed by the date and
/// HIFN  time the entry was added to the log.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR log/Queries the all the messages that are currently in the acquisition log
/// HIPAR log/and clears the acquisition log. If the acquisition log is empty, the
/// HIPAR log/number 0, 'No entry', is returned. For each error in the log, the
/// HIPAR log/aqcuisition log entry message is returned, followed by the date and
/// HIPAR log/time the entry was added to the log.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_DUAnalysisQuerySystemAcquisitionLogAll(ViSession instrSession,
                                                      ViInt32   bufferSize,
                                                      ViChar    log[],
                                                      ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL,
        RSDSA_ATTR_DU_ANALYSIS_SYSTEM_ACQUISITION_LOG_ALL,
        RS_VAL_DIRECT_USER_CALL,
        bufferSize, log, bytesRead),
        3, "Log");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the most recent acquisition log entry and deletes it. If the
/// HIFN  acquisition log is empty, the number 0, 'No entry', is returned.
/// HIFN  Returns the aqcuisition log entry message, followed by the date and
/// HIFN  time the entry was added to the log.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR entry/Queries the most recent acquisition log entry and deletes it. If the
/// HIPAR entry/acquisition log is empty, the number 0, 'No entry', is returned.
/// HIPAR entry/Returns the aqcuisition log entry message, followed by the date and
/// HIPAR entry/time the entry was added to the log.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_DUAnalysisQuerySystemAcquisitionLogNext(ViSession instrSession,
                                                       ViInt32   bufferSize,
                                                       ViChar    entry[],
                                                       ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL,
        RSDSA_ATTR_DU_ANALYSIS_SYSTEM_ACQUISITION_LOG_NEXT,
        RS_VAL_DIRECT_USER_CALL,
        bufferSize, entry, bytesRead),
        3, "Entry");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the acquisition log for the number of unread entries. If the
/// HIFN  acquisition log is empty, the error number 0, 'No error', is returned.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR count/Queries the acquisition log for the number of unread entries. If the
/// HIPAR count/acquisition log is empty, the error number 0, 'No error', is returned.
ViStatus rsdsa_DUAnalysisQuerySystemAcquisitionLogCount(ViSession instrSession,
                                                        ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_SYSTEM_ACQUISITION_LOG_COUNT, count),
            2, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes all entries in the acquisition log.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_DUAnalysisSystemAcquisitionLogClearAll(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_SYSTEM_ACQUISITION_LOG_CLEAR_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the downstream protocol for dynamic upstream analysis mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR standard/Specifies the downstream protocol for dynamic upstream analysis mode.
ViStatus rsdsa_DUAnalysisConfigureDownstreamStandard(ViSession instrSession,
                                                     ViInt32   standard)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_DOWNSTREAM_STANDARD, standard),
            2, "Standard");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the list of equalizer coefficients. A pair of signed
/// HIFN  floating-point values is returned for each coefficient, representing
/// HIFN  the real and imaginary parts. The number of coefficient pairs returned
/// HIFN  depends on the mode, the signal bandwidth, and for OFDM/OFDMA, the FFT
/// HIFN  size.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR equalizerCoefficients/Queries the list of equalizer coefficients. A pair of signed
/// HIPAR equalizerCoefficients/floating-point values is returned for each coefficient, representing
/// HIPAR equalizerCoefficients/the real and imaginary parts. The number of coefficient pairs returned
/// HIPAR equalizerCoefficients/depends on the mode, the signal bandwidth, and for OFDM/OFDMA, the FFT
/// HIPAR equalizerCoefficients/size.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_DUAnalysisQueryDownstreamEqualizerCoefficients(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViReal64  equalizerCoefficients[],
                            ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringAsViReal64ArrayToUserBuffer(instrSession, NULL,
        RSDSA_ATTR_DU_ANALYSIS_QUERY_DOWNSTREAM_EQUALIZER_COEFFICIENTS, NULL,
        bufferSize, equalizerCoefficients, bytesRead),
        3, "Equalizer Coefficients");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the constellation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR constellation/Defines the constellation.
ViStatus rsdsa_DUAnalysisConfigureDownstreamConstellation(ViSession instrSession,
                                                          ViInt32   constellation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_DOWNSTREAM_CONSTELLATION, constellation),
            2, "Constellation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Automatically determines the attenuation level and preamplifier state
/// HIFN  (on/off). Query the resultant attenuation level with
/// HIFN  RSDSA_ATTR_DUAN_CONFIGURE_ATTENUATION and the resultant preamplifier
/// HIFN  state with RSDSA_ATTR_DUAN_CONFIGURE_PREAMPLIFIER_ENABLED.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_DUAnalysisDownstreamAutoLevel(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_DOWNSTREAM_AUTO_LEVEL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the total attenuation for RF input in dB. If you set the
/// HIFN  attenuation manually, it is no longer coupled to the reference level,
/// HIFN  but the reference level is coupled to the attenuation. Thus, if the
/// HIFN  current reference level is not compatible with an attenuation that has
/// HIFN  been set manually, the command also adjusts the reference level.
/// HIFN  Therefore, the reference level set in
/// HIFN  DISPlay:[<Mode>:][WINDow:]TRACe[<Trace>]:Y[:SCALe]:RLEVel is no longer
/// HIFN  applied. Note: The query returns the actual attenuation setting, which
/// HIFN  can also be changed by RSDSA_ATTR_ED30_AUTO_LEVEL.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attenuation/Defines the total attenuation for RF input in dB. If you set the
/// HIPAR attenuation/attenuation manually, it is no longer coupled to the reference level,
/// HIPAR attenuation/but the reference level is coupled to the attenuation. Thus, if the
/// HIPAR attenuation/current reference level is not compatible with an attenuation that has
/// HIPAR attenuation/been set manually, the command also adjusts the reference level.
/// HIPAR attenuation/Therefore, the reference level set in
/// HIPAR attenuation/DISPlay:[<Mode>:][WINDow:]TRACe[<Trace>]:Y[:SCALe]:RLEVel is no longer
/// HIPAR attenuation/applied. Note: The query returns the actual attenuation setting, which
/// HIPAR attenuation/can also be changed by RSDSA_ATTR_ED30_AUTO_LEVEL.
ViStatus rsdsa_DUAnalysisConfigureDownstreamAttenuation(ViSession instrSession,
                                                        ViReal64  attenuation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_DOWNSTREAM_ATTENUATION, attenuation),
            2, "Attenuation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the preamplifier on and off. Note: The query returns the actual
/// HIFN  preamplifier state (which might also be changed by
/// HIFN  RSDSA_ATTR_DUAN_AUTO_LEVEL).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Turns the preamplifier on and off. Note: The query returns the actual
/// HIPAR enabled/preamplifier state (which might also be changed by
/// HIPAR enabled/RSDSA_ATTR_DUAN_AUTO_LEVEL).
ViStatus rsdsa_DUAnalysisConfigureDownstreamPreamplifierEnabled(
                            ViSession instrSession,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_DOWNSTREAM_PREAMPLIFIER_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the target upstream channel to lock to for dynamic upstream
/// HIFN  analysis mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelID/Specifies the target upstream channel to lock to for dynamic upstream
/// HIPAR channelID/analysis mode.
ViStatus rsdsa_DUAnalysisConfigureUpstreamChannelID(ViSession instrSession,
                                                    ViInt32   channelID)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_UPSTREAM_CHANNEL_ID, channelID),
            2, "Channel ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the total attenuation for RF input in dB. If you set the
/// HIFN  attenuation manually, it is no longer coupled to the reference level,
/// HIFN  but the reference level is coupled to the attenuation. Thus, if the
/// HIFN  current reference level is not compatible with an attenuation that has
/// HIFN  been set manually, the command also adjusts the reference level.
/// HIFN  Therefore, the reference level set in
/// HIFN  DISPlay:[<Mode>:][WINDow:]TRACe[<Trace>]:Y[:SCALe]:RLEVel is no longer
/// HIFN  applied. Note: The query returns the actual attenuation setting, which
/// HIFN  can also be changed by INPut:<Direction>:LEVel:AUTO.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attenuation/Defines the total attenuation for RF input in dB. If you set the
/// HIPAR attenuation/attenuation manually, it is no longer coupled to the reference level,
/// HIPAR attenuation/but the reference level is coupled to the attenuation. Thus, if the
/// HIPAR attenuation/current reference level is not compatible with an attenuation that has
/// HIPAR attenuation/been set manually, the command also adjusts the reference level.
/// HIPAR attenuation/Therefore, the reference level set in
/// HIPAR attenuation/DISPlay:[<Mode>:][WINDow:]TRACe[<Trace>]:Y[:SCALe]:RLEVel is no longer
/// HIPAR attenuation/applied. Note: The query returns the actual attenuation setting, which
/// HIPAR attenuation/can also be changed by INPut:<Direction>:LEVel:AUTO.
ViStatus rsdsa_DUAnalysisConfigureUpstreamAttenuation(ViSession instrSession,
                                                      ViReal64  attenuation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_UPSTREAM_ATTENUATION, attenuation),
            2, "Attenuation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the preamplifier on and off. Note: The query returns the actual
/// HIFN  preamplifier state (which might also be changed by
/// HIFN  RSDSA_ATTR_D31U_AUTO_LEVEL).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Turns the preamplifier on and off. Note: The query returns the actual
/// HIPAR enabled/preamplifier state (which might also be changed by
/// HIPAR enabled/RSDSA_ATTR_D31U_AUTO_LEVEL).
ViStatus rsdsa_DUAnalysisConfigureUpstreamPreamplifierEnabled(
                            ViSession instrSession,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_UPSTREAM_PREAMPLIFIER_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the SFP+ network output on and off.NOUTput
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Turns the SFP+ network output on and off.
ViStatus rsdsa_DUAnalysisConfigureSFPOutputEnabled(ViSession instrSession,
                                                   ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_SFP_OUTPUT_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables and configures the UDP port to be used for SFP+ IUC output.
/// HIFN  The UDP ports must be uniquely assigned to downstream OFDM profiles/PLC
/// HIFN  and upstream DOCSIS IUCs.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR iuc/Selects repcap IUC_OFDMA.
/// HIPAR udpPortEnabled/Turns the SFP+ IUC UDP port on and off.
/// HIPAR udpPort/Configures the UDP port to be used for SFP+ IUC output. The UDP ports
/// HIPAR udpPort/must be uniquely assigned to downstream OFDM profiles/PLC and upstream
/// HIPAR udpPort/DOCSIS IUCs.
ViStatus rsdsa_DUAnalysisConfigureSFPIUCUDPPort(ViSession instrSession,
                                                ViInt32   iuc,
                                                ViBoolean udpPortEnabled,
                                                ViInt32   udpPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxIUC = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "IUC_OFDMA", &maxIUC),
        2, "IUC");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, iuc, 1, maxIUC),
            2, "IUC");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_SFP_IUC_UDP_PORT_ENABLED,
            "IUC_OFDMA", iuc - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "IUC");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, repCap, RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_SFP_IUC_UDP_PORT_ENABLED, udpPortEnabled),
            3, "UDP Port Enabled");
    if (udpPortEnabled)
    {
        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, repCap, RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_SFP_IUC_UDP_PORT, udpPort),
                4, "UDP Port");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the alarm threshold for RMS MER overall. When this parameter
/// HIFN  falls below the set threshold, an alarm is generated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rmsMEROverall/Sets the alarm threshold for RMS MER overall. When this parameter
/// HIPAR rmsMEROverall/falls below the set threshold, an alarm is generated.
ViStatus rsdsa_DUAnalysisConfigureAlarmThresholdRMSMEROverall(
                            ViSession instrSession,
                            ViReal64  rmsMEROverall)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_ALARM_THRESHOLD_RMS_MER_OVERALL, rmsMEROverall),
            2, "RMS MER Overall");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the alarm thresholds for pre- and post-Reed-Solomon BER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR berPreRS/Sets the alarm threshold for pre-Reed-Solomon BER. When this parameter
/// HIPAR berPreRS/exceeds the set threshold, an alarm is generated.
/// HIPAR berPostRS/Sets the alarm threshold for post-Reed-Solomon BER. When this
/// HIPAR berPostRS/parameter exceeds the set threshold, an alarm is generated.
ViStatus rsdsa_DUAnalysisConfigureAlarmThresholdsBERReedSolomon(
                            ViSession instrSession,
                            ViReal64  berPreRS,
                            ViReal64  berPostRS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_ALARM_THRESHOLD_BER_PRE_REED_SOLOMON, berPreRS),
            2, "BER Pre-RS");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_ALARM_THRESHOLD_BER_POST_REED_SOLOMON, berPostRS),
            3, "BER Post-RS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the alarm thresholds for pre- and post-LDPC BER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR berPreLDPC/Sets the alarm threshold for pre-LDPC BER. When this parameter exceeds
/// HIPAR berPreLDPC/the set threshold, an alarm is generated.
/// HIPAR berPostLDPC/Sets the alarm threshold for post-LDPC BER. When this parameter
/// HIPAR berPostLDPC/exceeds the set threshold, an alarm is generated.
ViStatus rsdsa_DUAnalysisConfigureAlarmThresholdsBERLDPC(ViSession instrSession,
                                                         ViReal64  berPreLDPC,
                                                         ViReal64  berPostLDPC)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_ALARM_THRESHOLD_BER_PRE_LDPC, berPreLDPC),
            2, "BER Pre-LDPC");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_DU_ANALYSIS_ALARM_THRESHOLD_BER_POST_LDPC, berPostLDPC),
            3, "BER Post-LDPC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the dynamic SFD analysis mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_EnableDynamicSFDAnalysis(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_ENABLE_DYNAMIC_SFD_ANALYSIS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the burst demodulator lock status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstDemodulatorLockStatus/Queries the burst demodulator lock status.
ViStatus rsdsa_SFDAnalysisQueryBurstDemodulatorLockStatus(ViSession instrSession,
                                                          ViInt32*  burstDemodulatorLockStatus)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_BURST_DEMODULATOR_LOCK_STATUS, burstDemodulatorLockStatus),
            2, "Burst Demodulator Lock Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the forward error correction (FEC) lock status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fecLockStatus/Queries the forward error correction (FEC) lock status.
ViStatus rsdsa_SFDAnalysisQueryFECLockStatus(ViSession instrSession,
                                             ViInt32*  fecLockStatus)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_FEC_LOCK_STATUS, fecLockStatus),
            2, "FEC Lock Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the burst receive power level in the units selected by
/// HIFN  RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstReceivePower/Queries the burst receive power level in the units selected by
/// HIPAR burstReceivePower/RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_SFDAnalysisQueryBurstReceivePower(ViSession instrSession,
                                                 ViReal64* burstReceivePower)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_BURST_RECEIVE_POWER, burstReceivePower),
            2, "Burst Receiver Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the frequency offset in Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequencyOffset/Queries the frequency offset in Hz.
ViStatus rsdsa_SFDAnalysisQueryFrequencyOffset(ViSession instrSession,
                                               ViReal64* frequencyOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_FREQUENCY_OFFSET, frequencyOffset),
            2, "Frequency Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the burst timing offset from the time the burst was detected
/// HIFN  and the time the burst was supposed to occur, in seconds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstTimingOffset/Queries the burst timing offset from the time the burst was detected
/// HIPAR burstTimingOffset/and the time the burst was supposed to occur, in seconds.
ViStatus rsdsa_SFDAnalysisQueryBurstTimingOffset(ViSession instrSession,
                                                 ViReal64* burstTimingOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_BURST_TIMING_OFFSET, burstTimingOffset),
            2, "Burst Timing Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of minislots received.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR minislotsReceived/Queries the number of minislots received.
ViStatus rsdsa_SFDAnalysisQueryMinislotsReceived(ViSession instrSession,
                                                 ViInt64*  minislotsReceived)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt64(instrSession, NULL,
        RSDSA_ATTR_SFD_ANALYSIS_QUERY_MINISLOTS_RECEIVED,
        minislotsReceived), 2, "Minislot Received");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the percentage of valid bursts out of all bursts received over
/// HIFN  the last one second period.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstCoverage/Queries the percentage of valid bursts out of all bursts received over
/// HIPAR burstCoverage/the last one second period.
ViStatus rsdsa_SFDAnalysisQueryBurstCoverage(ViSession instrSession,
                                             ViReal64* burstCoverage)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_BURST_COVERAGE, burstCoverage),
            2, "Burst Coverage");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the SFD pilot status. If true, the SFD proprietary pilot is
/// HIFN  acquired.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sfdPilotAcquired/Queries the SFD pilot status. If true, the SFD proprietary pilot is
/// HIPAR sfdPilotAcquired/acquired.
ViStatus rsdsa_SFDAnalysisQuerySFDPilotAcquired(ViSession instrSession,
                                                ViInt32*  sfdPilotAcquired)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_SFD_PILOT_ACQUIRED, sfdPilotAcquired),
            2, "SFD Pilot Acquired");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the sign preferred for MER numerical display and plot axis
/// HIFN  labels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merSign/Selects the sign preferred for MER numerical display and plot axis
/// HIPAR merSign/labels.
ViStatus rsdsa_SFDAnalysisConfigureMERSign(ViSession instrSession,
                                           ViInt32   merSign)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_MER_SIGN, merSign),
            2, "MER Sign");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall RMS equalized MER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merRMS/Queries the overall RMS equalized MER.
ViStatus rsdsa_SFDAnalysisQueryMERRMS(ViSession instrSession,
                                      ViReal64* merRMS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_MER_RMS, merRMS),
            2, "MER RMS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall RMS partially equalized MER. For OFDMA mode, it is
/// HIFN  only valid for data bursts with grant sizes greater than 100 using
/// HIFN  pilots 4 or 11.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merRMSPartialyEqualized/Queries the overall RMS partially equalized MER. For OFDMA mode, it is
/// HIPAR merRMSPartialyEqualized/only valid for data bursts with grant sizes greater than 100 using
/// HIPAR merRMSPartialyEqualized/pilots 4 or 11.
ViStatus rsdsa_SFDAnalysisQueryMERRMSPartialyEqualized(ViSession instrSession,
                                                       ViReal64* merRMSPartialyEqualized)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_MER_RMS_PARTIALY_EQUALIZED, merRMSPartialyEqualized),
            2, "MER RMS Partialy Equalized");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall peak (lowest) MER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merPeak/Queries the overall peak (lowest) MER.
ViStatus rsdsa_SFDAnalysisQueryMERPeak(ViSession instrSession,
                                       ViReal64* merPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_MER_PEAK, merPeak),
            2, "MER Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the RMS MER for the pilot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merPilotRMS/Queries the RMS MER for the pilot.
ViStatus rsdsa_SFDAnalysisQueryMERPilotRMS(ViSession instrSession,
                                           ViReal64* merPilotRMS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_MER_PILOT_RMS, merPilotRMS),
            2, "MER Pilot RMS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the peak (lowest) MER for the pilot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merPilotPeak/Queries the peak (lowest) MER for the pilot.
ViStatus rsdsa_SFDAnalysisQueryMERPilotPeak(ViSession instrSession,
                                            ViReal64* merPilotPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_MER_PILOT_PEAK, merPilotPeak),
            2, "MER Pilot Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the constellations for the pilot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR constellations/Queries the constellations for the pilot.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_SFDAnalysisQueryPilotConstellations(ViSession instrSession,
                                                   ViInt32   bufferSize,
                                                   ViChar    constellations[],
                                                   ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_PILOT_CONSTELLATIONS, RS_VAL_DIRECT_USER_CALL, bufferSize, constellations, bytesRead),
        3, "NCP Constellations");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the RMS MER for the complementary pilot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merComplementaryPilotRMS/Queries the RMS MER for the complementary pilot.
ViStatus rsdsa_SFDAnalysisQueryMERComplementaryPilotRMS(ViSession instrSession,
                                                        ViReal64* merComplementaryPilotRMS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_MER_COMPLEMENTARY_PILOT_RMS, merComplementaryPilotRMS),
            2, "MER Complementary Pilot RMS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the peak (lowest) MER for the complementary pilot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merComplementaryPilotPeak/Queries the peak (lowest) MER for the complementary pilot.
ViStatus rsdsa_SFDAnalysisQueryMERComplementaryPilotPeak(ViSession instrSession,
                                                         ViReal64* merComplementaryPilotPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_MER_COMPLEMENTARY_PILOT_PEAK, merComplementaryPilotPeak),
            2, "MER Complementary Pilot Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the constellations for the complementary pilot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR constellations/Queries the constellations for the complementary pilot.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_SFDAnalysisQueryComplementaryPilotConstellations(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    constellations[],
                            ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_COMPLEMENTARY_PILOT_CONSTELLATIONS, RS_VAL_DIRECT_USER_CALL, bufferSize, constellations, bytesRead),
        3, "NCP Constellations");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the I/Q amplitude imbalance in dB.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR amplitudedB/Queries the I/Q amplitude imbalance in dB.
ViStatus rsdsa_SFDAnalysisQueryIQImbalanceAmplitudedB(ViSession instrSession,
                                                      ViReal64* amplitudedB)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_IQ_IMBALANCE_AMPLITUDE_DB, amplitudedB),
            2, "Amplitude dB");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the I/Q amplitude imbalance in percent.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR amplitudePercent/Queries the I/Q amplitude imbalance in percent.
ViStatus rsdsa_SFDAnalysisQueryIQImbalanceAmplitudePercent(ViSession instrSession,
                                                           ViReal64* amplitudePercent)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_IQ_IMBALANCE_AMPLITUDE_PERCENT, amplitudePercent),
            2, "Amplitude Percent");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the I/Q imbalance quadrature error in degrees.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR quadratureError/Queries the I/Q imbalance quadrature error in degrees.
ViStatus rsdsa_SFDAnalysisQueryIQImbalanceQuadratureError(ViSession instrSession,
                                                          ViReal64* quadratureError)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_IQ_IMBALANCE_QUADRATURE_ERROR, quadratureError),
            2, "Quadrature Error");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the settings for error analysis. Changing error analysis
/// HIFN  settings will reset the error analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR errorAnalysisMode/Specifies the mode of BER measurement.
/// HIPAR manualTestDepth/Defines the test depth of the minimum BER integration. The query
/// HIPAR manualTestDepth/returns the actual test depth setting, which may be automatically
/// HIPAR manualTestDepth/changed in automatic BER measurement mode.
/// HIPAR testDepthSource/Selects the BER source used for BER measurements.
/// HIPAR testDepthProgressionSource/Selects the BER source used for test depth progression.
ViStatus rsdsa_SFDAnalysisConfigureErrorAnalysis(ViSession instrSession,
                                                 ViInt32   errorAnalysisMode,
                                                 ViInt32   manualTestDepth,
                                                 ViInt32   testDepthSource,
                                                 ViInt32   testDepthProgressionSource)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_ERROR_ANALYSIS_MODE, errorAnalysisMode),
            2, "Error Analysis Mode");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH, manualTestDepth),
            3, "Manual Test Depth");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH_SOURCE, testDepthSource),
            4, "Test Depth Source");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH_PROGRESSION_SOURCE, testDepthProgressionSource),
            5, "Test Depth Progression Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Performs a BER reset that clears all related counters to zero and
/// HIFN  restarts the BER/CER measurement for the current mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_SFDAnalysisResetErrorAnalysis(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_RESET_ERROR_ANALYSIS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the elapsed time of the current BER/CER measurement: time
/// HIFN  elapsed since the start of the mode, or last BER reset
/// HIFN  (RSDSA_ATTR_D31U_RESET_ERROR_ANALYSIS), or last BER configuration
/// HIFN  change, in seconds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR elapsedTime/Queries the elapsed time of the current BER/CER measurement: time
/// HIPAR elapsedTime/elapsed since the start of the mode, or last BER reset
/// HIPAR elapsedTime/(RSDSA_ATTR_D31U_RESET_ERROR_ANALYSIS), or last BER configuration
/// HIPAR elapsedTime/change, in seconds.
ViStatus rsdsa_SFDAnalysisQueryBERElapsedTime(ViSession instrSession,
                                              ViReal64* elapsedTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_BER_ELAPSED_TIME, elapsedTime),
            2, "Elapsed Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the percentage of blocks processed for the current BER/CER
/// HIFN  measurement: blocks processed since the start of the mode, or last BER
/// HIFN  reset (CALCulate:[<Mode>:][AVERage:]BERatio:CLEar), or last BER
/// HIFN  configuration change. Expressed as the percentage of blocks processed
/// HIFN  relative to the current test depth. A block is 10^7 bits maximum.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR blocksProcessed/Queries the percentage of blocks processed for the current BER/CER
/// HIPAR blocksProcessed/measurement: blocks processed since the start of the mode, or last BER
/// HIPAR blocksProcessed/reset (CALCulate:[<Mode>:][AVERage:]BERatio:CLEar), or last BER
/// HIPAR blocksProcessed/configuration change. Expressed as the percentage of blocks processed
/// HIPAR blocksProcessed/relative to the current test depth. A block is 10^7 bits maximum.
ViStatus rsdsa_SFDAnalysisQueryBERBlocksProcessed(ViSession instrSession,
                                                  ViReal64* blocksProcessed)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_BER_BLOCKS_PROCESSED, blocksProcessed),
            2, "Blocks Processed");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of bursts expected since the signal was acquired.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstsExpected/Queries the number of bursts expected since the signal was acquired.
ViStatus rsdsa_SFDAnalysisQueryBERBurstsExpected(ViSession instrSession,
                                                 ViReal64* burstsExpected)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_BER_BURSTS_EXPECTED, burstsExpected),
            2, "Bursts Expected");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of bursts received since the signal was acquired.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstsReceived/Queries the number of bursts received since the signal was acquired.
ViStatus rsdsa_SFDAnalysisQueryBERBurstsReceived(ViSession instrSession,
                                                 ViReal64* burstsReceived)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_BER_BURSTS_RECEIVED, burstsReceived),
            2, "Bursts Received");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the pre-Reed-Solomon BER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR preRS/Queries the pre-Reed-Solomon BER.
ViStatus rsdsa_SFDAnalysisQueryBERPreReedSolomon(ViSession instrSession,
                                                 ViReal64* preRS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_BER_PRE_REED_SOLOMON, preRS),
            2, "PRE RS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of bits analyzed at the input of the
/// HIFN  Reed-Solomon decoder for the BER measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR preRSTotalBits/Queries the total number of bits analyzed at the input of the
/// HIPAR preRSTotalBits/Reed-Solomon decoder for the BER measurement.
ViStatus rsdsa_SFDAnalysisQueryBERPreReedSolomonTotalBits(ViSession instrSession,
                                                          ViReal64* preRSTotalBits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_BER_PRE_REED_SOLOMON_TOTAL_BITS, preRSTotalBits),
            2, "Pre RS Total bits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the post-Reed-Solomon BER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR postRS/Queries the post-Reed-Solomon BER.
ViStatus rsdsa_SFDAnalysisQueryBERPostReedSolomon(ViSession instrSession,
                                                  ViReal64* postRS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_BER_POST_REED_SOLOMON, postRS),
            2, "Post RS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of corrected bits at the output of the
/// HIFN  Reed-Solomon decoder for the BER measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR postRSCorrectedBits/Queries the total number of corrected bits at the output of the
/// HIPAR postRSCorrectedBits/Reed-Solomon decoder for the BER measurement.
ViStatus rsdsa_SFDAnalysisQueryBERPostReedSolomonCorrectedBits(
                            ViSession instrSession,
                            ViReal64* postRSCorrectedBits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_BER_POST_REED_SOLOMON_CORRECTED_BITS, postRSCorrectedBits),
            2, "Post RS Corrected Bits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of uncorrected bits at the output of the
/// HIFN  Reed-Solomon decoder for the BER measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR postRSUncorrectedBits/Queries the total number of uncorrected bits at the output of the
/// HIPAR postRSUncorrectedBits/Reed-Solomon decoder for the BER measurement.
ViStatus rsdsa_SFDAnalysisQueryBERPostReedSolomonUncorrectedBits(
                            ViSession instrSession,
                            ViReal64* postRSUncorrectedBits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_BER_POST_REED_SOLOMON_UNCORRECTED_BITS, postRSUncorrectedBits),
            2, "Post RS Uncorrected Bits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the codeword length(s) to be used for displayed status (i.e.,
/// HIFN  codeword length filter).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR codewordLengthFilter/Selects the codeword length(s) to be used for displayed status (i.e.,
/// HIPAR codewordLengthFilter/codeword length filter).
ViStatus rsdsa_SFDAnalysisConfigureCodewordLengthFilter(ViSession instrSession,
                                                        ViInt32   codewordLengthFilter)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_CODEWORD_LENGTH_FILTER, codewordLengthFilter),
            2, "Codeword Length Filter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the corrected CER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR corrected/Queries the corrected CER.
ViStatus rsdsa_SFDAnalysisQueryCERCorrected(ViSession instrSession,
                                            ViReal64* corrected)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_CER_CORRECTED, corrected),
            2, "Corrected");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the uncorrected CER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR uncorrected/Queries the uncorrected CER.
ViStatus rsdsa_SFDAnalysisQueryCERUncorrected(ViSession instrSession,
                                              ViReal64* uncorrected)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_CER_UNCORRECTED, uncorrected),
            2, "Uncorrected");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of codewords for the CER measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR totalCodewords/Queries the total number of codewords for the CER measurement.
ViStatus rsdsa_SFDAnalysisQueryCERTotalCodewords(ViSession instrSession,
                                                 ViReal64* totalCodewords)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_CER_TOTAL_CODEWORDS, totalCodewords),
            2, "Total Codewords");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of corrected CER codewords.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR correctedCodewords/Queries the total number of corrected CER codewords.
ViStatus rsdsa_SFDAnalysisQueryCERCorrectedCodewords(ViSession instrSession,
                                                     ViReal64* correctedCodewords)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_CER_CORRECTED_CODEWORDS, correctedCodewords),
            2, "Corrected Codewords");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of uncorrected CER codewords.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR uncorrectedCodewords/Queries the total number of uncorrected CER codewords.
ViStatus rsdsa_SFDAnalysisQueryCERUncorrectedCodewords(ViSession instrSession,
                                                       ViReal64* uncorrectedCodewords)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_CER_UNCORRECTED_CODEWORDS, uncorrectedCodewords),
            2, "Uncorrected Codewords");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the post-LDPC CER, for the codeword length currently selected
/// HIFN  by DISPlay:[<Mode>:][WINDow:]CLFilter <CwLength>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR postLDPC/Queries the post-LDPC CER, for the codeword length currently selected
/// HIPAR postLDPC/by DISPlay:[<Mode>:][WINDow:]CLFilter <CwLength>.
ViStatus rsdsa_SFDAnalysisQueryCERPostLDPC(ViSession instrSession,
                                           ViReal64* postLDPC)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_CER_POST_LDPC, postLDPC),
            2, "Post LDPC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of post-LDPC codeword errors, for the
/// HIFN  codeword length currently selected by
/// HIFN  DISPlay:[<Mode>:][WINDow:]CLFilter <CwLength>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR postLDPCCodewordErrors/Queries the total number of post-LDPC codeword errors, for the
/// HIPAR postLDPCCodewordErrors/codeword length currently selected by
/// HIPAR postLDPCCodewordErrors/DISPlay:[<Mode>:][WINDow:]CLFilter <CwLength>.
ViStatus rsdsa_SFDAnalysisQueryCERPostLDPCCodewordErrors(ViSession instrSession,
                                                         ViReal64* postLDPCCodewordErrors)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_CER_POST_LDPC_CODEWORD_ERRORS, postLDPCCodewordErrors),
            2, "Post LDPC Codeword Errors");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the average number of LDPC iterations, for the codeword length
/// HIFN  currently selected by DISPlay:[<Mode>:][WINDow:]CLFilter <CwLength>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR averageLDPCIterations/Queries the average number of LDPC iterations, for the codeword length
/// HIPAR averageLDPCIterations/currently selected by DISPlay:[<Mode>:][WINDow:]CLFilter <CwLength>.
ViStatus rsdsa_SFDAnalysisQueryCERAverageLDPCIterations(ViSession instrSession,
                                                        ViReal64* averageLDPCIterations)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_CER_AVERAGE_LDPC_ITERATIONS, averageLDPCIterations),
            2, "Average LDPC Iterations");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the constellation plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_SFDAnalysisDisplayConstellationPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_DISPLAY_CONSTELLATION_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the I/Q density plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_SFDAnalysisDisplayIQDensityPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_DISPLAY_I_Q_DENSITY_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the measurement mode for the I/Q density plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementMode/Selects the measurement mode for the I/Q density plot.
/// HIPAR windowWidth/Specifies the sliding window width in samples for the I/Q density plot
/// HIPAR windowWidth/in sliding window mode.
ViStatus rsdsa_SFDAnalysisConfigureIQMeasurementMode(ViSession instrSession,
                                                     ViInt32   measurementMode,
                                                     ViInt32   windowWidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_I_Q_MEASUREMENT_MODE, measurementMode),
            2, "Measurement Mode");
    if (measurementMode == RSDSA_VAL_IQ_MEASUREMENT_MODE_SLIDING_WINDOW)
    {
        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_I_Q_WINDOW_WIDTH, windowWidth),
                3, "Window Width");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the zoom setting for the I/Q density plot display. Note that
/// HIFN  the x and y origin are the top left corner of the window, as per the
/// HIFN  GUI settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR x/X origin (left-most window point), between 0 and 1.
/// HIPAR y/Y origin (top-most window point), between 0 and 1.
/// HIPAR zoom/Zoom setting, 1 to 128.
ViStatus rsdsa_SFDAnalysisSetIQDisplayZoom(ViSession instrSession,
                                           ViReal64  x,
                                           ViReal64  y,
                                           ViInt32   zoom)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_CheckRangeTableRangeViReal64(instrSession, &RsDSA_rngIQZoomOrigin, x),
        2, "X");
    viCheckParm(rsdsa_CheckRangeTableRangeViReal64(instrSession, &RsDSA_rngIQZoomOrigin, y),
        3, "Y");
    viCheckParm(rsdsa_CheckRangeTableRangeViInt32(instrSession, &RsDSA_rngIQZoomLevel, zoom),
        4, "Zoom");

    snprintf(param, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%f,%f,%ld", x, y, zoom);

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_HIDDEN_SFD_ANALYSIS_DISPLAY_I_Q_ZOOM, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Save the constellation I/Q data to the specified file. The file
/// HIFN  extension is .iq.tar. Note that there is no way to abort this command,
/// HIFN  and large file sizes could take a long time to capture depending on the
/// HIFN  input data, so use maxSamples or timeoutSeconds to limit the file size
/// HIFN  or time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/File path, uses .iq.tar extension.
/// HIPAR maxSamples/Sets the I/Q data file maximum size in samples.
/// HIPAR timeoutSeconds/Sets the maximum time in seconds to save data.
ViStatus rsdsa_SFDAnalysisCaptureIQDataToFile(ViSession instrSession,
                                              ViString  fileName,
                                              ViInt64   maxSamples,
                                              ViReal64  timeoutSeconds)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt64(instrSession, NULL,
        RSDSA_ATTR_SFD_ANALYSIS_IQ_DATA_EXPORT_MAX_SAMPLES,
        maxSamples),
        3, "Max Samples");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_IQ_DATA_EXPORT_TIMEOUT, timeoutSeconds),
            4, "Timeout Seconds");

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_CAPTURE_IQ_DATA_TO_FILENAME, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the MER vs. time plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_SFDAnalysisDisplayMERVsTimePlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_DISPLAY_MER_VS_TIME_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a time filter period to be used for displayed MER vs. Time
/// HIFN  plots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR period/Selects a time filter period to be used for displayed MER vs. Time
/// HIPAR period/plots.
ViStatus rsdsa_SFDAnalysisConfigureMERVsTimePlotFilterPeriod(ViSession instrSession,
                                                             ViInt32   period)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_MER_VS_TIME_PLOT_FILTER_PERIOD, period),
            2, "Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the BER vs. time plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_SFDAnalysisDisplayBERVsTimePlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_DISPLAY_BER_VS_TIME_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a time filter period to be used for displayed BER vs. Time
/// HIFN  plots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR period/Selects a time filter period to be used for displayed BER vs. Time
/// HIPAR period/plots.
ViStatus rsdsa_SFDAnalysisConfigureBERVsTimePlotFilterPeriod(ViSession instrSession,
                                                             ViInt32   period)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_BER_VS_TIME_PLOT_FILTER_PERIOD, period),
            2, "Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the POWer vs. time plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_SFDAnalysisDisplayPowerVsTimePlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_DISPLAY_POWER_VS_TIME_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a time filter period to be used for displayed Power vs. Time
/// HIFN  plots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR period/Selects a time filter period to be used for displayed Power vs. Time
/// HIPAR period/plots.
ViStatus rsdsa_SFDAnalysisConfigurePowerVsTimePlotFilterPeriod(
                            ViSession instrSession,
                            ViInt32   period)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_POWER_VS_TIME_PLOT_FILTER_PERIOD, period),
            2, "Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the MER vs. Subcarrier plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_SFDAnalysisDisplayMERVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_DISPLAY_MER_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the MER vs. minislot plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_SFDAnalysisDisplayMERVsMinislotPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_DISPLAY_MER_VS_MINISLOT_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays a downstream or upstream CCDF (Complementary Cumulative
/// HIFN  Distribution Function histogram as per CM-SP-PHYv3.1-I16 sections 9.3.8
/// HIFN  or 9.4.6 respectively). This command may take up to 30 seconds to
/// HIFN  execute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_SFDAnalysisDisplayCCDFHistogram(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_DISPLAY_CCDF_HISTOGRAM, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the peak CCDF power level in the units selected by
/// HIFN  RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ccdfPeak/Queries the peak CCDF power level in the units selected by
/// HIPAR ccdfPeak/RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_SFDAnalysisQueryCCDFPeak(ViSession instrSession,
                                        ViReal64* ccdfPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_CCDF_PEAK, ccdfPeak),
            2, "CCDF Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the channel analysis amplitude/phase vs. subcarrier plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_SFDAnalysisDisplayAmplitudePhaseVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_DISPLAY_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Scales the y-axis for the channel analysis amplitude/phase vs.
/// HIFN  subcarrier plot to best fit the measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_SFDAnalysisAutoScaleAmplitudePhaseVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_AUTO_SCALE_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the channel analysis amplitude/group delay vs. subcarrier
/// HIFN  plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_SFDAnalysisDisplayAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_DISPLAY_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Scales the y-axis for the channel analysis amplitude/group delay vs.
/// HIFN  subcarrier plot to best fit the measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_SFDAnalysisAutoScaleAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_AUTO_SCALE_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the channel analysis echo pattern plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_SFDAnalysisDisplayEchoPatternPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_DISPLAY_ECHO_PATTERN_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the spectrum analysis plot for DOCSIS 3.1 Downstream mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_SFDAnalysisDisplaySpectrumAnalysisPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_DISPLAY_SPECTRUM_ANALYSIS_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns off all markers including delta markers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_SFDAnalysisAllMarkersOff(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_ALL_MARKERS_OFF, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns off all delta markers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_SFDAnalysisAllDeltaMarkersOff(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_ALL_DELTA_MARKERS_OFF, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Clears all masks, including both the lower and upper masks.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_SFDAnalysisClearAllMasks(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_CLEAR_ALL_MASKS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns all masks on or off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Turns all masks on or off.
ViStatus rsdsa_SFDAnalysisConfigureDisplayAllMasksEnabled(ViSession instrSession,
                                                          ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DISPLAY_ALL_MASKS_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures amplitude settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR referenceLevel/Defines the reference level for the displayed trace in the units
/// HIPAR referenceLevel/selected by RSDSA_ATTR_POWER_UNITS. This setting is only used when
/// HIPAR referenceLevel/automatic y-axis scaling is turned off. If the reference level is set,
/// HIPAR referenceLevel/then the attenuation set in INPut:ATTenuation is no longer applied.
/// HIPAR scalePerDivision/Defines the scale per division in the units selected by
/// HIPAR scalePerDivision/RSDSA_ATTR_POWER_UNITS to be used for the y-axis for all trace
/// HIPAR scalePerDivision/displays, which all have 10 divisions on the y-axis. Note that this
/// HIPAR scalePerDivision/command sets the scale per division, whereas the user of the DSA GUI
/// HIPAR scalePerDivision/sets the entire range for the vertical axis.
ViStatus rsdsa_SFDAnalysisConfigureAmplitudeSettings(ViSession instrSession,
                                                     ViInt32   referenceLevel,
                                                     ViReal64  scalePerDivision)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_REFERENCE_LEVEL, referenceLevel),
            2, "Reference Level");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_SCALE_PER_DIVISION, scalePerDivision),
            3, "Scale Per Division");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Scales the y-axis for trace displays by automatically adjusting the
/// HIFN  y-axis reference level and scale per division to best fit the
/// HIFN  measurement results. Otherwise, the y-axis can be manually scaled using
/// HIFN  RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_REFERENCE_LEVEL and
/// HIFN  RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_SCALE_PER_DIVISION.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_SFDAnalysisAutoScaleYAxis(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_AUTO_SCALE_Y_AXIS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures bandwidth settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoAverageCount/When turned on, the DSA automatically determines the average setting.
/// HIPAR manualAverageCount/Defines the number of averages. A value of 1 indicates no averaging.
/// HIPAR manualAverageCount/Note: For OFDM spectrum analysis, setting an average that is not a
/// HIPAR manualAverageCount/multiple of 128 may cause artifacts between pilots.
ViStatus rsdsa_SFDAnalysisConfigureBandwidthSettings(ViSession instrSession,
                                                     ViBoolean autoAverageCount,
                                                     ViInt32   manualAverageCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_AUTO_AVERAGE_COUNT, autoAverageCount),
            2, "Auto Average Count");

    if (autoAverageCount == VI_FALSE)
    {
        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_MANUAL_AVERAGE_COUNT, manualAverageCount),
                3, "Manual Average Count");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures measurement settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR captureMode/Selects whether to use continuous captures or single trace captures.
/// HIPAR traceHold/Selects the measurement mode for the displayed trace. The DSA applies
/// HIPAR traceHold/minimum/maximum hold over multiple single traces initiated by
/// HIPAR traceHold/DISPlay:[<Mode>:][WINDow:]TRACe[<Trace>]:RSINgle command.
ViStatus rsdsa_SFDAnalysisConfigureMeasurementsSettings(ViSession instrSession,
                                                        ViInt32   captureMode,
                                                        ViInt32   traceHold)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_CAPTURE_MODE, captureMode),
            2, "Capture mode");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_TRACE_MODE, traceHold),
            3, "Trace Hold");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restarts a single sweep, rearming for the next trigger, if applicable.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_SFDAnalysisRunSingleSweep(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_RUN_SINGLE_SWEEP, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns individual markers on and off. If the specified marker is a
/// HIFN  delta marker, it is turned into a normal marker. The default position
/// HIFN  is at the maximum peak.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
/// HIPAR state/Turns individual markers on and off. If the specified marker is a
/// HIPAR state/delta marker, it is turned into a normal marker. The default position
/// HIPAR state/is at the maximum peak.
ViStatus rsdsa_SFDAnalysisConfigureMarkerState(ViSession instrSession,
                                               ViInt32   marker,
                                               ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_STATE,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, repCap, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_STATE, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a marker to a particular coordinate on the x-axis. If the
/// HIFN  specified marker is turned off, it is turned on first. If the specified
/// HIFN  marker is a delta marker, it is turned into a normal marker.
/// HIFN
/// HIFN     The units are Hz (frequency domain) or s (time domain, for zero span
/// HIFN  mode).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
/// HIPAR xPosition/Moves a marker to a particular coordinate on the x-axis. If the
/// HIPAR xPosition/specified marker is turned off, it is turned on first. If the specified
/// HIPAR xPosition/marker is a delta marker, it is turned into a normal marker.
ViStatus rsdsa_SFDAnalysisConfigureMarkerXPosition(ViSession instrSession,
                                                   ViInt32   marker,
                                                   ViReal64  xPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_X_POSITION,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_X_POSITION, xPosition),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the position of a marker on the y-axis. If the specified
/// HIFN  marker is turned off, it is turned on first. If the specified marker is
/// HIFN  a delta marker, it is turned into a normal marker.
/// HIFN
/// HIFN     The units are determined by RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
/// HIPAR yPosition/Queries the position of a marker on the y-axis. If the specified
/// HIPAR yPosition/marker is turned off, it is turned on first. If the specified marker is
/// HIPAR yPosition/a delta marker, it is turned into a normal marker.
ViStatus rsdsa_SFDAnalysisQueryMarkerYPosition(ViSession instrSession,
                                               ViInt32   marker,
                                               ViReal64* yPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_Y_POSITION,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_Y_POSITION, yPosition),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a marker to the highest amplitude displayed. If the specified
/// HIFN  marker is turned off, it is turned on first. If the specified marker is
/// HIFN  a delta marker, it is turned into a normal marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_SFDAnalysisMarkerToMaximumPeak(ViSession instrSession,
                                              ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_TO_MAXIMUM_PEAK,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_TO_MAXIMUM_PEAK, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a marker to the next peak to the left of the current marker
/// HIFN  position. If the specified marker is turned off, it is turned on first.
/// HIFN  If the specified marker is a delta marker, it is turned into a normal
/// HIFN  marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_SFDAnalysisMarkerToNextPeakLeft(ViSession instrSession,
                                               ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_LEFT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_LEFT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a marker to the next peak to the right of the current marker
/// HIFN  position. If the specified marker is turned off, it is turned on first.
/// HIFN  If the specified marker is a delta marker, it is turned into a normal
/// HIFN  marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_SFDAnalysisMarkerToNextPeakRight(ViSession instrSession,
                                                ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_RIGHT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_RIGHT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a marker to the next displayed point to the left of the current
/// HIFN  marker position. If the specified marker is turned off, it is turned on
/// HIFN  first. If the specified marker is a delta marker, it is turned into a
/// HIFN  normal marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_SFDAnalysisMarkerToNextPointLeft(ViSession instrSession,
                                                ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_LEFT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_LEFT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a marker to the next displayed point to the right of the current
/// HIFN  marker position. If the specified marker is turned off, it is turned on
/// HIFN  first. If the specified marker is a delta marker, it is turned into a
/// HIFN  normal marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_SFDAnalysisMarkerToNextPointRight(ViSession instrSession,
                                                 ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_RIGHT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_RIGHT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the reference level to the power level of a marker. If the
/// HIFN  specified marker is turned off, it is turned on first. If the specified
/// HIFN  marker is a delta marker, it is turned into a normal marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_SFDAnalysisMarkerSetPowerReferenceLevel(ViSession instrSession,
                                                       ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarker),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarker),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_SET_POWER_REFERENCE_LEVEL,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_SET_POWER_REFERENCE_LEVEL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns delta markers on and off. If the specified marker is a normal
/// HIFN  marker, it is turned into a delta marker. The default position is at
/// HIFN  the maximum peak.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deltaMarker/Selects repcap Delta.
/// HIPAR state/Turns delta markers on and off. If the specified marker is a normal
/// HIPAR state/marker, it is turned into a delta marker. The default position is at
/// HIPAR state/the maximum peak.
ViStatus rsdsa_SFDAnalysisConfigureDeltaMarkerState(ViSession instrSession,
                                                    ViInt32   deltaMarker,
                                                    ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaMarker),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deltaMarker, 1, maxDeltaMarker),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_STATE,
            "Delta", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, repCap, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_STATE, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a reference marker for a delta marker. Values of delta marker
/// HIFN  are relative to the reference marker. If the specified reference marker
/// HIFN  is turned off, it is turned on first. If the specified delta marker is
/// HIFN  turned off, it is turned on first. If the specified delta marker is a
/// HIFN  normal marker, it is turned into a delta marker. If the specified
/// HIFN  reference marker is the same as the specified delta marker, the marker
/// HIFN  is changed to a normal marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deltaMarker/Selects repcap Delta.
/// HIPAR linkedReferenceMarker/Selects a reference marker for a delta marker. Values of delta marker
/// HIPAR linkedReferenceMarker/are relative to the reference marker. If the specified reference marker
/// HIPAR linkedReferenceMarker/is turned off, it is turned on first. If the specified delta marker is
/// HIPAR linkedReferenceMarker/turned off, it is turned on first. If the specified delta marker is a
/// HIPAR linkedReferenceMarker/normal marker, it is turned into a delta marker. If the specified
/// HIPAR linkedReferenceMarker/reference marker is the same as the specified delta marker, the marker
/// HIPAR linkedReferenceMarker/is changed to a normal marker.
ViStatus rsdsa_SFDAnalysisConfigureDeltaMarkerLinkedReferenceMarker(
                            ViSession instrSession,
                            ViInt32   deltaMarker,
                            ViInt32   linkedReferenceMarker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaMarker),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deltaMarker, 1, maxDeltaMarker),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_LINKED_REFERENCE_MARKER,
            "Delta", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    if (linkedReferenceMarker == RSDSA_VAL_REFERENCE_MARKER_NORMAL)
        linkedReferenceMarker = deltaMarker;

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, repCap, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_LINKED_REFERENCE_MARKER, linkedReferenceMarker),
            3, "Linked Reference Marker");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to a particular absolute coordinate on the
/// HIFN  x-axis. If the specified marker is turned off, it is turned on first.
/// HIFN  If the specified marker is a normal marker, it is turned into a delta
/// HIFN  marker. The query returns the absolute position of the delta marker.
/// HIFN
/// HIFN     The units are Hz (frequency domain) or s (time domain, for zero span
/// HIFN  mode).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deltaMarker/Selects repcap Delta.
/// HIPAR absoluteXPosition/Moves a delta marker to a particular absolute coordinate on the
/// HIPAR absoluteXPosition/x-axis. If the specified marker is turned off, it is turned on first.
/// HIPAR absoluteXPosition/If the specified marker is a normal marker, it is turned into a delta
/// HIPAR absoluteXPosition/marker. The query returns the absolute position of the delta marker.
ViStatus rsdsa_SFDAnalysisConfigureDeltaMarkerAbsoluteXPosition(
                            ViSession instrSession,
                            ViInt32   deltaMarker,
                            ViReal64  absoluteXPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaMarker),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deltaMarker, 1, maxDeltaMarker),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_ABSOLUTE_X_POSITION,
            "Delta", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_ABSOLUTE_X_POSITION, absoluteXPosition),
            3, "Absolute X Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to a particular relative coordinate on the
/// HIFN  x-axis. If the specified marker is turned off, it is turned on first.
/// HIFN  If the specified marker is a normal marker, it is turned into a delta
/// HIFN  marker. The query returns the relative position of the delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deltaMarker/Selects repcap Delta.
/// HIPAR relativeXPosition/Moves a delta marker to a particular relative coordinate on the
/// HIPAR relativeXPosition/x-axis. If the specified marker is turned off, it is turned on first.
/// HIPAR relativeXPosition/If the specified marker is a normal marker, it is turned into a delta
/// HIPAR relativeXPosition/marker. The query returns the relative position of the delta marker.
ViStatus rsdsa_SFDAnalysisConfigureDeltaMarkerRelativeXPosition(
                            ViSession instrSession,
                            ViInt32   deltaMarker,
                            ViReal64  relativeXPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaMarker),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deltaMarker, 1, maxDeltaMarker),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_RELATIVE_X_POSITION,
            "Delta", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_RELATIVE_X_POSITION, relativeXPosition),
            3, "Relative X Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the absolute position of a delta marker on the y-axis. If the
/// HIFN  specified marker is turned off, it is turned on first. If the specified
/// HIFN  marker is a normal marker, it is turned into a delta marker.
/// HIFN
/// HIFN     The units are determined by RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deltaMarker/Selects repcap Delta.
/// HIPAR yPosition/Queries the absolute position of a delta marker on the y-axis. If the
/// HIPAR yPosition/specified marker is turned off, it is turned on first. If the specified
/// HIPAR yPosition/marker is a normal marker, it is turned into a delta marker.
ViStatus rsdsa_SFDAnalysisQueryDeltaMarkerYPosition(ViSession instrSession,
                                                    ViInt32   deltaMarker,
                                                    ViReal64* yPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaMarker),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deltaMarker, 1, maxDeltaMarker),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_Y_POSITION,
            "Delta", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_Y_POSITION, yPosition),
            3, "Y Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to the highest amplitude displayed. If the
/// HIFN  specified marker is turned off, it is turned on first. If the specified
/// HIFN  marker is a normal marker, it is turned into a delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
ViStatus rsdsa_SFDAnalysisDeltaMarkerToMaximumPeak(ViSession instrSession,
                                                   ViInt32   delta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaMarker),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDeltaMarker),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_MAXIMUM_PEAK,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_MAXIMUM_PEAK, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to the next peak to the left of the current
/// HIFN  marker position. If the specified marker is turned off, it is turned on
/// HIFN  first. If the specified marker is a normal marker, it is turned into a
/// HIFN  delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
ViStatus rsdsa_SFDAnalysisDeltaMarkerToNextPeakLeft(ViSession instrSession,
                                                    ViInt32   delta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaMarker),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDeltaMarker),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_LEFT,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_LEFT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to the next peak to the right of the current
/// HIFN  marker position. If the specified marker is turned off, it is turned on
/// HIFN  first. If the specified marker is a normal marker, it is turned into a
/// HIFN  delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
ViStatus rsdsa_SFDAnalysisDeltaMarkerToNextPeakRight(ViSession instrSession,
                                                     ViInt32   delta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaMarker),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDeltaMarker),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_RIGHT,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_RIGHT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to the next displayed point to the left of the
/// HIFN  current marker position. If the specified marker is turned off, it is
/// HIFN  turned on first. If the specified marker is a normal marker, it is
/// HIFN  turned into a delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
ViStatus rsdsa_SFDAnalysisDeltaMarkerToNextPointLeft(ViSession instrSession,
                                                     ViInt32   delta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaMarker),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDeltaMarker),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_LEFT,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_LEFT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to the next displayed point to the right of the
/// HIFN  current marker position. If the specified marker is turned off, it is
/// HIFN  turned on first. If the specified marker is a normal marker, it is
/// HIFN  turned into a delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
ViStatus rsdsa_SFDAnalysisDeltaMarkerToNextPointRight(ViSession instrSession,
                                                      ViInt32   delta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaMarker),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDeltaMarker),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_RIGHT,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_RIGHT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the reference level to the power level of a delta marker. If the
/// HIFN  specified marker is turned off, it is turned on first. If the specified
/// HIFN  marker is a normal marker, it is turned into a delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
ViStatus rsdsa_SFDAnalysisDeltaMarkerSetPowerReferenceLevel(ViSession instrSession,
                                                            ViInt32   delta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaMarker = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaMarker),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDeltaMarker),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_SET_POWER_REFERENCE_LEVEL,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_SET_POWER_REFERENCE_LEVEL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the bandpower between a specified delta marker and its
/// HIFN  reference marker in the frequency domain. An error is generated if the
/// HIFN  specified marker is turned off, or if the specified marker is a normal
/// HIFN  marker.
/// HIFN
/// HIFN     The units are selected by RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deltaMarker/Selects repcap Delta.
/// HIPAR bandwidthPowerResult/Queries the bandpower between a specified delta marker and its
/// HIPAR bandwidthPowerResult/reference marker in the frequency domain. An error is generated if the
/// HIPAR bandwidthPowerResult/specified marker is turned off, or if the specified marker is a normal
/// HIPAR bandwidthPowerResult/marker.
ViStatus rsdsa_SFDAnalysisQueryDeltaMarkerBandwidthPowerResult(
                            ViSession instrSession,
                            ViInt32   deltaMarker,
                            ViReal64* bandwidthPowerResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDelta = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDelta),
        2, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deltaMarker, 1, maxDelta),
            2, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_BANDWIDTH_POWER_RESULT,
            "Delta", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_BANDWIDTH_POWER_RESULT, bandwidthPowerResult),
            3, "Bandwidth Power Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns individual masks on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mask/Selects repcap Mask.
/// HIPAR visible/Turns individual masks on and off.
ViStatus rsdsa_SFDAnalysisConfigureMaskVisible(ViSession instrSession,
                                               ViInt32   mask,
                                               ViBoolean visible)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMask = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Mask", &maxMask),
        2, "Mask");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mask, 1, maxMask),
            2, "Mask");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_SFD_ANALYSIS_MASK_VISIBLE,
            "Mask", mask - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Mask");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, repCap, RSDSA_ATTR_SFD_ANALYSIS_MASK_VISIBLE, visible),
            3, "Visible");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Clears specified mask, including both the lower and upper masks.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mask/Selects repcap Mask.
ViStatus rsdsa_SFDAnalysisClearMask(ViSession instrSession,
                                    ViInt32   mask)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMask = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Mask", &maxMask),
        2, "Mask");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mask, 1, maxMask),
            2, "Mask");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_SFD_ANALYSIS_CLEAR_MASK,
            "Mask", mask - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Mask");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_SFD_ANALYSIS_CLEAR_MASK, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the specified lower mask with 1 to 20 points. Each point in
/// HIFN  the mask configuration must have a corresponding X and Y value. All
/// HIFN  frequency (X) entries are in Hz, and all amplitude (Y) entries are in
/// HIFN  the units selected by RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mask/Selects repcap Mask.
/// HIPAR points/Defines the length of the X and Y values arrays. A mask definition may
/// HIPAR points/have up to 20 points defined.
/// HIPAR xValues/An array of frequency (X axis) entries for the points of the lower
/// HIPAR xValues/mask, in Hz.
/// HIPAR yValues/An array of amplitude (Y axis) entries for the points of the lower
/// HIPAR yValues/mask, in the units specified by RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_SFDAnalysisConfigureLowerDisplayMask(ViSession instrSession,
                                                    ViInt32   mask,
                                                    ViInt32   points,
                                                    ViReal64  xValues[],
                                                    ViReal64  yValues[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMask = 0;
    ViChar   valuesParameter[RS_MAX_MESSAGE_BUF_SIZE] = { '\0' };
    ViChar*  p2buffer = valuesParameter;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Mask", &maxMask),
        2, "Mask");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mask, 1, maxMask),
            2, "Mask");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_HIDDEN_SFD_ANALYSIS_MASK_LOWER_CONFIGURATION,
            "Mask", mask - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Mask");

    for (int i = 0; i < points; i++)
    {
        p2buffer += snprintf(valuesParameter, RS_MAX_MESSAGE_BUF_SIZE - strlen(valuesParameter), "%f,%f", xValues[i], yValues[i]);
    }
    *(--p2buffer) = '\0';

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_HIDDEN_SFD_ANALYSIS_MASK_LOWER_CONFIGURATION, valuesParameter));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the specified upper mask with 1 to 20 points. Each point in
/// HIFN  the mask configuration must have a corresponding X and Y value. All
/// HIFN  frequency (X) entries are in Hz, and all amplitude (Y) entries are in
/// HIFN  the units selected by RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mask/Selects repcap Mask.
/// HIPAR points/Defines the length of the X and Y values arrays. A mask definition may
/// HIPAR points/have up to 20 points defined.
/// HIPAR xValues/An array of frequency (X axis) entries for the points of the upper
/// HIPAR xValues/mask, in Hz.
/// HIPAR yValues/An array of amplitude (Y axis) entries for the points of the upper
/// HIPAR yValues/mask, in the units specified by RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_SFDAnalysisConfigureUpperDisplayMask(ViSession instrSession,
                                                    ViInt32   mask,
                                                    ViInt32   points,
                                                    ViReal64  xValues[],
                                                    ViReal64  yValues[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMask = 0;
    ViChar   valuesParameter[RS_MAX_MESSAGE_BUF_SIZE] = { '\0' };
    ViChar*  p2buffer = valuesParameter;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Mask", &maxMask),
        2, "Mask");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mask, 1, maxMask),
            2, "Mask");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_HIDDEN_SFD_ANALYSIS_MASK_UPPER_CONFIGURATION,
            "Mask", mask - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Mask");

    for (int i = 0; i < points; i++)
    {
        p2buffer += snprintf(valuesParameter, RS_MAX_MESSAGE_BUF_SIZE - strlen(valuesParameter), "%f,%f", xValues[i], yValues[i]);
    }
    *(--p2buffer) = '\0';

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_HIDDEN_SFD_ANALYSIS_MASK_UPPER_CONFIGURATION, valuesParameter));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the specified trace data. Plot data is returned as x,y value
/// HIFN  pairs (x1,y1,x2,y2,...). 1054 points are displayed across the DSA
/// HIFN  screen. For I/Q density data, 65535 integers are always returned in row
/// HIFN  order, regardless of the zoom settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Selects repcap Trace.
/// HIPAR bufferSize/Size of output array.
/// HIPAR traceData/Queries the specified trace data. Plot data is returned as x,y value
/// HIPAR traceData/pairs (x1,y1,x2,y2,...). 1054 points are displayed across the DSA
/// HIPAR traceData/screen. For I/Q density data, 65535 integers are always returned in row
/// HIPAR traceData/order, regardless of the zoom settings.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_SFDAnalysisQueryTraceData(ViSession instrSession,
                                         ViInt32   trace,
                                         ViInt32   bufferSize,
                                         ViReal64  traceData[],
                                         ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxTrace = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Trace", &maxTrace),
        2, "Trace");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, maxTrace),
            2, "Trace");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_SFD_ANALYSIS_QUERY_TRACE_DATA,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trace");
    viCheckParm(RsCore_GetAttributeViStringAsViReal64ArrayToUserBuffer(instrSession, repCap,
        RSDSA_ATTR_SFD_ANALYSIS_QUERY_TRACE_DATA, NULL,
        bufferSize, traceData, bytesRead),
        4, "Trace Data");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the all the messages that are currently in the acquisition log
/// HIFN  and clears the acquisition log. If the acquisition log is empty, the
/// HIFN  number 0, 'No entry', is returned. For each error in the log, the
/// HIFN  aqcuisition log entry message is returned, followed by the date and
/// HIFN  time the entry was added to the log.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR log/Queries the all the messages that are currently in the acquisition log
/// HIPAR log/and clears the acquisition log. If the acquisition log is empty, the
/// HIPAR log/number 0, 'No entry', is returned. For each error in the log, the
/// HIPAR log/aqcuisition log entry message is returned, followed by the date and
/// HIPAR log/time the entry was added to the log.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_SFDAnalysisQuerySystemAcquisitionLogAll(ViSession instrSession,
                                                       ViInt32   bufferSize,
                                                       ViChar    log[],
                                                       ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL,
        RSDSA_ATTR_SFD_ANALYSIS_SYSTEM_ACQUISITION_LOG_ALL,
        RS_VAL_DIRECT_USER_CALL,
        bufferSize, log, bytesRead),
        3, "Log");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the most recent acquisition log entry and deletes it. If the
/// HIFN  acquisition log is empty, the number 0, 'No entry', is returned.
/// HIFN  Returns the aqcuisition log entry message, followed by the date and
/// HIFN  time the entry was added to the log.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR entry/Queries the most recent acquisition log entry and deletes it. If the
/// HIPAR entry/acquisition log is empty, the number 0, 'No entry', is returned.
/// HIPAR entry/Returns the aqcuisition log entry message, followed by the date and
/// HIPAR entry/time the entry was added to the log.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_SFDAnalysisQuerySystemAcquisitionLogNext(ViSession instrSession,
                                                        ViInt32   bufferSize,
                                                        ViChar    entry[],
                                                        ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL,
        RSDSA_ATTR_SFD_ANALYSIS_SYSTEM_ACQUISITION_LOG_NEXT,
        RS_VAL_DIRECT_USER_CALL,
        bufferSize, entry, bytesRead),
        3, "Log");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the acquisition log for the number of unread entries. If the
/// HIFN  acquisition log is empty, the error number 0, 'No error', is returned.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR count/Queries the acquisition log for the number of unread entries. If the
/// HIPAR count/acquisition log is empty, the error number 0, 'No error', is returned.
ViStatus rsdsa_SFDAnalysisQuerySystemAcquisitionLogCount(ViSession instrSession,
                                                         ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_SYSTEM_ACQUISITION_LOG_COUNT, count),
            2, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes all entries in the acquisition log.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_SFDAnalysisSystemAcquisitionLogClearAll(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_SYSTEM_ACQUISITION_LOG_CLEAR_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the SFD pilot frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sfdPilotFrequency/Specifies the SFD pilot frequency.
ViStatus rsdsa_ConfigureSFDAnalysisConfigureSFDPilotFrequency(
                            ViSession instrSession,
                            ViReal64  sfdPilotFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_SFD_PILOT_FREQUENCY, sfdPilotFrequency),
            2, "SFD Pilot Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a parameter to be used as a data filter for displayed
/// HIFN  constellation, I/Q density, MER vs. Time, and channel analysis plots,
/// HIFN  where applicable.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR plotDataFilter/Selects a parameter to be used as a data filter for displayed
/// HIPAR plotDataFilter/constellation, I/Q density, MER vs. Time, and channel analysis plots,
/// HIPAR plotDataFilter/where applicable.
ViStatus rsdsa_SFDAnalysisConfigurePlotDataFilter(ViSession instrSession,
                                                  ViInt32   plotDataFilter)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_PLOT_DATA_FILTER, plotDataFilter),
            2, "Plot Data Filter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the list of equalizer coefficients. A pair of signed
/// HIFN  floating-point values is returned for each coefficient, representing
/// HIFN  the real and imaginary parts. The number of coefficient pairs returned
/// HIFN  depends on the mode, the signal bandwidth, and for OFDM/OFDMA, the FFT
/// HIFN  size.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR equalizerCoefficients/Queries the list of equalizer coefficients. A pair of signed
/// HIPAR equalizerCoefficients/floating-point values is returned for each coefficient, representing
/// HIPAR equalizerCoefficients/the real and imaginary parts. The number of coefficient pairs returned
/// HIPAR equalizerCoefficients/depends on the mode, the signal bandwidth, and for OFDM/OFDMA, the FFT
/// HIPAR equalizerCoefficients/size.
/// HIPAR bytesRead/Returns the number of bytes that was read from the instrument. If this
/// HIPAR bytesRead/number is larger than the allocated Buffer Size, some data was lost.
/// HIPAR bytesRead/Consider reallocating the buffer and querying again.
ViStatus rsdsa_SFDAnalysisQueryDownstreamEqualizerCoefficients(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViReal64  equalizerCoefficients[],
                            ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringAsViReal64ArrayToUserBuffer(instrSession, NULL,
        RSDSA_ATTR_SFD_ANALYSIS_QUERY_DOWNSTREAM_EQUALIZER_COEFFICIENTS, NULL,
        bufferSize, equalizerCoefficients, bytesRead),
        3, "Equalizer Coefficients");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the total attenuation for RF input in dB. If you set the
/// HIFN  attenuation manually, it is no longer coupled to the reference level,
/// HIFN  but the reference level is coupled to the attenuation. Thus, if the
/// HIFN  current reference level is not compatible with an attenuation that has
/// HIFN  been set manually, the command also adjusts the reference level.
/// HIFN  Therefore, the reference level set in
/// HIFN  DISPlay:[<Mode>:][WINDow:]TRACe[<Trace>]:Y[:SCALe]:RLEVel is no longer
/// HIFN  applied. Note: The query returns the actual attenuation setting, which
/// HIFN  can also be changed by RSDSA_ATTR_ED30_AUTO_LEVEL.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attenuation/Defines the total attenuation for RF input in dB. If you set the
/// HIPAR attenuation/attenuation manually, it is no longer coupled to the reference level,
/// HIPAR attenuation/but the reference level is coupled to the attenuation. Thus, if the
/// HIPAR attenuation/current reference level is not compatible with an attenuation that has
/// HIPAR attenuation/been set manually, the command also adjusts the reference level.
/// HIPAR attenuation/Therefore, the reference level set in
/// HIPAR attenuation/DISPlay:[<Mode>:][WINDow:]TRACe[<Trace>]:Y[:SCALe]:RLEVel is no longer
/// HIPAR attenuation/applied. Note: The query returns the actual attenuation setting, which
/// HIPAR attenuation/can also be changed by RSDSA_ATTR_ED30_AUTO_LEVEL.
ViStatus rsdsa_SFDAnalysisConfigureDownstreamAttenuation(ViSession instrSession,
                                                         ViReal64  attenuation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_DOWNSTREAM_ATTENUATION, attenuation),
            2, "Attenuation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the preamplifier on and off. Note: The query returns the actual
/// HIFN  preamplifier state (which might also be changed by
/// HIFN  RSDSA_ATTR_SUAN_AUTO_LEVEL).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Turns the preamplifier on and off. Note: The query returns the actual
/// HIPAR enabled/preamplifier state (which might also be changed by
/// HIPAR enabled/RSDSA_ATTR_SUAN_AUTO_LEVEL).
ViStatus rsdsa_SFDAnalysisConfigureDownstreamPreamplifierEnabled(
                            ViSession instrSession,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_DOWNSTREAM_PREAMPLIFIER_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the total attenuation for RF input in dB. If you set the
/// HIFN  attenuation manually, it is no longer coupled to the reference level,
/// HIFN  but the reference level is coupled to the attenuation. Thus, if the
/// HIFN  current reference level is not compatible with an attenuation that has
/// HIFN  been set manually, the command also adjusts the reference level.
/// HIFN  Therefore, the reference level set in
/// HIFN  DISPlay:[<Mode>:][WINDow:]TRACe[<Trace>]:Y[:SCALe]:RLEVel is no longer
/// HIFN  applied. Note: The query returns the actual attenuation setting, which
/// HIFN  can also be changed by INPut:<Direction>:LEVel:AUTO.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attenuation/Defines the total attenuation for RF input in dB. If you set the
/// HIPAR attenuation/attenuation manually, it is no longer coupled to the reference level,
/// HIPAR attenuation/but the reference level is coupled to the attenuation. Thus, if the
/// HIPAR attenuation/current reference level is not compatible with an attenuation that has
/// HIPAR attenuation/been set manually, the command also adjusts the reference level.
/// HIPAR attenuation/Therefore, the reference level set in
/// HIPAR attenuation/DISPlay:[<Mode>:][WINDow:]TRACe[<Trace>]:Y[:SCALe]:RLEVel is no longer
/// HIPAR attenuation/applied. Note: The query returns the actual attenuation setting, which
/// HIPAR attenuation/can also be changed by INPut:<Direction>:LEVel:AUTO.
ViStatus rsdsa_SFDAnalysisConfigureUpstreamAttenuation(ViSession instrSession,
                                                       ViReal64  attenuation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_UPSTREAM_ATTENUATION, attenuation),
            2, "Attenuation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the preamplifier on and off. Note: The query returns the actual
/// HIFN  preamplifier state (which might also be changed by
/// HIFN  RSDSA_ATTR_D31U_AUTO_LEVEL).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Turns the preamplifier on and off. Note: The query returns the actual
/// HIPAR enabled/preamplifier state (which might also be changed by
/// HIPAR enabled/RSDSA_ATTR_D31U_AUTO_LEVEL).
ViStatus rsdsa_SFDAnalysisConfigureUpstreamPreamplifierEnabled(
                            ViSession instrSession,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_UPSTREAM_PREAMPLIFIER_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the SFP+ network output on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Turns the SFP+ network output on and off.
ViStatus rsdsa_SFDAnalysisConfigureSFPOutputEnabled(ViSession instrSession,
                                                    ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_SFP_OUTPUT_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables and configures the UDP port to be used for SFP+ IUC output.
/// HIFN  The UDP ports must be uniquely assigned to downstream OFDM profiles/PLC
/// HIFN  and upstream DOCSIS IUCs.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR iuc/Selects repcap IUC_OFDMA.
/// HIPAR enabled/Turns the SFP+ IUC UDP port on and off.
/// HIPAR udpPort/Configures the UDP port to be used for SFP+ IUC output. The UDP ports
/// HIPAR udpPort/must be uniquely assigned to downstream OFDM profiles/PLC and upstream
/// HIPAR udpPort/DOCSIS IUCs.
ViStatus rsdsa_SFDAnalysisConfigureSFPIUCUDPPort(ViSession instrSession,
                                                 ViInt32   iuc,
                                                 ViBoolean enabled,
                                                 ViInt32   udpPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxIUC = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "IUC_OFDMA", &maxIUC),
        2, "IUC");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, iuc, 1, maxIUC),
            2, "IUC");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_SFP_IUC_UDP_PORT_ENABLED,
            "IUC_OFDMA", iuc - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "IUC");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, repCap, RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_SFP_IUC_UDP_PORT_ENABLED, enabled),
            3, "Enabled");
    if (enabled)
    {
        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, repCap, RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_SFP_IUC_UDP_PORT, udpPort),
                4, "UDP Port");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the alarm threshold for RMS MER overall. When this parameter
/// HIFN  falls below the set threshold, an alarm is generated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rmsMEROverall/Sets the alarm threshold for RMS MER overall. When this parameter
/// HIPAR rmsMEROverall/falls below the set threshold, an alarm is generated.
ViStatus rsdsa_SFDAnalysisConfigureAlarmThresholdRMSMEROverall(
                            ViSession instrSession,
                            ViReal64  rmsMEROverall)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_ALARM_THRESHOLD_RMS_MER_OVERALL, rmsMEROverall),
            2, "RMS MER Overall");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the alarm thresholds for pre- and post-Reed-Solomon BER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR berPreRS/Sets the alarm threshold for pre-Reed-Solomon BER. When this parameter
/// HIPAR berPreRS/exceeds the set threshold, an alarm is generated.
/// HIPAR berPostRS/Sets the alarm threshold for post-Reed-Solomon BER. When this
/// HIPAR berPostRS/parameter exceeds the set threshold, an alarm is generated.
ViStatus rsdsa_SFDAnalysisConfigureAlarmThresholdsBERReedSolomon(
                            ViSession instrSession,
                            ViReal64  berPreRS,
                            ViReal64  berPostRS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_ALARM_THRESHOLD_BER_PRE_REED_SOLOMON, berPreRS),
            2, "BER Pre RS");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_ALARM_THRESHOLD_BER_POST_REED_SOLOMON, berPostRS),
            3, "BER Post RS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the alarm thresholds for pre- and post-LDPC BER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR berPreLDPC/Sets the alarm threshold for pre-LDPC BER. When this parameter exceeds
/// HIPAR berPreLDPC/the set threshold, an alarm is generated.
/// HIPAR berPostLDPC/Sets the alarm threshold for post-LDPC BER. When this parameter
/// HIPAR berPostLDPC/exceeds the set threshold, an alarm is generated.
ViStatus rsdsa_SFDAnalysisConfigureAlarmThresholdsBERLDPC(ViSession instrSession,
                                                          ViReal64  berPreLDPC,
                                                          ViReal64  berPostLDPC)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_ALARM_THRESHOLD_BER_PRE_LDPC, berPreLDPC),
            2, "BER Pre LDPC");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_ALARM_THRESHOLD_BER_POST_LDPC, berPostLDPC),
            3, "BER Post LDPC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the CM ranging analysis mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_EnableCMRangingAnalysis(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_ENABLE_CM_RANGING_ANALYSIS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the burst demodulator lock status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstDemodulatorLockStatus/Queries the burst demodulator lock status.
ViStatus rsdsa_QueryCMRangingBurstDemodulatorLockStatus(ViSession instrSession,
                                                        ViInt32*  burstDemodulatorLockStatus)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_QUERY_BURST_DEMODULATOR_LOCK_STATUS, burstDemodulatorLockStatus),
            2, "Burst Demodulator Lock Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the forward error correction (FEC) lock status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fecLockStatus/Queries the forward error correction (FEC) lock status.
ViStatus rsdsa_QueryCMRangingFECLockStatus(ViSession instrSession,
                                           ViInt32*  fecLockStatus)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_QUERY_FEC_LOCK_STATUS, fecLockStatus),
            2, "FEC Lock Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the burst receive power level in the units selected by
/// HIFN  RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstReceivePower/Queries the burst receive power level in the units selected by
/// HIPAR burstReceivePower/RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_QueryCMRangingBurstReceivePower(ViSession instrSession,
                                               ViReal64* burstReceivePower)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_CM_RANGING_QUERY_BURST_RECEIVE_POWER, burstReceivePower),
            2, "Burst Receive Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the frequency offset in Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequencyOffset/Queries the frequency offset in Hz.
ViStatus rsdsa_QueryCMRangingFrequencyOffset(ViSession instrSession,
                                             ViReal64* frequencyOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_CM_RANGING_QUERY_FREQUENCY_OFFSET, frequencyOffset),
            2, "Frequency Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the burst timing offset from the time the burst was detected
/// HIFN  and the time the burst was supposed to occur, in seconds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstTimingOffset/Queries the burst timing offset from the time the burst was detected
/// HIPAR burstTimingOffset/and the time the burst was supposed to occur, in seconds.
ViStatus rsdsa_QueryCMRangingBurstTimingOffset(ViSession instrSession,
                                               ViReal64* burstTimingOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_CM_RANGING_QUERY_BURST_TIMING_OFFSET, burstTimingOffset),
            2, "Burst Timing Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of minislots received.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR minislotsReceived/Queries the number of minislots received.
ViStatus rsdsa_QueryCMRangingMinislotsReceived(ViSession instrSession,
                                               ViInt64*  minislotsReceived)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt64(instrSession, NULL, RSDSA_ATTR_CM_RANGING_QUERY_MINISLOTS_RECEIVED, minislotsReceived),
        2, "Minislots Received");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the percentage of valid bursts out of all bursts received over
/// HIFN  the last one second period.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstCoverage/Queries the percentage of valid bursts out of all bursts received over
/// HIPAR burstCoverage/the last one second period.
ViStatus rsdsa_QueryCMRangingBurstCoverage(ViSession instrSession,
                                           ViReal64* burstCoverage)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_CM_RANGING_QUERY_BURST_COVERAGE, burstCoverage),
            2, "Burst Coverage");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the upstream mode detected via the SFD pilot signal in SFD
/// HIFN  upstream analysis mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR detectedUpstreamMode/Queries the upstream mode detected via the SFD pilot signal in SFD
/// HIPAR detectedUpstreamMode/upstream analysis mode.
ViStatus rsdsa_SFDAnalysisQueryDetectedUpstreamMode(ViSession instrSession,
                                                    ViInt32*  detectedUpstreamMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_QUERY_DETECTED_UPSTREAM_MODE, detectedUpstreamMode),
            2, "Detected Upstream Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the sign preferred for MER numerical display and plot axis
/// HIFN  labels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merSign/Selects the sign preferred for MER numerical display and plot axis
/// HIPAR merSign/labels.
ViStatus rsdsa_ConfigureCMRangingMERSign(ViSession instrSession,
                                         ViInt32   merSign)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_CONFIGURE_MER_SIGN, merSign),
            2, "MER Sign");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall RMS equalized MER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merRMS/Queries the overall RMS equalized MER.
ViStatus rsdsa_QueryCMRangingMERRMS(ViSession instrSession,
                                    ViReal64* merRMS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_CM_RANGING_QUERY_MER_RMS, merRMS),
            2, "MER RMS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall RMS partially equalized MER. For OFDMA mode, it is
/// HIFN  only valid for data bursts with grant sizes greater than 100 using
/// HIFN  pilots 4 or 11.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merRMSPartialyEqualized/Queries the overall RMS partially equalized MER. For OFDMA mode, it is
/// HIPAR merRMSPartialyEqualized/only valid for data bursts with grant sizes greater than 100 using
/// HIPAR merRMSPartialyEqualized/pilots 4 or 11.
ViStatus rsdsa_QueryCMRangingMERRMSPartialyEqualized(ViSession instrSession,
                                                     ViReal64* merRMSPartialyEqualized)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_CM_RANGING_QUERY_MER_RMS_PARTIALY_EQUALIZED, merRMSPartialyEqualized),
            2, "MER RMS Partialy Equalized");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall peak (lowest) MER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merPeak/Queries the overall peak (lowest) MER.
ViStatus rsdsa_QueryCMRangingMERPeak(ViSession instrSession,
                                     ViReal64* merPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_CM_RANGING_QUERY_MER_PEAK, merPeak),
            2, "MER Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the RMS MER for the pilot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merPilotRMS/Queries the RMS MER for the pilot.
ViStatus rsdsa_QueryCMRangingMERPilotRMS(ViSession instrSession,
                                         ViReal64* merPilotRMS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_CM_RANGING_QUERY_MER_PILOT_RMS, merPilotRMS),
            2, "MER Pilot RMS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the peak (lowest) MER for the pilot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merPilotPeak/Queries the peak (lowest) MER for the pilot.
ViStatus rsdsa_QueryCMRangingMERPilotPeak(ViSession instrSession,
                                          ViReal64* merPilotPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_CM_RANGING_QUERY_MER_PILOT_PEAK, merPilotPeak),
            2, "MER Pilot Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the constellations for the pilot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR pilotConstellations/Queries the constellations for the pilot.
ViStatus rsdsa_QueryCMRangingPilotConstellations(ViSession instrSession,
                                                 ViInt32   bufferSize,
                                                 ViChar    pilotConstellations[],
                                                 ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSDSA_ATTR_CM_RANGING_QUERY_PILOT_CONSTELLATIONS, RS_VAL_DIRECT_USER_CALL, bufferSize, pilotConstellations, bytesRead),
        3, "Pilot Constellations");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the RMS MER for the complementary pilot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merComplementaryPilotRMS/Queries the RMS MER for the complementary pilot.
ViStatus rsdsa_QueryCMRangingMERComplementaryPilotRMS(ViSession instrSession,
                                                      ViReal64* merComplementaryPilotRMS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_CM_RANGING_QUERY_MER_COMPLEMENTARY_PILOT_RMS, merComplementaryPilotRMS),
            2, "MER Complementary Pilot RMS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the peak (lowest) MER for the complementary pilot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merComplementaryPilotPeak/Queries the peak (lowest) MER for the complementary pilot.
ViStatus rsdsa_QueryCMRangingMERComplementaryPilotPeak(ViSession instrSession,
                                                       ViReal64* merComplementaryPilotPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_CM_RANGING_QUERY_MER_COMPLEMENTARY_PILOT_PEAK, merComplementaryPilotPeak),
            2, "MER Complementary Pilot Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the constellations for the complementary pilot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR pilotConstellations/Queries the constellations for the complementary pilot.
ViStatus rsdsa_QueryCMRangingComplementaryPilotConstellations(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    pilotConstellations[],
                            ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSDSA_ATTR_CM_RANGING_QUERY_COMPLEMENTARY_PILOT_CONSTELLATIONS, RS_VAL_DIRECT_USER_CALL, bufferSize, pilotConstellations, bytesRead),
        3, "Pilot Constellations");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the RMS MER for the IUC currently selected by
/// HIFN  DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merIUCRMS/Queries the RMS MER for the IUC currently selected by
/// HIPAR merIUCRMS/DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>.
ViStatus rsdsa_QueryCMRangingMERIUCRMS(ViSession instrSession,
                                       ViReal64* merIUCRMS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_CM_RANGING_QUERY_MER_IUC_RMS, merIUCRMS),
            2, "MER IUC RMS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the peak (lowest) MER for the IUC currently selected by
/// HIFN  DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merIUCPeak/Queries the peak (lowest) MER for the IUC currently selected by
/// HIPAR merIUCPeak/DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>.
ViStatus rsdsa_QueryCMRangingMERIUCPeak(ViSession instrSession,
                                        ViReal64* merIUCPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_CM_RANGING_QUERY_MER_IUC_PEAK, merIUCPeak),
            2, "MER IUC Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the constellations for the currently selected IUC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR iucConstellations/Queries the constellations for the currently selected IUC.
ViStatus rsdsa_QueryCMRangingIUCConstellations(ViSession instrSession,
                                               ViInt32   bufferSize,
                                               ViChar    iucConstellations[],
                                               ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSDSA_ATTR_CM_RANGING_QUERY_IUC_CONSTELLATIONS, RS_VAL_DIRECT_USER_CALL, bufferSize, iucConstellations, bytesRead),
        3, "IUC Constellations");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the I/Q amplitude imbalance in dB.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR iqImbalanceAmplitudedB/Queries the I/Q amplitude imbalance in dB.
ViStatus rsdsa_QueryCMRangingIQImbalanceAmplitudedB(ViSession instrSession,
                                                    ViReal64* iqImbalanceAmplitudedB)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_CM_RANGING_QUERY_IQ_IMBALANCE_AMPLITUDE_DB, iqImbalanceAmplitudedB),
            2, "IQ Imbalance Amplitude dB");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the I/Q amplitude imbalance in percent.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR iqImbalanceAmplitudePercent/Queries the I/Q amplitude imbalance in percent.
ViStatus rsdsa_QueryCMRangingIQImbalanceAmplitudePercent(ViSession instrSession,
                                                         ViReal64* iqImbalanceAmplitudePercent)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_CM_RANGING_QUERY_IQ_IMBALANCE_AMPLITUDE_PERCENT, iqImbalanceAmplitudePercent),
            2, "IQ Imbalance Amplitud Percent");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the I/Q imbalance quadrature error in degrees.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR iqImbalanceQuadratureError/Queries the I/Q imbalance quadrature error in degrees.
ViStatus rsdsa_QueryCMRangingIQImbalanceQuadratureError(ViSession instrSession,
                                                        ViReal64* iqImbalanceQuadratureError)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_CM_RANGING_QUERY_IQ_IMBALANCE_QUADRATURE_ERROR, iqImbalanceQuadratureError),
            2, "IQ Imbalance Quadrature Error");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the constellation plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_CMRangingDisplayConstellationPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_CM_RANGING_DISPLAY_CONSTELLATION_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the I/Q density plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_CMRangingDisplayIQDensityPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_CM_RANGING_DISPLAY_I_Q_DENSITY_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the IQ Measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Selects the measurement mode for the I/Q density plot.
/// HIPAR windowWidth/Specifies the sliding window width in samples for the I/Q density plot
/// HIPAR windowWidth/in sliding window mode.
ViStatus rsdsa_ConfigureCMRangingIQMeasurement(ViSession instrSession,
                                               ViInt32   mode,
                                               ViInt32   windowWidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_CONFIGURE_I_Q_MEASUREMENT_MODE, mode),
            2, "Mode");

    if (mode == RSDSA_VAL_IQ_MEASUREMENT_MODE_SLIDING_WINDOW)
        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_CONFIGURE_I_Q_WINDOW_WIDTH, windowWidth),
                2, "Window Width");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the zoom setting for the I/Q density plot display. Note that
/// HIFN  the x and y origin are the top left corner of the window, as per the
/// HIFN  GUI settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR x/X origin (left-most window point), between 0 and 1.
/// HIPAR y/Y origin (top-most window point), between 0 and 1.
/// HIPAR z/Zoom setting, 1 to 128.
ViStatus rsdsa_CMRangingSetIQDisplayZoom(ViSession instrSession,
                                         ViReal64  x,
                                         ViReal64  y,
                                         ViInt32   z)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, x, 0, 1),
            2, "X");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, y, 0, 1),
            3, "Y");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, z, 1, 128),
            4, "Z");

    snprintf(param, RS_MAX_MESSAGE_BUF_SIZE, "%f,%f,%ld", x, y, z);

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_HIDDEN_CM_RANGING_DISPLAY_I_Q_ZOOM, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Save the constellation I/Q data to the specified file. The file
/// HIFN  extension is .iq.tar. Note that there is no way to abort this command,
/// HIFN  and large file sizes could take a long time to capture depending on the
/// HIFN  input data, so use maxSamples or timeoutSeconds to limit the file size
/// HIFN  or time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filename/File path, uses .iq.tar extension.
/// HIPAR maxSamples/Sets the I/Q data file maximum size in samples.
/// HIPAR timeoutSeconds/Sets the maximum time in seconds to save data.
ViStatus rsdsa_CMRangingCaptureIQDataToFile(ViSession instrSession,
                                            ViInt64   maxSamples,
                                            ViReal64  timeoutSeconds,
                                            ViString  filename)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_IQ_DATA_EXPORT_MAX_SAMPLES, maxSamples),
        2, "Max Samples");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_IQ_DATA_EXPORT_TIMEOUT, timeoutSeconds),
            3, "Timeout Seconds");

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_SFD_ANALYSIS_CAPTURE_IQ_DATA_TO_FILENAME, filename),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the MER vs. time plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_CMRangingDisplayMERVsTimePlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_CM_RANGING_DISPLAY_MER_VS_TIME_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a time filter period to be used for displayed MER vs. Time
/// HIFN  plots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merVsTimePlotFilterPeriod/Selects a time filter period to be used for displayed MER vs. Time
/// HIPAR merVsTimePlotFilterPeriod/plots.
ViStatus rsdsa_ConfigureCMRangingMERVsTimePlotFilterPeriod(ViSession instrSession,
                                                           ViInt32   merVsTimePlotFilterPeriod)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_CONFIGURE_MER_VS_TIME_PLOT_FILTER_PERIOD, merVsTimePlotFilterPeriod),
            2, "MER Vs Time Plot Filter Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the BER vs. time plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_CMRangingDisplayBERVsTimePlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_CM_RANGING_DISPLAY_BER_VS_TIME_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a time filter period to be used for displayed BER vs. Time
/// HIFN  plots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR berVsTimePlotFilterPeriod/Selects a time filter period to be used for displayed BER vs. Time
/// HIPAR berVsTimePlotFilterPeriod/plots.
ViStatus rsdsa_ConfigureCMRangingBERVsTimePlotFilterPeriod(ViSession instrSession,
                                                           ViInt32   berVsTimePlotFilterPeriod)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_CONFIGURE_BER_VS_TIME_PLOT_FILTER_PERIOD, berVsTimePlotFilterPeriod),
            2, "BER Vs Time Plot Filter Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the POWer vs. time plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_CMRangingDisplayPowerVsTimePlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_CM_RANGING_DISPLAY_POWER_VS_TIME_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a time filter period to be used for displayed Power vs. Time
/// HIFN  plots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR powVsTimePlotFilterPeriod/Selects a time filter period to be used for displayed Power vs. Time
/// HIPAR powVsTimePlotFilterPeriod/plots.
ViStatus rsdsa_ConfigureCMRangingPowerVsTimePlotFilterPeriod(ViSession instrSession,
                                                             ViInt32   powVsTimePlotFilterPeriod)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_CONFIGURE_POWER_VS_TIME_PLOT_FILTER_PERIOD, powVsTimePlotFilterPeriod),
            2, "Pow Vs Time Plot Filter Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the MER vs. Subcarrier plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_CMRangingDisplayMERVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_CM_RANGING_DISPLAY_MER_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the MER vs. minislot plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_CMRangingDisplayMERVsMinislotPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_CM_RANGING_DISPLAY_MER_VS_MINISLOT_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays a downstream or upstream CCDF (Complementary Cumulative
/// HIFN  Distribution Function histogram as per CM-SP-PHYv3.1-I16 sections 9.3.8
/// HIFN  or 9.4.6 respectively). This command may take up to 30 seconds to
/// HIFN  execute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_CMRangingDisplayCCDFHistogram(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_CM_RANGING_DISPLAY_CCDF_HISTOGRAM, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the peak CCDF power level in the units selected by
/// HIFN  RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cmRangingQueryCCDFPeak/Queries the peak CCDF power level in the units selected by
/// HIPAR cmRangingQueryCCDFPeak/RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_QueryCMRangingCCDFPeak(ViSession instrSession,
                                      ViReal64* ccdfPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_CM_RANGING_QUERY_CCDF_PEAK, ccdfPeak),
            2, "CCDF Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the channel analysis amplitude/phase vs. subcarrier plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_CMRangingDisplayAmplitudePhaseVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_CM_RANGING_DISPLAY_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Scales the y-axis for the channel analysis amplitude/phase vs.
/// HIFN  subcarrier plot to best fit the measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_CMRangingAutoScaleAmplitudePhaseVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_CM_RANGING_AUTO_SCALE_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the channel analysis amplitude/group delay vs. subcarrier
/// HIFN  plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_CMRangingDisplayAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_CM_RANGING_DISPLAY_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Scales the y-axis for the channel analysis amplitude/group delay vs.
/// HIFN  subcarrier plot to best fit the measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_CMRangingAutoScaleAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_CM_RANGING_AUTO_SCALE_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the channel analysis echo pattern plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_CMRangingDisplayEchoPatternPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_CM_RANGING_DISPLAY_ECHO_PATTERN_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the spectrum analysis plot for DOCSIS 3.1 Downstream mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_CMRangingDisplaySpectrumAnalysisPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_CM_RANGING_DISPLAY_SPECTRUM_ANALYSIS_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the specified trace data. Plot data is returned as x,y value
/// HIFN  pairs ( x1,y1,x2,y2,...). 1054 points are displayed across the DSA
/// HIFN  screen. For I/Q density data, 65535 integers are always returned in row
/// HIFN  order, regardless of the zoom settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Selects repcap Trace.
/// HIPAR bufferSize/Size of output array.
/// HIPAR cmRangingQueryTraceData/Queries the specified trace data. Plot data is returned as x,y value
/// HIPAR cmRangingQueryTraceData/pairs ( x1,y1,x2,y2,...). 1054 points are displayed across the DSA
/// HIPAR cmRangingQueryTraceData/screen. For I/Q density data, 65535 integers are always returned in row
/// HIPAR cmRangingQueryTraceData/order, regardless of the zoom settings.
ViStatus rsdsa_QueryCMRangingTraceData(ViSession instrSession,
                                       ViInt32   trace,
                                       ViInt32   bufferSize,
                                       ViReal64  traceData[],
                                       ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxTrace = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Trace", &maxTrace),
        2, "Trace");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, maxTrace),
            2, "Trace");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_CM_RANGING_QUERY_TRACE_DATA,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trace");
    viCheckParm(RsCore_GetAttributeViStringAsViReal64ArrayToUserBuffer(instrSession, repCap,
        RSDSA_ATTR_CM_RANGING_QUERY_TRACE_DATA, NULL,
        bufferSize, traceData, bytesRead),
        4, "Trace Data");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the all the messages that are currently in the acquisition log
/// HIFN  and clears the acquisition log. If the acquisition log is empty, the
/// HIFN  number 0, 'No entry', is returned. For each error in the log, the
/// HIFN  aqcuisition log entry message is returned, followed by the date and
/// HIFN  time the entry was added to the log.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR acquisitionLogAll/Queries the all the messages that are currently in the acquisition log
/// HIPAR acquisitionLogAll/and clears the acquisition log. If the acquisition log is empty, the
/// HIPAR acquisitionLogAll/number 0, 'No entry', is returned. For each error in the log, the
/// HIPAR acquisitionLogAll/aqcuisition log entry message is returned, followed by the date and
/// HIPAR acquisitionLogAll/time the entry was added to the log.
ViStatus rsdsa_QueryCMRangingSystemAcquisitionLogAll(ViSession instrSession,
                                                     ViInt32   bufferSize,
                                                     ViChar    acquisitionLogAll[],
                                                     ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSDSA_ATTR_CM_RANGING_SYSTEM_ACQUISITION_LOG_ALL, RS_VAL_DIRECT_USER_CALL, bufferSize, acquisitionLogAll, bytesRead),
        3, "Acquisition Log All");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the most recent acquisition log entry and deletes it. If the
/// HIFN  acquisition log is empty, the number 0, 'No entry', is returned.
/// HIFN  Returns the aqcuisition log entry message, followed by the date and
/// HIFN  time the entry was added to the log.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR acquisitionLogNext/Queries the most recent acquisition log entry and deletes it. If the
/// HIPAR acquisitionLogNext/acquisition log is empty, the number 0, 'No entry', is returned.
/// HIPAR acquisitionLogNext/Returns the aqcuisition log entry message, followed by the date and
/// HIPAR acquisitionLogNext/time the entry was added to the log.
ViStatus rsdsa_QueryCMRangingSystemAcquisitionLogNext(ViSession instrSession,
                                                      ViInt32   bufferSize,
                                                      ViChar    acquisitionLogNext[],
                                                      ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSDSA_ATTR_CM_RANGING_SYSTEM_ACQUISITION_LOG_NEXT, RS_VAL_DIRECT_USER_CALL, bufferSize, acquisitionLogNext, bytesRead),
        3, "Acquisition Log Next");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the acquisition log for the number of unread entries. If the
/// HIFN  acquisition log is empty, the error number 0, 'No error', is returned.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR acquisitionLogCount/Queries the acquisition log for the number of unread entries. If the
/// HIPAR acquisitionLogCount/acquisition log is empty, the error number 0, 'No error', is returned.
ViStatus rsdsa_QueryCMRangingSystemAcquisitionLogCount(ViSession instrSession,
                                                       ViInt32*  acquisitionLogCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_SYSTEM_ACQUISITION_LOG_COUNT, acquisitionLogCount),
            2, "Acquisition Log Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes all entries in the acquisition log.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_CMRangingSystemAcquisitionLogClearAll(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_CM_RANGING_SYSTEM_ACQUISITION_LOG_CLEAR_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a parameter to be used as a data filter for displayed
/// HIFN  constellation, I/Q density, MER vs. Time, and channel analysis plots,
/// HIFN  where applicable.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR plotDataFilter/Selects a parameter to be used as a data filter for displayed
/// HIPAR plotDataFilter/constellation, I/Q density, MER vs. Time, and channel analysis plots,
/// HIPAR plotDataFilter/where applicable.
ViStatus rsdsa_ConfigureCMRangingPlotDataFilter(ViSession instrSession,
                                                ViInt32   plotDataFilter)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_CONFIGURE_PLOT_DATA_FILTER, plotDataFilter),
            2, "Plot Data Filter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the downstream mode used for CM ranging analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR downstreamMode/Specifies the downstream mode used for CM ranging analysis.
ViStatus rsdsa_ConfigureCMRangingDownstreamMode(ViSession instrSession,
                                                ViInt32   downstreamMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_CONFIGURE_DOWNSTREAM_MODE, downstreamMode),
            2, "Downstream Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the SFD transmit power in CM ranging analysis mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sfdDownstreamTransmitPower/Specifies the SFD transmit power in CM ranging analysis mode.
ViStatus rsdsa_ConfigureCMRangingSFDDownstreamTransmitPower(ViSession instrSession,
                                                            ViReal64  sfdDownstreamTransmitPower)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_CM_RANGING_CONFIGURE_SFD_DOWNSTREAM_TRANSMIT_POWER, sfdDownstreamTransmitPower),
            2, "SFD Downstream Transmit Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the ASI output on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR asiOutputEnabled/Turns the ASI output on and off.
ViStatus rsdsa_ConfigureCMRangingDownstreamASIOutputEnabled(ViSession instrSession,
                                                            ViBoolean asiOutputEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_CM_RANGING_CONFIGURE_DOWNSTREAM_ASI_OUTPUT_ENABLED, asiOutputEnabled),
            2, "ASI Output Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the EuroDOCSIS 3.0
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequency/Specifies the center frequency of the channel. If the spectrum
/// HIPAR frequency/location is changed by CONFigure:CHANnel:SPECtrum:FREQuency, the center
/// HIPAR frequency/frequency is adapted accordingly.
/// HIPAR constellation/Defines the constellation.
ViStatus rsdsa_ConfigureCMRangingED30(ViSession instrSession,
                                      ViReal64  frequency,
                                      ViInt32   constellation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_CM_RANGING_ED30_CONFIGURE_FREQUENCY, frequency),
            2, "Frequency");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_ED30_CONFIGURE_CONSTELLATION, constellation),
            3, "Constellation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  COnfigures the DOCSIS 3.0
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequency/Specifies the center frequency of the channel. If the spectrum
/// HIPAR frequency/location is changed by CONFigure:CHANnel:SPECtrum:FREQuency, the center
/// HIPAR frequency/frequency is adapted accordingly.
/// HIPAR constellation/Defines the constellation.
/// HIPAR interleaver/Specifies the interleaver type for DOCSIS 3.0 in CM ranging analysis
/// HIPAR interleaver/mode.
ViStatus rsdsa_ConfigureCMRangingD30D(ViSession instrSession,
                                      ViReal64  frequency,
                                      ViInt32   constellation,
                                      ViInt32   interleaver)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D30D_CONFIGURE_FREQUENCY, frequency),
            2, "Frequency");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D30D_CONFIGURE_CONSTELLATION, constellation),
            3, "Constellation");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CONFIGURE_CHANNEL_INTERLEAVER, interleaver),
            4, "Interleaver");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the SC-QAM J.83C.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequency/Specifies the center frequency of the channel. If the spectrum
/// HIPAR frequency/location is changed by CONFigure:CHANnel:SPECtrum:FREQuency, the center
/// HIPAR frequency/frequency is adapted accordingly.
/// HIPAR constellation/Defines the constellation.
ViStatus rsdsa_ConfigureCMRangingD30J(ViSession instrSession,
                                      ViReal64  frequency,
                                      ViInt32   constellation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D30J_CONFIGURE_FREQUENCY, frequency),
            2, "Frequency");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D30J_CONFIGURE_CONSTELLATION, constellation),
            3, "Constellation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the DOCSIS 3.1 OFDM
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequency/Specifies the center frequency of the channel. If the spectrum
/// HIPAR frequency/location is changed by CONFigure:CHANnel:SPECtrum:FREQuency, the center
/// HIPAR frequency/frequency is adapted accordingly.
/// HIPAR cyclicPrefixLength/Defines the cyclic prefix length, which determines where the useful
/// HIPAR cyclicPrefixLength/data starts and allows the application to detect delay spreads during
/// HIPAR cyclicPrefixLength/transmission. The longer the delay spread, the longer the CP must be.
/// HIPAR cyclicPrefixLength/This command and query do not apply to
/// HIPAR timeInterleaverDepth/Specifies the OFDM time interleaver depth. This control do not apply
/// HIPAR timeInterleaverDepth/to OFDM automatic acquisition mode.
ViStatus rsdsa_ConfigureCMRangingD31D(ViSession instrSession,
                                      ViReal64  frequency,
                                      ViInt32   cyclicPrefixLength,
                                      ViInt32   timeInterleaverDepth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31D_CONFIGURE_FREQUENCY, frequency),
            2, "Frequency");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31D_CONFIGURE_CYCLIC_PREFIX_LENGTH, cyclicPrefixLength),
            3, "Cyclic Prefix Length");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31D_CONFIGURE_TIME_INTERLEAVER_DEPTH, timeInterleaverDepth),
            4, "Time Interleaver Depth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the DOCSIS 3.1 OFDM Subcarrier.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subcarrier0Frequency/Specifies the center frequency of subcarrier 0 of the channel, which
/// HIPAR subcarrier0Frequency/defines the beginning of the OFDM spectrum. The default value is
/// HIPAR subcarrier0Frequency/derived from the current center frequency. If the center frequency is
/// HIPAR subcarrier0Frequency/changed by CONFigure:CHANnel:FREQuency,
/// HIPAR subcarrierSpacing/Defines the FFT length, which determines the subcarrier spacing and
/// HIPAR subcarrierSpacing/other parameters. This control do not apply to OFDM automatic
/// HIPAR subcarrierSpacing/acquisition mode.
ViStatus rsdsa_ConfigureCMRangingD31DSubcarrier(ViSession instrSession,
                                                ViReal64  subcarrier0Frequency,
                                                ViInt32   subcarrierSpacing)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31D_CONFIGURE_SUBCARRIER_0_FREQUENCY, subcarrier0Frequency),
            2, "Subcarrier 0 Frequency");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31D_CONFIGURE_SUBCARRIER_SPACING, subcarrierSpacing),
            3, "Subcarrier Spacing");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the OFDM subcarrier configuration file, which contains
/// HIFN  downstream subcarrier configuration information. If the command
/// HIFN  specifies an empty string, the subcarrier configuration file selection
/// HIFN  is cleared. This command and query do not apply to OFDM automatic
/// HIFN  acquisition mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subcarrierConfigFile/Loads the OFDM subcarrier configuration file, which contains
/// HIPAR subcarrierConfigFile/downstream subcarrier configuration information. If the command
/// HIPAR subcarrierConfigFile/specifies an empty string, the subcarrier configuration file selection
/// HIPAR subcarrierConfigFile/is cleared. This command and query do not apply to OFDM automatic
/// HIPAR subcarrierConfigFile/acquisition mode.
ViStatus rsdsa_ConfigureCMRangingD31DSubcarrierConfigFile(ViSession instrSession,
                                                          ViString  subcarrierConfigFile)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31D_CONFIGURE_SUBCARRIER_CONFIG_FILE, subcarrierConfigFile),
            2, "Subcarrier Config File");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the downstream.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attenuation/Defines the total attenuation for RF input in dB. If you set the
/// HIPAR attenuation/attenuation manually, it is no longer coupled to the reference level,
/// HIPAR attenuation/but the reference level is coupled to the attenuation. Thus, if the
/// HIPAR attenuation/current reference level is not compatible with an attenuation that has
/// HIPAR attenuation/been set manually, the command also adjusts the reference level.
/// HIPAR attenuation/Therefore, the reference level set in
/// HIPAR attenuation/DISPlay:[<Mode>:][WINDow:]TRACe[<Trace>]:Y[:SCALe]:RLEVel is no longer
/// HIPAR attenuation/applied. Note: The query returns the actual attenuation setting, which
/// HIPAR attenuation/can also be changed by RSDSA_ATTR_CMRA_AUTO_LEVEL.
/// HIPAR preamplifierEnabled/Turns the preamplifier on and off. Note: The query returns the actual
/// HIPAR preamplifierEnabled/preamplifier state (which might also be changed by
/// HIPAR preamplifierEnabled/RSDSA_ATTR_CMRA_AUTO_LEVEL).
ViStatus rsdsa_ConfigureCMRangingDownstream(ViSession instrSession,
                                            ViReal64  attenuation,
                                            ViBoolean preamplifierEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_CM_RANGING_CONFIGURE_DOWNSTREAM_ATTENUATION, attenuation),
            2, "Attenuation");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_CM_RANGING_CONFIGURE_DOWNSTREAM_PREAMPLIFIER_ENABLED, preamplifierEnabled),
            3, "Preamplifier Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the list of equalizer coefficients. A pair of signed
/// HIFN  floating-point values is returned for each coefficient, representing
/// HIFN  the real and imaginary parts. The number of coefficient pairs returned
/// HIFN  depends on the mode, the signal bandwidth, and for OFDM/OFDMA, the FFT
/// HIFN  size.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR equalizerCoefficients/Queries the list of equalizer coefficients. A pair of signed
/// HIPAR equalizerCoefficients/floating-point values is returned for each coefficient, representing
/// HIPAR equalizerCoefficients/the real and imaginary parts. The number of coefficient pairs returned
/// HIPAR equalizerCoefficients/depends on the mode, the signal bandwidth, and for OFDM/OFDMA, the FFT
/// HIPAR equalizerCoefficients/size.
ViStatus rsdsa_QueryCMRangingDownstreamEqualizerCoefficients(ViSession instrSession,
                                                             ViInt32   bufferSize,
                                                             ViReal64  equalizerCoefficients[],
                                                             ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringAsViReal64ArrayToUserBuffer(instrSession, NULL,
        RSDSA_ATTR_CM_RANGING_QUERY_DOWNSTREAM_EQUALIZER_COEFFICIENTS, NULL,
        bufferSize, equalizerCoefficients, bytesRead),
        3, "Equalizer Coefficients");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the upstream mode used for CM ranging analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR upstreamMode/Specifies the upstream mode used for CM ranging analysis.
ViStatus rsdsa_ConfigureCMRangingUpstreamMode(ViSession instrSession,
                                              ViInt32   upstreamMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_CONFIGURE_UPSTREAM_MODE, upstreamMode),
            2, "Upstream Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an IUC to be used as a data filter for displayed status and
/// HIFN  plots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR plotDataFilterIUC/Selects an IUC to be used as a data filter for displayed status and
/// HIPAR plotDataFilterIUC/plots.
ViStatus rsdsa_ConfigureCMRangingUpstreamPlotDataFilterIUC(ViSession instrSession,
                                                           ViInt32   plotDataFilterIUC)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_CONFIGURE_UPSTREAM_PLOT_DATA_FILTER_IUC, plotDataFilterIUC),
            2, "Plot Data Filter IUC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the DOCSIS 3.0 ATDMA.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequency/Specifies the center frequency of the channel. If the spectrum
/// HIPAR frequency/location is changed by CONFigure:CHANnel:SPECtrum:FREQuency, the center
/// HIPAR frequency/frequency is adapted accordingly.
/// HIPAR constellation/Defines the constellation.
/// HIPAR channelSymbolRate/Specifies the symbol rate.
/// HIPAR lastCodewordLength/Specifies the last codeword length.
/// HIPAR guardTimeSize/Specifies the guard time size.
/// HIPAR minislotSize/Specifies the minislot size.
ViStatus rsdsa_ConfigureCMRangingD30A(ViSession instrSession,
                                      ViReal64  frequency,
                                      ViInt32   constellation,
                                      ViInt32   channelSymbolRate,
                                      ViInt32   lastCodewordLength,
                                      ViInt32   guardTimeSize,
                                      ViInt32   minislotSize)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D30A_CONFIGURE_FREQUENCY, frequency),
            2, "Frequency");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D30A_CONFIGURE_CONSTELLATION, constellation),
            3, "Constellation");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D30A_CONFIGURE_CHANNEL_SYMBOL_RATE, channelSymbolRate),
            4, "Channel Symbol Rate");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D30A_CONFIGURE_LAST_CODEWORD_LENGTH, lastCodewordLength),
            5, "Last Codeword Length");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D30A_CONFIGURE_GUARD_TIME_SIZE, guardTimeSize),
            6, "Guard Time Size");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D30A_CONFIGURE_MINISLOT_SIZE, minislotSize),
            7, "Minislot Size");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the FEC parameter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fecParameterT/Specifies the FEC parameter T.
/// HIPAR fecParameterK/Specifies the FEC parameter k.
ViStatus rsdsa_ConfigureCMRangingD30AFECParameter(ViSession instrSession,
                                                  ViInt32   fecParameterT,
                                                  ViInt32   fecParameterK)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D30A_CONFIGURE_FEC_PARAMETER_T, fecParameterT),
            2, "FEC Parameter T");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D30A_CONFIGURE_FEC_PARAMETER_K, fecParameterK),
            3, "FEC Parameter K");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the preamble.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR pattern/Specifies the preamble pattern.
/// HIPAR preambleLength/Specifies the preamble length.
/// HIPAR offset/Specifies the preamble offset.
/// HIPAR type/Specifies the preamble type.
ViStatus rsdsa_ConfigureCMRangingD30APreamble(ViSession instrSession,
                                              ViString  pattern,
                                              ViInt32   preambleLength,
                                              ViInt32   offset,
                                              ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D30A_CONFIGURE_PREAMBLE_PATTERN, pattern),
            2, "Pattern");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D30A_CONFIGURE_PREAMBLE_LENGTH, preambleLength),
            3, "Preamble Length");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D30A_CONFIGURE_PREAMBLE_OFFSET, offset),
            4, "Offset");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D30A_CONFIGURE_PREAMBLE_TYPE, type),
            5, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the scrambler.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Turns scrambling on and off.
/// HIPAR seed/Specifies the scrambler seed.
ViStatus rsdsa_ConfigureCMRangingD30AScrambler(ViSession instrSession,
                                               ViBoolean enabled,
                                               ViString  seed)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D30A_CONFIGURE_SCRAMBLER_ENABLED, enabled),
            2, "Enabled");
    if (enabled)
        viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D30A_CONFIGURE_SCRAMBLER_SEED, seed),
                3, "Seed");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the Reed Solomon.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Specifies the Reed-Solomon interleaver mode.
/// HIPAR interleaverDepth/Specifies the Reed-Solomon interleaver depth, used when the
/// HIPAR interleaverDepth/Reed-Solomon interleaver mode is fixed. Valid range is 2 to
/// HIPAR interleaverDepth/floor[2048/(k+2T)].
/// HIPAR interleaverBlockSize/Specifies the Reed-Solomon interleaver block size, used when the
/// HIPAR interleaverBlockSize/Reed-Solomon interleaver mode is dynamic. Valid range is 2(k+2T) to
/// HIPAR interleaverBlockSize/2048.
ViStatus rsdsa_ConfigureCMRangingD30AReedSolomon(ViSession instrSession,
                                                 ViInt32   mode,
                                                 ViInt32   interleaverDepth,
                                                 ViInt32   interleaverBlockSize)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D30A_CONFIGURE_REED_SOLOMON_MODE, mode),
            2, "Mode");

    if (mode == RSDSA_VAL_REED_SOLOMON_MODE_FIXED)
        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D30A_CONFIGURE_REED_SOLOMON_INTERLEAVER_DEPTH, interleaverDepth),
                3, "Interleaver Depth");

    if (mode == RSDSA_VAL_REED_SOLOMON_MODE_DYNAMIC)
        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D30A_CONFIGURE_REED_SOLOMON_INTERLEAVER_BLOCK_SIZE, interleaverBlockSize),
                4, "Interleaver BlockSize");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the Exclusion Band.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR band/Selects repcap Band.
/// HIPAR start/Defines the start subcarrier of each exclusion band.
/// HIPAR width/Defines the width of each exclusion band in subcarriers.
ViStatus rsdsa_ConfigureCMRangingD31UExclusionBand(ViSession instrSession,
                                                   ViInt32   band,
                                                   ViInt32   start,
                                                   ViInt32   width)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxBand = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Band", &maxBand),
        2, "Band");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, band, 1, maxBand),
            2, "Band");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_EXCLUSION_BAND_START,
            "Band", band - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Band");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, repCap, RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_EXCLUSION_BAND_START, start),
            3, "Start");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, repCap, RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_EXCLUSION_BAND_WIDTH, width),
            4, "Width");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the wideband probe.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR startSubcarrier/Defines the start subcarrier for wideband probe.
/// HIPAR subcarrierSkip/Defines the subcarrier skip for wideband probe.
/// HIPAR staggerEnabled/Controls the stagger for wideband probe.
/// HIPAR symbolInFrame/Defines the symbol in frame for wideband probe.
ViStatus rsdsa_ConfigureCMRangingD31UWidebandProbe(ViSession instrSession,
                                                   ViInt32   startSubcarrier,
                                                   ViInt32   subcarrierSkip,
                                                   ViBoolean staggerEnabled,
                                                   ViInt32   symbolInFrame)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_WIDEBAND_PROBE_START_SUBCARRIER, startSubcarrier),
            2, "Start Subcarrier");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_WIDEBAND_PROBE_SUBCARRIER_SKIP, subcarrierSkip),
            3, "Subcarrier Skip");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_WIDEBAND_PROBE_STAGGER_ENABLED, staggerEnabled),
            4, "Stagger Enabled");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_WIDEBAND_PROBE_SYMBOL_IN_FRAME, symbolInFrame),
            5, "Symbol In Frame");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the BW request.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR numberOfMinislots/Defines the number of minislots in frame for bandwidth requests.
/// HIPAR subslot/Defines the subslot for bandwidth requests.
ViStatus rsdsa_ConfigureCMRangingD31UBWRequest(ViSession instrSession,
                                               ViInt32   numberOfMinislots,
                                               ViInt32   subslot)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_BW_REQUEST_NUMBER_OF_MINISLOTS, numberOfMinislots),
            2, "Number Of Minislots");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_BW_REQUEST_SUBSLOT, subslot),
            3, "Subslot");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the DOCSIS 3.1 OFDMA.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequency/Specifies the center frequency of the channel. If the spectrum
/// HIPAR frequency/location is changed by CONFigure:CHANnel:SPECtrum:FREQuency, the center
/// HIPAR frequency/frequency is adapted accordingly.
/// HIPAR rollOff/Defines the roll-off period, which depends on the channel bandwidth
/// HIPAR rollOff/and the number of excluded carriers within the channel. The larger the
/// HIPAR rollOff/roll-off period, the more time transmission takes; however, the more
/// HIPAR rollOff/useful subcarriers are available in the frequency domain. This command
/// HIPAR rollOff/and query do not apply to OFDM automatic acquisition mode.
/// HIPAR cyclicPrefixLength/Defines the cyclic prefix length, which determines where the useful
/// HIPAR cyclicPrefixLength/data starts and allows the application to detect delay spreads during
/// HIPAR cyclicPrefixLength/transmission. The longer the delay spread, the longer the CP must be.
/// HIPAR cyclicPrefixLength/This command and query do not apply to
/// HIPAR symbolsPerFrame/Defines the number of symbols per frame to be expected.
/// HIPAR subcarrierSpacing/Defines the FFT length, which determines the subcarrier spacing and
/// HIPAR subcarrierSpacing/other parameters. This control do not apply to OFDM automatic
/// HIPAR subcarrierSpacing/acquisition mode.
ViStatus rsdsa_ConfigureCMRangingD31U(ViSession instrSession,
                                      ViReal64  frequency,
                                      ViInt32   rollOff,
                                      ViInt32   cyclicPrefixLength,
                                      ViInt32   symbolsPerFrame,
                                      ViInt32   subcarrierSpacing)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_FREQUENCY, frequency),
            2, "Frequency");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_ROLL_OFF, rollOff),
        3, "Roll Off")

        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_CYCLIC_PREFIX_LENGTH, cyclicPrefixLength),
                4, "Cyclic Prefix Length");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_SYMBOLS_PER_FRAME, symbolsPerFrame),
            5, "Symbols Per Frame");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_SUBCARRIER_SPACING, subcarrierSpacing),
            6, "Subcarrier Spacing");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Controls the randomizer (scrambler).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Controls the randomizer (scrambler).
/// HIPAR seed/Specifies the randomizer (scrambler) seed.
ViStatus rsdsa_ConfigureCMRangingD31URandomizer(ViSession instrSession,
                                                ViBoolean enabled,
                                                ViString  seed)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_RANDOMIZER_ENABLED, enabled),
            2, "Enabled");

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_RANDOMIZER_SEED, seed),
            3, "Seed");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the minislot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR configurationMode/Selects the minislot configuration mode for OFDMA.
ViStatus rsdsa_ConfigureCMRangingD31UMinislotMode(ViSession instrSession,
                                                  ViInt32   configurationMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_MINISLOT_CONFIGURATION_MODE, configurationMode),
            2, "Configuration Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the OFDMA minislot configuration file, which contains upstream
/// HIFN  minislot configuration information. Refer to section 4.7 for the
/// HIFN  configuration file format and contents.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR configurationFile/Loads the OFDMA minislot configuration file, which contains upstream
/// HIPAR configurationFile/minislot configuration information. Refer to section 4.7 for the
/// HIPAR configurationFile/configuration file format and contents.
ViStatus rsdsa_ConfigureCMRangingD31UMinislotConfigurationFile(
                            ViSession instrSession,
                            ViString  configurationFile)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_MINISLOT_CONFIGURATION_FILE, configurationFile),
            2, "Configuration File");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the Minislot Manual.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR constellation/Defines the constellation. This setting is overridden by the optional
/// HIPAR constellation/OFDMA Minislot Configuration File.
/// HIPAR pilotPattern/Defines the pilot pattern. This setting is overridden by the optional
/// HIPAR pilotPattern/OFDMA Minislot Configuration File.
/// HIPAR startingMinislot/Defines the starting minislot. This setting is overridden by the
/// HIPAR startingMinislot/optional OFDMA Minislot Configuration File.
/// HIPAR numberOfMinislots/Specifies the number of minislots. This setting is overridden by the
/// HIPAR numberOfMinislots/optional OFDMA Minislot Configuration File.
ViStatus rsdsa_ConfigureCMRangingD31UMinislotManual(ViSession instrSession,
                                                    ViInt32   constellation,
                                                    ViInt32   pilotPattern,
                                                    ViInt64   startingMinislot,
                                                    ViInt32   numberOfMinislots)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_CONSTELLATION, constellation),
            2, "Constellation");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_PILOT_PATTERN, pilotPattern),
            3, "Pilot Pattern");

    viCheckParm(rsdsa_SetAttributeViInt64(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_STARTING_MINISLOT, startingMinislot),
        4, "Starting Minislot");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_NUMBER_OF_MINISLOTS, numberOfMinislots),
            5, "Number Of Minislots");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the active subcarrier.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR firstActiveSubcarrier/Defines the first subcarrier in the active region.
/// HIPAR lastActiveSubcarrier/Defines the last subcarrier in the active region.
ViStatus rsdsa_ConfigureCMRangingD31UActiveSubcarrier(ViSession instrSession,
                                                      ViInt32   firstActiveSubcarrier,
                                                      ViInt32   lastActiveSubcarrier)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_FIRST_ACTIVE_SUBCARRIER, firstActiveSubcarrier),
            2, "First Active Subcarrier");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_LAST_ACTIVE_SUBCARRIER, lastActiveSubcarrier),
            3, "Last Active Subcarrier");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the starting minislot in frame for initial ranging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR minislotInFrame/Defines the starting minislot in frame for initial ranging.
/// HIPAR numberOfMinislots/Defines the number of minislots for initial ranging.
/// HIPAR numberOfSubcarriers/Defines the number of subcarriers for initial ranging (must be an even
/// HIPAR numberOfSubcarriers/number).
ViStatus rsdsa_ConfigureCMRangingD31UInitialRanging(ViSession instrSession,
                                                    ViInt32   minislotInFrame,
                                                    ViInt32   numberOfMinislots,
                                                    ViInt32   numberOfSubcarriers)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_INITIAL_RANGING_STARTING_MINISLOT_IN_FRAME, minislotInFrame),
            2, "Minislot In Frame");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_INITIAL_RANGING_NUMBER_OF_MINISLOTS, numberOfMinislots),
            3, "Number Of Minislots");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_INITIAL_RANGING_NUMBER_OF_SUBCARRIERS, numberOfSubcarriers),
            4, "Number Of Subcarriers");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the preamble for initial ranging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR pattern/Specifies the preamble pattern for initial ranging.
/// HIPAR length/Specifies the preamble length for initial ranging.
/// HIPAR offset/Specifies the preamble offset for initial ranging.
ViStatus rsdsa_ConfigureCMRangingD31UInitialRangingPreamble(ViSession instrSession,
                                                            ViString  pattern,
                                                            ViInt32   length,
                                                            ViInt32   offset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_INITIAL_RANGING_PREAMBLE_PATTERN, pattern),
            2, "Patter");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_INITIAL_RANGING_PREAMBLE_LENGTH, length),
            3, "Length");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_INITIAL_RANGING_PREAMBLE_OFFSET, offset),
            4, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the fine ranging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR startingMinislotInFrame/Defines the starting minislot in frame for fine ranging.
/// HIPAR numberOfMinislots/Defines the number of minislots for fine ranging.
/// HIPAR numberOfSubcarriers/Defines the number of subcarriers for fine ranging (must be an even
/// HIPAR numberOfSubcarriers/number).
ViStatus rsdsa_ConfigureCMRangingD31UFineRanging(ViSession instrSession,
                                                 ViInt32   startingMinislotInFrame,
                                                 ViInt32   numberOfMinislots,
                                                 ViInt32   numberOfSubcarriers)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_FINE_RANGING_STARTING_MINISLOT_IN_FRAME, startingMinislotInFrame),
            2, "Starting Minislot In Frame");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_FINE_RANGING_NUMBER_OF_MINISLOTS, numberOfMinislots),
            3, "Number Of Minislots");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_FINE_RANGING_NUMBER_OF_SUBCARRIERS, numberOfSubcarriers),
            4, "Number Of Subcarriers");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the Fine Ranging Preamble.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR pattern/Specifies the preamble pattern for fine ranging.
/// HIPAR offset/Specifies the preamble offset for fine ranging.
ViStatus rsdsa_ConfigureCMRangingD31UFineRangingPreamble(ViSession instrSession,
                                                         ViString  pattern,
                                                         ViInt32   offset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_FINE_RANGING_PREAMBLE_PATTERN, pattern),
            2, "Patter");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_FINE_RANGING_PREAMBLE_OFFSET, offset),
            3, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the SFP+ network output on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Turns the SFP+ network output on and off.
ViStatus rsdsa_ConfigureCMRangingSFPOutputEnabled(ViSession instrSession,
                                                  ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_CM_RANGING_CONFIGURE_SFP_OUTPUT_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the IUC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR iuc_ofdma/Selects repcap IUC_OFDMA.
/// HIPAR enabled/Turns the SFP+ IUC UDP port on and off.
/// HIPAR udpPort/Configures the UDP port to be used for SFP+ IUC output. The UDP ports
/// HIPAR udpPort/must be uniquely assigned to downstream OFDM profiles/PLC and upstream
/// HIPAR udpPort/DOCSIS IUCs.
ViStatus rsdsa_ConfigureCMRangingSFPIUCUDPPort(ViSession instrSession,
                                               ViInt32   iuc_ofdma,
                                               ViBoolean enabled,
                                               ViInt32   udpPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxOFDMA = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "IUC_OFDMA", &maxOFDMA),
        2, "IUC OFDMA");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, iuc_ofdma, 1, maxOFDMA),
            2, "IUC OFDMA");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_CM_RANGING_CONFIGURE_SFP_IUC_UDP_PORT,
            "IUC_OFDMA", iuc_ofdma - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "IUC OFDMA");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, repCap, RSDSA_ATTR_CM_RANGING_CONFIGURE_SFP_IUC_UDP_PORT_ENABLED, enabled),
            3, "Enabled");

    if (enabled)
    {
        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, repCap, RSDSA_ATTR_CM_RANGING_CONFIGURE_SFP_IUC_UDP_PORT, udpPort),
                4, "UDP Port");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Automatically determines the attenuation level and preamplifier state
/// HIFN  (on/off).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_CMRangingUpstreamAutoLevel(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_CM_RANGING_UPSTREAM_AUTO_LEVEL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the Signal Path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attenuation/Defines the total attenuation for RF input in dB. If you set the
/// HIPAR attenuation/attenuation manually, it is no longer coupled to the reference level,
/// HIPAR attenuation/but the reference level is coupled to the attenuation. Thus, if the
/// HIPAR attenuation/current reference level is not compatible with an attenuation that has
/// HIPAR attenuation/been set manually, the command also adjusts the reference level.
/// HIPAR attenuation/Therefore, the reference level set in
/// HIPAR attenuation/DISPlay:[<Mode>:][WINDow:]TRACe[<Trace>]:Y[:SCALe]:RLEVel is no longer
/// HIPAR attenuation/applied. Note: The query returns the actual attenuation setting, which
/// HIPAR attenuation/can also be changed by INPut:<Direction>:LEVel:AUTO.
/// HIPAR preamplifierEnabled/Turns the preamplifier on and off. Note: The query returns the actual
/// HIPAR preamplifierEnabled/preamplifier state (which might also be changed by
/// HIPAR preamplifierEnabled/RSDSA_ATTR_CM_RANGING_D31U_AUTO_LEVEL).
ViStatus rsdsa_ConfigureCMRangingUpstream(ViSession instrSession,
                                          ViReal64  attenuation,
                                          ViBoolean preamplifierEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_CM_RANGING_CONFIGURE_UPSTREAM_ATTENUATION, attenuation),
            2, "Attenuation");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_CM_RANGING_CONFIGURE_UPSTREAM_PREAMPLIFIER_ENABLED, preamplifierEnabled),
            3, "Preamplifier Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the upstream DOCSIS 3.1 OFDMA mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_EnableDOCSIS31OFDMAMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_ENABLE_DOCSIS_31_OFDMA_MODE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an IUC to be used as a data filter for displayed status and
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mapGenIUC/Selects an IUC to be used as a data filter for displayed status and
/// HIPAR mapGenIUC/plots.
ViStatus rsdsa_ConfigureD31UMapGenIUC(ViSession instrSession,
                                      ViInt32   mapGenIUC)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_MAP_GEN_IUC, mapGenIUC),
            2, "Map Gen IUC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the burst demodulator lock status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstDemodulatorLock/Queries the burst demodulator lock status.
ViStatus rsdsa_QueryD31UBurstDemodulatorLockStatus(ViSession instrSession,
                                                   ViInt32*  burstDemodulatorLock)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_QUERY_BURST_DEMODULATOR_LOCK_STATUS, burstDemodulatorLock),
            2, "Burst Demodulator Lock");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the forward error correction (FEC) lock status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fecLock/Queries the forward error correction (FEC) lock status.
ViStatus rsdsa_QueryD31UFECLockStatus(ViSession instrSession,
                                      ViInt32*  fecLock)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_QUERY_FEC_LOCK_STATUS, fecLock),
            2, "FEC Lock");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the burst receive power level in the units selected by
/// HIFN  RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR readBurstReceivePower/Queries the burst receive power level in the units selected by
/// HIPAR readBurstReceivePower/RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_QueryD31UReadBurstReceivePower(ViSession instrSession,
                                              ViReal64* readBurstReceivePower)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_READ_BURST_RECEIVE_POWER, readBurstReceivePower),
            2, "Read Burst Recieve Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the input signal power level, per 1.6 MHz, in the units
/// HIFN  selected by RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR signalPowerPer1600kHz/Queries the input signal power level, per 1.6 MHz, in the units
/// HIPAR signalPowerPer1600kHz/selected by RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_QueryD31USignalPowerPer1600kHz(ViSession instrSession,
                                              ViReal64* signalPowerPer1600kHz)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_QUERY_SIGNAL_POWER_PER_1600KHZ, signalPowerPer1600kHz),
            2, "Signal Power Per 1600 kHz");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the frequency offset in Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequencyOffset/Queries the frequency offset in Hz.
ViStatus rsdsa_QueryD31UFrequencyOffset(ViSession instrSession,
                                        ViReal64* frequencyOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_QUERY_FREQUENCY_OFFSET, frequencyOffset),
            2, "Frequency Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the burst timing offset from the time the burst was detected
/// HIFN  and the time the burst was supposed to occur, in seconds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstTimingOffset/Queries the burst timing offset from the time the burst was detected
/// HIPAR burstTimingOffset/and the time the burst was supposed to occur, in seconds.
ViStatus rsdsa_QueryD31UBurstTimingOffset(ViSession instrSession,
                                          ViReal64* burstTimingOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_QUERY_BURST_TIMING_OFFSET, burstTimingOffset),
            2, "Burst Timing Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the payload data rate in Mbps (megabits per second) for the
/// HIFN  IUC currently selected by DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR payloadDataRateIUC/Queries the payload data rate in Mbps (megabits per second) for the
/// HIPAR payloadDataRateIUC/IUC currently selected by DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>.
ViStatus rsdsa_QueryD31UPayloadDataRateIUC(ViSession instrSession,
                                           ViReal64* payloadDataRateIUC)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_QUERY_PAYLOAD_DATA_RATE_IUC, payloadDataRateIUC),
            2, "Payload Data Rate IUC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of minislots received.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR minislotsReceived/Queries the number of minislots received.
ViStatus rsdsa_QueryD31UMinislotsReceived(ViSession instrSession,
                                          ViInt64*  minislotsReceived)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt64(instrSession, NULL, RSDSA_ATTR_D31U_QUERY_MINISLOTS_RECEIVED, minislotsReceived),
        2, "Minislots Recieved");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the percentage of valid bursts out of all bursts received over
/// HIFN  the last one second period.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstCoverage/Queries the percentage of valid bursts out of all bursts received over
/// HIPAR burstCoverage/the last one second period.
ViStatus rsdsa_QueryD31UBurstCoverage(ViSession instrSession,
                                      ViReal64* burstCoverage)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_QUERY_BURST_COVERAGE, burstCoverage),
            2, "Burst Coverage");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the sign preferred for MER numerical display and plot axis
/// HIFN  labels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merSign/Selects the sign preferred for MER numerical display and plot axis
/// HIPAR merSign/labels.
ViStatus rsdsa_ConfigureD31UMERSign(ViSession instrSession,
                                    ViInt32   merSign)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_MER_SIGN, merSign),
            2, "MER Sign");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall RMS equalized MER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merRMS/Queries the overall RMS equalized MER.
ViStatus rsdsa_QueryD31UMERRMS(ViSession instrSession,
                               ViReal64* merRMS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_QUERY_MER_RMS, merRMS),
            2, "MER RMS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall RMS partially equalized MER. For OFDMA mode, it is
/// HIFN  only valid for data bursts with grant sizes greater than 100 using
/// HIFN  pilots 4 or 11.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merRMSPartialyEqualized/Queries the overall RMS partially equalized MER. For OFDMA mode, it is
/// HIPAR merRMSPartialyEqualized/only valid for data bursts with grant sizes greater than 100 using
/// HIPAR merRMSPartialyEqualized/pilots 4 or 11.
ViStatus rsdsa_QueryD31UMERRMSPartialyEqualized(ViSession instrSession,
                                                ViReal64* merRMSPartialyEqualized)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_QUERY_MER_RMS_PARTIALY_EQUALIZED, merRMSPartialyEqualized),
            2, "MER RMS Partially Equalized");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall peak (lowest) MER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merPeak/Queries the overall peak (lowest) MER.
ViStatus rsdsa_QueryD31UMERPeak(ViSession instrSession,
                                ViReal64* merPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_QUERY_MER_PEAK, merPeak),
            2, "MER Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the RMS MER for the pilot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merPilotRMS/Queries the RMS MER for the pilot.
ViStatus rsdsa_QueryD31UMERPilotRMS(ViSession instrSession,
                                    ViReal64* merPilotRMS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_QUERY_MER_PILOT_RMS, merPilotRMS),
            2, "MER Pilot RMS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the peak (lowest) MER for the pilot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merPilotPeak/Queries the peak (lowest) MER for the pilot.
ViStatus rsdsa_QueryD31UMERPilotPeak(ViSession instrSession,
                                     ViReal64* merPilotPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_QUERY_MER_PILOT_PEAK, merPilotPeak),
            2, "MER Pilot Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the constellations for the pilot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR pilotConstellations/Queries the constellations for the pilot.
ViStatus rsdsa_QueryD31UPilotConstellations(ViSession instrSession,
                                            ViInt32   bufferSize,
                                            ViChar    pilotConstellations[],
                                            ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSDSA_ATTR_D31U_QUERY_PILOT_CONSTELLATIONS, RS_VAL_DIRECT_USER_CALL, bufferSize, pilotConstellations, bytesRead),
        2, "Pilot Constellation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the RMS MER for the complementary pilot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merComplementaryPilotRMS/Queries the RMS MER for the complementary pilot.
ViStatus rsdsa_QueryD31UMERComplementaryPilotRMS(ViSession instrSession,
                                                 ViReal64* merComplementaryPilotRMS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_QUERY_MER_COMPLEMENTARY_PILOT_RMS, merComplementaryPilotRMS),
            2, "MER Complementary Pilot RMS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the peak (lowest) MER for the complementary pilot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merComplementaryPilotPeak/Queries the peak (lowest) MER for the complementary pilot.
ViStatus rsdsa_QueryD31UMERComplementaryPilotPeak(ViSession instrSession,
                                                  ViReal64* merComplementaryPilotPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_QUERY_MER_COMPLEMENTARY_PILOT_PEAK, merComplementaryPilotPeak),
            2, "MER Complimentary Pilot Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the constellations for the complementary pilot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR pilotConstellations/Queries the constellations for the complementary pilot.
ViStatus rsdsa_QueryD31UComplementaryPilotConstellations(ViSession instrSession,
                                                         ViInt32   bufferSize,
                                                         ViChar    pilotConstellations[],
                                                         ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSDSA_ATTR_D31U_QUERY_COMPLEMENTARY_PILOT_CONSTELLATIONS, RS_VAL_DIRECT_USER_CALL, bufferSize, pilotConstellations, bytesRead),
        2, "Pilot Constellation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the RMS MER for the IUC currently selected by
/// HIFN  DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merMapGenIUCRMS/Queries the RMS MER for the IUC currently selected by
/// HIPAR merMapGenIUCRMS/DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>.
ViStatus rsdsa_QueryD31UMERMapGenIUCRMS(ViSession instrSession,
                                        ViReal64* merMapGenIUCRMS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_QUERY_MER_MAP_GEN_IUC_RMS, merMapGenIUCRMS),
            2, "MER Map Gen IUC RMS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the peak (lowest) MER for the IUC currently selected by
/// HIFN  DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merMapGenIUCPeak/Queries the peak (lowest) MER for the IUC currently selected by
/// HIPAR merMapGenIUCPeak/DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>.
ViStatus rsdsa_QueryD31UMERMapGenIUCPeak(ViSession instrSession,
                                         ViReal64* merMapGenIUCPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_QUERY_MER_MAP_GEN_IUC_PEAK, merMapGenIUCPeak),
            2, "MER Map Gen IUC Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the constellations for the currently selected IUC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR iucConstellations/Queries the constellations for the currently selected IUC.
ViStatus rsdsa_QueryD31UIUCConstellations(ViSession instrSession,
                                          ViInt32   bufferSize,
                                          ViChar    iucConstellations[],
                                          ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSDSA_ATTR_D31U_QUERY_IUC_CONSTELLATIONS, RS_VAL_DIRECT_USER_CALL, bufferSize, iucConstellations, bytesRead),
        2, "IUC Constellation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Performs a BER reset that clears all related counters to zero and
/// HIFN  restarts the BER/CER measurement for the current mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_ResetD31UErrorAnalysis(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31U_RESET_ERROR_ANALYSIS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures Error Analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR testDepth/Defines the test depth of the minimum BER integration. The query
/// HIPAR testDepth/returns the actual test depth setting, which may be automatically
/// HIPAR testDepth/changed in automatic BER measurement mode.
/// HIPAR mode/Specifies the mode of BER measurement.
/// HIPAR testDepthSource/Selects the BER source used for BER measurements.
/// HIPAR progressionSource/Selects the BER source used for test depth progression.
ViStatus rsdsa_ConfigureD31UErrorAnalysis(ViSession instrSession,
                                          ViInt32   mode,
                                          ViInt32   testDepth,
                                          ViInt32   testDepthSource,
                                          ViInt32   progressionSource)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_ERROR_ANALYSIS_MODE, mode),
            2, "Mode");

    if (mode == RSDSA_VAL_ERROR_ANALYSIS_MANUAL)
    {
        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH, testDepth),
                3, "Test Depth");
    }

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH_SOURCE, testDepthSource),
            4, "Test Depth Source");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH_PROGRESSION_SOURCE, progressionSource),
            5, "Progression Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the elapsed time of the current BER/CER measurement: time
/// HIFN  elapsed since the start of the mode, or last BER reset
/// HIFN  (RSDSA_ATTR_D31U_RESET_ERROR_ANALYSIS), or last BER configuration
/// HIFN  change, in seconds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR berElapsedTime/Queries the elapsed time of the current BER/CER measurement: time
/// HIPAR berElapsedTime/elapsed since the start of the mode, or last BER reset
/// HIPAR berElapsedTime/(RSDSA_ATTR_D31U_RESET_ERROR_ANALYSIS), or last BER configuration
/// HIPAR berElapsedTime/change, in seconds.
ViStatus rsdsa_QueryD31UBERElapsedTime(ViSession instrSession,
                                       ViReal64* berElapsedTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_QUERY_BER_ELAPSED_TIME, berElapsedTime),
            2, "BER Elapsed Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the percentage of blocks processed for the current BER/CER
/// HIFN  measurement: blocks processed since the start of the mode, or last BER
/// HIFN  reset (CALCulate:[<Mode>:][AVERage:]BERatio:CLEar), or last BER
/// HIFN  configuration change. Expressed as the percentage of blocks processed
/// HIFN  relative to the current test depth. A block is 10^7 bits maximum.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR berBlocksProcessed/Queries the percentage of blocks processed for the current BER/CER
/// HIPAR berBlocksProcessed/measurement: blocks processed since the start of the mode, or last BER
/// HIPAR berBlocksProcessed/reset (CALCulate:[<Mode>:][AVERage:]BERatio:CLEar), or last BER
/// HIPAR berBlocksProcessed/configuration change. Expressed as the percentage of blocks processed
/// HIPAR berBlocksProcessed/relative to the current test depth. A block is 10^7 bits maximum.
ViStatus rsdsa_QueryD31UBERBlocksProcessed(ViSession instrSession,
                                           ViReal64* berBlocksProcessed)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_QUERY_BER_BLOCKS_PROCESSED, berBlocksProcessed),
            2, "BER Blocks Processed");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of bursts expected since the signal was acquired.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR berBurstsExpected/Queries the number of bursts expected since the signal was acquired.
ViStatus rsdsa_QueryD31UBERBurstsExpected(ViSession instrSession,
                                          ViReal64* berBurstsExpected)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_QUERY_BER_BURSTS_EXPECTED, berBurstsExpected),
            2, "BER Bursts Expected");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of bursts received since the signal was acquired.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR berBurstsReceived/Queries the number of bursts received since the signal was acquired.
ViStatus rsdsa_QueryD31UBERBurstsReceived(ViSession instrSession,
                                          ViReal64* berBurstsReceived)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_QUERY_BER_BURSTS_RECEIVED, berBurstsReceived),
            2, "BER Bursts Received");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the pre-LDPC BER for the IUC currently selected by
/// HIFN  DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR berIUCPreLDPC/Queries the pre-LDPC BER for the IUC currently selected by
/// HIPAR berIUCPreLDPC/DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>.
ViStatus rsdsa_QueryD31UBERIUCPreLDPC(ViSession instrSession,
                                      ViReal64* berIUCPreLDPC)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_QUERY_BER_IUC_PRE_LDPC, berIUCPreLDPC),
            2, "BER IUC Pre-LDPC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of bits analyzed at the input of the LDPC
/// HIFN  decoder for the BER measurement, for the IUC currently selected by
/// HIFN  DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR berIUCPreLDPCTotalBits/Queries the total number of bits analyzed at the input of the LDPC
/// HIPAR berIUCPreLDPCTotalBits/decoder for the BER measurement, for the IUC currently selected by
/// HIPAR berIUCPreLDPCTotalBits/DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>.
ViStatus rsdsa_QueryD31UBERIUCPreLDPCTotalBits(ViSession instrSession,
                                               ViReal64* berIUCPreLDPCTotalBits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_QUERY_BER_IUC_PRE_LDPC_TOTAL_BITS, berIUCPreLDPCTotalBits),
            2, "BER IUC Pre-LDPC Total Bits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the post-LDPC BER for the IUC currently selected by
/// HIFN  DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR berIUCPostLDPC/Queries the post-LDPC BER for the IUC currently selected by
/// HIPAR berIUCPostLDPC/DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>.
ViStatus rsdsa_QueryD31UBERIUCPostLDPC(ViSession instrSession,
                                       ViReal64* berIUCPostLDPC)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_QUERY_BER_IUC_POST_LDPC, berIUCPostLDPC),
            2, "BER IUC Post-LDPC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of bits analyzed at the output of the LDPC
/// HIFN  decoder for the BER measurement, for the IUC currently selected by
/// HIFN  DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>. Note that the bits for
/// HIFN  undecodable codewords are not included in this total.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR berIUCPostLDPCTotalBits/Queries the total number of bits analyzed at the output of the LDPC
/// HIPAR berIUCPostLDPCTotalBits/decoder for the BER measurement, for the IUC currently selected by
/// HIPAR berIUCPostLDPCTotalBits/DISPlay:[<Mode>:][WINDow:]IUCFilter <Iuc>. Note that the bits for
/// HIPAR berIUCPostLDPCTotalBits/undecodable codewords are not included in this total.
ViStatus rsdsa_QueryD31UBERIUCPostLDPCTotalBits(ViSession instrSession,
                                                ViReal64* berIUCPostLDPCTotalBits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_QUERY_BER_IUC_POST_LDPC_TOTAL_BITS, berIUCPostLDPCTotalBits),
            2, "BER IUC Post-LDPC Total Bits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the codeword length(s) to be used for displayed status (i.e.,
/// HIFN  codeword length filter).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR codewordLengthFilter/Selects the codeword length(s) to be used for displayed status (i.e.,
/// HIPAR codewordLengthFilter/codeword length filter).
ViStatus rsdsa_ConfigureD31UCodewordLengthFilter(ViSession instrSession,
                                                 ViInt32   codewordLengthFilter)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_CODEWORD_LENGTH_FILTER, codewordLengthFilter),
            2, "Codeword Length Filter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the post-LDPC CER, for the codeword length currently selected
/// HIFN  by DISPlay:[<Mode>:][WINDow:]CLFilter <CwLength>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR postLDPC/Queries the post-LDPC CER, for the codeword length currently selected
/// HIPAR postLDPC/by DISPlay:[<Mode>:][WINDow:]CLFilter <CwLength>.
ViStatus rsdsa_QueryD31UCERPostLDPC(ViSession instrSession,
                                    ViReal64* postLDPC)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_QUERY_CER_POST_LDPC, postLDPC),
            2, "Post-LDPC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of post-LDPC codeword errors, for the
/// HIFN  codeword length currently selected by
/// HIFN  DISPlay:[<Mode>:][WINDow:]CLFilter <CwLength>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR postLDPCCodewordErrors/Queries the total number of post-LDPC codeword errors, for the
/// HIPAR postLDPCCodewordErrors/codeword length currently selected by
/// HIPAR postLDPCCodewordErrors/DISPlay:[<Mode>:][WINDow:]CLFilter <CwLength>.
ViStatus rsdsa_QueryD31UCERPostLDPCCodewordErrors(ViSession instrSession,
                                                  ViReal64* postLDPCCodewordErrors)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_QUERY_CER_POST_LDPC_CODEWORD_ERRORS, postLDPCCodewordErrors),
            2, "Post-LDPC Codeword Errors");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the average number of LDPC iterations, for the codeword length
/// HIFN  currently selected by DISPlay:[<Mode>:][WINDow:]CLFilter <CwLength>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR averageLDPCIterations/Queries the average number of LDPC iterations, for the codeword length
/// HIPAR averageLDPCIterations/currently selected by DISPlay:[<Mode>:][WINDow:]CLFilter <CwLength>.
ViStatus rsdsa_QueryD31UCERAverageLDPCIterations(ViSession instrSession,
                                                 ViReal64* averageLDPCIterations)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_QUERY_CER_AVERAGE_LDPC_ITERATIONS, averageLDPCIterations),
            2, "Average LDPC Iterations");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of bits in the average payload per codeword,
/// HIFN  for the IUC currently selected by DISPlay:[<Mode>:][WINDow:]IUCFilter
/// HIFN  <Iuc>. For OFDMA, this command also applies to the codeword length
/// HIFN  currently selected by DISPlay:[<Mode>:][WINDow:]CLFilter <CwLength>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR payloadPerCodewordPerIUC/Queries the total number of bits in the average payload per codeword,
/// HIPAR payloadPerCodewordPerIUC/for the IUC currently selected by DISPlay:[<Mode>:][WINDow:]IUCFilter
/// HIPAR payloadPerCodewordPerIUC/<Iuc>. For OFDMA, this command also applies to the codeword length
/// HIPAR payloadPerCodewordPerIUC/currently selected by DISPlay:[<Mode>:][WINDow:]CLFilter <CwLength>.
ViStatus rsdsa_QueryD31UCERAvgPayloadPerCodewordPerIUC(ViSession instrSession,
                                                       ViReal64* payloadPerCodewordPerIUC)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_QUERY_CER_AVG_PAYLOAD_PER_CODEWORD_PER_IUC, payloadPerCodewordPerIUC),
            2, "Payload Per Codeword Per IUC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the percentage of codewords covered (i.e., not discarded), for
/// HIFN  the IUC currently selected by DISPlay:[<Mode>:][WINDow:]IUCFilter
/// HIFN  <Iuc>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cerCodewordCoverageIUC/Queries the percentage of codewords covered (i.e., not discarded), for
/// HIPAR cerCodewordCoverageIUC/the IUC currently selected by DISPlay:[<Mode>:][WINDow:]IUCFilter
/// HIPAR cerCodewordCoverageIUC/<Iuc>.
ViStatus rsdsa_QueryD31UCERCodewordCoverageIUC(ViSession instrSession,
                                               ViReal64* cerCodewordCoverageIUC)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_QUERY_CER_CODEWORD_COVERAGE_IUC, cerCodewordCoverageIUC),
            2, "CER Codeword Coverage IUC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the constellation plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31UDisplayConstellationPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31U_DISPLAY_CONSTELLATION_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the I/Q density plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31UDisplayIQDensityPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31U_DISPLAY_I_Q_DENSITY_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the I/Q density plot.tion.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Selects the measurement mode for the I/Q density plot.
/// HIPAR windowWidth/Specifies the sliding window width in samples for the I/Q density plot
/// HIPAR windowWidth/in sliding window mode.
ViStatus rsdsa_ConfigureD31UIQ(ViSession instrSession,
                               ViInt32   mode,
                               ViInt32   windowWidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_I_Q_MEASUREMENT_MODE, mode),
            2, "Mode");

    if (mode == RSDSA_VAL_IQ_MEASUREMENT_MODE_SLIDING_WINDOW)
        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_I_Q_WINDOW_WIDTH, windowWidth),
                3, "Window Width");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the zoom setting for the I/Q density plot display. Note that
/// HIFN  the x and y origin are the top left corner of the window, as per the
/// HIFN  GUI settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR x/X origin (left-most window point), between 0 and 1.
/// HIPAR y/Y origin (top-most window point), between 0 and 1.
/// HIPAR zoom/Zoom setting, 1 to 128.
ViStatus rsdsa_D31USetIQDisplayZoom(ViSession instrSession,
                                    ViReal64  x,
                                    ViReal64  y,
                                    ViInt32   zoom)
{
    ViStatus error = VI_SUCCESS;
    ViChar   parameterString[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_CheckRangeTableRangeViReal64(instrSession, &RsDSA_rngIQZoomOrigin, x),
        2, "X");
    viCheckParm(rsdsa_CheckRangeTableRangeViReal64(instrSession, &RsDSA_rngIQZoomOrigin, y),
        3, "Y");
    viCheckParm(rsdsa_CheckRangeTableRangeViInt32(instrSession, &RsDSA_rngIQZoomLevel, zoom),
        4, "Zoom");

    snprintf(parameterString, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%f,%f,%ld", x, y, zoom);

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_HIDDEN_D31U_DISPLAY_I_Q_ZOOM, parameterString));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Save the constellation I/Q data to the specified file. The file
/// HIFN  extension is .iq.tar. Note that there is no way to abort this command,
/// HIFN  and large file sizes could take a long time to capture depending on the
/// HIFN  input data, so use maxSamples or timeoutSeconds to limit the file size
/// HIFN  or time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filename/File path, uses .iq.tar extension.
/// HIPAR maxSamples/Sets the I/Q data file maximum size in samples.
/// HIPAR timeoutSeconds/Sets the maximum time in seconds to save data.
ViStatus rsdsa_D31UCaptureIQDataToFile(ViSession instrSession,
                                       ViString  filename,
                                       ViInt64   maxSamples,
                                       ViReal64  timeoutSeconds)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt64(instrSession, NULL, RSDSA_ATTR_D31U_IQ_DATA_EXPORT_MAX_SAMPLES, maxSamples),
        3, "Max Samples");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_IQ_DATA_EXPORT_TIMEOUT, timeoutSeconds),
            4, "Timeout Seconds");

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31U_CAPTURE_IQ_DATA_TO_FILENAME, filename),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the MER vs. time plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31UDisplayMERVsTimePlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31U_DISPLAY_MER_VS_TIME_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a time filter period to be used for displayed MER vs. Time
/// HIFN  plots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filterPeriod/Selects a time filter period to be used for displayed MER vs. Time
/// HIPAR filterPeriod/plots.
ViStatus rsdsa_ConfigureD31UMERVsTimePlotFilterPeriod(ViSession instrSession,
                                                      ViInt32   filterPeriod)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_MER_VS_TIME_PLOT_FILTER_PERIOD, filterPeriod),
            2, "Filter Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the BER vs. time plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31UDisplayBERVsTimePlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31U_DISPLAY_BER_VS_TIME_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a time filter period to be used for displayed BER vs. Time
/// HIFN  plots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filterPeriod/Selects a time filter period to be used for displayed BER vs. Time
/// HIPAR filterPeriod/plots.
ViStatus rsdsa_ConfigureD31UBERVsTimePlotFilterPeriod(ViSession instrSession,
                                                      ViInt32   filterPeriod)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_BER_VS_TIME_PLOT_FILTER_PERIOD, filterPeriod),
            2, "Filter Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the POWer vs. time plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31UDisplayPowerVsTimePlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31U_DISPLAY_POWER_VS_TIME_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a time filter period to be used for displayed Power vs. Time
/// HIFN  plots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filterPeriod/Selects a time filter period to be used for displayed Power vs. Time
/// HIPAR filterPeriod/plots.
ViStatus rsdsa_ConfigureD31UPowerVsTimePlotFilterPeriod(ViSession instrSession,
                                                        ViInt32   filterPeriod)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_POWER_VS_TIME_PLOT_FILTER_PERIOD, filterPeriod),
            2, "Filter Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the MER vs. Subcarrier plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31UDisplayMERVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31U_DISPLAY_MER_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the MER vs. minislot plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31UDisplayMERVsMinislotPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31U_DISPLAY_MER_VS_MINISLOT_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays a downstream or upstream CCDF (Complementary Cumulative
/// HIFN  Distribution Function histogram as per CM-SP-PHYv3.1-I16 sections 9.3.8
/// HIFN  or 9.4.6 respectively). This command may take up to 30 seconds to
/// HIFN  execute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31UDisplayCCDFHistogram(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31U_DISPLAY_CCDF_HISTOGRAM, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the peak CCDF power level in the units selected by
/// HIFN  RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR d31uQueryCCDFPeak/Queries the peak CCDF power level in the units selected by
/// HIPAR d31uQueryCCDFPeak/RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_QueryD31UCCDFPeak(ViSession instrSession,
                                 ViReal64* CCDFPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_QUERY_CCDF_PEAK, CCDFPeak),
        2, "CCDF Peak")

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the channel analysis amplitude/phase vs. subcarrier plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31UDisplayAmplitudePhaseVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31U_DISPLAY_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Scales the y-axis for the channel analysis amplitude/phase vs.
/// HIFN  subcarrier plot to best fit the measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31UAutoScaleAmplitudePhaseVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31U_AUTO_SCALE_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the channel analysis amplitude/group delay vs. subcarrier
/// HIFN  plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31UDisplayAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31U_DISPLAY_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Scales the y-axis for the channel analysis amplitude/group delay vs.
/// HIFN  subcarrier plot to best fit the measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31UAutoScaleAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31U_AUTO_SCALE_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the channel analysis echo pattern plot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31UDisplayEchoPatternPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31U_DISPLAY_ECHO_PATTERN_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the spectrum analysis plot for DOCSIS 3.1 Downstream mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31UDisplaySpectrumAnalysisPlot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31U_DISPLAY_SPECTRUM_ANALYSIS_PLOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restarts a single sweep, rearming for the next trigger, if applicable.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31USpectrumAnalysisRunSingleSweep(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_RUN_SINGLE_SWEEP, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Scales the y-axis for trace displays by automatically adjusting the
/// HIFN  y-axis reference level and scale per division to best fit the
/// HIFN  measurement results. Otherwise, the y-axis can be manually scaled using
/// HIFN  DISPlay:[<Mode>:][WINDow:]TRACe[<Trace>]:Y[:SCALe]:RLEVel and
/// HIFN  DISPlay:[<Mode>:][WINDow:]TRACe[<Trace>]:Y[:SCALe]:PDIVision.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31USpectrumAnalysisAmplitudeAutoScale(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_AUTO_SCALE_Y_AXIS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures spectrum analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR traceMode/Selects the measurement mode for the displayed trace. The DSA applies
/// HIPAR traceMode/minimum/maximum hold over multiple single traces.
/// HIPAR captureMode/Selects whether to use continuous captures or single trace captures.
/// HIPAR automaticAverages/When turned on, the DSA automatically determines the average setting.
ViStatus rsdsa_ConfigureD31USpectrumAnalysis(ViSession instrSession,
                                             ViInt32   traceMode,
                                             ViInt32   captureMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_CONFIGURE_TRACE_HOLD, traceMode),
            2, "Trace Mode");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_CONFIGURE_CAPTURE_MODE, captureMode),
            3, "Capture Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures spectrum analysis amplitude.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR scalePerDivision/Defines the scale per division in the units selected by
/// HIPAR scalePerDivision/RSDSA_ATTR_POWER_UNITS to be used for the y-axis for all trace
/// HIPAR scalePerDivision/displays, which all have 10 divisions on the y-axis. Note that this
/// HIPAR scalePerDivision/command sets the scale per division, whereas the user of the DSA GUI
/// HIPAR scalePerDivision/sets the entire range for the vertical axis.
/// HIPAR referenceLevel/Defines the reference level for the displayed trace in the units
/// HIPAR referenceLevel/selected by RSDSA_ATTR_POWER_UNITS. This setting is only used when
/// HIPAR referenceLevel/automatic y-axis scaling is turned off. If the reference level is set,
/// HIPAR referenceLevel/then the attenuation set in INPut:ATTenuation is no longer applied.
ViStatus rsdsa_ConfigureD31USpectrumAnalysisAmplitude(ViSession instrSession,
                                                      ViReal64  scalePerDivision,
                                                      ViInt32   referenceLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_AMPLITUDE_CONFIGURE_SCALE_PER_DIVISION, scalePerDivision),
            2, "Scale Per Division");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_AMPLITUDE_CONFIGURE_REFERENCE_LEVEL, referenceLevel),
            3, "Reference Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the spectrum analysis averages.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR automaticAverages/When turned on, the DSA automatically determines the average setting.
/// HIPAR count/Defines the number of averages. A value of 1 indicates no averaging.
/// HIPAR count/Note: For OFDM spectrum analysis, setting an average that is not a
/// HIPAR count/multiple of 128 may cause artifacts between pilots.
ViStatus rsdsa_ConfigureD31USpectrumAnalysisAverages(ViSession instrSession,
                                                     ViInt32   averageCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_CONFIGURE_MANUAL_AVERAGE_COUNT, averageCount),
            2, "Average Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns all masks on or off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Turns all masks on or off.
ViStatus rsdsa_ConfigureD31USpectrumAnalysisDisplayAllMasksEnabled(
                            ViSession instrSession,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DISPLAY_ALL_MASKS_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns off all markers including delta markers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31USpectrumAnalysisAllMarkersOff(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_ALL_MARKERS_OFF, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns off all delta markers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31USpectrumAnalysisAllDeltaMarkersOff(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_ALL_DELTA_MARKERS_OFF, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Clears all masks, including both the lower and upper masks.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31USpectrumAnalysisClearAllMasks(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_CLEAR_ALL_MASKS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns individual markers on and off. If the specified marker is a
/// HIFN  delta marker, it is turned into a normal marker. The default position
/// HIFN  is at the maximum peak.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
/// HIPAR enabled/Turns individual markers on and off. If the specified marker is a
/// HIPAR enabled/delta marker, it is turned into a normal marker. The default position
/// HIPAR enabled/is at the maximum peak.
/// HIPAR analysisMarkerXPosition/Moves a marker to a particular coordinate on the x-axis. If the
/// HIPAR analysisMarkerXPosition/specified marker is turned off, it is turned on first. If the specified
/// HIPAR analysisMarkerXPosition/marker is a delta marker, it is turned into a normal marker.
ViStatus rsdsa_ConfigureD31USpectrumAnalysisMarker(ViSession instrSession,
                                                   ViInt32   marker,
                                                   ViBoolean enabled,
                                                   ViReal64  analysisMarkerXPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarkersCnt = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarkersCnt),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarkersCnt),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_MARKER_STATE,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, repCap, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_MARKER_STATE, enabled),
            3, "Enabled");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_MARKER_X_POSITION, analysisMarkerXPosition),
            4, "Analysis Marker X Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the position of a marker on the y-axis. If the specified
/// HIFN  marker is turned off, it is turned on first. If the specified marker is
/// HIFN  a delta marker, it is turned into a normal marker.
/// HIFN
/// HIFN     The units are determined by RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
/// HIPAR yPosition/Queries the position of a marker on the y-axis. If the specified
/// HIPAR yPosition/marker is turned off, it is turned on first. If the specified marker is
/// HIPAR yPosition/a delta marker, it is turned into a normal marker.
ViStatus rsdsa_QueryD31USpectrumAnalysisMarkerYPosition(ViSession instrSession,
                                                        ViInt32   marker,
                                                        ViReal64* yPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarkersCnt = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarkersCnt),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarkersCnt),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_MARKER_Y_POSITION,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_MARKER_Y_POSITION, yPosition),
            3, "Y Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a marker to the highest amplitude displayed. If the specified
/// HIFN  marker is turned off, it is turned on first. If the specified marker is
/// HIFN  a delta marker, it is turned into a normal marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_D31USpectrumAnalysisMarkerToMaximumPeak(ViSession instrSession,
                                                       ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarkersCnt = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarkersCnt),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarkersCnt),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_MARKER_TO_MAXIMUM_PEAK,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_MARKER_TO_MAXIMUM_PEAK, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a marker to the next peak to the left of the current marker
/// HIFN  position. If the specified marker is turned off, it is turned on first.
/// HIFN  If the specified marker is a delta marker, it is turned into a normal
/// HIFN  marker.
/// HIFN     CALCulate:[<Mode>:]MARKer<Marker>[:FUNCtion]:MAXimum:LEFT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_D31USpectrumAnalysisMarkerToNextPeakLeft(ViSession instrSession,
                                                        ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarkersCnt = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarkersCnt),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarkersCnt),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_LEFT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_LEFT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a marker to the next peak to the right of the current marker
/// HIFN  position. If the specified marker is turned off, it is turned on first.
/// HIFN  If the specified marker is a delta marker, it is turned into a normal
/// HIFN  marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_D31USpectrumAnalysisMarkerToNextPeakRight(ViSession instrSession,
                                                         ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarkersCnt = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarkersCnt),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarkersCnt),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_RIGHT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_RIGHT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a marker to the next displayed point to the left of the current
/// HIFN  marker position. If the specified marker is turned off, it is turned on
/// HIFN  first. If the specified marker is a delta marker, it is turned into a
/// HIFN  normal marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_D31USpectrumAnalysisMarkerToNextPointLeft(ViSession instrSession,
                                                         ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarkersCnt = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarkersCnt),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarkersCnt),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_LEFT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_LEFT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a marker to the next displayed point to the right of the current
/// HIFN  marker position. If the specified marker is turned off, it is turned on
/// HIFN  first. If the specified marker is a delta marker, it is turned into a
/// HIFN  normal marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_D31USpectrumAnalysisMarkerToNextPointRight(ViSession instrSession,
                                                          ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarkersCnt = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarkersCnt),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarkersCnt),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_RIGHT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_RIGHT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the center frequency to the frequency of a marker. If the
/// HIFN  specified marker is turned off, it is turned on first. If the specified
/// HIFN  marker is a delta marker, it is turned into a normal marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_D31USpectrumAnalysisMarkerSetCenterFrequency(ViSession instrSession,
                                                            ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarkersCnt = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarkersCnt),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarkersCnt),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_MARKER_SET_CENTER_FREQUENCY,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_MARKER_SET_CENTER_FREQUENCY, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the reference level to the power level of a marker. If the
/// HIFN  specified marker is turned off, it is turned on first. If the specified
/// HIFN  marker is a delta marker, it is turned into a normal marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects repcap Marker.
ViStatus rsdsa_D31USpectrumAnalysisMarkerSetPowerReferenceLevel(
                            ViSession instrSession,
                            ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMarkersCnt = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &maxMarkersCnt),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, maxMarkersCnt),
            2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_MARKER_SET_POWER_REFERENCE_LEVEL,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_MARKER_SET_POWER_REFERENCE_LEVEL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the Delta Marker
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
/// HIPAR enabled/Turns delta markers on and off. If the specified marker is a normal
/// HIPAR enabled/marker, it is turned into a delta marker. The default position is at
/// HIPAR enabled/the maximum peak.
ViStatus rsdsa_ConfigureD31USpectrumAnalysisDeltaMarkerEnabled(
                            ViSession instrSession,
                            ViInt32   delta,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaCnt = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaCnt),
        2, "Delta");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDeltaCnt),
            2, "Delta");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_STATE,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, repCap, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_STATE, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the linked reference marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
/// HIPAR linkedReferenceMarker/Selects a reference marker for a delta marker. Values of delta marker
/// HIPAR linkedReferenceMarker/are relative to the reference marker. If the specified reference marker
/// HIPAR linkedReferenceMarker/is turned off, it is turned on first. If the specified delta marker is
/// HIPAR linkedReferenceMarker/turned off, it is turned on first. If the specified delta marker is a
/// HIPAR linkedReferenceMarker/normal marker, it is turned into a delta marker. If the specified
/// HIPAR linkedReferenceMarker/reference marker is the same as the specified delta marker, the marker
/// HIPAR linkedReferenceMarker/is changed to a normal marker.
ViStatus rsdsa_ConfigureD31USpectrumAnalysisDeltaMarkerLinkedReferenceMarker(
                            ViSession instrSession,
                            ViInt32   delta,
                            ViInt32   linkedReferenceMarker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaCnt = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaCnt),
        2, "Delta");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDeltaCnt),
            2, "Delta");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_LINKED_REFERENCE_MARKER,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta");

    if (linkedReferenceMarker == RSDSA_VAL_REFERENCE_MARKER_NORMAL)
        linkedReferenceMarker = delta;

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, repCap, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_LINKED_REFERENCE_MARKER, linkedReferenceMarker),
            3, "Linked Reference Marker");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to a particular absolute coordinate on the
/// HIFN  x-axis. If the specified marker is turned off, it is turned on first.
/// HIFN  If the specified marker is a normal marker, it is turned into a delta
/// HIFN  marker. The query returns the absolute position of the delta marker.
/// HIFN
/// HIFN     The units are Hz (frequency domain) or s (time domain, for zero span
/// HIFN  mode).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
/// HIPAR xPosition/Moves a delta marker to a particular absolute coordinate on the
/// HIPAR xPosition/x-axis. If the specified marker is turned off, it is turned on first.
/// HIPAR xPosition/If the specified marker is a normal marker, it is turned into a delta
/// HIPAR xPosition/marker. The query returns the absolute position of the delta marker.
ViStatus rsdsa_ConfigureD31USpectrumAnalysisDeltaMarkerAbsoluteXPosition(
                            ViSession instrSession,
                            ViInt32   delta,
                            ViReal64  xPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaCnt = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaCnt),
        2, "Delta");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDeltaCnt),
            2, "Delta");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_ABSOLUTE_X_POSITION,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_ABSOLUTE_X_POSITION, xPosition),
            3, "X Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to a particular relative coordinate on the
/// HIFN  x-axis. If the specified marker is turned off, it is turned on first.
/// HIFN  If the specified marker is a normal marker, it is turned into a delta
/// HIFN  marker. The query returns the relative position of the delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
/// HIPAR xPosition/Moves a delta marker to a particular relative coordinate on the
/// HIPAR xPosition/x-axis. If the specified marker is turned off, it is turned on first.
/// HIPAR xPosition/If the specified marker is a normal marker, it is turned into a delta
/// HIPAR xPosition/marker. The query returns the relative position of the delta marker.
ViStatus rsdsa_ConfigureD31USpectrumAnalysisDeltaMarkerRelativeXPosition(
                            ViSession instrSession,
                            ViInt32   delta,
                            ViReal64  xPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaCnt = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaCnt),
        2, "Delta");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDeltaCnt),
            2, "Delta");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_RELATIVE_X_POSITION,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_RELATIVE_X_POSITION, xPosition),
            3, "X Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the absolute position of a delta marker on the y-axis. If the
/// HIFN  specified marker is turned off, it is turned on first. If the specified
/// HIFN  marker is a normal marker, it is turned into a delta marker.
/// HIFN
/// HIFN     The units are determined by RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
/// HIPAR yPosition/Queries the absolute position of a delta marker on the y-axis. If the
/// HIPAR yPosition/specified marker is turned off, it is turned on first. If the specified
/// HIPAR yPosition/marker is a normal marker, it is turned into a delta marker.
ViStatus rsdsa_QueryD31USpectrumAnalysisDeltaMarkerYPosition(ViSession instrSession,
                                                             ViInt32   delta,
                                                             ViReal64* yPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaCnt = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaCnt),
        2, "Delta");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDeltaCnt),
            2, "Delta");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_Y_POSITION,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta");

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_Y_POSITION, yPosition),
            3, "Y Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to the highest amplitude displayed. If the
/// HIFN  specified marker is turned off, it is turned on first. If the specified
/// HIFN  marker is a normal marker, it is turned into a delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
ViStatus rsdsa_D31USpectrumAnalysisDeltaMarkerToMaximumPeak(ViSession instrSession,
                                                            ViInt32   delta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaCnt = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaCnt),
        2, "Delta");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDeltaCnt),
            2, "Delta");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_MAXIMUM_PEAK,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_MAXIMUM_PEAK, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to the next peak to the left of the current
/// HIFN  marker position. If the specified marker is turned off, it is turned on
/// HIFN  first. If the specified marker is a normal marker, it is turned into a
/// HIFN  delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
ViStatus rsdsa_D31USpectrumAnalysisDeltaMarkerToNextPeakLeft(ViSession instrSession,
                                                             ViInt32   delta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaCnt = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaCnt),
        2, "Delta");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDeltaCnt),
            2, "Delta");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_LEFT,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_LEFT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to the next peak to the right of the current
/// HIFN  marker position. If the specified marker is turned off, it is turned on
/// HIFN  first. If the specified marker is a normal marker, it is turned into a
/// HIFN  delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
ViStatus rsdsa_D31USpectrumAnalysisDeltaMarkerToNextPeakRight(
                            ViSession instrSession,
                            ViInt32   delta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaCnt = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaCnt),
        2, "Delta");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDeltaCnt),
            2, "Delta");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_RIGHT,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_RIGHT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to the next displayed point to the left of the
/// HIFN  current marker position. If the specified marker is turned off, it is
/// HIFN  turned on first. If the specified marker is a normal marker, it is
/// HIFN  turned into a delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
ViStatus rsdsa_D31USpectrumAnalysisDeltaMarkerToNextPointLeft(
                            ViSession instrSession,
                            ViInt32   delta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaCnt = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaCnt),
        2, "Delta");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDeltaCnt),
            2, "Delta");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_LEFT,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_LEFT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to the next displayed point to the right of the
/// HIFN  current marker position. If the specified marker is turned off, it is
/// HIFN  turned on first. If the specified marker is a normal marker, it is
/// HIFN  turned into a delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
ViStatus rsdsa_D31USpectrumAnalysisDeltaMarkerToNextPointRight(
                            ViSession instrSession,
                            ViInt32   delta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaCnt = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaCnt),
        2, "Delta");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDeltaCnt),
            2, "Delta");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_RIGHT,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_RIGHT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the center frequency to the frequency of a delta marker. If the
/// HIFN  specified marker is turned off, it is turned on first. If the specified
/// HIFN  marker is a normal marker, it is turned into a delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
ViStatus rsdsa_D31USpectrumAnalysisDeltaMarkerSetCenterFrequency(
                            ViSession instrSession,
                            ViInt32   delta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaCnt = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaCnt),
        2, "Delta");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDeltaCnt),
            2, "Delta");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_SET_CENTER_FREQUENCY,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_SET_CENTER_FREQUENCY, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the reference level to the power level of a delta marker. If the
/// HIFN  specified marker is turned off, it is turned on first. If the specified
/// HIFN  marker is a normal marker, it is turned into a delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
ViStatus rsdsa_D31USpectrumAnalysisDeltaMarkerSetPowerReferenceLevel(
                            ViSession instrSession,
                            ViInt32   delta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaCnt = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaCnt),
        2, "Delta");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDeltaCnt),
            2, "Delta");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_SET_POWER_REFERENCE_LEVEL,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_SET_POWER_REFERENCE_LEVEL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the bandpower between a specified delta marker and its
/// HIFN  reference marker in the frequency domain. An error is generated if the
/// HIFN  specified marker is turned off, or if the specified marker is a normal
/// HIFN  marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delta/Selects repcap Delta.
/// HIPAR bandwidthPowerResult/Queries the bandpower between a specified delta marker and its
/// HIPAR bandwidthPowerResult/reference marker in the frequency domain. An error is generated if the
/// HIPAR bandwidthPowerResult/specified marker is turned off, or if the specified marker is a normal
/// HIPAR bandwidthPowerResult/marker.
ViStatus rsdsa_QueryD31USpectrumAnalysisDeltaMarkerBandwidthPowerResult(
                            ViSession instrSession,
                            ViInt32   delta,
                            ViReal64* bandwidthPowerResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxDeltaCnt = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Delta", &maxDeltaCnt),
        2, "Delta");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delta, 1, maxDeltaCnt),
            2, "Delta");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_BANDWIDTH_POWER_RESULT,
            "Delta", delta - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta");

    viCheckParm(rsdsa_GetAttributeViReal64(instrSession, repCap, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_BANDWIDTH_POWER_RESULT, bandwidthPowerResult),
            3, "Bandwidth Power Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns individual masks on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mask/Selects repcap Mask.
/// HIPAR maskVisible/Turns individual masks on and off.
ViStatus rsdsa_ConfigureD31USpectrumAnalysisMaskVisible(ViSession instrSession,
                                                        ViInt32   mask,
                                                        ViBoolean maskVisible)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMask = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Mask", &maxMask),
        2, "Mask");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mask, 1, maxMask),
            2, "Mask");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_MASK_VISIBLE,
            "Mask", mask - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Mask");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, repCap, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_MASK_VISIBLE, maskVisible),
            3, "Mask Visible");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Clears specified mask, including both the lower and upper masks.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mask/Selects repcap Mask.
ViStatus rsdsa_D31USpectrumAnalysisClearMask(ViSession instrSession,
                                             ViInt32   mask)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxMask = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Mask", &maxMask),
        2, "Mask");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mask, 1, maxMask),
            2, "Mask");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_CLEAR_MASK,
            "Mask", mask - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Mask");

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_CLEAR_MASK, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the specified lower mask. Each point in the mask
/// HIFN  configuration must have a corresponding X and Y value. All frequency
/// HIFN  (X) entries are in Hz, and all amplitude (Y) entries are in the units
/// HIFN  selected by RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mask/Selects repcap Mask.
/// HIPAR x/An array of frequency (X axis) entries for the points of the lower
/// HIPAR x/mask, in Hz.
/// HIPAR y/An array of amplitude (Y axis) entries for the points of the lower
/// HIPAR y/mask, in the units specified by RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_ConfigureD31USpectrumAnalysisLowerMask(ViSession instrSession,
                                                      ViInt32   mask,
                                                      ViInt32   points,
                                                      ViReal64  xValues[],
                                                      ViReal64  yValues[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   param[RS_MAX_MESSAGE_BUF_SIZE] = { '\0' };
    ViChar*  p2buffer = param;
    ViInt32  maxMask = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Mask", &maxMask),
        2, "Mask");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mask, 1, maxMask),
            2, "Mask");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_HIDDEN_D31U_SPECTRUM_ANALYSIS_MASK_LOWER_CONFIGURATION,
            "Mask", mask - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Mask");

    for (int i = 0; i < points; i++)
    {
        p2buffer = p2buffer + snprintf(param, RS_MAX_MESSAGE_BUF_SIZE - strlen(param), "%f,%f", xValues[i], yValues[i]);
    }
    *(--p2buffer) = '\0';

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_HIDDEN_D31U_SPECTRUM_ANALYSIS_MASK_LOWER_CONFIGURATION, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the specified upper  mask. Each point in the mask
/// HIFN  configuration must have a corresponding X and Y value. All frequency
/// HIFN  (X) entries are in Hz, and all amplitude (Y) entries are in the units
/// HIFN  selected by RSDSA_ATTR_POWER_UNITS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mask/Selects repcap Mask.
/// HIPAR x/An array of frequency (X axis) entries for the points of the upper
/// HIPAR x/mask, in Hz.
/// HIPAR y/An array of amplitude (Y axis) entries for the points of the upper
/// HIPAR y/mask, in the units specified by RSDSA_ATTR_POWER_UNITS.
ViStatus rsdsa_ConfigureD31USpectrumAnalysisUpperMask(ViSession instrSession,
                                                      ViInt32   mask,
                                                      ViInt32   points,
                                                      ViReal64  xValues[],
                                                      ViReal64  yValues[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   param[RS_MAX_MESSAGE_BUF_SIZE] = { '\0' };
    ViChar*  p2buffer = param;
    ViInt32  maxMask = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Mask", &maxMask),
        2, "Mask");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mask, 1, maxMask),
            2, "Mask");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_HIDDEN_D31U_SPECTRUM_ANALYSIS_MASK_UPPER_CONFIGURATION,
            "Mask", mask - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Mask");

    for (int i = 0; i < points; i++)
    {
        p2buffer = p2buffer + snprintf(param, RS_MAX_MESSAGE_BUF_SIZE - strlen(param), "%f,%f", xValues[i], yValues[i]);
    }
    *(--p2buffer) = '\0';

    checkErr(rsdsa_SetAttributeViString(instrSession, repCap, RSDSA_ATTR_HIDDEN_D31U_SPECTRUM_ANALYSIS_MASK_UPPER_CONFIGURATION, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the specified trace data. Plot data is returned as x,y value
/// HIFN  pairs ( x1,y1,x2,y2,...). 1054 points are displayed across the DSA
/// HIFN  screen. For I/Q density data, 65535 integers are always returned in row
/// HIFN  order, regardless of the zoom settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Selects repcap Trace.
/// HIPAR bufferSize/Size of output array.
/// HIPAR traceData/Queries the specified trace data. Plot data is returned as x,y value
/// HIPAR traceData/pairs ( x1,y1,x2,y2,...). 1054 points are displayed across the DSA
/// HIPAR traceData/screen. For I/Q density data, 65535 integers are always returned in row
/// HIPAR traceData/order, regardless of the zoom settings.
ViStatus rsdsa_QueryD31UTraceData(ViSession instrSession,
                                  ViInt32   trace,
                                  ViInt32   bufferSize,
                                  ViReal64  traceData[],
                                  ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxTrace = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Trace", &maxTrace),
        2, "Trace");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, maxTrace),
            2, "Trace");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31U_QUERY_TRACE_DATA,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trace");
    viCheckParm(RsCore_GetAttributeViStringAsViReal64ArrayToUserBuffer(instrSession, repCap,
        RSDSA_ATTR_D31U_QUERY_TRACE_DATA, NULL,
        bufferSize, traceData, bytesRead),
        4, "Trace Data");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the all the messages that are currently in the acquisition log
/// HIFN  and clears the acquisition log. If the acquisition log is empty, the
/// HIFN  number 0, 'No entry', is returned. For each error in the log, the
/// HIFN  aqcuisition log entry message is returned, followed by the date and
/// HIFN  time the entry was added to the log.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR acquisitionLogAll/Queries the all the messages that are currently in the acquisition log
/// HIPAR acquisitionLogAll/and clears the acquisition log. If the acquisition log is empty, the
/// HIPAR acquisitionLogAll/number 0, 'No entry', is returned. For each error in the log, the
/// HIPAR acquisitionLogAll/aqcuisition log entry message is returned, followed by the date and
/// HIPAR acquisitionLogAll/time the entry was added to the log.
ViStatus rsdsa_QueryD31USystemAcquisitionLogAll(ViSession instrSession,
                                                ViInt32   bufferSize,
                                                ViChar    acquisitionLogAll[],
                                                ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSDSA_ATTR_D31U_SYSTEM_ACQUISITION_LOG_ALL, RS_VAL_DIRECT_USER_CALL, bufferSize, acquisitionLogAll, bytesRead),
        2, "Acquisition Log All");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the most recent acquisition log entry and deletes it. If the
/// HIFN  acquisition log is empty, the number 0, 'No entry', is returned.
/// HIFN  Returns the aqcuisition log entry message, followed by the date and
/// HIFN  time the entry was added to the log.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR acquisitionLogNext/Queries the most recent acquisition log entry and deletes it. If the
/// HIPAR acquisitionLogNext/acquisition log is empty, the number 0, 'No entry', is returned.
/// HIPAR acquisitionLogNext/Returns the aqcuisition log entry message, followed by the date and
/// HIPAR acquisitionLogNext/time the entry was added to the log.
ViStatus rsdsa_QueryD31USystemAcquisitionLogNext(ViSession instrSession,
                                                 ViInt32   bufferSize,
                                                 ViChar    acquisitionLogNext[],
                                                 ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSDSA_ATTR_D31U_SYSTEM_ACQUISITION_LOG_NEXT, RS_VAL_DIRECT_USER_CALL, bufferSize, acquisitionLogNext, bytesRead),
        2, "Acquisition Log Next");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the acquisition log for the number of unread entries. If the
/// HIFN  acquisition log is empty, the error number 0, 'No error', is returned.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR acquisitionLogCount/Queries the acquisition log for the number of unread entries. If the
/// HIPAR acquisitionLogCount/acquisition log is empty, the error number 0, 'No error', is returned.
ViStatus rsdsa_QueryD31USystemAcquisitionLogCount(ViSession instrSession,
                                                  ViInt32*  acquisitionLogCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_SYSTEM_ACQUISITION_LOG_COUNT, acquisitionLogCount),
            2, "Acquisition Log Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes all entries in the acquisition log.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31USystemAcquisitionLogClearAll(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31U_SYSTEM_ACQUISITION_LOG_CLEAR_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Automatically determines the attenuation level and preamplifier state
/// HIFN  (on/off). Query the resultant attenuation level with
/// HIFN  RSDSA_ATTR_D31U_CONFIGURE_ATTENUATION and the resultant preamplifier
/// HIFN  state with RSDSA_ATTR_D31U_CONFIGURE_PREAMPLIFIER_ENABLED.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_D31UAutoLevel(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31U_AUTO_LEVEL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the DOCSIS 3.1 OFDMA
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR plotDataFilter/Selects a parameter to be used as a data filter for displayed
/// HIPAR plotDataFilter/constellation, I/Q density, MER vs. Time, and channel analysis plots,
/// HIPAR plotDataFilter/where applicable.
/// HIPAR frequency/Specifies the center frequency of the channel. If the spectrum
/// HIPAR frequency/location is changed by CONFigure:CHANnel:SPECtrum:FREQuency, the center
/// HIPAR frequency/frequency is adapted accordingly.
/// HIPAR rollOff/Defines the roll-off period, which depends on the channel bandwidth
/// HIPAR rollOff/and the number of excluded carriers within the channel. The larger the
/// HIPAR rollOff/roll-off period, the more time transmission takes; however, the more
/// HIPAR rollOff/useful subcarriers are available in the frequency domain. This command
/// HIPAR rollOff/and query do not apply to OFDM automatic acquisition mode.
/// HIPAR cyclicPrefixLength/Defines the cyclic prefix length, which determines where the useful
/// HIPAR cyclicPrefixLength/data starts and allows the application to detect delay spreads during
/// HIPAR cyclicPrefixLength/transmission. The longer the delay spread, the longer the CP must be.
/// HIPAR cyclicPrefixLength/This command and query do not apply to OFDM automatic acquisition mode.
/// HIPAR cyclicPrefixLength/
/// HIPAR symbolsPerFrame/Defines the number of symbols per frame to be expected.
/// HIPAR subcarrierSpacing/Defines the FFT length, which determines the subcarrier spacing and
/// HIPAR subcarrierSpacing/other parameters. This control do not apply to OFDM automatic
/// HIPAR subcarrierSpacing/acquisition mode.
ViStatus rsdsa_ConfigureD31U(ViSession instrSession,
                             ViInt32   plotDataFilter,
                             ViReal64  frequency,
                             ViInt32   rollOff,
                             ViInt32   cyclicPrefixLength,
                             ViInt32   symbolsPerFrame,
                             ViInt32   subcarrierSpacing)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_PLOT_DATA_FILTER, plotDataFilter),
            2, "Plot Data Filter");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_FREQUENCY, frequency),
            3, "Frequency");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_ROLL_OFF, rollOff),
            4, "Roll Off");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_CYCLIC_PREFIX_LENGTH, cyclicPrefixLength),
            5, "Cyclic Prefix Length");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_SYMBOLS_PER_FRAME, symbolsPerFrame),
            6, "Symbols Per Frame");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_SUBCARRIER_SPACING, subcarrierSpacing),
            7, "Subcarrier Spacing");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the randomizer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Controls the randomizer (scrambler).
/// HIPAR seed/Specifies the randomizer (scrambler) seed.
ViStatus rsdsa_ConfigureD31URandomizer(ViSession instrSession,
                                       ViBoolean enabled,
                                       ViString  seed)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_RANDOMIZER_ENABLED, enabled),
            2, "Enabled");
    if (enabled)
        viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_RANDOMIZER_SEED, seed),
                3, "Seed");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the exclusion band.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR band/Selects repcap Band.
/// HIPAR start/Defines the start subcarrier of each exclusion band.
/// HIPAR width/Defines the width of each exclusion band in subcarriers.
ViStatus rsdsa_ConfigureD31UExclusionBand(ViSession instrSession,
                                          ViInt32   band,
                                          ViInt32   start,
                                          ViInt32   width)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxBand = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Band", &maxBand),
        2, "Band");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, band, 1, maxBand),
            2, "Band");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31U_CONFIGURE_EXCLUSION_BAND_START,
            "Band", band - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Band");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, repCap, RSDSA_ATTR_D31U_CONFIGURE_EXCLUSION_BAND_START, start),
            3, "Start");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, repCap, RSDSA_ATTR_D31U_CONFIGURE_EXCLUSION_BAND_WIDTH, width),
            4, "Width");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the start subcarrier for wideband probe.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR startSubcarrier/Defines the start subcarrier for wideband probe.
/// HIPAR subcarrierSkip/Defines the subcarrier skip for wideband probe.
/// HIPAR stagger/Controls the stagger for wideband probe.
/// HIPAR symbolInFrame/Defines the symbol in frame for wideband probe.
ViStatus rsdsa_ConfigureD31UWidebandProbe(ViSession instrSession,
                                          ViInt32   startSubcarrier,
                                          ViInt32   subcarrierSkip,
                                          ViBoolean stagger,
                                          ViInt32   symbolInFrame)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_WIDEBAND_PROBE_START_SUBCARRIER, startSubcarrier),
            2, "Start Subcarrier");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_WIDEBAND_PROBE_SUBCARRIER_SKIP, subcarrierSkip),
            3, "Subcarrier Skip");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_WIDEBAND_PROBE_STAGGER_ENABLED, stagger),
            4, "Stagger");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_WIDEBAND_PROBE_SYMBOL_IN_FRAME, symbolInFrame),
            5, "Symbol In Frame");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the BW request.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR numberOfMinislots/Defines the number of minislots in frame for bandwidth requests.
/// HIPAR subslot/Defines the subslot for bandwidth requests.
ViStatus rsdsa_ConfigureD31UBWRequest(ViSession instrSession,
                                      ViInt32   numberOfMinislots,
                                      ViInt32   subslot)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_BW_REQUEST_NUMBER_OF_MINISLOTS, numberOfMinislots),
            2, "Number Of Minislots");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_BW_REQUEST_SUBSLOT, subslot),
            3, "Subslot");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the signal path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attenuation/Defines the total attenuation for RF input in dB. If you set the
/// HIPAR attenuation/attenuation manually, it is no longer coupled to the reference level,
/// HIPAR attenuation/but the reference level is coupled to the attenuation. Thus, if the
/// HIPAR attenuation/current reference level is not compatible with an attenuation that has
/// HIPAR attenuation/been set manually, the command also adjusts the reference level.
/// HIPAR attenuation/Therefore, the reference level set in
/// HIPAR attenuation/DISPlay:[<Mode>:][WINDow:]TRACe[<Trace>]:Y[:SCALe]:RLEVel is no longer
/// HIPAR attenuation/applied. Note: The query returns the actual attenuation setting, which
/// HIPAR attenuation/can also be changed by RSDSA_ATTR_D31U_AUTO_LEVEL.
/// HIPAR preamplifier/Turns the preamplifier on and off. Note: The query returns the actual
/// HIPAR preamplifier/preamplifier state (which might also be changed by
/// HIPAR preamplifier/RSDSA_ATTR_D31U_AUTO_LEVEL).
/// HIPAR prefilter/Controls a prefilter that is constructed based on the subcarrier
/// HIPAR prefilter/layout of the channel, which allows filtering out of unwanted nearby
/// HIPAR prefilter/interferers or adjacent channels.
/// HIPAR prefilterLength/Selects the length of the digital filter used when the prefilter is
/// HIPAR prefilterLength/enabled. A longer filter allows for more aggressive rejection of
/// HIPAR prefilterLength/adjacent channels.
ViStatus rsdsa_ConfigureD31USignalPath(ViSession instrSession,
                                       ViReal64  attenuation,
                                       ViBoolean preamplifier,
                                       ViBoolean prefilter,
                                       ViInt32   prefilterLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_SIGNAL_PATH_ATTENUATION, attenuation),
            2, "Attenuation");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_SIGNAL_PATH_PREAMPLIFIER_ENABLED, preamplifier),
            3, "Preamplifier");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_SIGNAL_PATH_PREFILTER_ENABLED, prefilter),
            4, "Prefilter");

    if (prefilter)
        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_SIGNAL_PATH_PREFILTER_LENGTH, prefilterLength),
                5, "Prefilter Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the scheduling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR scheduling/This command is only valid in standalone receiver mode. In standalone
/// HIPAR scheduling/mode there are no MAP messages going into the system, so the user must
/// HIPAR scheduling/specify details of the burst scheduling. This parameter determines of
/// HIPAR scheduling/the receiver should wait for a single burst on
/// HIPAR repeatInterval/This command is only valid in standalone receiver mode. In standalone
/// HIPAR repeatInterval/mode there are no MAP messages going into the system, so the user must
/// HIPAR repeatInterval/specify details of the burst scheduling. This parameter determines
/// HIPAR repeatInterval/periodic repeat interval that burst will arrive in
ViStatus rsdsa_ConfigureD31UScheduling(ViSession instrSession,
                                       ViInt32   scheduling,
                                       ViInt32   repeatInterval)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_SCHEDULING, scheduling),
            2, "Scheduling");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_SCHEDULING_REPEAT_INTERVAL, repeatInterval),
            3, "Repeat Interval");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the trigger input offset for upstream DOCSIS modes. Sets
/// HIFN  the offset applied to the internal DOCSIS clock upon the system
/// HIFN  detecting an external input trigger. It is used to synchronize the
/// HIFN  timing for the reception of upstream bursts. Burst
/// HIFN     timing impairments can be simulated by manipulating this offset. Note
/// HIFN  this offset is additive on the triggering system offset, which is set
/// HIFN  via the zerospan mode. See section 3.9 for triggering details.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR inputTriggerOffset/Configures the trigger input offset for upstream DOCSIS modes. Sets
/// HIPAR inputTriggerOffset/the offset applied to the internal DOCSIS clock upon the system
/// HIPAR inputTriggerOffset/detecting an external input trigger. It is used to synchronize the
/// HIPAR inputTriggerOffset/timing for the reception of upstream bursts. Burst
ViStatus rsdsa_ConfigureD31UInputTriggerOffset(ViSession instrSession,
                                               ViReal64  inputTriggerOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_INPUT_TRIGGER_OFFSET, inputTriggerOffset),
            2, "Input Trigger Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the alarm threshold.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rmsMEROverall/Sets the alarm threshold for RMS MER overall. When this parameter
/// HIPAR rmsMEROverall/falls below the set threshold, an alarm is generated.
/// HIPAR berPreLDPC/Sets the alarm threshold for pre-LDPC BER. When this parameter exceeds
/// HIPAR berPreLDPC/the set threshold, an alarm is generated.
/// HIPAR berPostLDPC/Sets the alarm threshold for post-LDPC BER. When this parameter
/// HIPAR berPostLDPC/exceeds the set threshold, an alarm is generated.
ViStatus rsdsa_ConfigureD31UAlarmThreshold(ViSession instrSession,
                                           ViReal64  rmsMEROverall,
                                           ViReal64  berPreLDPC,
                                           ViReal64  berPostLDPC)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_ALARM_THRESHOLD_RMS_MER_OVERALL, rmsMEROverall),
            2, "RMS MER Overall");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_ALARM_THRESHOLD_BER_PRE_LDPC, berPreLDPC),
            3, "BER Pre-LDPC");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_D31U_ALARM_THRESHOLD_BER_POST_LDPC, berPostLDPC),
            4, "BER Post-LDPC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the number of bursts used for the moving average applied to
/// HIFN  overall MER readings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR merBursts/Specifies the number of bursts used for the moving average applied to
/// HIPAR merBursts/overall MER readings.
ViStatus rsdsa_ConfigureD31UMERBursts(ViSession instrSession,
                                      ViInt32   merBursts)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_MER_BURSTS, merBursts),
            2, "MER Burst");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the minislot configuration mode for OFDMA.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Selects the minislot configuration mode for OFDMA.
ViStatus rsdsa_ConfigureD31UMinislotConfigurationMode(ViSession instrSession,
                                                      ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_MINISLOT_CONFIGURATION_MODE, mode),
            2, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the OFDMA minislot configuration file, which contains upstream
/// HIFN  minislot configuration information. Refer to section 4.7 for the
/// HIFN  configuration file format and contents.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filename/Loads the OFDMA minislot configuration file, which contains upstream
/// HIPAR filename/minislot configuration information. Refer to section 4.7 for the
/// HIPAR filename/configuration file format and contents.
ViStatus rsdsa_ConfigureD31UMinislotConfigurationFile(ViSession instrSession,
                                                      ViString  filename)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_MINISLOT_CONFIGURATION_FILE, filename),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the Minislot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR constellation/Defines the constellation. This setting is overridden by the optional
/// HIPAR constellation/OFDMA Minislot Configuration File.
/// HIPAR pilotPattern/Defines the pilot pattern. This setting is overridden by the optional
/// HIPAR pilotPattern/OFDMA Minislot Configuration File.
/// HIPAR startingMinislot/Defines the starting minislot. This setting is overridden by the
/// HIPAR startingMinislot/optional OFDMA Minislot Configuration File.
/// HIPAR numberOfMinislots/Specifies the number of minislots. This setting is overridden by the
/// HIPAR numberOfMinislots/optional OFDMA Minislot Configuration File.
/// HIPAR firstActiveSubcarrier/Defines the first subcarrier in the active region.
/// HIPAR lastActiveSubcarrier/Defines the last subcarrier in the active region.
ViStatus rsdsa_ConfigureD31UMinislot(ViSession instrSession,
                                     ViInt32   constellation,
                                     ViInt32   pilotPattern,
                                     ViInt64   startingMinislot,
                                     ViInt32   numberOfMinislots,
                                     ViInt32   firstActiveSubcarrier,
                                     ViInt32   lastActiveSubcarrier)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_CONSTELLATION, constellation),
            2, "Constellation");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_PILOT_PATTERN, pilotPattern),
            3, "Pilot Pattern");

    viCheckParm(rsdsa_SetAttributeViInt64(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_STARTING_MINISLOT, startingMinislot),
        4, "Starting Minislot");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_NUMBER_OF_MINISLOTS, numberOfMinislots),
            5, "Number Of Minislots");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_FIRST_ACTIVE_SUBCARRIER, firstActiveSubcarrier),
            6, "First Active Subcarrier");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_LAST_ACTIVE_SUBCARRIER, lastActiveSubcarrier),
            7, "Last Active Subcarrier");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the Initial Ranging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR minislotInFrame/Defines the starting minislot in frame for initial ranging.
/// HIPAR numberOfMinislots/Defines the number of minislots for initial ranging.
/// HIPAR numberOfSubcarriers/Defines the number of subcarriers for initial ranging (must be an even
/// HIPAR numberOfSubcarriers/number).
ViStatus rsdsa_ConfigureD31UInitialRanging(ViSession instrSession,
                                           ViInt32   minislotInFrame,
                                           ViInt32   numberOfMinislots,
                                           ViInt32   numberOfSubcarriers)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_INITIAL_RANGING_STARTING_MINISLOT_IN_FRAME, minislotInFrame),
            2, "Minislot In Frame");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_INITIAL_RANGING_NUMBER_OF_MINISLOTS, numberOfMinislots),
            3, "Number Of Minislots");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_INITIAL_RANGING_NUMBER_OF_SUBCARRIERS, numberOfSubcarriers),
            4, "Number Of Subcarriers");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the ranging preamble.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR pattern/Specifies the preamble pattern for initial ranging.
/// HIPAR length/Specifies the preamble length for initial ranging.
/// HIPAR offset/Specifies the preamble offset for initial ranging.
ViStatus rsdsa_ConfigureD31UInitialRangingPreamble(ViSession instrSession,
                                                   ViString  pattern,
                                                   ViInt32   length,
                                                   ViInt32   offset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_INITIAL_RANGING_PREAMBLE_PATTERN, pattern),
            2, "Pattern");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_INITIAL_RANGING_PREAMBLE_LENGTH, length),
            3, "Length");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_INITIAL_RANGING_PREAMBLE_OFFSET, offset),
            3, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the Fine Ranging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR startingMinislotInFrame/Defines the starting minislot in frame for fine ranging.
/// HIPAR numberOfMinislots/Defines the number of minislots for fine ranging.
/// HIPAR numberOfSubcarriers/Defines the number of subcarriers for fine ranging (must be an even
/// HIPAR numberOfSubcarriers/number).
ViStatus rsdsa_ConfigureD31UFineRanging(ViSession instrSession,
                                        ViInt32   startingMinislotInFrame,
                                        ViInt32   numberOfMinislots,
                                        ViInt32   numberOfSubcarriers)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_FINE_RANGING_STARTING_MINISLOT_IN_FRAME, startingMinislotInFrame),
            2, "Starting Minislot In Frame");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_FINE_RANGING_NUMBER_OF_MINISLOTS, numberOfMinislots),
            3, "Number Of Minislots");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_FINE_RANGING_NUMBER_OF_SUBCARRIERS, numberOfSubcarriers),
            4, "Number Of Subcarriers");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the fine ranging preamble.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR pattern/Specifies the preamble pattern for fine ranging.
/// HIPAR offset/Specifies the preamble offset for fine ranging.
ViStatus rsdsa_ConfigureD31UFineRangingPreamble(ViSession instrSession,
                                                ViString  pattern,
                                                ViInt32   offset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_FINE_RANGING_PREAMBLE_PATTERN, pattern),
            2, "Pattern");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_FINE_RANGING_PREAMBLE_OFFSET, offset),
            3, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the SFP+ network output on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Turns the SFP+ network output on and off.
ViStatus rsdsa_ConfigureD31USFPOutputEnabled(ViSession instrSession,
                                             ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_D31U_CONFIGURE_SFP_OUTPUT_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the SFP+ IUC UDP port on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR iuc/Selects repcap IUC_OFDMA.
/// HIPAR enabled/Turns the SFP+ IUC UDP port on and off.
/// HIPAR udpPort/Configures the UDP port to be used for SFP+ IUC output. The UDP ports
/// HIPAR udpPort/must be uniquely assigned to downstream OFDM profiles/PLC and upstream
/// HIPAR udpPort/DOCSIS IUCs.
ViStatus rsdsa_ConfigureD31USFPIUCUDPPort(ViSession instrSession,
                                          ViInt32   iuc,
                                          ViBoolean enabled,
                                          ViInt32   udpPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxIUC = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "IUC_OFDMA", &maxIUC),
        2, "IUC");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, iuc, 1, maxIUC),
            2, "IUC");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSDSA_ATTR_D31U_CONFIGURE_SFP_IUC_UDP_PORT_ENABLED,
            "IUC_OFDMA", iuc - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "IUC");

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, repCap, RSDSA_ATTR_D31U_CONFIGURE_SFP_IUC_UDP_PORT_ENABLED, enabled),
            3, "Enabled");

    if (enabled)
        viCheckParm(rsdsa_SetAttributeViInt32(instrSession, repCap, RSDSA_ATTR_D31U_CONFIGURE_SFP_IUC_UDP_PORT, udpPort),
                4, "UDP Port");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Automatically determines the attenuation level and preamplifier state
/// HIFN  (on/off). Query the resultant attenuation level with
/// HIFN  INPut:<Direction>:ATTenuation? and the resultant preamplifier state
/// HIFN  with INPut:<Direction>:GAIN:STATe?.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_RFInputDownstreamAutomaticSettings(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_RF_INPUT_DOWNSTREAM_DETECT_ATTENUATION, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures RF Input Downstream settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attenuation/Defines the total attenuation for RF input in dB. If you set the
/// HIPAR attenuation/attenuation manually, it is no longer coupled to the reference level,
/// HIPAR attenuation/but the reference level is coupled to the attenuation. Thus, if the
/// HIPAR attenuation/current reference level is not compatible with an attenuation that has
/// HIPAR attenuation/been set manually, the command also adjusts the reference level.
/// HIPAR attenuation/Therefore, the reference level set in
/// HIPAR attenuation/DISPlay:[<Mode>:][WINDow:]TRACe[<Trace>]:Y[:SCALe]:RLEVel is no longer
/// HIPAR attenuation/applied. Note: The query returns the actual attenuation setting, which
/// HIPAR attenuation/can also be changed by INPut:<Direction>:LEVel:AUTO.
/// HIPAR offsetLevel/Defines the RF level offset in dB.
ViStatus rsdsa_ConfigureRFInputDownstreamSettings(ViSession instrSession,
                                                  ViReal64  attenuation,
                                                  ViReal64  offsetLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_RF_INPUT_DOWNSTREAM_ATTENUATION, attenuation),
            2, "Attenuation");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_RF_INPUT_DOWNSTREAM_OFFSET_LEVEL, offsetLevel),
            3, "Offset Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the preamplifier on and off. Note: The query returns the actual
/// HIFN  preamplifier state (which might also be changed by
/// HIFN  INPut:<Direction>:LEVel:AUTO).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Turns the preamplifier on and off. Note: The query returns the actual
/// HIPAR enabled/preamplifier state (which might also be changed by
/// HIPAR enabled/INPut:<Direction>:LEVel:AUTO).
ViStatus rsdsa_ConfigureRFInputDownstreamPreamplifierEnabled(ViSession instrSession,
                                                             ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_RF_INPUT_DOWNSTREAM_PREAMPLIFIER_STATE, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Automatically determines the attenuation level and preamplifier state
/// HIFN  (on/off). Query the resultant attenuation level with
/// HIFN  INPut:<Direction>:ATTenuation? and the resultant preamplifier state
/// HIFN  with INPut:<Direction>:GAIN:STATe?.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_RFInputUpstreamAutomaticSettings(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_RF_INPUT_UPSTREAM_DETECT_ATTENUATION, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures RF Input Downstream attenuation and offset level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attenuation/Defines the total attenuation for RF input in dB. If you set the
/// HIPAR attenuation/attenuation manually, it is no longer coupled to the reference level,
/// HIPAR attenuation/but the reference level is coupled to the attenuation. Thus, if the
/// HIPAR attenuation/current reference level is not compatible with an attenuation that has
/// HIPAR attenuation/been set manually, the command also adjusts the reference level.
/// HIPAR attenuation/Therefore, the reference level set in
/// HIPAR attenuation/DISPlay:[<Mode>:][WINDow:]TRACe[<Trace>]:Y[:SCALe]:RLEVel is no longer
/// HIPAR attenuation/applied. Note: The query returns the actual attenuation setting, which
/// HIPAR attenuation/can also be changed by INPut:<Direction>:LEVel:AUTO.
/// HIPAR offsetLevel/Defines the RF level offset in dB.
ViStatus rsdsa_ConfigureRFInputUpstreamSettings(ViSession instrSession,
                                                ViReal64  attenuation,
                                                ViReal64  offsetLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_RF_INPUT_UPSTREAM_ATTENUATION, attenuation),
            2, "Attenuation");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_RF_INPUT_UPSTREAM_OFFSET_LEVEL, offsetLevel),
            3, "Offset Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the preamplifier on and off. Note: The query returns the actual
/// HIFN  preamplifier state (which might also be changed by
/// HIFN  INPut:<Direction>:LEVel:AUTO).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Turns the preamplifier on and off. Note: The query returns the actual
/// HIPAR enabled/preamplifier state (which might also be changed by
/// HIPAR enabled/INPut:<Direction>:LEVel:AUTO).
ViStatus rsdsa_ConfigureRFInputUpstreamPreamplifierEnabled(ViSession instrSession,
                                                           ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_RF_INPUT_UPSTREAM_PREAMPLIFIER_STATE, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Perform MER optimization.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_RFInputOptimizeMER(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_RF_INPUT_OPTIMIZE_MER, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disable the trigger input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Enables or disable the trigger input.
ViStatus rsdsa_ConfigureTriggerInputEnabled(ViSession instrSession,
                                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_TRIGGER_INPUT_STATE, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the trigger input offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR offset/Configures the trigger input offset in seconds for spectrum analyzer
/// HIPAR offset/zero span mode.
ViStatus rsdsa_ConfigureTriggerInputOffset(ViSession instrSession,
                                           ViReal64  offset)
{
    ViStatus error = VI_SUCCESS;

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_TRIGGER_INPUT_OFFSET, offset),
            2, "Offset");

Error:
    return error;
}

/// HIFN  Configures the trigger input slope.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR slope/Configures the trigger input signal slope.
ViStatus rsdsa_ConfigureTriggerInputSlope(ViSession instrSession,
                                          ViInt32   slope)
{
    ViStatus error = VI_SUCCESS;

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_TRIGGER_INPUT_SLOPE, slope),
            2, "Slope");

Error:
    return error;
}

/// HIFN  Configures the trigger input polarity.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR polarity/Configures the trigger input signal polarity.
ViStatus rsdsa_ConfigureTriggerInputPolarity(ViSession instrSession,
                                             ViInt32   polarity)
{
    ViStatus error = VI_SUCCESS;

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_TRIGGER_INPUT_POLARITY, polarity),
            2, "Polarity");

Error:
    return error;
}

/// HIFN  Restarts a single sweep waiting for the trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_RestartTriggerInput(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_TRIGGER_INPUT_RESTART, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Generates an input trigger, with the configured polarity and offset.
/// HIFN  This will not generate a trigger if RSDSA_ATTR_TRIGGER_INPUT_STATE is
/// HIFN  OFF. Note that the input trigger is tied to the output trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_GenerateTriggerInputPulse(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_TRIGGER_INPUT_GENERATE_PULSE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables the trigger output.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Enables or disables the trigger output.
ViStatus rsdsa_ConfigureTriggerOutputEnabled(ViSession instrSession,
                                             ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_TRIGGER_OUTPUT_STATE, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the trigger output settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR offset/Configures the trigger output offset in seconds.
/// HIPAR polarity/Configures the trigger output signal polarity.
/// HIPAR pulseLength/Defines the width in seconds of the pulse generated at the trigger
/// HIPAR pulseLength/output.
/// HIPAR cyclePeriod/Configures the trigger output repeating cycle period in seconds.
ViStatus rsdsa_ConfigureTriggerOutputSettings(ViSession instrSession,
                                              ViReal64  offset,
                                              ViInt32   polarity,
                                              ViReal64  pulseLength,
                                              ViReal64  cyclePeriod)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_TRIGGER_OUTPUT_OFFSET, offset),
            2, "Offset");

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_TRIGGER_OUTPUT_POLARITY, polarity),
            3, "Polarity");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_TRIGGER_OUTPUT_PULSE_LENGTH, pulseLength),
            4, "Pulse Length");

    viCheckParm(rsdsa_SetAttributeViReal64(instrSession, NULL, RSDSA_ATTR_TRIGGER_OUTPUT_CYCLE_PERIOD, cyclePeriod),
            5, "Cycle Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the measurement mode of the DSA. See section 4.1 for the list
/// HIFN  of modes that can be active simultaneously.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instrumentMode/Selects the measurement mode of the DSA. See section 4.1 for the list
/// HIPAR instrumentMode/of modes that can be active simultaneously.
ViStatus rsdsa_ConfigureInstrumentMode(ViSession instrSession,
                                       ViInt32   instrumentMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_INSTRUMENT_MODE, instrumentMode),
            2, "Instrument Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the active modes, as per the list of modes that can be active
/// HIFN  simultaneously as listed in section 4.1.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instrumentActiveModes/Queries the active modes, as per the list of modes that can be active
/// HIPAR instrumentActiveModes/simultaneously as listed in section 4.1.
ViStatus rsdsa_QueryInstrumentModeStatus(ViSession instrSession,
                                         ViInt32   bufferSize,
                                         ViChar    instrumentActiveModes[],
                                         ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSDSA_ATTR_INSTRUMENT_MODE_STATUS, RS_VAL_DIRECT_USER_CALL, bufferSize, instrumentActiveModes, bytesRead),
        2, "Instrument Active Modes");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts a firmware update using the specified image file on the DSA
/// HIFN  (extension .img). To store the update file, the MMEMory:DATA command is
/// HIFN  used.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR firmwareUpdate/Starts a firmware update using the specified image file on the DSA
/// HIPAR firmwareUpdate/(extension .img). To store the update file, the MMEMory:DATA command is
/// HIPAR firmwareUpdate/used.
ViStatus rsdsa_ConfigureSystemFirmwareUpdate(ViSession instrSession,
                                             ViString  firmwareUpdate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_SYSTEM_FIRMWARE_UPDATE, firmwareUpdate),
            2, "Firmware Update");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Used to enter the date and time for the internal calendar and clock.
/// HIFN  The date and time setting is used for file and alarm log timestamps.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR year/Year: Current 4-digit year
/// HIPAR month/Month: 1 to 12
/// HIPAR day/Day: 1 to 31
/// HIPAR hour/Hour: 0 to 23
/// HIPAR minute/Minute: 0 to 59
/// HIPAR second/Second: 0 to 59
ViStatus rsdsa_SetSystemDateTime(ViSession instrSession,
                                 ViInt32   year,
                                 ViInt32   month,
                                 ViInt32   day,
                                 ViInt32   hour,
                                 ViInt32   minute,
                                 ViInt32   second)
{
    ViStatus error = VI_SUCCESS;
    ViChar   paramDate[RS_MAX_MESSAGE_LEN] = "";
    ViChar   paramTime[RS_MAX_MESSAGE_LEN] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            3, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            4, "Day");

    snprintf(paramDate, RS_MAX_MESSAGE_LEN, "%ld,%ld,%ld", year, month, day);

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_HIDDEN_SYSTEM_DATE, paramDate));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            5, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minute, 0, 59),
            6, "Minute");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, second, 0, 59),
            7, "Second");

    snprintf(paramTime, RS_MAX_MESSAGE_LEN, "%ld,%ld,%ld", hour, minute, second);

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_HIDDEN_SYSTEM_TIME, paramTime));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the current date and time for the internal calendar and clock.
/// HIFN  The date and time setting is used for file and alarm log timestamps.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SYSTem:DATE?
/// HIFN     SYSTem:TIME?
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR year/Year: Current 4-digit year
/// HIPAR month/Month: 1 to 12
/// HIPAR day/Day: 1 to 31
/// HIPAR hour/Hour: 0 to 23
/// HIPAR minute/Minute: 0 to 59
/// HIPAR second/Second: 0 to 59
ViStatus rsdsa_QuerySystemDateTime(ViSession instrSession,
                                   ViInt32*  year,
                                   ViInt32*  month,
                                   ViInt32*  day,
                                   ViInt32*  hour,
                                   ViInt32*  minute,
                                   ViInt32*  second)
{
    ViStatus error = VI_SUCCESS;
    ViChar   dateStr[RS_MAX_MESSAGE_LEN] = "";
    ViChar   timeStr[RS_MAX_MESSAGE_LEN] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_GetAttributeViString(instrSession, NULL, RSDSA_ATTR_HIDDEN_SYSTEM_DATE, RS_MAX_MESSAGE_LEN, dateStr));

    checkErr(RsCore_GetViInt32AtCsvIndex(dateStr, 0, year));

    checkErr(RsCore_GetViInt32AtCsvIndex(dateStr, 1, month));

    checkErr(RsCore_GetViInt32AtCsvIndex(dateStr, 2, day));

    checkErr(rsdsa_GetAttributeViString(instrSession, NULL, RSDSA_ATTR_HIDDEN_SYSTEM_TIME, RS_MAX_MESSAGE_LEN, timeStr));

    checkErr(RsCore_GetViInt32AtCsvIndex(timeStr, 0, hour));

    checkErr(RsCore_GetViInt32AtCsvIndex(timeStr, 1, minute));

    checkErr(RsCore_GetViInt32AtCsvIndex(timeStr, 2, second));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Reboots the instrument, including the operating system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_SystemReboot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_SYSTEM_REBOOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Shuts down the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_SystemShutdown(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_SYSTEM_SHUTDOWN, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Resets the DSA to its factory preset or the last preset defined by
/// HIFN  SYSTem:PRESet:STORe, except for network settings. If PresetType is not
/// HIFN  specified, the DSA is reset to its factory preset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR systemPreset/Resets the DSA to its factory preset or the last preset defined by
/// HIPAR systemPreset/SYSTem:PRESet:STORe, except for network settings. If PresetType is not
/// HIPAR systemPreset/specified, the DSA is reset to its factory preset.
ViStatus rsdsa_ConfigureSystemPreset(ViSession instrSession,
                                     ViInt32   systemPreset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_SYSTEM_PRESET, systemPreset),
            2, "System Preset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Saves the current DSA configuration as the new default instrument
/// HIFN  preset, instead of the factory preset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_SystemPresetStore(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_SYSTEM_PRESET_STORE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Controls the remote desktop.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR remoteDesktop/Controls the remote desktop.
ViStatus rsdsa_ConfigureSystemRemoteDesktop(ViSession instrSession,
                                            ViBoolean remoteDesktop)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViBoolean(instrSession, NULL, RSDSA_ATTR_SYSTEM_REMOTE_DESKTOP, remoteDesktop),
            2, "Remote Desktop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the units of the y-axis for all power-based measurement
/// HIFN  displays: dBm, dBV, dBmV, or dBV.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR powerUnits/Selects the units of the y-axis for all power-based measurement
/// HIPAR powerUnits/displays: dBm, dBV, dBmV, or dBV.
ViStatus rsdsa_ConfigurePowerUnits(ViSession instrSession,
                                   ViInt32   powerUnits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_POWER_UNITS, powerUnits),
            2, "Power Units");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of files in the current directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR catalogLength/Queries the number of files in the current directory.
ViStatus rsdsa_QueryMemoryFileCatalogLength(ViSession instrSession,
                                            ViInt32*  catalogLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_MEMORY_FILE_CATALOG_LENGTH, catalogLength),
            2, "Catalog Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries a list of the files in the current directory along with the
/// HIFN  used and free space.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR directoryUsedBytes/Used bytes in directory.
/// HIPAR freeBytesOnDisk/Free bytes on disk. This is an approximation as compression is used
/// HIPAR freeBytesOnDisk/when files are stored and the amount of compression depends on the file
/// HIPAR freeBytesOnDisk/contents.
ViStatus rsdsa_ListCurrentDirectoryFiles(ViSession instrSession,
                                         ViInt32*  usedBytes,
                                         ViInt32*  freeBytes,
                                         ViInt32   bufferSize,
                                         ViChar    fileCatalog[],
                                         ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  dataRead = NULL;
    ViChar*  p2buf;
    ViChar   usedBytesStr[RS_PARAM_BUF_SIZE] = "";
    ViChar   freeBytesStr[RS_PARAM_BUF_SIZE] = "";
    ViInt32  preambleLength = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringUnknownLen(instrSession, NULL, RSDSA_ATTR_HIDDEN_MEMORY_FILE_CATALOG, RS_VAL_DIRECT_USER_CALL, &dataRead),
        5, "File Catalog");

    checkErr(RsCore_GetViInt32AtCsvIndex(dataRead, 0, usedBytes));
    checkErr(RsCore_GetViStringAtCsvIndex(dataRead, 0, VI_FALSE, RS_PARAM_BUF_SIZE, usedBytesStr));

    checkErr(RsCore_GetViInt32AtCsvIndex(dataRead, 1, freeBytes));
    checkErr(RsCore_GetViStringAtCsvIndex(dataRead, 1, VI_FALSE, RS_PARAM_BUF_SIZE, freeBytesStr));

    preambleLength = (int)strlen(usedBytesStr) + (int)strlen(freeBytesStr) + 2 /*Commas*/;
    p2buf = dataRead + preambleLength;
    checkErr(RsCore_CopyToUserBufferAsciiData(fileCatalog, bufferSize, p2buf));
    *bytesRead = (int)strlen(dataRead) - preambleLength;

Error:
    if (dataRead) free(dataRead);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Changes the default directory to that specified by the directory name.
/// HIFN  It is possible to change to a higher directory using two dots ...
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR currentDirectory/Changes the default directory to that specified by the directory name.
/// HIPAR currentDirectory/It is possible to change to a higher directory using two dots ...
ViStatus rsdsa_ConfigureMemoryCurrentDirectory(ViSession instrSession,
                                               ViString  currentDirectory)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_MEMORY_CURRENT_DIRECTORY, currentDirectory),
            2, "Current Directory");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Copies directories and files. The source and destination can include
/// HIFN  pathnames.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR source/Source file path.
/// HIPAR destination/Destination file path.
ViStatus rsdsa_CopyFileOrDirectory(ViSession instrSession,
                                   ViString  source,
                                   ViString  destination)
{
    ViStatus error = VI_SUCCESS;
    ViChar   parm[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(parm, RS_MAX_MESSAGE_BUF_SIZE, "\'%s\',\'%s\'", source, destination);

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_HIDDEN_MEMORY_COPY_FILE_OR_DIRECTORY, parm));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a file from one location to another location in the directory
/// HIFN  tree. Can be used to rename a file by not specifying a path for the
/// HIFN  file destination.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR source/Source file path.
/// HIPAR destination/Destination file path.
ViStatus rsdsa_MoveFileOrDirectory(ViSession instrSession,
                                   ViString  source,
                                   ViString  destination)
{
    ViStatus error = VI_SUCCESS;
    ViChar   parm[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(parm, RS_MAX_MESSAGE_BUF_SIZE, "\'%s\',\'%s\'", source, destination);

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_HIDDEN_MEMORY_MOVE_FILE_OR_DIRECTORY, parm));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes the specified file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filename/File path.
ViStatus rsdsa_DeleteFile(ViSession instrSession,
                          ViString  filename)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_HIDDEN_MEMORY_DELETE_FILE, filename),
            2, "Filename");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of subdirectories in the current directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR directoryCatalogLength/Queries the number of subdirectories in the current directory.
ViStatus rsdsa_QueryMemoryDirectoryCatalogLength(ViSession instrSession,
                                                 ViInt32*  directoryCatalogLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_MEMORY_DIRECTORY_CATALOG_LENGTH, directoryCatalogLength),
            2, "Directory Catalog Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries a list of the subdirectories in the current directory.
/// HIFN
/// HIFN     The array contains one entry for each subdirectory: <Subdirectory
/// HIFN  name>,<Subdirectory name>,...
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     MMEMory:DCATalog?
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSDSA_ATTR_HIDDEN_MEMORY_SUBDIRECTORY_CATALOG
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR directoryusedbytes/Used bytes in directory.
/// HIPAR freebytesondisk/Free bytes on disk. This is an approximation as compression is used
/// HIPAR freebytesondisk/when files are stored and the amount of compression depends on the file
/// HIPAR freebytesondisk/contents.
ViStatus rsdsa_ListCurrentDirectorySubdirectories(ViSession instrSession,
                                                  ViInt32*  usedBytes,
                                                  ViInt32*  freeBytes,
                                                  ViInt32   bufferSize,
                                                  ViChar    fileCatalog[],
                                                  ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  dataRead = NULL;
    ViChar*  p2buf;
    ViChar   usedBytesStr[RS_PARAM_BUF_SIZE] = "";
    ViChar   freeBytesStr[RS_PARAM_BUF_SIZE] = "";
    ViInt32  preambleLength = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringUnknownLen(instrSession, NULL, RSDSA_ATTR_HIDDEN_MEMORY_SUBDIRECTORY_CATALOG, RS_VAL_DIRECT_USER_CALL, &dataRead),
        5, "File Catalog");

    checkErr(RsCore_GetViInt32AtCsvIndex(dataRead, 0, usedBytes));
    checkErr(RsCore_GetViStringAtCsvIndex(dataRead, 0, VI_FALSE, RS_PARAM_BUF_SIZE, usedBytesStr));

    checkErr(RsCore_GetViInt32AtCsvIndex(dataRead, 1, freeBytes));
    checkErr(RsCore_GetViStringAtCsvIndex(dataRead, 1, VI_FALSE, RS_PARAM_BUF_SIZE, freeBytesStr));

    preambleLength = (int)strlen(usedBytesStr) + (int)strlen(freeBytesStr) + 2 /*Commas*/;
    p2buf = dataRead + preambleLength;
    checkErr(RsCore_CopyToUserBufferAsciiData(fileCatalog, bufferSize, p2buf));
    *bytesRead = (int)strlen(dataRead) - preambleLength;

Error:
    if (dataRead) free(dataRead);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Creates a new subdirectory in the current directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR directory/Directory name.
ViStatus rsdsa_CreateDirectory(ViSession instrSession,
                               ViString  directory)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_HIDDEN_MEMORY_CREATE_DIRECTORY, directory),
            2, "Directory");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes the specified subdirectory in the current directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR directory/Subdirectory name.
ViStatus rsdsa_DeleteDirectory(ViSession instrSession,
                               ViString  directory)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_HIDDEN_MEMORY_DELETE_DIRECTORY, directory),
            2, "Directory");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current unit settings to the specified configuration file.
/// HIFN  Configurations stored via the local GUI, remote GUI, SCPI, and SNMP
/// HIFN  interfaces are stored and recalled separately via those respective
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filename/File path.
ViStatus rsdsa_SaveConfigurationFile(ViSession instrSession,
                                     ViString  filename)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_HIDDEN_MEMORY_SAVE_CONFIGURATION_FILE, filename),
            2, "Filename");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restores the entire DSA configuration that was previously stored via
/// HIFN  SCPI. Configurations stored via the local GUI, remote GUI, SCPI, and
/// HIFN  SNMP interfaces are stored and recalled separately via those respective
/// HIFN  interfaces.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filename/File path of a configuration that was previously stored by remote
/// HIPAR filename/control.
ViStatus rsdsa_LoadConfigurationFile(ViSession instrSession,
                                     ViString  filename)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_HIDDEN_MEMORY_LOAD_CONFIGURATION_FILE, filename),
            2, "Filename");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restores the factory default configuration to the DSA instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_LoadFactoryConfiguration(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_MEMORY_LOAD_FACTORY_CONFIGURATION, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Takes a snapshot of the screen and stores the image to the specified
/// HIFN  file. The file extension is .png.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filename/File path.
ViStatus rsdsa_SaveScreenCapture(ViSession instrSession,
                                 ViString  filename)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_HIDDEN_MEMORY_SAVE_SCREEN_CAPTURE, filename),
            2, "Filename");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the constellation I/Q data for the last selected measurement
/// HIFN  mode to the specified I/Q data file. The file extension is .iq.tar.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filename/File path.
ViStatus rsdsa_SaveIQConstellationData(ViSession instrSession,
                                       ViString  filename)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_HIDDEN_MEMORY_SAVE_IQ_CONSTELLATION_DATA, filename),
            2, "Filename");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the specified trace for the last selected measurement mode to
/// HIFN  the specified configuration file. The file extension is .csv.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Selects repcap Trace.
/// HIPAR filename/File path.
ViStatus rsdsa_StoreTraceData(ViSession instrSession,
                              ViInt32   trace,
                              ViString  filename)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViInt32  maxTrace = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Trace", &maxTrace),
        2, "Trace");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, maxTrace),
            2, "Trace");

    snprintf(param, RS_MAX_MESSAGE_BUF_SIZE, "%ld,\'%s\'", trace, filename);

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_HIDDEN_MEMORY_STORE_TRACE_DATA, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Ejects the specified USB memory device. This command has no effect and
/// HIFN  generates an error if the device is not mounted.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR usbDevice/USB memory device
ViStatus rsdsa_EjectUSB(ViSession instrSession,
                        ViInt32   usbDevice)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_HIDDEN_MEMORY_EJECT_USB, usbDevice),
            2, "USB Device");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Status preset. Resets the ENABle part of the STATus:OPERation and
/// HIFN  STATus:QUEStionable registers are set to 0; i.e., all events in these
/// HIFN  registers are not passed on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_StatusPreset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_STATUS_PRESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN   Returns the decimal value of the ESR and then clears the register to
/// HIFN  0.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR eventStatusRegister/Returns the decimal value of the ESR and then clears the register to
/// HIPAR eventStatusRegister/0.
ViStatus rsdsa_QueryEventStatusRegister(ViSession instrSession,
                                        ViInt32*  eventStatusRegister)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_QUERY_EVENT_STATUS_REGISTER, eventStatusRegister),
            2, "Event Status Register");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Operation Enable Register (16-bit mask). A bit value of 1 allows the
/// HIFN  corresponding bit in the Operation Event Register to set bit 7 in the
/// HIFN  STB.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR operationEnableRegister/Operation Enable Register (16-bit mask). A bit value of 1 allows the
/// HIPAR operationEnableRegister/corresponding bit in the Operation Event Register to set bit 7 in the
/// HIPAR operationEnableRegister/STB.
ViStatus rsdsa_ConfigureOperationEnableRegister(ViSession instrSession,
                                                ViInt32   operationEnableRegister)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_OPERATION_ENABLE_REGISTER, operationEnableRegister),
            2, "Operation Enable Register");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Operation Status Register (16-bit read-only). Provides status of the
/// HIFN  current activities of normal DSA operation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR operationalStatusRegister/Operation Status Register (16-bit read-only). Provides status of the
/// HIPAR operationalStatusRegister/current activities of normal DSA operation.
ViStatus rsdsa_QueryOperationalStatusRegister(ViSession instrSession,
                                              ViInt32*  operationalStatusRegister)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_QUERY_OPERATIONAL_STATUS_REGISTER, operationalStatusRegister),
            2, "Operation Status Register");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Operation Event Register (16-bit read-only). A bit value of 1
/// HIFN  indicates the corresponding bit in the Operation Status Register
/// HIFN  changed state. Reading the Operation Event Register clears all the bits
/// HIFN  in the register.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR operationEventRegister/Operation Event Register (16-bit read-only). A bit value of 1
/// HIPAR operationEventRegister/indicates the corresponding bit in the Operation Status Register
/// HIPAR operationEventRegister/changed state. Reading the Operation Event Register clears all the bits
/// HIPAR operationEventRegister/in the register.
ViStatus rsdsa_QueryOperationEventRegister(ViSession instrSession,
                                           ViInt32*  operationEventRegister)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_QUERY_OPERATION_EVENT_REGISTER, operationEventRegister),
            2, "Operation Event Register");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Questionable Enable Register (16-bit mask). A bit value of 1 allows
/// HIFN  the corresponding bit in the Questionable Event Register to set a bit
/// HIFN  in the STB.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR questionableEnableRegister/Questionable Enable Register (16-bit mask). A bit value of 1 allows
/// HIPAR questionableEnableRegister/the corresponding bit in the Questionable Event Register to set a bit
/// HIPAR questionableEnableRegister/in the STB.
ViStatus rsdsa_ConfigureQuestionableEnableRegister(ViSession instrSession,
                                                   ViInt32   questionableEnableRegister)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_QUESTIONABLE_ENABLE_REGISTER, questionableEnableRegister),
            2, "Questionable Enable Register");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Questionable Status Register (16-bit read-only). Provides status on
/// HIFN  instrument states that are outside of specifications. Bit values of 0
/// HIFN  indicate a normal condition and 1 indicates an abnormal condition.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR questionableStatusRegister/Questionable Status Register (16-bit read-only). Provides status on
/// HIPAR questionableStatusRegister/instrument states that are outside of specifications. Bit values of 0
/// HIPAR questionableStatusRegister/indicate a normal condition and 1 indicates an abnormal condition.
ViStatus rsdsa_QueryQuestionableStatusRegister(ViSession instrSession,
                                               ViInt32*  questionableStatusRegister)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_QUERY_QUESTIONABLE_STATUS_REGISTER, questionableStatusRegister),
            2, "Questionable Status Register");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Questionable Event Register (16-bit read-only). A bit value of 1
/// HIFN  indicates the corresponding bit in the Questionable Condition Register
/// HIFN  has changed. Reading the register clears all the bits in the register.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR questionableEventRegister/Questionable Event Register (16-bit read-only). A bit value of 1
/// HIPAR questionableEventRegister/indicates the corresponding bit in the Questionable Condition Register
/// HIPAR questionableEventRegister/has changed. Reading the register clears all the bits in the register.
ViStatus rsdsa_QueryQuestionableEventRegister(ViSession instrSession,
                                              ViInt32*  questionableEventRegister)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_GetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_QUERY_QUESTIONABLE_EVENT_REGISTER, questionableEventRegister),
            2, "Questionable Event Register");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Resets all network settings to factory defaults.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_SystemNetworkReset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_SYSTEM_NETWORK_RESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the DSA IP address, subnet mask, and gateway parameters for the
/// HIFN  network connection used to control and monitor the DSA. This command
/// HIFN  set the network type to static.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ipAddress/Sets the DSA IP address, subnet mask, and gateway parameters for the
/// HIPAR ipAddress/network connection used to control and monitor the DSA. This command
/// HIPAR ipAddress/set the network type to static.
/// HIPAR subnet/Sets the DSA IP address, subnet mask, and gateway parameters for the
/// HIPAR subnet/network connection used to control and monitor the DSA. This command
/// HIPAR subnet/set the network type to static.
/// HIPAR gateway/Sets the DSA IP address, subnet mask, and gateway parameters for the
/// HIPAR gateway/network connection used to control and monitor the DSA. This command
/// HIPAR gateway/set the network type to static.
ViStatus rsdsa_ConfigureSystemNetwork(ViSession instrSession,
                                      ViString  ipAddress,
                                      ViString  subnet,
                                      ViString  gateway)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(param, RS_MAX_MESSAGE_BUF_SIZE, "\"%s\",\"%s\",\"%s\"", ipAddress, subnet, gateway);

    checkErr(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_HIDDEN_SYSTEM_CONFIGURE_NETWORK, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the DSA network type. Setting the network type to static also
/// HIFN  resets the static network configuration to factory defaults:
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR networkType/Selects the DSA network type. Setting the network type to static also
/// HIPAR networkType/resets the static network configuration to factory defaults:
ViStatus rsdsa_ConfigureSystemNetworkType(ViSession instrSession,
                                          ViInt32   networkType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_SYSTEM_CONFIGURE_NETWORK_TYPE, networkType),
            2, "Network Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the DSA network interface MAC address.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR dsaMACAddress/Queries the DSA network interface MAC address.
ViStatus rsdsa_QuerySystemDSAMACAddress(ViSession instrSession,
                                        ViInt32   bufferSize,
                                        ViChar    MACAddress[],
                                        ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSDSA_ATTR_SYSTEM_QUERY_DSA_MAC_ADDRESS, RS_VAL_DIRECT_USER_CALL, bufferSize, MACAddress, bytesRead),
        2, "MAC Address");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the SFP+ IP address, in dotted decimal form.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ipAddress/Configures the SFP+ IP address, in dotted decimal form.
ViStatus rsdsa_ConfigureSystemSFPPlusIPAddress(ViSession instrSession,
                                               ViString  ipAddress)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_SYSTEM_CONFIGURE_SFP_PLUS_IP_ADDRESS, ipAddress),
            2, "IP Address");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the SFP+ network subnet mask, in dotted decimal format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subnetMask/Configures the SFP+ network subnet mask, in dotted decimal format.
ViStatus rsdsa_ConfigureSystemSFPPlusSubnetMask(ViSession instrSession,
                                                ViString  subnetMask)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_SYSTEM_CONFIGURE_SFP_PLUS_SUBNET_MASK, subnetMask),
            2, "Subnet Mask");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the SFP+ network gateway, in dotted decimal format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR gateway/Configures the SFP+ network gateway, in dotted decimal format.
ViStatus rsdsa_ConfigureSystemSFPPlusGateway(ViSession instrSession,
                                             ViString  gateway)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_SYSTEM_CONFIGURE_SFP_PLUS_GATEWAY, gateway),
            2, "Gateway");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the SFP+ destination IP address, in dotted decimal format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR destinationIPAddress/Configures the SFP+ destination IP address, in dotted decimal format.
ViStatus rsdsa_ConfigureSystemSFPPlusDestinationIPAddress(ViSession instrSession,
                                                          ViString  destinationIPAddress)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_SYSTEM_CONFIGURE_SFP_PLUS_DESTINATION_IP_ADDRESS, destinationIPAddress),
            2, "Destination IP Address");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the SFP+ output data rate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR dataRate/Configures the SFP+ output data rate.
ViStatus rsdsa_ConfigureSystemSFPPlusDataRate(ViSession instrSession,
                                              ViInt32   dataRate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_SYSTEM_CONFIGURE_SFP_PLUS_DATA_RATE, dataRate),
            2, "Data Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the SFD network option; i.e., whether IP address or host
/// HIFN  name will be used.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR networkOption/Configures the SFD network option; i.e., whether IP address or host
/// HIPAR networkOption/name will be used.
ViStatus rsdsa_ConfigureSystemSFDNetworkOption(ViSession instrSession,
                                               ViInt32   networkOption)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViInt32(instrSession, NULL, RSDSA_ATTR_SYSTEM_CONFIGURE_SFD_NETWORK_OPTION, networkOption),
            2, "Network Option");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the SFD network host name. Only the SFD host name or the IP
/// HIFN  address need to be specified, depending upon
/// HIFN  RSDSA_ATTR_SYSTEM_CONFIGURE_SFD_NETWORK_OPTION.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR hostName/Configures the SFD network host name. Only the SFD host name or the IP
/// HIPAR hostName/address need to be specified, depending upon
/// HIPAR hostName/RSDSA_ATTR_SYSTEM_CONFIGURE_SFD_NETWORK_OPTION.
ViStatus rsdsa_ConfigureSystemSFDHostName(ViSession instrSession,
                                          ViString  hostName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_SYSTEM_CONFIGURE_SFD_HOST_NAME, hostName),
            2, "Host Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the SFD IP address that is used to control and monitor the
/// HIFN  SFD in dotted decimal format. Only the SFD host name or the IP address
/// HIFN  need to be specified, depending upon
/// HIFN  RSDSA_ATTR_SYSTEM_CONFIGURE_SFD_NETWORK_OPTION.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ipAddress/Configures the SFD IP address that is used to control and monitor the
/// HIPAR ipAddress/SFD in dotted decimal format. Only the SFD host name or the IP address
/// HIPAR ipAddress/need to be specified, depending upon
/// HIPAR ipAddress/RSDSA_ATTR_SYSTEM_CONFIGURE_SFD_NETWORK_OPTION.
ViStatus rsdsa_ConfigureSystemSFDIPAddress(ViSession instrSession,
                                           ViString  ipAddress)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsdsa_SetAttributeViString(instrSession, NULL, RSDSA_ATTR_SYSTEM_CONFIGURE_SFD_IP_ADDRESS, ipAddress),
            2, "IP Address");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Creates a file containing the specified data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filename/File path.
/// HIPAR data/The bytes to write to the destination file.
ViStatus rsdsa_WriteFromFileToInstrument(ViSession instrSession,
                                         ViString  source,
                                         ViString  destination)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmdFormated[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  cmd = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_BuildAttrCommandString(instrSession, &g_RSDSA_ATTR_HIDDEN_MEMORY_CREATE_FILE, NULL, &cmd));

    snprintf(cmdFormated, RS_MAX_MESSAGE_BUF_SIZE, "%s '%s',", cmd, destination);

    checkErr(RsCore_WriteBinaryDataFromFile(instrSession, cmdFormated, source));

Error:
    if (cmd) free(cmd);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the data for the file indicated. The file name can include
/// HIFN  pathnames.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filename/The file name can include pathnames.
ViStatus rsdsa_ReadToFileFromInstrument(ViSession instrSession,
                                        ViString  source,
                                        ViString  destination)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmdFormated[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  cmd = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_BuildAttrCommandString(instrSession, &g_RSDSA_ATTR_HIDDEN_MEMORY_READ_FILE, NULL, &cmd));

    snprintf(cmdFormated, RS_MAX_MESSAGE_BUF_SIZE, "%s '%s'", cmd, source);

    checkErr(RsCore_QueryBinaryDataBlockToFile(instrSession, cmdFormated, destination, RS_VAL_TRUNCATE));

Error:
    if (cmd) free(cmd);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sends a command to trigger the RF signal generator.
/// HIFN  Call this function if you set a trigger source to a software trigger
/// HIFN  value.  Below is a list of attributes and functions which can set the
/// HIFN  trigger source to a software trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_SendSoftwareTrigger(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  command = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_BuildAttrCommandString(instrSession, &g_RSDSA_ATTR_TRIGGER_INPUT_GENERATE_PULSE, NULL, &command));

    checkErr(RsCore_WriteAttributeCommand(instrSession, &g_RSDSA_ATTR_TRIGGER_INPUT_GENERATE_PULSE, command));

Error:
    if (command) free(command);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function resets the instrument to a known state and sends
/// HIFN  initialization commands to the instrument.  The initialization commands
/// HIFN  set instrument settings such as Headers Off, Short Command form, and
/// HIFN  Data Transfer Binary to the state necessary for the operation of the
/// HIFN  instrument driver.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_Reset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (RsCore_Simulating(instrSession) == VI_FALSE)
    {
        checkErr(rsdsa_ClearStatus(instrSession));
        checkErr(RsCore_Write(instrSession, "*RST"));
        checkErr(RsCore_QueryViStringShort(instrSession, "*OPC?", NULL));
    }

    checkErr(rsdsa_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function runs the instrument's self test routine and returns the
/// HIFN  test result(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR selfTestResult/This control contains the value returned from the instrument self
/// HIPAR selfTestResult/test.  Zero means success.  For any other code, see the device's
/// HIPAR selfTestResult/operator's manual.
/// HIPAR selfTestMessage/Returns the self-test response string from the instrument. See the
/// HIPAR selfTestMessage/device's operation manual for an explanation of the string's contents.
ViStatus rsdsa_SelfTest(ViSession instrSession,
                        ViInt16*  selfTestResult,
                        ViChar    selfTestMessage[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, selfTestResult), 2, "Null address for Test Result");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, selfTestMessage), 3, "Null address for Test Message");

    checkErr(RsCore_SelfTest(instrSession, selfTestResult, 48, selfTestMessage));

    checkErr(rsdsa_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the revision numbers of the instrument driver
/// HIFN  and instrument firmware.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instrumentDriverRevision/Returns the instrument driver software revision numbers in the form of
/// HIPAR instrumentDriverRevision/a string.
/// HIPAR firmwareRevision/Returns the instrument firmware revision numbers in the form of a
/// HIPAR firmwareRevision/string.
ViStatus rsdsa_RevisionQuery(ViSession instrSession,
                             ViChar    instrumentDriverRevision[],
                             ViChar    firmwareRevision[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, instrumentDriverRevision), 2, "Null address for Instrument Driver Revision");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, firmwareRevision), 3, "Null address for Firmware Revision");

    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RS_ATTR_SPECIFIC_DRIVER_REVISION, 0, 256, instrumentDriverRevision));
    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RS_ATTR_INSTRUMENT_FIRMWARE_REVISION, 0, 256, firmwareRevision));

    checkErr(rsdsa_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function reads an error code and a message from the instrument's
/// HIFN  error queue.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR errorCode/Returns the error code read from the instrument's error queue.
/// HIPAR errorMessage/Returns the error message string read from the instrument's error
/// HIPAR errorMessage/message queue.
ViStatus rsdsa_error_query(ViSession instrSession,
                           ViInt32*  errorCode,
                           ViChar    errorMessage[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorCode), 2, "Null address for Error Code");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorMessage), 3, "Null address for Error Message");

    checkErr(RsCore_ErrorQueryAll(instrSession, errorCode, 1024, errorMessage));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function converts a status code returned by an instrument driver
/// HIFN  function into a user-readable string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR errorCode/Pass the Status parameter that is returned from any of the instrument
/// HIPAR errorCode/driver functions.
/// HIPAR errorMessage/Returns the user-readable message string that corresponds to the
/// HIPAR errorMessage/status code you specify.
ViStatus rsdsa_error_message(ViSession instrSession,
                             ViStatus  errorCode,
                             ViChar    errorMessage[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    static   RsCoreStringValueTable errorTable = { RSDSA_ERROR_CODES_AND_MSGS, {0, NULL} };

    (void)(RsCore_LockSession(instrSession));

    // all VISA and RS error codes are handled as well as codes in the table
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorMessage), 3, "Null address for Error Message");

    checkErr(RsCore_GetSpecificDriverStatusDesc(instrSession, errorCode, errorMessage, 256, errorTable));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function switches ON/OFF various error checkings performed inside
/// HIFN  of the driver.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR optionChecking/This control switches driver option checking On or Off. If the option
/// HIPAR optionChecking/checking is ON, the driver checks the presence of the required
/// HIPAR optionChecking/instrument option before it sends the command to the instrument. Switch
/// HIPAR optionChecking/it OFF in special cases when you are sure that your instrument accepts
/// HIPAR optionChecking/the command. In any case the option checking on the instrument site is
/// HIPAR optionChecking/always active, and will cause an error if the command is not supported.
/// HIPAR rangeChecking/This control switches driver parameter range checking On or Off. If
/// HIPAR rangeChecking/the range checking is ON, the driver checks the range of the input
/// HIPAR rangeChecking/parameter before it sends the command to the instrument. Switch it OFF
/// HIPAR rangeChecking/in special cases when you are sure that your instrument accepts the
/// HIPAR rangeChecking/values outside the driver-restricted range. In any case, the parameter
/// HIPAR rangeChecking/range checking on the instrument site is always active, and will cause
/// HIPAR rangeChecking/an error if value is outside the instrument-accepted range.
/// HIPAR statusChecking/This control enables or disables instrument state checking (reading of
/// HIPAR statusChecking/the status byte and checking it for error). All driver functions are
/// HIPAR statusChecking/using state checking. Switch this function to ON when debugging your
/// HIPAR statusChecking/application. For better bus throughput and instrument performance
/// HIPAR statusChecking/switch it OFF.
ViStatus rsdsa_ConfigureErrorChecking(ViSession instrSession,
                                      ViBoolean optionChecking,
                                      ViBoolean rangeChecking,
                                      ViBoolean statusChecking)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->optionChecking = optionChecking;
    checkErr(RsCore_SetAttributeViBoolean(instrSession, "", RS_ATTR_QUERY_INSTRUMENT_STATUS, 0, statusChecking));
    checkErr(RsCore_SetAttributeViBoolean(instrSession, "", RS_ATTR_RANGE_CHECK, 0, rangeChecking));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function clears status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_ClearStatus(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ClearStatus(instrSession));

    checkErr(rsdsa_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the ID Query response string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR idQueryResponse/Returns the ID Query response string. The array should consist of at
/// HIPAR idQueryResponse/least 256 elements.
ViStatus rsdsa_IDQueryResponse(ViSession instrSession,
                               ViInt32   bufferSize,
                               ViChar    IDQueryResponse[],
                               ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_QueryViString(instrSession, "*IDN?", bufferSize, IDQueryResponse));
    if (returnedValues)
        *returnedValues = (ViInt32)strlen(IDQueryResponse);

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stops further command processing until all commands sent before *WAI
/// HIFN  have been executed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_ProcessAllPreviousCommands(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_WriteInstrData(instrSession, "*WAI"));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the OPC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR opc/Queries the OPC.
ViStatus rsdsa_QueryOPC(ViSession instrSession,
                        ViInt32*  OPC)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsdsa_QueryViInt32(instrSession, "*OPC?", OPC));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the OPC timeout.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR opcTimeout/This control sets the timeout value.
ViStatus rsdsa_SetOPCTimeout(ViSession instrSession,
                             ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_SetAttributeViInt32(instrSession, NULL, RS_ATTR_OPC_TIMEOUT, 0, (ViUInt32)timeout));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the OPC timeout.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR timeout/This control returns the timeout value.
ViStatus rsdsa_GetOPCTimeout(ViSession instrSession,
                             ViInt32*  timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, RS_ATTR_OPC_TIMEOUT, 0, timeout));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function specifies the minimum timeout value to use (in
/// HIFN  milliseconds) when accessing the device associated with the given
/// HIFN  session.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR visaTimeout/This control sets the timeout value.
ViStatus rsdsa_SetVISATimeout(ViSession instrSession,
                              ViUInt32  VISATimeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_SetVisaTimeout(instrSession, VISATimeout));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the minimum timeout value to use (in
/// HIFN  milliseconds) when accessing the device associated with the given
/// HIFN  session.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR visaTimeout/This control gets the timeout value.
ViStatus rsdsa_GetVISATimeout(ViSession instrSession,
                              ViUInt32* VISATimeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetVisaTimeout(instrSession, VISATimeout));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If TRUE (default after init is FALSE), the driver calls SYST:ERR?
/// HIFN  automatically in CheckStatus() when Instrument Status Error is
/// HIFN  detected.
/// HIFN     Use the function rsdsa_GetError afterwards to get the detailed
/// HIFN  information about the instrument error(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoSystErrQuery/If TRUE (default after init is FALSE), the driver calls SYST:ERR?
/// HIPAR autoSystErrQuery/automatically in CheckStatus() when Instrument Status Error is
/// HIPAR autoSystErrQuery/detected.
ViStatus rsdsa_ConfigureAutoSystemErrQuery(ViSession instrSession,
                                           ViBoolean autoSystErrQuery)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->autoSystErrQuery = autoSystErrQuery;

Error:
    return error;
}

/// HIFN  Session-related mutex switch.
/// HIFN     If TRUE (default after init is FALSE), the driver sets a
/// HIFN  mutual-exclusion mechanism for every driver's hi-level function and
/// HIFN  attribute access.
/// HIFN     Set this value to TRUE if you plan to use one session in more than one
/// HIFN  thread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multiThreadLocking/Session-related mutex switch.
ViStatus rsdsa_ConfigureMultiThreadLocking(ViSession instrSession,
                                           ViBoolean multiThreadLocking)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->multiThreadLocking = multiThreadLocking;

Error:
    return error;
}

/// HIFN  This function configures attribute data type mismatch reporting.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Enables/disables data type mismatch reporting.
ViStatus rsdsa_ConfigureAttributeDataTypeMismatchReporting(ViSession instrSession,
                                                           ViBoolean state)
{  
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->reportAttrDataTypeMismatch = state;

Error:
    return error;
}

/// HIFN  Retrieves detailed description of the last error that occurred in the
/// HIFN  entered driver session. The latest error overwrites the description. If
/// HIFN  you enter NULL as the instrSession parameter, the driver returns
/// HIFN  the last error that occured in the current execution thread.
/// HIFN     The function also clears the error - calls the rsdsa_ClearError() at
/// HIFN  the end.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR code/Return value - returns the last status code. If the returned code is
/// HIPAR code/0, no error occurred since the last call of the rsdsa_GetError() or
/// HIPAR code/rsdsa_ClearError()
/// HIPAR buffersize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR buffersize/Description parameter. If the error description is longer, it is
/// HIPAR buffersize/truncated and properly 0-terminated.
/// HIPAR description/Returns the error description for the session or execution thread. If
/// HIPAR description/there is no description, the function returns an empty string.
ViStatus rsdsa_GetError(ViSession instrSession,
                        ViStatus* code,
                        ViInt32   bufferSize,
                        ViChar    description[])
{
    ViStatus error = VI_SUCCESS;

    (void)(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, code), 2, "Null address for Error");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, description), 4, "Null address for Description");

    checkErr(RsCore_GetErrorCompleteDescription(instrSession, &rsdsa_error_message, code, bufferSize, description));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Clears the information about the last error that occured for the
/// HIFN  entered session. If you enter NULL as the instrSession, the
/// HIFN  function clears the error for the current execution thread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsdsa_ClearError(ViSession instrSession)
{
    return RsCore_ClearErrorInfo(instrSession);
}

/// HIFN  This function writes a user-specified string to the instrument.
/// HIFN
/// HIFN     Note:  This function bypasses IVI attribute state caching.  Therefore,
/// HIFN  when you call this function, the cached values for all attributes will
/// HIFN  be invalidated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
ViStatus rsdsa_WriteInstrData(ViSession instrSession,
                              ViString  writeBuffer)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    error = RsCore_Write(instrSession, writeBuffer);

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the ViBoolean value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViBoolean value
ViStatus rsdsa_QueryViBoolean(ViSession  instrSession,
                              ViString   command,
                              ViBoolean* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, (ViInt32)strlen(command), 1, 2147483647), 2,
        "Command (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViBoolean(instrSession, command, value));

    checkErr(rsdsa_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the ViInt32 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViInt32 value
ViStatus rsdsa_QueryViInt32(ViSession instrSession,
                            ViString  command,
                            ViInt32*  value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, (ViInt32)strlen(command), 1, 2147483647), 2,
        "Command (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViInt32(instrSession, command, value));

    checkErr(rsdsa_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the ViReal64 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViReal64 value
ViStatus rsdsa_QueryViReal64(ViSession instrSession,
                             ViString  command,
                             ViReal64* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, (ViInt32)strlen(command), 1, 2147483647), 2,
        "Command (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViReal64(instrSession, command, value));

    checkErr(rsdsa_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the ViString value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Value parameter.
/// HIPAR value/Returns the string value.
ViStatus rsdsa_QueryViString(ViSession instrSession,
                             ViString  command,
                             ViInt32   bufferSize,
                             ViChar    value[])
{
    ViStatus error = VI_SUCCESS;
    ViChar*  pbuffer = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, command),
        2, "Command");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value),
        4, "Value");

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, command, &pbuffer));

    checkErr(rsdsa_CheckStatus(instrSession));

    checkErr(RsCore_CopyToUserBufferAsciiData(value, bufferSize, pbuffer));

Error:
    if (pbuffer) free(pbuffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Function used by Hi-level functions to obtain a repCap name from:
/// HIFN         - attributeID
/// HIFN         - repCapNameID
/// HIFN         - 0-based index in the repCapNames
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR repcapNameID/RepCap Name ID.
/// HIPAR index/Zero based index in the repCapNames.
/// HIPAR bufferSize/Pass the number of bytes in the string you allocate for the
/// HIPAR bufferSize/repCapName.
/// HIPAR repcapname/String returning RepCap name.
ViStatus rsdsa_GetAttributeRepCapName(ViSession instrSession,
                                      ViUInt32  attributeID,
                                      ViString  repcapNameID,
                                      ViInt32   index,
                                      ViInt32   bufferSize,
                                      ViChar    repcapname[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, attributeID, repcapNameID, index, bufferSize, repcapname));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the repeated capability id(s) that belongs to
/// HIFN  the attribute you specify. More then one id is listed in comma
/// HIFN  separated string.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Repeated Capability Id(s) parameter.
/// HIPAR repeatedCapabilityIds/The buffer in which the function returns the repeated capability id(s)
/// HIPAR repeatedCapabilityIds/belonging to the attribute. The buffer must be of type ViChar and have
/// HIPAR repeatedCapabilityIds/at least as many bytes as indicated in the Buffer Size parameter.
ViStatus rsdsa_GetAttributeRepeatedCapabilityIds(ViSession instrSession,
                                                 ViUInt32  attributeID,
                                                 ViInt32   bufferSize,
                                                 ViChar    repeatedCapabilityIDs[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bufferSize, 1, RS_MAX_SHORT_MESSAGE_BUF_SIZE),
            3, "Buffer Size");

    checkErr(RsCore_GetAttributeRepCapNameIds(instrSession, attributeID, bufferSize, repeatedCapabilityIDs));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the repeated capability name(s) that belongs to
/// HIFN  the attribute and single repeated capability id you may specify. More
/// HIFN  then one name is listed in comma separated string.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR repeatedCapabilityId/If the attribute applies to a repeated capability such as a trace,
/// HIPAR repeatedCapabilityId/this parameter defines the Id of the repeated capability on which to
/// HIPAR repeatedCapabilityId/query repeated capability id name(s).
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Repeated Capability Id Name(s) parameter.
/// HIPAR repeatedCapabilityIdNames/The buffer in which the function returns the repeated capability id
/// HIPAR repeatedCapabilityIdNames/name(s) belonging to the attribute. The buffer must be of type ViChar
/// HIPAR repeatedCapabilityIdNames/and have at least as many bytes as indicated in the Buffer Size
/// HIPAR repeatedCapabilityIdNames/parameter.
ViStatus rsdsa_GetAttributeRepeatedCapabilityIdNames(ViSession instrSession,
                                                     ViUInt32  attributeID,
                                                     ViString  repeatedCapabilityId,
                                                     ViInt32   bufferSize,
                                                     ViChar    repeatedCapabilityIdNames[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bufferSize, 1, RS_MAX_SHORT_MESSAGE_BUF_SIZE),
            4, "Buffer Size");

    checkErr(RsCore_GetAttributeRepCapNamesAll(instrSession, attributeID, repeatedCapabilityId, bufferSize, repeatedCapabilityIdNames));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
* Function: rsdsa_SetAttribute<type> Functions
* Purpose:  These functions enable the instrument driver user to set
*           attribute values directly.  There are typesafe versions for
*           ViInt32, ViInt64, ViReal64, ViString, and ViBoolean data types.
*****************************************************************************/

/// HIFN  Sets the value of a ViInt32 attribute directly.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR repeatedCapabilityName/A comma-separated list of any required repeated capability IDs.
/// HIPAR attributeID/The attribute ID constant.
/// HIPAR attributeValue/The value to set to the attribute.
ViStatus rsdsa_SetAttributeViInt32(ViSession instrSession,
                                   ViString  repeatedCapabilityName,
                                   ViAttr    attributeID,
                                   ViInt32   attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);
    error = RsCore_SetAttributeViInt32(instrSession, repeatedCapabilityName, attributeID, RS_VAL_DIRECT_USER_CALL, attributeValue);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the value of a ViInt64 attribute directly.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR repeatedCapabilityName/A comma-separated list of any required repeated capability IDs.
/// HIPAR attributeID/The attribute ID constant.
/// HIPAR attributeValue/The value to set to the attribute.
ViStatus rsdsa_SetAttributeViInt64(ViSession instrSession,
                                   ViString  repeatedCapabilityName,
                                   ViAttr    attributeID,
                                   ViInt64   attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);
    error = RsCore_SetAttributeViInt64(instrSession, repeatedCapabilityName, attributeID, RS_VAL_DIRECT_USER_CALL, attributeValue);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the value of a ViReal64 attribute directly.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR repeatedCapabilityName/A comma-separated list of any required repeated capability IDs.
/// HIPAR attributeID/The attribute ID constant.
/// HIPAR attributeValue/The value to set to the attribute.
ViStatus rsdsa_SetAttributeViReal64(ViSession instrSession,
                                    ViString  repeatedCapabilityName,
                                    ViAttr    attributeID,
                                    ViReal64  attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);
    error = RsCore_SetAttributeViReal64(instrSession, repeatedCapabilityName, attributeID, RS_VAL_DIRECT_USER_CALL, attributeValue);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the value of a ViString attribute directly.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR repeatedCapabilityName/A comma-separated list of any required repeated capability IDs.
/// HIPAR attributeID/The attribute ID constant.
/// HIPAR attributeValue/The value to set to the attribute.
ViStatus rsdsa_SetAttributeViString(ViSession instrSession,
                                    ViString  repeatedCapabilityName,
                                    ViAttr    attributeID,
                                    ViString  attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);
    error = RsCore_SetAttributeViStringOrRawString(instrSession, repeatedCapabilityName, attributeID, RS_VAL_DIRECT_USER_CALL, attributeValue);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the value of a ViBoolean attribute directly.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR repeatedCapabilityName/A comma-separated list of any required repeated capability IDs.
/// HIPAR attributeID/The attribute ID constant.
/// HIPAR attributeValue/The value to set to the attribute.
ViStatus rsdsa_SetAttributeViBoolean(ViSession instrSession,
                                     ViString  repeatedCapabilityName,
                                     ViAttr    attributeID,
                                     ViBoolean attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);
    error = RsCore_SetAttributeViBoolean(instrSession, repeatedCapabilityName, attributeID, RS_VAL_DIRECT_USER_CALL, attributeValue);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
* Function: rsdsa_GetAttribute<type> Functions
* Purpose:  These functions enable the instrument driver user to get
*           attribute values directly.  There are typesafe versions for
*           ViInt32, ViInt64, ViReal64, ViString, and ViBoolean data types.
*****************************************************************************/

/// HIFN  Gets the value of a ViInt32 attribute directly.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR repeatedCapabilityName/A comma-separated list of any required repeated capability IDs.
/// HIPAR attributeID/The attribute ID constant.
/// HIPAR attributeValue/The value returned by the attribute query.
ViStatus rsdsa_GetAttributeViInt32(ViSession instrSession,
                                   ViString  repeatedCapabilityName,
                                   ViAttr    attributeID,
                                   ViInt32*  attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);
    error = RsCore_GetAttributeViInt32(instrSession, repeatedCapabilityName, attributeID, RS_VAL_DIRECT_USER_CALL, attributeValue);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Gets the value of a ViInt64 attribute directly.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR repeatedCapabilityName/A comma-separated list of any required repeated capability IDs.
/// HIPAR attributeID/The attribute ID constant.
/// HIPAR attributeValue/The value returned by the attribute query.
ViStatus rsdsa_GetAttributeViInt64(ViSession instrSession,
                                   ViString  repeatedCapabilityName,
                                   ViAttr    attributeID,
                                   ViInt64*  attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);
    error = RsCore_GetAttributeViInt64(instrSession, repeatedCapabilityName, attributeID, RS_VAL_DIRECT_USER_CALL, attributeValue);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Gets the value of a ViReal64 attribute directly.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR repeatedCapabilityName/A comma-separated list of any required repeated capability IDs.
/// HIPAR attributeID/The attribute ID constant.
/// HIPAR attributeValue/The value returned by the attribute query.
ViStatus rsdsa_GetAttributeViReal64(ViSession instrSession,
                                    ViString  repeatedCapabilityName,
                                    ViAttr    attributeID,
                                    ViReal64* attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);
    error = RsCore_GetAttributeViReal64(instrSession, repeatedCapabilityName, attributeID, RS_VAL_DIRECT_USER_CALL, attributeValue);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Gets the value of a ViString directly.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR repeatedCapabilityName/A comma-separated list of any required repeated capability IDs.
/// HIPAR attributeID/The attribute ID constant.
/// HIPAR bufSize/The size of the buffer allocated for the returned value.
/// HIPAR attributeValue/The value returned by the attribute query.
ViStatus rsdsa_GetAttributeViString(ViSession instrSession,
                                    ViString  repeatedCapabilityName,
                                    ViAttr    attributeID,
                                    ViInt32   bufferSize,
                                    ViChar    attributeValue[])
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);
    error = RsCore_GetAttributeViStringOrRawString(instrSession, repeatedCapabilityName, attributeID, RS_VAL_DIRECT_USER_CALL, bufferSize, attributeValue);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Gets the value of a ViBoolean attribute directly.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR repeatedCapabilityName/A comma-separated list of any required repeated capability IDs.
/// HIPAR attributeID/The attribute ID constant.
/// HIPAR attributeValue/The value returned by the attribute query.
ViStatus rsdsa_GetAttributeViBoolean(ViSession  instrSession,
                                     ViString   repeatedCapabilityName,
                                     ViAttr     attributeID,
                                     ViBoolean* attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);
    error = RsCore_GetAttributeViBoolean(instrSession, repeatedCapabilityName, attributeID, RS_VAL_DIRECT_USER_CALL, attributeValue);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: rsdsa_CheckStatus
 * Purpose:  This function checks the status of the instrument to detect
 *           whether the instrument has encountered an error. This function
 *           is called at the end of most exported driver functions. If the
 *           instrument reports an error, this function returns
 *           RS_ERROR_INSTRUMENT_SPECIFIC. The user can set the
 *           rscma_ATTR_QUERY_INSTRUMENT_STATUS attribute to VI_FALSE to disable this
 *           check and increase execution speed.
 *
 *           Note:  Call this function only when the session is locked.
 *****************************************************************************/
ViStatus rsdsa_CheckStatus(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    if (RsCore_QueryInstrStatus(instrSession) && !RsCore_Simulating(instrSession))
    {
        checkErr(RsCore_CheckStatus(instrSession, VI_SUCCESS));
    }

Error:
    return error;
}

/*****************************************************************************
 * Function: rsdsa_close
 * Purpose:  This function closes the instrument.
 *
 *           Note:  This function must unlock the session before calling
 *           Rs_Dispose.
 *****************************************************************************/
ViStatus rsdsa_close(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ViClose(instrSession));

Error:
    (void)RsCore_Dispose(instrSession);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
